/* valaprojectgenerator.vala
 *
 * Copyright (C) 2007-2008  Jürg Billeter
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.

 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.

 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 *
 * Author:
 * 	Jürg Billeter <j@bitron.ch>
 * 	Andrea Del Signore <sejerpz@tin.it>
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <glib/gi18n-lib.h>
#include <float.h>
#include <math.h>
#include <glib/gstdio.h>
#include <config.h>
#include <stdio.h>
#include <gobject/gvaluecollector.h>


#define VALA_TYPE_PROJECT_TYPE (vala_project_type_get_type ())

#define VALA_TYPE_PROJECT_LICENSE (vala_project_license_get_type ())

#define VALA_TYPE_PROJECT_GENERATOR (vala_project_generator_get_type ())
#define VALA_PROJECT_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGenerator))
#define VALA_PROJECT_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorClass))
#define VALA_IS_PROJECT_GENERATOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VALA_TYPE_PROJECT_GENERATOR))
#define VALA_IS_PROJECT_GENERATOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VALA_TYPE_PROJECT_GENERATOR))
#define VALA_PROJECT_GENERATOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorClass))

typedef struct _ValaProjectGenerator ValaProjectGenerator;
typedef struct _ValaProjectGeneratorClass ValaProjectGeneratorClass;
typedef struct _ValaProjectGeneratorPrivate ValaProjectGeneratorPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))
#define _g_option_context_free0(var) ((var == NULL) ? NULL : (var = (g_option_context_free (var), NULL)))
#define _vala_project_generator_unref0(var) ((var == NULL) ? NULL : (var = (vala_project_generator_unref (var), NULL)))
typedef struct _ValaParamSpecProjectGenerator ValaParamSpecProjectGenerator;

typedef enum  {
	VALA_PROJECT_TYPE_CONSOLE_APPLICATION,
	VALA_PROJECT_TYPE_GTK_APPLICATION
} ValaProjectType;

typedef enum  {
	VALA_PROJECT_LICENSE_GPL2,
	VALA_PROJECT_LICENSE_GPL3,
	VALA_PROJECT_LICENSE_LGPL2,
	VALA_PROJECT_LICENSE_LGPL3
} ValaProjectLicense;

struct _ValaProjectGenerator {
	GTypeInstance parent_instance;
	volatile int ref_count;
	ValaProjectGeneratorPrivate * priv;
};

struct _ValaProjectGeneratorClass {
	GTypeClass parent_class;
	void (*finalize) (ValaProjectGenerator *self);
};

struct _ValaProjectGeneratorPrivate {
	GtkDialog* config_dialog;
	GtkFileChooserButton* project_folder_button;
	GtkComboBox* project_type_combobox;
	GtkComboBox* license_combobox;
	GtkEntry* name_entry;
	GtkEntry* email_entry;
	char* project_path;
	char* project_name;
	char* namespace_name;
	char* make_name;
	char* upper_case_make_name;
	char* real_name;
	char* email_address;
	ValaProjectType project_type;
	ValaProjectLicense project_license;
};

struct _ValaParamSpecProjectGenerator {
	GParamSpec parent_instance;
};


static char* vala_project_generator_option_project_path;
static char* vala_project_generator_option_project_path = NULL;
static ValaProjectType vala_project_generator_option_project_type;
static ValaProjectType vala_project_generator_option_project_type = VALA_PROJECT_TYPE_GTK_APPLICATION;
static ValaProjectLicense vala_project_generator_option_project_license;
static ValaProjectLicense vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_LGPL2;
static gboolean vala_project_generator_option_version;
static gboolean vala_project_generator_option_version = FALSE;
static char* vala_project_generator_option_author;
static char* vala_project_generator_option_author = NULL;
static char* vala_project_generator_option_email;
static char* vala_project_generator_option_email = NULL;
static char* vala_project_generator_option_project_name;
static char* vala_project_generator_option_project_name = NULL;
static char** vala_project_generator_option_files;
static gint vala_project_generator_option_files_length1;
static char** vala_project_generator_option_files = NULL;
static gint vala_project_generator_option_files_length1 = 0;
static gint vala_project_generator_option_files_size = 0;
static gpointer vala_project_generator_parent_class = NULL;

GType vala_project_type_get_type (void);
GType vala_project_license_get_type (void);
gpointer vala_project_generator_ref (gpointer instance);
void vala_project_generator_unref (gpointer instance);
GParamSpec* vala_param_spec_project_generator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vala_value_set_project_generator (GValue* value, gpointer v_object);
gpointer vala_value_get_project_generator (const GValue* value);
GType vala_project_generator_get_type (void);
#define VALA_PROJECT_GENERATOR_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VALA_TYPE_PROJECT_GENERATOR, ValaProjectGeneratorPrivate))
enum  {
	VALA_PROJECT_GENERATOR_DUMMY_PROPERTY
};
gboolean vala_project_generator_option_parse_callback (const char* option_name, const char* value, void* data, GError** error);
ValaProjectGenerator* vala_project_generator_new (void);
ValaProjectGenerator* vala_project_generator_construct (GType object_type);
static GtkHBox* vala_project_generator_create_hbox (ValaProjectGenerator* self, const char* title, GtkSizeGroup* size_group);
static void vala_project_generator_build_dialog_ui (ValaProjectGenerator* self);
GtkResponseType vala_project_generator_ask_parameters (ValaProjectGenerator* self);
static void vala_project_generator_write_autogen_sh (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_configure_ac (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_toplevel_makefile_am (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_src_makefile_am (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_main_vala (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_mainwindow_vala (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_authors (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_maintainers (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_linguas (ValaProjectGenerator* self, GError** error);
static void vala_project_generator_write_potfiles (ValaProjectGenerator* self, GError** error);
static char* vala_project_generator_get_automake_path (ValaProjectGenerator* self);
void vala_project_generator_create_project (ValaProjectGenerator* self);
static gboolean vala_project_generator_get_automake_has_native_vala_support (ValaProjectGenerator* self);
static char* vala_project_generator_generate_source_file_header (ValaProjectGenerator* self, const char* filename);
static gint vala_project_generator_main (char** args, int args_length1);
static void vala_project_generator_finalize (ValaProjectGenerator* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GOptionEntry VALA_PROJECT_GENERATOR_options[] = {{"projectdir", 'p', 0, G_OPTION_ARG_FILENAME, &vala_project_generator_option_project_path, "Project directory", "DIRECTORY"}, {"type", 't', 0, G_OPTION_ARG_CALLBACK, (void*) vala_project_generator_option_parse_callback, "Project TYPE: gtk+, console", "TYPE"}, {"license", 'l', 0, G_OPTION_ARG_CALLBACK, (void*) vala_project_generator_option_parse_callback, "License TYPE: gpl2, gpl3, lgpl2, lgpl3", "TYPE"}, {"version", (gchar) 0, 0, G_OPTION_ARG_NONE, &vala_project_generator_option_version, "Display version number", NULL}, {"author", 'a', 0, G_OPTION_ARG_STRING, &vala_project_generator_option_author, "Author name", "NAME"}, {"email", 'e', 0, G_OPTION_ARG_STRING, &vala_project_generator_option_email, "Author email address", "EMAIL"}, {"", (gchar) 0, 0, G_OPTION_ARG_FILENAME_ARRAY, &vala_project_generator_option_files, "Project NAME", "NAME"}, {NULL}};



GType vala_project_type_get_type (void) {
	static GType vala_project_type_type_id = 0;
	if (G_UNLIKELY (vala_project_type_type_id == 0)) {
		static const GEnumValue values[] = {{VALA_PROJECT_TYPE_CONSOLE_APPLICATION, "VALA_PROJECT_TYPE_CONSOLE_APPLICATION", "console-application"}, {VALA_PROJECT_TYPE_GTK_APPLICATION, "VALA_PROJECT_TYPE_GTK_APPLICATION", "gtk-application"}, {0, NULL, NULL}};
		vala_project_type_type_id = g_enum_register_static ("ValaProjectType", values);
	}
	return vala_project_type_type_id;
}



GType vala_project_license_get_type (void) {
	static GType vala_project_license_type_id = 0;
	if (G_UNLIKELY (vala_project_license_type_id == 0)) {
		static const GEnumValue values[] = {{VALA_PROJECT_LICENSE_GPL2, "VALA_PROJECT_LICENSE_GPL2", "gpl2"}, {VALA_PROJECT_LICENSE_GPL3, "VALA_PROJECT_LICENSE_GPL3", "gpl3"}, {VALA_PROJECT_LICENSE_LGPL2, "VALA_PROJECT_LICENSE_LGPL2", "lgpl2"}, {VALA_PROJECT_LICENSE_LGPL3, "VALA_PROJECT_LICENSE_LGPL3", "lgpl3"}, {0, NULL, NULL}};
		vala_project_license_type_id = g_enum_register_static ("ValaProjectLicense", values);
	}
	return vala_project_license_type_id;
}


gboolean vala_project_generator_option_parse_callback (const char* option_name, const char* value, void* data, GError** error) {
	gboolean result;
	GError * _inner_error_;
	gboolean _tmp0_;
	g_return_val_if_fail (option_name != NULL, FALSE);
	g_return_val_if_fail (value != NULL, FALSE);
	_inner_error_ = NULL;
	_tmp0_ = FALSE;
	if (_vala_strcmp0 (option_name, "--type") == 0) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = _vala_strcmp0 (option_name, "-t") == 0;
	}
	if (_tmp0_) {
		if (_vala_strcmp0 (value, "gtk+") == 0) {
			vala_project_generator_option_project_type = VALA_PROJECT_TYPE_GTK_APPLICATION;
		} else {
			if (_vala_strcmp0 (value, "console") == 0) {
				vala_project_generator_option_project_type = VALA_PROJECT_TYPE_CONSOLE_APPLICATION;
			} else {
				char* _tmp1_;
				GError* _tmp2_;
				_inner_error_ = (_tmp2_ = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, _tmp1_ = g_strdup_printf (_ ("project of type %s is not supported"), value)), _g_free0 (_tmp1_), _tmp2_);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_OPTION_ERROR) {
						g_propagate_error (error, _inner_error_);
						return FALSE;
					} else {
						g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
						g_clear_error (&_inner_error_);
						return FALSE;
					}
				}
			}
		}
	} else {
		gboolean _tmp3_;
		_tmp3_ = FALSE;
		if (_vala_strcmp0 (option_name, "--license") == 0) {
			_tmp3_ = TRUE;
		} else {
			_tmp3_ = _vala_strcmp0 (option_name, "-l") == 0;
		}
		if (_tmp3_) {
			if (_vala_strcmp0 (value, "gpl2") == 0) {
				vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_GPL2;
			} else {
				if (_vala_strcmp0 (value, "gpl3") == 0) {
					vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_GPL3;
				} else {
					if (_vala_strcmp0 (value, "lgpl2") == 0) {
						vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_LGPL2;
					} else {
						if (_vala_strcmp0 (value, "lgpl3") == 0) {
							vala_project_generator_option_project_license = VALA_PROJECT_LICENSE_LGPL3;
						} else {
							char* _tmp4_;
							GError* _tmp5_;
							_inner_error_ = (_tmp5_ = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, _tmp4_ = g_strdup_printf (_ ("license of type %s is not available"), value)), _g_free0 (_tmp4_), _tmp5_);
							if (_inner_error_ != NULL) {
								if (_inner_error_->domain == G_OPTION_ERROR) {
									g_propagate_error (error, _inner_error_);
									return FALSE;
								} else {
									g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
									g_clear_error (&_inner_error_);
									return FALSE;
								}
							}
						}
					}
				}
			}
		} else {
			char* _tmp6_;
			GError* _tmp7_;
			_inner_error_ = (_tmp7_ = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_UNKNOWN_OPTION, _tmp6_ = g_strdup_printf (_ ("unknown option %s"), option_name)), _g_free0 (_tmp6_), _tmp7_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_OPTION_ERROR) {
					g_propagate_error (error, _inner_error_);
					return FALSE;
				} else {
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return FALSE;
				}
			}
		}
	}
	result = TRUE;
	return result;
}


ValaProjectGenerator* vala_project_generator_construct (GType object_type) {
	ValaProjectGenerator* self;
	char* _tmp0_;
	char* _tmp1_;
	self = (ValaProjectGenerator*) g_type_create_instance (object_type);
	self->priv->project_path = (_tmp0_ = g_strdup (vala_project_generator_option_project_path), _g_free0 (self->priv->project_path), _tmp0_);
	self->priv->project_name = (_tmp1_ = g_strdup (vala_project_generator_option_project_name), _g_free0 (self->priv->project_name), _tmp1_);
	return self;
}


ValaProjectGenerator* vala_project_generator_new (void) {
	return vala_project_generator_construct (VALA_TYPE_PROJECT_GENERATOR);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static GtkHBox* vala_project_generator_create_hbox (ValaProjectGenerator* self, const char* title, GtkSizeGroup* size_group) {
	GtkHBox* result;
	GtkHBox* hbox;
	GtkVBox* vbox;
	GtkLabel* label;
	GtkLabel* _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (title != NULL, NULL);
	g_return_val_if_fail (size_group != NULL, NULL);
	hbox = g_object_ref_sink ((GtkHBox*) gtk_hbox_new (FALSE, 6));
	vbox = _g_object_ref0 (self->priv->config_dialog->vbox);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) hbox, FALSE, FALSE, (guint) 0);
	gtk_widget_show ((GtkWidget*) hbox);
	label = g_object_ref_sink ((GtkLabel*) gtk_label_new ("    "));
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	gtk_widget_show ((GtkWidget*) label);
	label = (_tmp0_ = g_object_ref_sink ((GtkLabel*) gtk_label_new (title)), _g_object_unref0 (label), _tmp0_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	g_object_set ((GtkMisc*) label, "xalign", (float) 0, NULL);
	gtk_widget_show ((GtkWidget*) label);
	gtk_size_group_add_widget (size_group, (GtkWidget*) label);
	result = hbox;
	_g_object_unref0 (vbox);
	_g_object_unref0 (label);
	return result;
}


static void vala_project_generator_build_dialog_ui (ValaProjectGenerator* self) {
	GtkDialog* _tmp0_;
	GtkVBox* vbox;
	GtkSizeGroup* size_group;
	GtkLabel* label;
	GtkHBox* hbox;
	GtkLabel* _tmp1_;
	GtkHBox* _tmp5_;
	GtkComboBox* _tmp6_;
	GtkHBox* _tmp7_;
	GtkComboBox* _tmp8_;
	GtkLabel* _tmp9_;
	GtkHBox* _tmp10_;
	GtkEntry* _tmp11_;
	GtkHBox* _tmp14_;
	GtkEntry* _tmp15_;
	GtkWidget* ok_button;
	g_return_if_fail (self != NULL);
	self->priv->config_dialog = (_tmp0_ = g_object_ref_sink ((GtkDialog*) gtk_dialog_new ()), _g_object_unref0 (self->priv->config_dialog), _tmp0_);
	gtk_window_set_title ((GtkWindow*) self->priv->config_dialog, "Vala Project Generator");
	gtk_container_set_border_width ((GtkContainer*) self->priv->config_dialog, (guint) 12);
	gtk_dialog_set_has_separator (self->priv->config_dialog, FALSE);
	vbox = _g_object_ref0 (self->priv->config_dialog->vbox);
	size_group = gtk_size_group_new (GTK_SIZE_GROUP_HORIZONTAL);
	label = NULL;
	hbox = NULL;
	gtk_box_set_spacing ((GtkBox*) vbox, 6);
	label = (_tmp1_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("<b>Project</b>")), _g_object_unref0 (label), _tmp1_);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	gtk_label_set_use_markup (label, TRUE);
	g_object_set ((GtkMisc*) label, "xalign", (float) 0, NULL);
	gtk_widget_show ((GtkWidget*) label);
	if (vala_project_generator_option_project_path == NULL) {
		GtkHBox* _tmp2_;
		GtkFileChooserButton* _tmp3_;
		hbox = (_tmp2_ = vala_project_generator_create_hbox (self, "Project folder:", size_group), _g_object_unref0 (hbox), _tmp2_);
		self->priv->project_folder_button = (_tmp3_ = g_object_ref_sink ((GtkFileChooserButton*) gtk_file_chooser_button_new ("Select a project folder", GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER)), _g_object_unref0 (self->priv->project_folder_button), _tmp3_);
		gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->project_folder_button, TRUE, TRUE, (guint) 0);
		gtk_widget_show ((GtkWidget*) self->priv->project_folder_button);
	} else {
		char* _tmp4_;
		self->priv->project_path = (_tmp4_ = g_strdup (vala_project_generator_option_project_path), _g_free0 (self->priv->project_path), _tmp4_);
	}
	hbox = (_tmp5_ = vala_project_generator_create_hbox (self, "Project type:", size_group), _g_object_unref0 (hbox), _tmp5_);
	self->priv->project_type_combobox = (_tmp6_ = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), _g_object_unref0 (self->priv->project_type_combobox), _tmp6_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->project_type_combobox, TRUE, TRUE, (guint) 0);
	gtk_combo_box_append_text (self->priv->project_type_combobox, "Console Application");
	gtk_combo_box_append_text (self->priv->project_type_combobox, "GTK+ Application");
	gtk_combo_box_set_active (self->priv->project_type_combobox, (gint) vala_project_generator_option_project_type);
	gtk_widget_show ((GtkWidget*) self->priv->project_type_combobox);
	hbox = (_tmp7_ = vala_project_generator_create_hbox (self, "License:", size_group), _g_object_unref0 (hbox), _tmp7_);
	self->priv->license_combobox = (_tmp8_ = g_object_ref_sink ((GtkComboBox*) gtk_combo_box_new_text ()), _g_object_unref0 (self->priv->license_combobox), _tmp8_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->license_combobox, TRUE, TRUE, (guint) 0);
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU General Public License, version 2 or later");
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU General Public License, version 3 or later");
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU Lesser General Public License, version 2.1 or later");
	gtk_combo_box_append_text (self->priv->license_combobox, "GNU Lesser General Public License, version 3 or later");
	gtk_combo_box_set_active (self->priv->license_combobox, (gint) vala_project_generator_option_project_license);
	gtk_widget_show ((GtkWidget*) self->priv->license_combobox);
	label = (_tmp9_ = g_object_ref_sink ((GtkLabel*) gtk_label_new ("<b>Author</b>")), _g_object_unref0 (label), _tmp9_);
	gtk_box_pack_start ((GtkBox*) vbox, (GtkWidget*) label, FALSE, FALSE, (guint) 0);
	gtk_label_set_use_markup (label, TRUE);
	g_object_set ((GtkMisc*) label, "xalign", (float) 0, NULL);
	gtk_widget_show ((GtkWidget*) label);
	hbox = (_tmp10_ = vala_project_generator_create_hbox (self, "Name:", size_group), _g_object_unref0 (hbox), _tmp10_);
	self->priv->name_entry = (_tmp11_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), _g_object_unref0 (self->priv->name_entry), _tmp11_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->name_entry, TRUE, TRUE, (guint) 0);
	if (vala_project_generator_option_author != NULL) {
		char* _tmp12_;
		self->priv->real_name = (_tmp12_ = g_strdup (vala_project_generator_option_author), _g_free0 (self->priv->real_name), _tmp12_);
	} else {
		char* _tmp13_;
		self->priv->real_name = (_tmp13_ = g_strdup (g_getenv ("REAL_NAME")), _g_free0 (self->priv->real_name), _tmp13_);
	}
	if (self->priv->real_name != NULL) {
		gtk_entry_set_text (self->priv->name_entry, self->priv->real_name);
	}
	gtk_widget_show ((GtkWidget*) self->priv->name_entry);
	hbox = (_tmp14_ = vala_project_generator_create_hbox (self, "E-mail address:", size_group), _g_object_unref0 (hbox), _tmp14_);
	self->priv->email_entry = (_tmp15_ = g_object_ref_sink ((GtkEntry*) gtk_entry_new ()), _g_object_unref0 (self->priv->email_entry), _tmp15_);
	gtk_box_pack_start ((GtkBox*) hbox, (GtkWidget*) self->priv->email_entry, TRUE, TRUE, (guint) 0);
	if (vala_project_generator_option_email != NULL) {
		char* _tmp16_;
		self->priv->email_address = (_tmp16_ = g_strdup (vala_project_generator_option_email), _g_free0 (self->priv->email_address), _tmp16_);
	} else {
		char* _tmp17_;
		self->priv->email_address = (_tmp17_ = g_strdup (g_getenv ("EMAIL_ADDRESS")), _g_free0 (self->priv->email_address), _tmp17_);
	}
	if (self->priv->email_address != NULL) {
		gtk_entry_set_text (self->priv->email_entry, self->priv->email_address);
	}
	gtk_widget_show ((GtkWidget*) self->priv->email_entry);
	gtk_dialog_add_button (self->priv->config_dialog, GTK_STOCK_CANCEL, (gint) GTK_RESPONSE_CANCEL);
	ok_button = _g_object_ref0 (gtk_dialog_add_button (self->priv->config_dialog, "Create Project", (gint) GTK_RESPONSE_OK));
	gtk_widget_grab_default (ok_button);
	_g_object_unref0 (vbox);
	_g_object_unref0 (size_group);
	_g_object_unref0 (label);
	_g_object_unref0 (hbox);
	_g_object_unref0 (ok_button);
}


GtkResponseType vala_project_generator_ask_parameters (ValaProjectGenerator* self) {
	GtkResponseType result;
	GtkResponseType response;
	g_return_val_if_fail (self != NULL, 0);
	response = 0;
	vala_project_generator_build_dialog_ui (self);
	response = (GtkResponseType) gtk_dialog_run (self->priv->config_dialog);
	if (response == GTK_RESPONSE_OK) {
		char* _tmp4_;
		char* _tmp5_;
		if (vala_project_generator_option_project_path == NULL) {
			char* _tmp0_;
			self->priv->project_path = (_tmp0_ = gtk_file_chooser_get_current_folder ((GtkFileChooser*) self->priv->project_folder_button), _g_free0 (self->priv->project_path), _tmp0_);
		} else {
			char* _tmp1_;
			self->priv->project_path = (_tmp1_ = g_strdup (vala_project_generator_option_project_path), _g_free0 (self->priv->project_path), _tmp1_);
		}
		if (vala_project_generator_option_project_name == NULL) {
			char* _tmp2_;
			self->priv->project_name = (_tmp2_ = g_path_get_basename (self->priv->project_path), _g_free0 (self->priv->project_name), _tmp2_);
		} else {
			char* _tmp3_;
			self->priv->project_name = (_tmp3_ = g_strdup (vala_project_generator_option_project_name), _g_free0 (self->priv->project_name), _tmp3_);
		}
		self->priv->project_type = (ValaProjectType) gtk_combo_box_get_active (self->priv->project_type_combobox);
		self->priv->project_license = (ValaProjectLicense) gtk_combo_box_get_active (self->priv->license_combobox);
		self->priv->real_name = (_tmp4_ = g_strdup (gtk_entry_get_text (self->priv->name_entry)), _g_free0 (self->priv->real_name), _tmp4_);
		self->priv->email_address = (_tmp5_ = g_strdup (gtk_entry_get_text (self->priv->email_entry)), _g_free0 (self->priv->email_address), _tmp5_);
	}
	result = response;
	return result;
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


void vala_project_generator_create_project (ValaProjectGenerator* self) {
	GError * _inner_error_;
	GString* project_name_str;
	GString* make_name_str;
	GString* namespace_name_str;
	char* _tmp7_;
	char* _tmp11_;
	char* _tmp10_;
	char* _tmp9_;
	char* _tmp8_;
	char* _tmp12_;
	char* _tmp13_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	project_name_str = g_string_new ("");
	make_name_str = g_string_new ("");
	namespace_name_str = g_string_new ("");
	{
		gint i;
		i = 0;
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				gunichar c;
				gboolean _tmp1_;
				gboolean _tmp2_;
				gboolean _tmp3_;
				if (!_tmp0_) {
					i++;
				}
				_tmp0_ = FALSE;
				if (!(i < g_utf8_strlen (self->priv->project_name, -1))) {
					break;
				}
				c = g_utf8_get_char (g_utf8_offset_to_pointer (self->priv->project_name, i));
				_tmp1_ = FALSE;
				_tmp2_ = FALSE;
				_tmp3_ = FALSE;
				if (c >= 'a') {
					_tmp3_ = c <= 'z';
				} else {
					_tmp3_ = FALSE;
				}
				if (_tmp3_) {
					_tmp2_ = TRUE;
				} else {
					gboolean _tmp4_;
					_tmp4_ = FALSE;
					if (c >= 'A') {
						_tmp4_ = c <= 'Z';
					} else {
						_tmp4_ = FALSE;
					}
					_tmp2_ = _tmp4_;
				}
				if (_tmp2_) {
					_tmp1_ = TRUE;
				} else {
					gboolean _tmp5_;
					_tmp5_ = FALSE;
					if (c >= '0') {
						_tmp5_ = c <= '9';
					} else {
						_tmp5_ = FALSE;
					}
					_tmp1_ = _tmp5_;
				}
				if (_tmp1_) {
					g_string_append_unichar (project_name_str, c);
					g_string_append_unichar (make_name_str, c);
					g_string_append_unichar (namespace_name_str, c);
				} else {
					gboolean _tmp6_;
					_tmp6_ = FALSE;
					if (c == '-') {
						_tmp6_ = TRUE;
					} else {
						_tmp6_ = c == ' ';
					}
					if (_tmp6_) {
						g_string_append_unichar (project_name_str, (gunichar) '-');
						g_string_append_unichar (make_name_str, (gunichar) '_');
					}
				}
			}
		}
	}
	self->priv->project_name = (_tmp7_ = g_strdup (project_name_str->str), _g_free0 (self->priv->project_name), _tmp7_);
	self->priv->namespace_name = (_tmp11_ = g_strconcat (_tmp9_ = g_utf8_strup (_tmp8_ = string_substring (namespace_name_str->str, (glong) 0, (glong) 1), -1), _tmp10_ = string_substring (namespace_name_str->str, (glong) 1, g_utf8_strlen (namespace_name_str->str, -1) - 1), NULL), _g_free0 (self->priv->namespace_name), _tmp11_);
	_g_free0 (_tmp10_);
	_g_free0 (_tmp9_);
	_g_free0 (_tmp8_);
	self->priv->make_name = (_tmp12_ = g_strdup (make_name_str->str), _g_free0 (self->priv->make_name), _tmp12_);
	self->priv->upper_case_make_name = (_tmp13_ = g_utf8_strup (self->priv->make_name, -1), _g_free0 (self->priv->upper_case_make_name), _tmp13_);
	{
		char* _tmp14_;
		char* _tmp15_;
		char* _tmp16_;
		char* _tmp17_;
		char* _tmp18_;
		char* _tmp19_;
		char* s;
		char* automake_path;
		char* license_filename;
		gboolean _tmp32_;
		vala_project_generator_write_autogen_sh (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_configure_ac (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		g_mkdir (_tmp14_ = g_strconcat (self->priv->project_path, "/src", NULL), 0777);
		_g_free0 (_tmp14_);
		g_mkdir (_tmp15_ = g_strconcat (self->priv->project_path, "/po", NULL), 0777);
		_g_free0 (_tmp15_);
		vala_project_generator_write_toplevel_makefile_am (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_src_makefile_am (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		if (self->priv->project_type == VALA_PROJECT_TYPE_CONSOLE_APPLICATION) {
			vala_project_generator_write_main_vala (self, &_inner_error_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
		} else {
			if (self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION) {
				vala_project_generator_write_mainwindow_vala (self, &_inner_error_);
				if (_inner_error_ != NULL) {
					if (_inner_error_->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
			}
		}
		vala_project_generator_write_authors (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_maintainers (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_linguas (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		vala_project_generator_write_potfiles (self, &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		g_file_set_contents (_tmp16_ = g_strconcat (self->priv->project_path, "/NEWS", NULL), "", (glong) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_g_free0 (_tmp16_);
		g_file_set_contents (_tmp17_ = g_strconcat (self->priv->project_path, "/README", NULL), "", (glong) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_g_free0 (_tmp17_);
		g_file_set_contents (_tmp18_ = g_strconcat (self->priv->project_path, "/ChangeLog", NULL), "", (glong) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_g_free0 (_tmp18_);
		g_file_set_contents (_tmp19_ = g_strconcat (self->priv->project_path, "/po/ChangeLog", NULL), "", (glong) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				goto __catch0_g_file_error;
			}
			goto __finally0;
		}
		_g_free0 (_tmp19_);
		s = NULL;
		automake_path = vala_project_generator_get_automake_path (self);
		if (automake_path != NULL) {
			char* install_filename;
			install_filename = g_strconcat (automake_path, "/INSTALL", NULL);
			if (g_file_test (install_filename, G_FILE_TEST_EXISTS)) {
				char* _tmp22_;
				gboolean _tmp21_;
				char* _tmp20_;
				char* _tmp23_;
				_tmp20_ = NULL;
				_tmp21_ = g_file_get_contents (install_filename, &_tmp20_, NULL, &_inner_error_);
				s = (_tmp22_ = _tmp20_, _g_free0 (s), _tmp22_);
				_tmp21_;
				if (_inner_error_ != NULL) {
					_g_free0 (install_filename);
					_g_free0 (s);
					_g_free0 (automake_path);
					if (_inner_error_->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
				g_file_set_contents (_tmp23_ = g_strconcat (self->priv->project_path, "/INSTALL", NULL), s, (glong) (-1), &_inner_error_);
				if (_inner_error_ != NULL) {
					_g_free0 (install_filename);
					_g_free0 (s);
					_g_free0 (automake_path);
					if (_inner_error_->domain == G_FILE_ERROR) {
						goto __catch0_g_file_error;
					}
					goto __finally0;
				}
				_g_free0 (_tmp23_);
			}
			_g_free0 (install_filename);
		}
		license_filename = NULL;
		if (self->priv->project_license == VALA_PROJECT_LICENSE_GPL2) {
			char* _tmp24_;
			license_filename = (_tmp24_ = g_strdup (PACKAGE_DATADIR "/licenses/gpl-2.0.txt"), _g_free0 (license_filename), _tmp24_);
			if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
				char* _tmp25_;
				license_filename = (_tmp25_ = g_strdup ("/usr/share/common-licenses/GPL-2"), _g_free0 (license_filename), _tmp25_);
			}
		} else {
			if (self->priv->project_license == VALA_PROJECT_LICENSE_LGPL2) {
				char* _tmp26_;
				license_filename = (_tmp26_ = g_strdup (PACKAGE_DATADIR "/licenses/lgpl-2.1.txt"), _g_free0 (license_filename), _tmp26_);
				if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
					char* _tmp27_;
					license_filename = (_tmp27_ = g_strdup ("/usr/share/common-licenses/LGPL-2.1"), _g_free0 (license_filename), _tmp27_);
				}
			} else {
				if (self->priv->project_license == VALA_PROJECT_LICENSE_GPL3) {
					char* _tmp28_;
					license_filename = (_tmp28_ = g_strdup (PACKAGE_DATADIR "/licenses/gpl-3.0.txt"), _g_free0 (license_filename), _tmp28_);
					if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
						char* _tmp29_;
						license_filename = (_tmp29_ = g_strdup ("/usr/share/common-licenses/GPL-3"), _g_free0 (license_filename), _tmp29_);
					}
				} else {
					if (self->priv->project_license == VALA_PROJECT_LICENSE_LGPL3) {
						char* _tmp30_;
						license_filename = (_tmp30_ = g_strdup (PACKAGE_DATADIR "/licenses/lgpl-3.0.txt"), _g_free0 (license_filename), _tmp30_);
						if (!g_file_test (license_filename, G_FILE_TEST_EXISTS)) {
							char* _tmp31_;
							license_filename = (_tmp31_ = g_strdup ("/usr/share/common-licenses/LGPL-3"), _g_free0 (license_filename), _tmp31_);
						}
					}
				}
			}
		}
		_tmp32_ = FALSE;
		if (license_filename != NULL) {
			_tmp32_ = g_file_test (license_filename, G_FILE_TEST_EXISTS);
		} else {
			_tmp32_ = FALSE;
		}
		if (_tmp32_) {
			char* _tmp35_;
			gboolean _tmp34_;
			char* _tmp33_;
			char* _tmp36_;
			_tmp33_ = NULL;
			_tmp34_ = g_file_get_contents (license_filename, &_tmp33_, NULL, &_inner_error_);
			s = (_tmp35_ = _tmp33_, _g_free0 (s), _tmp35_);
			_tmp34_;
			if (_inner_error_ != NULL) {
				_g_free0 (s);
				_g_free0 (automake_path);
				_g_free0 (license_filename);
				if (_inner_error_->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
			g_file_set_contents (_tmp36_ = g_strconcat (self->priv->project_path, "/COPYING", NULL), s, (glong) (-1), &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_free0 (s);
				_g_free0 (automake_path);
				_g_free0 (license_filename);
				if (_inner_error_->domain == G_FILE_ERROR) {
					goto __catch0_g_file_error;
				}
				goto __finally0;
			}
			_g_free0 (_tmp36_);
		}
		_g_free0 (s);
		_g_free0 (automake_path);
		_g_free0 (license_filename);
	}
	goto __finally0;
	__catch0_g_file_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_critical ("valaprojectgenerator.vala:318: Error while creating project: %s", e->message);
			_g_error_free0 (e);
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		_g_string_free0 (project_name_str);
		_g_string_free0 (make_name_str);
		_g_string_free0 (namespace_name_str);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_g_string_free0 (project_name_str);
	_g_string_free0 (make_name_str);
	_g_string_free0 (namespace_name_str);
}


static void vala_project_generator_write_autogen_sh (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	GString* s;
	char* _tmp0_;
	char* _tmp1_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_string_new ("");
	g_string_append (s, "#!/bin/sh\n");
	g_string_append (s, "# Run this to generate all the initial makefiles, etc.\n\n");
	g_string_append (s, "srcdir=`dirname $0`\n");
	g_string_append (s, "test -z \"$srcdir\" && srcdir=.\n\n");
	g_string_append_printf (s, "PKG_NAME=\"%s\"\n\n", self->priv->project_name);
	g_string_append (s, ". gnome-autogen.sh\n");
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/autogen.sh", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	g_chmod (_tmp1_ = g_strconcat (self->priv->project_path, "/autogen.sh", NULL), 0755);
	_g_free0 (_tmp1_);
	_g_string_free0 (s);
}


static void vala_project_generator_write_configure_ac (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	gboolean use_gtk;
	GString* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	use_gtk = self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION;
	s = g_string_new ("");
	g_string_append_printf (s, "AC_INIT([%s], [0.1.0], [%s], [%s])\n", self->priv->project_name, self->priv->email_address, self->priv->project_name);
	g_string_append (s, "AC_CONFIG_SRCDIR([Makefile.am])\n");
	g_string_append (s, "AC_CONFIG_HEADERS(config.h)\n");
	g_string_append (s, "AM_INIT_AUTOMAKE([dist-bzip2])\n");
	g_string_append (s, "AM_MAINTAINER_MODE\n\n");
	g_string_append (s, "AC_PROG_CC\n");
	g_string_append (s, "AM_PROG_CC_C_O\n");
	g_string_append (s, "AC_DISABLE_STATIC\n");
	g_string_append (s, "AC_PROG_LIBTOOL\n\n");
	g_string_append (s, "AC_PATH_PROG(VALAC, valac, valac)\n");
	g_string_append (s, "AC_SUBST(VALAC)\n\n");
	g_string_append (s, "AH_TEMPLATE([GETTEXT_PACKAGE], [Package name for gettext])\n");
	g_string_append_printf (s, "GETTEXT_PACKAGE=%s\n", self->priv->project_name);
	g_string_append (s, "AC_DEFINE_UNQUOTED(GETTEXT_PACKAGE, \"$GETTEXT_PACKAGE\")\n");
	g_string_append (s, "AC_SUBST(GETTEXT_PACKAGE)\n");
	g_string_append (s, "AM_GLIB_GNU_GETTEXT\n");
	g_string_append (s, "IT_PROG_INTLTOOL([0.35.0])\n\n");
	g_string_append (s, "AC_SUBST(CFLAGS)\n");
	g_string_append (s, "AC_SUBST(CPPFLAGS)\n");
	g_string_append (s, "AC_SUBST(LDFLAGS)\n\n");
	g_string_append (s, "GLIB_REQUIRED=2.12.0\n");
	if (use_gtk) {
		g_string_append (s, "GTK_REQUIRED=2.10.0\n");
	}
	g_string_append (s, "\n");
	g_string_append_printf (s, "PKG_CHECK_MODULES(%s, glib-2.0 >= $GLIB_REQUIRED gobject-2.0 >= $GLIB_REQUIRED", self->priv->upper_case_make_name);
	if (use_gtk) {
		g_string_append (s, " gtk+-2.0 >= $GTK_REQUIRED");
	}
	g_string_append (s, ")\n");
	g_string_append_printf (s, "AC_SUBST(%s_CFLAGS)\n", self->priv->upper_case_make_name);
	g_string_append_printf (s, "AC_SUBST(%s_LIBS)\n\n", self->priv->upper_case_make_name);
	g_string_append (s, "AC_CONFIG_FILES([Makefile\n");
	g_string_append (s, "\tsrc/Makefile\n");
	g_string_append (s, "\tpo/Makefile.in])\n\n");
	g_string_append (s, "AC_OUTPUT\n");
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/configure.ac", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	_g_string_free0 (s);
}


static void vala_project_generator_write_toplevel_makefile_am (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	GString* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_string_new ("");
	g_string_append (s, "NULL = \n\n");
	g_string_append (s, "#Build in these directories:\n\n");
	g_string_append (s, "SUBDIRS = \\\n");
	g_string_append (s, "\tsrc \\\n");
	g_string_append (s, "\tpo \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append_printf (s, "%sdocdir = ${prefix}/doc/%s\n", self->priv->project_name, self->priv->project_name);
	g_string_append_printf (s, "%sdoc_DATA = \\\n", self->priv->project_name);
	g_string_append (s, "\tChangeLog \\\n");
	g_string_append (s, "\tREADME \\\n");
	g_string_append (s, "\tCOPYING \\\n");
	g_string_append (s, "\tAUTHORS \\\n");
	g_string_append (s, "\tINSTALL \\\n");
	g_string_append (s, "\tNEWS\\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "EXTRA_DIST = \\\n");
	g_string_append_printf (s, "\t$(%sdoc_DATA) \\\n", self->priv->project_name);
	g_string_append (s, "\tintltool-extract.in \\\n");
	g_string_append (s, "\tintltool-merge.in \\\n");
	g_string_append (s, "\tintltool-update.in\\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "DISTCLEANFILES = \\\n");
	g_string_append (s, "\tintltool-extract \\\n");
	g_string_append (s, "\tintltool-merge \\\n");
	g_string_append (s, "\tintltool-update \\\n");
	g_string_append (s, "\tpo/.intltool-merge-cache \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/Makefile.am", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	_g_string_free0 (s);
}


static void vala_project_generator_write_src_makefile_am (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	gboolean use_gtk;
	gboolean am_has_vala_support;
	GString* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	use_gtk = self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION;
	am_has_vala_support = vala_project_generator_get_automake_has_native_vala_support (self);
	s = g_string_new ("");
	g_string_append (s, "NULL = \n\n");
	g_string_append (s, "AM_CPPFLAGS = \\\n");
	g_string_append_printf (s, "\t$(%s_CFLAGS) \\\n", self->priv->upper_case_make_name);
	g_string_append (s, "\t-include $(CONFIG_HEADER) \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	if (!am_has_vala_support) {
		g_string_append_printf (s, "BUILT_SOURCES = %s.vala.stamp\n\n", self->priv->project_name);
	}
	g_string_append_printf (s, "bin_PROGRAMS = %s\n\n", self->priv->project_name);
	if (am_has_vala_support) {
		g_string_append_printf (s, "%s_SOURCES = \\\n", self->priv->make_name);
	} else {
		g_string_append_printf (s, "%s_VALASOURCES = \\\n", self->priv->make_name);
	}
	if (use_gtk) {
		g_string_append (s, "\tmainwindow.vala \\\n");
	} else {
		g_string_append (s, "\tmain.vala \\\n");
	}
	g_string_append (s, "\t$(NULL)\n\n");
	if (am_has_vala_support) {
		if (use_gtk) {
			g_string_append_printf (s, "%s_VALAFLAGS = --pkg gtk+-2.0\n", self->priv->make_name);
			g_string_append (s, "\n");
		}
	} else {
		g_string_append_printf (s, "%s_SOURCES = \\\n", self->priv->make_name);
		g_string_append_printf (s, "\t$(%s_VALASOURCES:.vala=.c) \\\n", self->priv->make_name);
		g_string_append (s, "\t$(NULL)\n\n");
		g_string_append_printf (s, "%s.vala.stamp: $(%s_VALASOURCES)\n", self->priv->project_name, self->priv->make_name);
		g_string_append (s, "\t$(VALAC) -C ");
		if (use_gtk) {
			g_string_append (s, "--pkg gtk+-2.0 ");
		}
		g_string_append (s, "--basedir $(top_srcdir)/src $^\n");
		g_string_append (s, "\ttouch $@\n\n");
	}
	g_string_append_printf (s, "%s_LDADD = \\\n", self->priv->make_name);
	g_string_append_printf (s, "\t$(%s_LIBS) \\\n", self->priv->upper_case_make_name);
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "EXTRA_DIST = \\\n");
	if (!am_has_vala_support) {
		g_string_append_printf (s, "\t$(%s_VALASOURCES) \\\n", self->priv->make_name);
		g_string_append_printf (s, "\t%s.vala.stamp \\\n", self->priv->project_name);
	}
	g_string_append (s, "\t$(NULL)\n\n");
	g_string_append (s, "DISTCLEANFILES = \\\n");
	g_string_append (s, "\t$(NULL)\n\n");
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/src/Makefile.am", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	_g_string_free0 (s);
}


static char* vala_project_generator_generate_source_file_header (ValaProjectGenerator* self, const char* filename) {
	char* result;
	GString* s;
	GTimeVal _tmp0_ = {0};
	GTimeVal tv;
	GDate _tmp1_ = {0};
	GDate d;
	char* license_name;
	char* license_version;
	char* program_type;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (filename != NULL, NULL);
	s = g_string_new ("");
	tv = (g_get_current_time (&_tmp0_), _tmp0_);
	d = (memset (&_tmp1_, 0, sizeof (GDate)), _tmp1_);
	g_date_set_time_val (&d, &tv);
	g_string_append_printf (s, "/* %s\n", filename);
	g_string_append (s, " *\n");
	g_string_append_printf (s, " * Copyright (C) %d  %s\n", (gint) g_date_get_year (&d), self->priv->real_name);
	g_string_append (s, " *\n");
	license_name = g_strdup ("");
	license_version = NULL;
	program_type = NULL;
	switch (self->priv->project_license) {
		case VALA_PROJECT_LICENSE_GPL2:
		{
			char* _tmp2_;
			char* _tmp3_;
			char* _tmp4_;
			license_name = (_tmp2_ = g_strdup ("GNU General Public License"), _g_free0 (license_name), _tmp2_);
			license_version = (_tmp3_ = g_strdup ("2"), _g_free0 (license_version), _tmp3_);
			program_type = (_tmp4_ = g_strdup ("program"), _g_free0 (program_type), _tmp4_);
			break;
		}
		case VALA_PROJECT_LICENSE_GPL3:
		{
			char* _tmp5_;
			char* _tmp6_;
			char* _tmp7_;
			license_name = (_tmp5_ = g_strdup ("GNU General Public License"), _g_free0 (license_name), _tmp5_);
			license_version = (_tmp6_ = g_strdup ("3"), _g_free0 (license_version), _tmp6_);
			program_type = (_tmp7_ = g_strdup ("program"), _g_free0 (program_type), _tmp7_);
			break;
		}
		case VALA_PROJECT_LICENSE_LGPL2:
		{
			char* _tmp8_;
			char* _tmp9_;
			char* _tmp10_;
			license_name = (_tmp8_ = g_strdup ("GNU Lesser General Public License"), _g_free0 (license_name), _tmp8_);
			license_version = (_tmp9_ = g_strdup ("2.1"), _g_free0 (license_version), _tmp9_);
			program_type = (_tmp10_ = g_strdup ("library"), _g_free0 (program_type), _tmp10_);
			break;
		}
		case VALA_PROJECT_LICENSE_LGPL3:
		{
			char* _tmp11_;
			char* _tmp12_;
			char* _tmp13_;
			license_name = (_tmp11_ = g_strdup ("GNU Lesser General Public License"), _g_free0 (license_name), _tmp11_);
			license_version = (_tmp12_ = g_strdup ("3"), _g_free0 (license_version), _tmp12_);
			program_type = (_tmp13_ = g_strdup ("library"), _g_free0 (program_type), _tmp13_);
			break;
		}
	}
	g_string_append_printf (s, " * This %s is free software: you can redistribute it and/or modify\n", program_type);
	g_string_append_printf (s, " * it under the terms of the %s as published by\n", license_name);
	g_string_append_printf (s, " * the Free Software Foundation, either version %s of the License, or\n", license_version);
	g_string_append (s, " * (at your option) any later version.\n");
	g_string_append (s, " *\n");
	g_string_append_printf (s, " * This %s is distributed in the hope that it will be useful,\n", program_type);
	g_string_append (s, " * but WITHOUT ANY WARRANTY; without even the implied warranty of\n");
	g_string_append (s, " * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the\n");
	g_string_append_printf (s, " * %s for more details.\n", license_name);
	g_string_append (s, " *\n");
	g_string_append_printf (s, " * You should have received a copy of the %s\n", license_name);
	g_string_append_printf (s, " * along with this %s.  If not, see <http://www.gnu.org/licenses/>.\n", program_type);
	g_string_append (s, " *\n");
	g_string_append (s, " * Author:\n");
	g_string_append_printf (s, " * \t%s <%s>\n", self->priv->real_name, self->priv->email_address);
	g_string_append (s, " */\n\n");
	result = g_strdup (s->str);
	_g_string_free0 (s);
	_g_free0 (license_name);
	_g_free0 (license_version);
	_g_free0 (program_type);
	return result;
}


static void vala_project_generator_write_main_vala (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	GString* s;
	char* _tmp0_;
	char* _tmp1_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_string_new ("");
	g_string_append (s, _tmp0_ = vala_project_generator_generate_source_file_header (self, "main.vala"));
	_g_free0 (_tmp0_);
	g_string_append (s, "using GLib;\n");
	g_string_append (s, "\n");
	g_string_append_printf (s, "public class %s.Main : Object {\n", self->priv->namespace_name);
	g_string_append (s, "\tpublic Main () {\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tpublic void run () {\n");
	g_string_append (s, "\t\tstdout.printf (\"Hello, world!\\n\");\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tstatic int main (string[] args) {\n");
	g_string_append (s, "\t\tvar main = new Main ();\n");
	g_string_append (s, "\t\tmain.run ();\n");
	g_string_append (s, "\t\treturn 0;\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "}\n");
	g_file_set_contents (_tmp1_ = g_strconcat (self->priv->project_path, "/src/main.vala", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp1_);
	_g_string_free0 (s);
}


static void vala_project_generator_write_mainwindow_vala (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	GString* s;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_string_new ("");
	g_string_append (s, _tmp0_ = vala_project_generator_generate_source_file_header (self, "mainwindow.vala"));
	_g_free0 (_tmp0_);
	g_string_append (s, "using GLib;\n");
	g_string_append (s, "using Gtk;\n");
	g_string_append (s, "\n");
	g_string_append_printf (s, "public class %s.MainWindow : Window {\n", self->priv->namespace_name);
	g_string_append (s, "\tprivate TextBuffer text_buffer;\n");
	g_string_append (s, "\tprivate string filename;\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tpublic MainWindow () {\n");
	g_string_append_printf (s, "\t\ttitle = \"%s\";\n", _tmp1_ = g_strescape (self->priv->project_name, ""));
	_g_free0 (_tmp1_);
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tconstruct {\n");
	g_string_append (s, "\t\tset_default_size (600, 400);\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tdestroy += Gtk.main_quit;\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar vbox = new VBox (false, 0);\n");
	g_string_append (s, "\t\tadd (vbox);\n");
	g_string_append (s, "\t\tvbox.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar toolbar = new Toolbar ();\n");
	g_string_append (s, "\t\tvbox.pack_start (toolbar, false, false, 0);\n");
	g_string_append (s, "\t\ttoolbar.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar button = new ToolButton.from_stock (Gtk.STOCK_SAVE);\n");
	g_string_append (s, "\t\ttoolbar.insert (button, -1);\n");
	g_string_append (s, "\t\tbutton.is_important = true;\n");
	g_string_append (s, "\t\tbutton.clicked += on_save_clicked;\n");
	g_string_append (s, "\t\tbutton.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar scrolled_window = new ScrolledWindow (null, null);\n");
	g_string_append (s, "\t\tvbox.pack_start (scrolled_window, true, true, 0);\n");
	g_string_append (s, "\t\tscrolled_window.hscrollbar_policy = PolicyType.AUTOMATIC;\n");
	g_string_append (s, "\t\tscrolled_window.vscrollbar_policy = PolicyType.AUTOMATIC;\n");
	g_string_append (s, "\t\tscrolled_window.show ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\ttext_buffer = new TextBuffer (null);\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar text_view = new TextView.with_buffer (text_buffer);\n");
	g_string_append (s, "\t\tscrolled_window.add (text_view);\n");
	g_string_append (s, "\t\ttext_view.show ();\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tpublic void run () {\n");
	g_string_append (s, "\t\tshow ();\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tGtk.main ();\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tprivate void on_save_clicked (ToolButton button) {\n");
	g_string_append (s, "\t\tif (filename == null) {\n");
	g_string_append (s, "\t\t\tvar dialog = new FileChooserDialog (_(\"Save File\"), this, FileChooserAction.SAVE,\n");
	g_string_append (s, "\t\t\t\tGtk.STOCK_CANCEL, ResponseType.CANCEL,\n");
	g_string_append (s, "\t\t\t\tGtk.STOCK_SAVE, ResponseType.ACCEPT);\n");
	g_string_append (s, "\t\t\tdialog.set_do_overwrite_confirmation (true);\n");
	g_string_append (s, "\t\t\tif (dialog.run () == ResponseType.ACCEPT) {\n");
	g_string_append (s, "\t\t\t\tfilename = dialog.get_filename ();\n");
	g_string_append (s, "\t\t\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\t\tdialog.destroy ();\n");
	g_string_append (s, "\t\t\tif (filename == null) {\n");
	g_string_append (s, "\t\t\t\treturn;\n");
	g_string_append (s, "\t\t\t}\n");
	g_string_append (s, "\t\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\ttry {\n");
	g_string_append (s, "\t\t\tTextIter start_iter, end_iter;\n");
	g_string_append (s, "\t\t\ttext_buffer.get_bounds (out start_iter, out end_iter);\n");
	g_string_append (s, "\t\t\tstring text = text_buffer.get_text (start_iter, end_iter, true);\n");
	g_string_append (s, "\t\t\tFileUtils.set_contents (filename, text, -1);\n");
	g_string_append (s, "\t\t} catch (FileError e) {\n");
	g_string_append (s, "\t\t\tcritical (\"Error while trying to save file: %s\", e.message);\n");
	g_string_append (s, "\t\t\tfilename = null;\n");
	g_string_append (s, "\t\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "\tstatic int main (string[] args) {\n");
	g_string_append (s, "\t\tGtk.init (ref args);\n");
	g_string_append (s, "\n");
	g_string_append (s, "\t\tvar window = new MainWindow ();\n");
	g_string_append (s, "\t\twindow.run ();\n");
	g_string_append (s, "\t\treturn 0;\n");
	g_string_append (s, "\t}\n");
	g_string_append (s, "\n");
	g_string_append (s, "}\n");
	g_file_set_contents (_tmp2_ = g_strconcat (self->priv->project_path, "/src/mainwindow.vala", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp2_);
	_g_string_free0 (s);
}


static void vala_project_generator_write_potfiles (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	gboolean use_gtk;
	GString* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	use_gtk = self->priv->project_type == VALA_PROJECT_TYPE_GTK_APPLICATION;
	s = g_string_new ("");
	g_string_append (s, "[encoding: UTF-8]\n");
	g_string_append (s, "# List of source files which contain translatable strings.\n");
	if (use_gtk) {
		g_string_append (s, "src/mainwindow.vala\n");
	} else {
		g_string_append (s, "src/main.vala\n");
	}
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/po/POTFILES.in", NULL), s->str, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_string_free0 (s);
			return;
		} else {
			_g_string_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	if (use_gtk) {
		char* _tmp1_;
		g_file_set_contents (_tmp1_ = g_strconcat (self->priv->project_path, "/po/POTFILES.skip", NULL), "src/mainwindow.c\n", (glong) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				g_propagate_error (error, _inner_error_);
				_g_string_free0 (s);
				return;
			} else {
				_g_string_free0 (s);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
		_g_free0 (_tmp1_);
	} else {
		char* _tmp2_;
		g_file_set_contents (_tmp2_ = g_strconcat (self->priv->project_path, "/po/POTFILES.skip", NULL), "src/main.c\n", (glong) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_FILE_ERROR) {
				g_propagate_error (error, _inner_error_);
				_g_string_free0 (s);
				return;
			} else {
				_g_string_free0 (s);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
		}
		_g_free0 (_tmp2_);
	}
	_g_string_free0 (s);
}


static void vala_project_generator_write_linguas (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	char* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_strdup ("# please keep this list sorted alphabetically\n#\n");
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/po/LINGUAS", NULL), s, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_free0 (s);
			return;
		} else {
			_g_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	_g_free0 (s);
}


static void vala_project_generator_write_authors (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	char* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_strdup_printf ("%s <%s>\n", self->priv->real_name, self->priv->email_address);
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/AUTHORS", NULL), s, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_free0 (s);
			return;
		} else {
			_g_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	_g_free0 (s);
}


static void vala_project_generator_write_maintainers (ValaProjectGenerator* self, GError** error) {
	GError * _inner_error_;
	char* s;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	s = g_strdup_printf ("%s\nE-mail: %s\n", self->priv->real_name, self->priv->email_address);
	g_file_set_contents (_tmp0_ = g_strconcat (self->priv->project_path, "/MAINTAINERS", NULL), s, (glong) (-1), &_inner_error_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_FILE_ERROR) {
			g_propagate_error (error, _inner_error_);
			_g_free0 (s);
			return;
		} else {
			_g_free0 (s);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return;
		}
	}
	_g_free0 (_tmp0_);
	_g_free0 (s);
}


static char* vala_project_generator_get_automake_path (ValaProjectGenerator* self) {
	char* result;
	char** _tmp1_;
	gint automake_paths_size;
	gint automake_paths_length1;
	char** _tmp0_;
	char** automake_paths;
	g_return_val_if_fail (self != NULL, NULL);
	_tmp0_ = NULL;
	automake_paths = (_tmp1_ = (_tmp0_ = g_new0 (char*, 3 + 1), _tmp0_[0] = g_strdup ("/usr/share/automake"), _tmp0_[1] = g_strdup ("/usr/share/automake-1.10"), _tmp0_[2] = g_strdup ("/usr/share/automake-1.9"), _tmp0_), automake_paths_length1 = 3, automake_paths_size = automake_paths_length1, _tmp1_);
	{
		char** automake_path_collection;
		int automake_path_collection_length1;
		int automake_path_it;
		automake_path_collection = automake_paths;
		automake_path_collection_length1 = automake_paths_length1;
		for (automake_path_it = 0; automake_path_it < automake_paths_length1; automake_path_it = automake_path_it + 1) {
			char* automake_path;
			automake_path = g_strdup (automake_path_collection[automake_path_it]);
			{
				if (g_file_test (automake_path, G_FILE_TEST_IS_DIR)) {
					result = automake_path;
					automake_paths = (_vala_array_free (automake_paths, automake_paths_length1, (GDestroyNotify) g_free), NULL);
					return result;
				}
				_g_free0 (automake_path);
			}
		}
	}
	result = NULL;
	automake_paths = (_vala_array_free (automake_paths, automake_paths_length1, (GDestroyNotify) g_free), NULL);
	return result;
}


static gboolean vala_project_generator_get_automake_has_native_vala_support (ValaProjectGenerator* self) {
	gboolean result;
	GError * _inner_error_;
	gboolean res;
	g_return_val_if_fail (self != NULL, FALSE);
	_inner_error_ = NULL;
	res = FALSE;
	{
		char* am_output;
		char* _tmp2_;
		gboolean _tmp1_;
		char* _tmp0_;
		am_output = NULL;
		_tmp0_ = NULL;
		_tmp1_ = g_spawn_command_line_sync ("automake --version", &_tmp0_, NULL, NULL, &_inner_error_);
		am_output = (_tmp2_ = _tmp0_, _g_free0 (am_output), _tmp2_);
		_tmp1_;
		if (_inner_error_ != NULL) {
			_g_free0 (am_output);
			goto __catch1_g_error;
			goto __finally1;
		}
		if (am_output != NULL) {
			gint _tmp4__length1;
			char** _tmp4_;
			char** _tmp3_;
			char* _tmp5_;
			char* first_line;
			char** _tmp7_;
			gint tmps_size;
			gint tmps_length1;
			char** _tmp6_;
			char** tmps;
			first_line = (_tmp5_ = g_strdup ((_tmp4_ = _tmp3_ = g_strsplit (am_output, "\n", 2), _tmp4__length1 = _vala_array_length (_tmp3_), _tmp4_)[0]), _tmp4_ = (_vala_array_free (_tmp4_, _tmp4__length1, (GDestroyNotify) g_free), NULL), _tmp5_);
			tmps = (_tmp7_ = _tmp6_ = g_strsplit (first_line, " ", 0), tmps_length1 = _vala_array_length (_tmp6_), tmps_size = tmps_length1, _tmp7_);
			if (tmps_length1 > 0) {
				char* ver;
				char** _tmp9_;
				char** _tmp8_;
				gboolean _tmp10_;
				gboolean _tmp11_;
				ver = g_strdup (tmps[tmps_length1 - 1]);
				tmps = (_tmp9_ = _tmp8_ = g_strsplit (ver, ".", 0), tmps = (_vala_array_free (tmps, tmps_length1, (GDestroyNotify) g_free), NULL), tmps_length1 = _vala_array_length (_tmp8_), tmps_size = tmps_length1, _tmp9_);
				_tmp10_ = FALSE;
				_tmp11_ = FALSE;
				if (tmps_length1 >= 2) {
					_tmp11_ = atoi (tmps[0]) >= 1;
				} else {
					_tmp11_ = FALSE;
				}
				if (_tmp11_) {
					_tmp10_ = atoi (tmps[1]) >= 11;
				} else {
					_tmp10_ = FALSE;
				}
				if (_tmp10_) {
					res = TRUE;
				}
				_g_free0 (ver);
			}
			_g_free0 (first_line);
			tmps = (_vala_array_free (tmps, tmps_length1, (GDestroyNotify) g_free), NULL);
		}
		_g_free0 (am_output);
	}
	goto __finally1;
	__catch1_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("valaprojectgenerator.vala:757: Cannot spawn automake: %s. Native vala support test failed.", err->message);
			_g_error_free0 (err);
		}
	}
	__finally1:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
	result = res;
	return result;
}


static gint vala_project_generator_main (char** args, int args_length1) {
	gint result;
	GError * _inner_error_;
	ValaProjectGenerator* generator;
	gboolean _tmp2_;
	_inner_error_ = NULL;
	gtk_init (&args_length1, &args);
	bindtextdomain (GETTEXT_PACKAGE, NULL);
	{
		GOptionContext* opt_context;
		opt_context = g_option_context_new ("- Vala Project Generator");
		g_option_context_set_help_enabled (opt_context, TRUE);
		g_option_context_add_main_entries (opt_context, VALA_PROJECT_GENERATOR_options, NULL);
		g_option_context_parse (opt_context, &args_length1, &args, &_inner_error_);
		if (_inner_error_ != NULL) {
			_g_option_context_free0 (opt_context);
			if (_inner_error_->domain == G_OPTION_ERROR) {
				goto __catch2_g_option_error;
			}
			goto __finally2;
		}
		if (vala_project_generator_option_files != NULL) {
			char* _tmp0_;
			if (vala_project_generator_option_files[1] != NULL) {
				_inner_error_ = g_error_new_literal (G_OPTION_ERROR, G_OPTION_ERROR_BAD_VALUE, _ ("Just a single project name can be specified on the command line"));
				if (_inner_error_ != NULL) {
					_g_option_context_free0 (opt_context);
					if (_inner_error_->domain == G_OPTION_ERROR) {
						goto __catch2_g_option_error;
					}
					goto __finally2;
				}
			}
			vala_project_generator_option_project_name = (_tmp0_ = g_strdup (vala_project_generator_option_files[0]), _g_free0 (vala_project_generator_option_project_name), _tmp0_);
			if (vala_project_generator_option_project_path == NULL) {
				char* _tmp1_;
				vala_project_generator_option_project_path = (_tmp1_ = g_get_current_dir (), _g_free0 (vala_project_generator_option_project_path), _tmp1_);
			}
		}
		_g_option_context_free0 (opt_context);
	}
	goto __finally2;
	__catch2_g_option_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			fprintf (stdout, "%s\n", e->message);
			fprintf (stdout, _ ("Run '%s --help' to see a full list of available command line options.\n"), args[0]);
			result = 1;
			_g_error_free0 (e);
			return result;
		}
	}
	__finally2:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return 0;
	}
	if (vala_project_generator_option_version) {
		fprintf (stdout, "vala-gen-project %s\n", PACKAGE_VERSION);
		result = 0;
		return result;
	}
	generator = vala_project_generator_new ();
	_tmp2_ = FALSE;
	if (vala_project_generator_option_project_name != NULL) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = vala_project_generator_ask_parameters (generator) == GTK_RESPONSE_OK;
	}
	if (_tmp2_) {
		vala_project_generator_create_project (generator);
		result = 0;
		_vala_project_generator_unref0 (generator);
		return result;
	}
	result = 1;
	_vala_project_generator_unref0 (generator);
	return result;
}


int main (int argc, char ** argv) {
	g_type_init ();
	return vala_project_generator_main (argv, argc);
}


static void vala_value_project_generator_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vala_value_project_generator_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vala_project_generator_unref (value->data[0].v_pointer);
	}
}


static void vala_value_project_generator_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vala_project_generator_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vala_value_project_generator_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vala_value_project_generator_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		ValaProjectGenerator* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vala_project_generator_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vala_value_project_generator_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	ValaProjectGenerator** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vala_project_generator_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vala_param_spec_project_generator (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	ValaParamSpecProjectGenerator* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VALA_TYPE_PROJECT_GENERATOR), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vala_value_get_project_generator (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_PROJECT_GENERATOR), NULL);
	return value->data[0].v_pointer;
}


void vala_value_set_project_generator (GValue* value, gpointer v_object) {
	ValaProjectGenerator* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VALA_TYPE_PROJECT_GENERATOR));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VALA_TYPE_PROJECT_GENERATOR));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vala_project_generator_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vala_project_generator_unref (old);
	}
}


static void vala_project_generator_class_init (ValaProjectGeneratorClass * klass) {
	vala_project_generator_parent_class = g_type_class_peek_parent (klass);
	VALA_PROJECT_GENERATOR_CLASS (klass)->finalize = vala_project_generator_finalize;
	g_type_class_add_private (klass, sizeof (ValaProjectGeneratorPrivate));
}


static void vala_project_generator_instance_init (ValaProjectGenerator * self) {
	self->priv = VALA_PROJECT_GENERATOR_GET_PRIVATE (self);
	self->ref_count = 1;
}


static void vala_project_generator_finalize (ValaProjectGenerator* obj) {
	ValaProjectGenerator * self;
	self = VALA_PROJECT_GENERATOR (obj);
	_g_object_unref0 (self->priv->config_dialog);
	_g_object_unref0 (self->priv->project_folder_button);
	_g_object_unref0 (self->priv->project_type_combobox);
	_g_object_unref0 (self->priv->license_combobox);
	_g_object_unref0 (self->priv->name_entry);
	_g_object_unref0 (self->priv->email_entry);
	_g_free0 (self->priv->project_path);
	_g_free0 (self->priv->project_name);
	_g_free0 (self->priv->namespace_name);
	_g_free0 (self->priv->make_name);
	_g_free0 (self->priv->upper_case_make_name);
	_g_free0 (self->priv->real_name);
	_g_free0 (self->priv->email_address);
}


GType vala_project_generator_get_type (void) {
	static GType vala_project_generator_type_id = 0;
	if (vala_project_generator_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vala_value_project_generator_init, vala_value_project_generator_free_value, vala_value_project_generator_copy_value, vala_value_project_generator_peek_pointer, "p", vala_value_project_generator_collect_value, "p", vala_value_project_generator_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (ValaProjectGeneratorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vala_project_generator_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (ValaProjectGenerator), 0, (GInstanceInitFunc) vala_project_generator_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vala_project_generator_type_id = g_type_register_fundamental (g_type_fundamental_next (), "ValaProjectGenerator", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vala_project_generator_type_id;
}


gpointer vala_project_generator_ref (gpointer instance) {
	ValaProjectGenerator* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vala_project_generator_unref (gpointer instance) {
	ValaProjectGenerator* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VALA_PROJECT_GENERATOR_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




