/*
 *  vbfconfignode.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define VBF_TYPE_CONFIG_NODE (vbf_config_node_get_type ())
#define VBF_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNode))
#define VBF_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))
#define VBF_IS_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_CONFIG_NODE))
#define VBF_IS_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_CONFIG_NODE))
#define VBF_CONFIG_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))

typedef struct _VbfConfigNode VbfConfigNode;
typedef struct _VbfConfigNodeClass VbfConfigNodeClass;
typedef struct _VbfConfigNodePrivate VbfConfigNodePrivate;

struct _VbfConfigNode {
	GObject parent_instance;
	VbfConfigNodePrivate * priv;
	VbfConfigNode* parent;
};

struct _VbfConfigNodeClass {
	GObjectClass parent_class;
	char* (*to_string) (VbfConfigNode* self);
};


static gpointer vbf_config_node_parent_class = NULL;

GType vbf_config_node_get_type (void);
enum  {
	VBF_CONFIG_NODE_DUMMY_PROPERTY
};
char* vbf_config_node_to_string (VbfConfigNode* self);
static char* vbf_config_node_real_to_string (VbfConfigNode* self);
VbfConfigNode* vbf_config_node_construct (GType object_type);
static void vbf_config_node_finalize (GObject* obj);



static char* vbf_config_node_real_to_string (VbfConfigNode* self) {
	g_return_val_if_fail (self != NULL, NULL);
	g_critical ("Type `%s' does not implement abstract method `vbf_config_node_to_string'", g_type_name (G_TYPE_FROM_INSTANCE (self)));
	return NULL;
}


char* vbf_config_node_to_string (VbfConfigNode* self) {
	return VBF_CONFIG_NODE_GET_CLASS (self)->to_string (self);
}


VbfConfigNode* vbf_config_node_construct (GType object_type) {
	VbfConfigNode * self;
	self = (VbfConfigNode*) g_object_new (object_type, NULL);
	return self;
}


static void vbf_config_node_class_init (VbfConfigNodeClass * klass) {
	vbf_config_node_parent_class = g_type_class_peek_parent (klass);
	VBF_CONFIG_NODE_CLASS (klass)->to_string = vbf_config_node_real_to_string;
	G_OBJECT_CLASS (klass)->finalize = vbf_config_node_finalize;
}


static void vbf_config_node_instance_init (VbfConfigNode * self) {
	self->parent = NULL;
}


static void vbf_config_node_finalize (GObject* obj) {
	VbfConfigNode * self;
	self = VBF_CONFIG_NODE (obj);
	G_OBJECT_CLASS (vbf_config_node_parent_class)->finalize (obj);
}


GType vbf_config_node_get_type (void) {
	static GType vbf_config_node_type_id = 0;
	if (vbf_config_node_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfConfigNodeClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_config_node_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfConfigNode), 0, (GInstanceInitFunc) vbf_config_node_instance_init, NULL };
		vbf_config_node_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfConfigNode", &g_define_type_info, G_TYPE_FLAG_ABSTRACT);
	}
	return vbf_config_node_type_id;
}




