/*
 *  vbfiprojectmanager.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define VBF_TYPE_IPROJECT_MANAGER (vbf_iproject_manager_get_type ())
#define VBF_IPROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_IPROJECT_MANAGER, VbfIProjectManager))
#define VBF_IS_IPROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_IPROJECT_MANAGER))
#define VBF_IPROJECT_MANAGER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VBF_TYPE_IPROJECT_MANAGER, VbfIProjectManagerIface))

typedef struct _VbfIProjectManager VbfIProjectManager;
typedef struct _VbfIProjectManagerIface VbfIProjectManagerIface;

#define VBF_TYPE_CONFIG_NODE (vbf_config_node_get_type ())
#define VBF_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNode))
#define VBF_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))
#define VBF_IS_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_CONFIG_NODE))
#define VBF_IS_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_CONFIG_NODE))
#define VBF_CONFIG_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))

typedef struct _VbfConfigNode VbfConfigNode;
typedef struct _VbfConfigNodeClass VbfConfigNodeClass;

#define VBF_TYPE_PROJECT (vbf_project_get_type ())
#define VBF_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_PROJECT, VbfProject))
#define VBF_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_PROJECT, VbfProjectClass))
#define VBF_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_PROJECT))
#define VBF_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_PROJECT))
#define VBF_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_PROJECT, VbfProjectClass))

typedef struct _VbfProject VbfProject;
typedef struct _VbfProjectClass VbfProjectClass;

struct _VbfIProjectManagerIface {
	GTypeInterface parent_iface;
	gboolean (*probe) (VbfIProjectManager* self, const char* project_file);
	VbfProject* (*open) (VbfIProjectManager* self, const char* project_file);
	void (*refresh) (VbfIProjectManager* self, VbfProject* project);
};



GType vbf_config_node_get_type (void);
GType vbf_project_get_type (void);
GType vbf_iproject_manager_get_type (void);
gboolean vbf_iproject_manager_probe (VbfIProjectManager* self, const char* project_file);
VbfProject* vbf_iproject_manager_open (VbfIProjectManager* self, const char* project_file);
void vbf_iproject_manager_refresh (VbfIProjectManager* self, VbfProject* project);



gboolean vbf_iproject_manager_probe (VbfIProjectManager* self, const char* project_file) {
	return VBF_IPROJECT_MANAGER_GET_INTERFACE (self)->probe (self, project_file);
}


VbfProject* vbf_iproject_manager_open (VbfIProjectManager* self, const char* project_file) {
	return VBF_IPROJECT_MANAGER_GET_INTERFACE (self)->open (self, project_file);
}


void vbf_iproject_manager_refresh (VbfIProjectManager* self, VbfProject* project) {
	VBF_IPROJECT_MANAGER_GET_INTERFACE (self)->refresh (self, project);
}


static void vbf_iproject_manager_base_init (VbfIProjectManagerIface * iface) {
	static gboolean initialized = FALSE;
	if (!initialized) {
		initialized = TRUE;
	}
}


GType vbf_iproject_manager_get_type (void) {
	static GType vbf_iproject_manager_type_id = 0;
	if (vbf_iproject_manager_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfIProjectManagerIface), (GBaseInitFunc) vbf_iproject_manager_base_init, (GBaseFinalizeFunc) NULL, (GClassInitFunc) NULL, (GClassFinalizeFunc) NULL, NULL, 0, 0, (GInstanceInitFunc) NULL, NULL };
		vbf_iproject_manager_type_id = g_type_register_static (G_TYPE_INTERFACE, "VbfIProjectManager", &g_define_type_info, 0);
		g_type_interface_add_prerequisite (vbf_iproject_manager_type_id, G_TYPE_OBJECT);
	}
	return vbf_iproject_manager_type_id;
}




