/*
 *  vbfmodule.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>


#define VBF_TYPE_MODULE (vbf_module_get_type ())
#define VBF_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_MODULE, VbfModule))
#define VBF_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_MODULE, VbfModuleClass))
#define VBF_IS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_MODULE))
#define VBF_IS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_MODULE))
#define VBF_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_MODULE, VbfModuleClass))

typedef struct _VbfModule VbfModule;
typedef struct _VbfModuleClass VbfModuleClass;
typedef struct _VbfModulePrivate VbfModulePrivate;

#define VBF_TYPE_CONFIG_NODE (vbf_config_node_get_type ())
#define VBF_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNode))
#define VBF_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))
#define VBF_IS_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_CONFIG_NODE))
#define VBF_IS_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_CONFIG_NODE))
#define VBF_CONFIG_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))

typedef struct _VbfConfigNode VbfConfigNode;
typedef struct _VbfConfigNodeClass VbfConfigNodeClass;

#define VBF_TYPE_PROJECT (vbf_project_get_type ())
#define VBF_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_PROJECT, VbfProject))
#define VBF_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_PROJECT, VbfProjectClass))
#define VBF_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_PROJECT))
#define VBF_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_PROJECT))
#define VBF_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_PROJECT, VbfProjectClass))

typedef struct _VbfProject VbfProject;
typedef struct _VbfProjectClass VbfProjectClass;

#define VBF_TYPE_PACKAGE (vbf_package_get_type ())
#define VBF_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_PACKAGE, VbfPackage))
#define VBF_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_PACKAGE, VbfPackageClass))
#define VBF_IS_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_PACKAGE))
#define VBF_IS_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_PACKAGE))
#define VBF_PACKAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_PACKAGE, VbfPackageClass))

typedef struct _VbfPackage VbfPackage;
typedef struct _VbfPackageClass VbfPackageClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

struct _VbfModule {
	GObject parent_instance;
	VbfModulePrivate * priv;
	char* id;
	char* name;
	VbfProject* project;
};

struct _VbfModuleClass {
	GObjectClass parent_class;
};

struct _VbfModulePrivate {
	GeeList* packages;
};


static gpointer vbf_module_parent_class = NULL;

GType vbf_module_get_type (void);
GType vbf_config_node_get_type (void);
GType vbf_project_get_type (void);
GType vbf_package_get_type (void);
#define VBF_MODULE_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_MODULE, VbfModulePrivate))
enum  {
	VBF_MODULE_DUMMY_PROPERTY
};
VbfModule* vbf_module_new (VbfProject* project, const char* id);
VbfModule* vbf_module_construct (GType object_type, VbfProject* project, const char* id);
GeeList* vbf_module_get_packages (VbfModule* self);
void vbf_module_add_package (VbfModule* self, VbfPackage* package);
static void vbf_module_finalize (GObject* obj);



VbfModule* vbf_module_construct (GType object_type, VbfProject* project, const char* id) {
	VbfModule * self;
	char* _tmp0_;
	char* _tmp1_;
	g_return_val_if_fail (project != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	self = (VbfModule*) g_object_new (object_type, NULL);
	self->id = (_tmp0_ = g_strdup (id), _g_free0 (self->id), _tmp0_);
	self->name = (_tmp1_ = g_strdup (id), _g_free0 (self->name), _tmp1_);
	self->project = project;
	return self;
}


VbfModule* vbf_module_new (VbfProject* project, const char* id) {
	return vbf_module_construct (VBF_TYPE_MODULE, project, id);
}


GeeList* vbf_module_get_packages (VbfModule* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeList*) gee_read_only_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->packages);
	return result;
}


void vbf_module_add_package (VbfModule* self, VbfPackage* package) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (package != NULL);
	gee_collection_add ((GeeCollection*) self->priv->packages, package);
}


static void vbf_module_class_init (VbfModuleClass * klass) {
	vbf_module_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfModulePrivate));
	G_OBJECT_CLASS (klass)->finalize = vbf_module_finalize;
}


static void vbf_module_instance_init (VbfModule * self) {
	self->priv = VBF_MODULE_GET_PRIVATE (self);
	self->priv->packages = (GeeList*) gee_array_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vbf_module_finalize (GObject* obj) {
	VbfModule * self;
	self = VBF_MODULE (obj);
	_g_free0 (self->id);
	_g_free0 (self->name);
	_gee_collection_object_unref0 (self->priv->packages);
	G_OBJECT_CLASS (vbf_module_parent_class)->finalize (obj);
}


GType vbf_module_get_type (void) {
	static GType vbf_module_type_id = 0;
	if (vbf_module_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfModuleClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_module_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfModule), 0, (GInstanceInitFunc) vbf_module_instance_init, NULL };
		vbf_module_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfModule", &g_define_type_info, 0);
	}
	return vbf_module_type_id;
}




