/*
 *  vbfpackage.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>


#define VBF_TYPE_PACKAGE (vbf_package_get_type ())
#define VBF_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_PACKAGE, VbfPackage))
#define VBF_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_PACKAGE, VbfPackageClass))
#define VBF_IS_PACKAGE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_PACKAGE))
#define VBF_IS_PACKAGE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_PACKAGE))
#define VBF_PACKAGE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_PACKAGE, VbfPackageClass))

typedef struct _VbfPackage VbfPackage;
typedef struct _VbfPackageClass VbfPackageClass;
typedef struct _VbfPackagePrivate VbfPackagePrivate;

#define VBF_TYPE_CONFIG_NODE (vbf_config_node_get_type ())
#define VBF_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNode))
#define VBF_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))
#define VBF_IS_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_CONFIG_NODE))
#define VBF_IS_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_CONFIG_NODE))
#define VBF_CONFIG_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))

typedef struct _VbfConfigNode VbfConfigNode;
typedef struct _VbfConfigNodeClass VbfConfigNodeClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _VbfPackage {
	GObject parent_instance;
	VbfPackagePrivate * priv;
	char* id;
	char* name;
	char* constraint;
	VbfConfigNode* version;
};

struct _VbfPackageClass {
	GObjectClass parent_class;
};


static gpointer vbf_package_parent_class = NULL;

GType vbf_package_get_type (void);
GType vbf_config_node_get_type (void);
enum  {
	VBF_PACKAGE_DUMMY_PROPERTY
};
VbfPackage* vbf_package_new (const char* id);
VbfPackage* vbf_package_construct (GType object_type, const char* id);
static void vbf_package_finalize (GObject* obj);



VbfPackage* vbf_package_construct (GType object_type, const char* id) {
	VbfPackage * self;
	char* _tmp0_;
	char* _tmp1_;
	g_return_val_if_fail (id != NULL, NULL);
	self = (VbfPackage*) g_object_new (object_type, NULL);
	self->id = (_tmp0_ = g_strdup (id), _g_free0 (self->id), _tmp0_);
	self->name = (_tmp1_ = g_strdup (id), _g_free0 (self->name), _tmp1_);
	return self;
}


VbfPackage* vbf_package_new (const char* id) {
	return vbf_package_construct (VBF_TYPE_PACKAGE, id);
}


static void vbf_package_class_init (VbfPackageClass * klass) {
	vbf_package_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vbf_package_finalize;
}


static void vbf_package_instance_init (VbfPackage * self) {
}


static void vbf_package_finalize (GObject* obj) {
	VbfPackage * self;
	self = VBF_PACKAGE (obj);
	_g_free0 (self->id);
	_g_free0 (self->name);
	_g_free0 (self->constraint);
	_g_object_unref0 (self->version);
	G_OBJECT_CLASS (vbf_package_parent_class)->finalize (obj);
}


GType vbf_package_get_type (void) {
	static GType vbf_package_type_id = 0;
	if (vbf_package_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfPackageClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_package_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfPackage), 0, (GInstanceInitFunc) vbf_package_instance_init, NULL };
		vbf_package_type_id = g_type_register_static (G_TYPE_OBJECT, "VbfPackage", &g_define_type_info, 0);
	}
	return vbf_package_type_id;
}




