/*
 *  vbfproject.vala - Vala Build Framework library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gio/gio.h>


#define VBF_TYPE_CONFIG_NODE (vbf_config_node_get_type ())
#define VBF_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNode))
#define VBF_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))
#define VBF_IS_CONFIG_NODE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_CONFIG_NODE))
#define VBF_IS_CONFIG_NODE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_CONFIG_NODE))
#define VBF_CONFIG_NODE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_CONFIG_NODE, VbfConfigNodeClass))

typedef struct _VbfConfigNode VbfConfigNode;
typedef struct _VbfConfigNodeClass VbfConfigNodeClass;
typedef struct _VbfConfigNodePrivate VbfConfigNodePrivate;

#define VBF_TYPE_PROJECT (vbf_project_get_type ())
#define VBF_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_PROJECT, VbfProject))
#define VBF_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_PROJECT, VbfProjectClass))
#define VBF_IS_PROJECT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_PROJECT))
#define VBF_IS_PROJECT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_PROJECT))
#define VBF_PROJECT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_PROJECT, VbfProjectClass))

typedef struct _VbfProject VbfProject;
typedef struct _VbfProjectClass VbfProjectClass;
typedef struct _VbfProjectPrivate VbfProjectPrivate;

#define VBF_TYPE_IPROJECT_MANAGER (vbf_iproject_manager_get_type ())
#define VBF_IPROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_IPROJECT_MANAGER, VbfIProjectManager))
#define VBF_IS_IPROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_IPROJECT_MANAGER))
#define VBF_IPROJECT_MANAGER_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VBF_TYPE_IPROJECT_MANAGER, VbfIProjectManagerIface))

typedef struct _VbfIProjectManager VbfIProjectManager;
typedef struct _VbfIProjectManagerIface VbfIProjectManagerIface;

#define VBF_TYPE_GROUP (vbf_group_get_type ())
#define VBF_GROUP(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_GROUP, VbfGroup))
#define VBF_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_GROUP, VbfGroupClass))
#define VBF_IS_GROUP(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_GROUP))
#define VBF_IS_GROUP_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_GROUP))
#define VBF_GROUP_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_GROUP, VbfGroupClass))

typedef struct _VbfGroup VbfGroup;
typedef struct _VbfGroupClass VbfGroupClass;

#define VBF_TYPE_MODULE (vbf_module_get_type ())
#define VBF_MODULE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_MODULE, VbfModule))
#define VBF_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_MODULE, VbfModuleClass))
#define VBF_IS_MODULE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_MODULE))
#define VBF_IS_MODULE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_MODULE))
#define VBF_MODULE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_MODULE, VbfModuleClass))

typedef struct _VbfModule VbfModule;
typedef struct _VbfModuleClass VbfModuleClass;

#define VBF_TYPE_VARIABLE (vbf_variable_get_type ())
#define VBF_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_VARIABLE, VbfVariable))
#define VBF_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_VARIABLE, VbfVariableClass))
#define VBF_IS_VARIABLE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_VARIABLE))
#define VBF_IS_VARIABLE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_VARIABLE))
#define VBF_VARIABLE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_VARIABLE, VbfVariableClass))

typedef struct _VbfVariable VbfVariable;
typedef struct _VbfVariableClass VbfVariableClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _VbfGroupPrivate VbfGroupPrivate;

#define VBF_TYPE_TARGET (vbf_target_get_type ())
#define VBF_TARGET(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_TARGET, VbfTarget))
#define VBF_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_TARGET, VbfTargetClass))
#define VBF_IS_TARGET(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_TARGET))
#define VBF_IS_TARGET_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_TARGET))
#define VBF_TARGET_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_TARGET, VbfTargetClass))

typedef struct _VbfTarget VbfTarget;
typedef struct _VbfTargetClass VbfTargetClass;

#define VBF_TYPE_FILE (vbf_file_get_type ())
#define VBF_FILE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_FILE, VbfFile))
#define VBF_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_FILE, VbfFileClass))
#define VBF_IS_FILE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_FILE))
#define VBF_IS_FILE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_FILE))
#define VBF_FILE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_FILE, VbfFileClass))

typedef struct _VbfFile VbfFile;
typedef struct _VbfFileClass VbfFileClass;

#define VBF_TYPE_SOURCE (vbf_source_get_type ())
#define VBF_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VBF_TYPE_SOURCE, VbfSource))
#define VBF_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VBF_TYPE_SOURCE, VbfSourceClass))
#define VBF_IS_SOURCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VBF_TYPE_SOURCE))
#define VBF_IS_SOURCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VBF_TYPE_SOURCE))
#define VBF_SOURCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VBF_TYPE_SOURCE, VbfSourceClass))

typedef struct _VbfSource VbfSource;
typedef struct _VbfSourceClass VbfSourceClass;
typedef struct _VbfFilePrivate VbfFilePrivate;

#define VBF_TYPE_FILE_TYPES (vbf_file_types_get_type ())
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

struct _VbfConfigNode {
	GObject parent_instance;
	VbfConfigNodePrivate * priv;
	VbfConfigNode* parent;
};

struct _VbfConfigNodeClass {
	GObjectClass parent_class;
	char* (*to_string) (VbfConfigNode* self);
};

struct _VbfIProjectManagerIface {
	GTypeInterface parent_iface;
	gboolean (*probe) (VbfIProjectManager* self, const char* project_file);
	VbfProject* (*open) (VbfIProjectManager* self, const char* project_file);
	void (*refresh) (VbfIProjectManager* self, VbfProject* project);
};

struct _VbfProject {
	VbfConfigNode parent_instance;
	VbfProjectPrivate * priv;
	char* id;
	char* name;
	char* url;
	char* version;
	char* working_dir;
	VbfIProjectManager* backend;
};

struct _VbfProjectClass {
	VbfConfigNodeClass parent_class;
};

struct _VbfProjectPrivate {
	GeeList* groups;
	GeeList* modules;
	GeeList* variables;
	GeeList* file_mons;
	gboolean in_refresh;
};

struct _VbfGroup {
	GObject parent_instance;
	VbfGroupPrivate * priv;
	char* name;
	char* id;
	VbfProject* project;
};

struct _VbfGroupClass {
	GObjectClass parent_class;
};

typedef enum  {
	VBF_FILE_TYPES_UNKNOWN,
	VBF_FILE_TYPES_DATA,
	VBF_FILE_TYPES_VALA_SOURCE
} VbfFileTypes;

struct _VbfFile {
	GObject parent_instance;
	VbfFilePrivate * priv;
	char* name;
	char* filename;
	char* uri;
	VbfFileTypes type;
	VbfTarget* target;
};

struct _VbfFileClass {
	GObjectClass parent_class;
};


static gpointer vbf_project_parent_class = NULL;

GType vbf_config_node_get_type (void);
GType vbf_project_get_type (void);
GType vbf_iproject_manager_get_type (void);
GType vbf_group_get_type (void);
GType vbf_module_get_type (void);
GType vbf_variable_get_type (void);
#define VBF_PROJECT_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VBF_TYPE_PROJECT, VbfProjectPrivate))
enum  {
	VBF_PROJECT_DUMMY_PROPERTY
};
VbfConfigNode* vbf_config_node_construct (GType object_type);
VbfProject* vbf_project_new (const char* id);
VbfProject* vbf_project_construct (GType object_type, const char* id);
GeeList* vbf_project_get_groups (VbfProject* self);
VbfGroup* vbf_project_get_group (VbfProject* self, const char* id);
void vbf_project_add_group (VbfProject* self, VbfGroup* group);
GeeList* vbf_project_get_modules (VbfProject* self);
void vbf_project_add_module (VbfProject* self, VbfModule* module);
GeeList* vbf_project_get_variables (VbfProject* self);
void vbf_project_add_variable (VbfProject* self, VbfVariable* variable);
static char* vbf_project_real_to_string (VbfConfigNode* base);
GType vbf_target_get_type (void);
GeeList* vbf_group_get_targets (VbfGroup* self);
GType vbf_file_get_type (void);
GType vbf_source_get_type (void);
GeeList* vbf_target_get_sources (VbfTarget* self);
GType vbf_file_types_get_type (void);
char* vbf_project_get_all_source_files (VbfProject* self);
void vbf_project_cleanup_file_monitors (VbfProject* self);
void vbf_project_clear (VbfProject* self);
static void vbf_project_on_project_file_changed (VbfProject* self, GFileMonitor* sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type);
static void _vbf_project_on_project_file_changed_g_file_monitor_changed (GFileMonitor* _sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer self);
void vbf_project_setup_file_monitors (VbfProject* self);
void vbf_project_update (VbfProject* self);
void vbf_iproject_manager_refresh (VbfIProjectManager* self, VbfProject* project);
static void vbf_project_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



VbfProject* vbf_project_construct (GType object_type, const char* id) {
	VbfProject * self;
	char* _tmp0_;
	g_return_val_if_fail (id != NULL, NULL);
	self = (VbfProject*) vbf_config_node_construct (object_type);
	self->id = (_tmp0_ = g_strdup (id), _g_free0 (self->id), _tmp0_);
	return self;
}


VbfProject* vbf_project_new (const char* id) {
	return vbf_project_construct (VBF_TYPE_PROJECT, id);
}


GeeList* vbf_project_get_groups (VbfProject* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeList*) gee_read_only_list_new (VBF_TYPE_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->groups);
	return result;
}


VbfGroup* vbf_project_get_group (VbfProject* self, const char* id) {
	VbfGroup* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->priv->groups);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			if (_vala_strcmp0 (group->id, id) == 0) {
				result = group;
				_gee_collection_object_unref0 (_group_it);
				return result;
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	result = NULL;
	return result;
}


void vbf_project_add_group (VbfProject* self, VbfGroup* group) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (group != NULL);
	gee_collection_add ((GeeCollection*) self->priv->groups, group);
}


GeeList* vbf_project_get_modules (VbfProject* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeList*) gee_read_only_list_new (VBF_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->modules);
	return result;
}


void vbf_project_add_module (VbfProject* self, VbfModule* module) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (module != NULL);
	gee_collection_add ((GeeCollection*) self->priv->modules, module);
}


GeeList* vbf_project_get_variables (VbfProject* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GeeList*) gee_read_only_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, self->priv->variables);
	return result;
}


void vbf_project_add_variable (VbfProject* self, VbfVariable* variable) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (variable != NULL);
	gee_collection_add ((GeeCollection*) self->priv->variables, variable);
}


static char* vbf_project_real_to_string (VbfConfigNode* base) {
	VbfProject * self;
	char* result;
	self = (VbfProject*) base;
	result = g_strdup_printf ("%s %s: %s", self->name, self->version, self->id);
	return result;
}


char* vbf_project_get_all_source_files (VbfProject* self) {
	char* result;
	char* res;
	g_return_val_if_fail (self != NULL, NULL);
	res = g_strdup ("");
	{
		GeeIterator* _group_it;
		_group_it = gee_iterable_iterator ((GeeIterable*) self->priv->groups);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp0_;
				GeeIterator* _tmp1_;
				GeeIterator* _target_it;
				_target_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp2_;
						GeeIterator* _tmp3_;
						GeeIterator* _source_it;
						_source_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
						while (TRUE) {
							VbfSource* source;
							char* _tmp4_;
							if (!gee_iterator_next (_source_it)) {
								break;
							}
							source = (VbfSource*) gee_iterator_get (_source_it);
							res = (_tmp4_ = g_strconcat (res, "\"", ((VbfFile*) source)->filename, "\"", NULL), _g_free0 (res), _tmp4_);
							_g_object_unref0 (source);
						}
						_gee_collection_object_unref0 (_source_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	result = res;
	return result;
}


void vbf_project_clear (VbfProject* self) {
	g_return_if_fail (self != NULL);
	gee_collection_clear ((GeeCollection*) self->priv->groups);
	gee_collection_clear ((GeeCollection*) self->priv->variables);
	gee_collection_clear ((GeeCollection*) self->priv->modules);
	vbf_project_cleanup_file_monitors (self);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static void _vbf_project_on_project_file_changed_g_file_monitor_changed (GFileMonitor* _sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type, gpointer self) {
	vbf_project_on_project_file_changed (self, _sender, file, other_file, event_type);
}


void vbf_project_setup_file_monitors (VbfProject* self) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	{
		char* fname;
		GFile* file;
		char* _tmp2_;
		GFile* _tmp3_;
		GFileMonitor* file_mon;
		fname = NULL;
		file = NULL;
		{
			GeeIterator* _group_it;
			_group_it = gee_iterable_iterator ((GeeIterable*) self->priv->groups);
			while (TRUE) {
				VbfGroup* group;
				char* _tmp0_;
				GFile* _tmp1_;
				GFileMonitor* file_mon;
				if (!gee_iterator_next (_group_it)) {
					break;
				}
				group = (VbfGroup*) gee_iterator_get (_group_it);
				fname = (_tmp0_ = g_build_filename (group->id, "Makefile.am", NULL), _g_free0 (fname), _tmp0_);
				file = (_tmp1_ = g_file_new_for_path (fname), _g_object_unref0 (file), _tmp1_);
				file_mon = _g_object_ref0 (g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, &_inner_error_));
				if (_inner_error_ != NULL) {
					_g_object_unref0 (group);
					_gee_collection_object_unref0 (_group_it);
					_g_free0 (fname);
					_g_object_unref0 (file);
					goto __catch0_g_error;
					goto __finally0;
				}
				gee_collection_add ((GeeCollection*) self->priv->file_mons, file_mon);
				g_signal_connect_object (file_mon, "changed", (GCallback) _vbf_project_on_project_file_changed_g_file_monitor_changed, self, 0);
				_g_object_unref0 (group);
				_g_object_unref0 (file_mon);
			}
			_gee_collection_object_unref0 (_group_it);
		}
		fname = (_tmp2_ = g_build_filename (self->id, "configure.ac", NULL), _g_free0 (fname), _tmp2_);
		file = (_tmp3_ = g_file_new_for_path (fname), _g_object_unref0 (file), _tmp3_);
		file_mon = _g_object_ref0 (g_file_monitor (file, G_FILE_MONITOR_NONE, NULL, &_inner_error_));
		if (_inner_error_ != NULL) {
			_g_free0 (fname);
			_g_object_unref0 (file);
			goto __catch0_g_error;
			goto __finally0;
		}
		gee_collection_add ((GeeCollection*) self->priv->file_mons, file_mon);
		g_signal_connect_object (file_mon, "changed", (GCallback) _vbf_project_on_project_file_changed_g_file_monitor_changed, self, 0);
		_g_free0 (fname);
		_g_object_unref0 (file);
		_g_object_unref0 (file_mon);
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_critical ("vbfproject.vala:136: setup_file_monitors error: %s", err->message);
			_g_error_free0 (err);
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


void vbf_project_cleanup_file_monitors (VbfProject* self) {
	g_return_if_fail (self != NULL);
	{
		GeeIterator* _file_mon_it;
		_file_mon_it = gee_iterable_iterator ((GeeIterable*) self->priv->file_mons);
		while (TRUE) {
			GFileMonitor* file_mon;
			guint _tmp0_;
			if (!gee_iterator_next (_file_mon_it)) {
				break;
			}
			file_mon = (GFileMonitor*) gee_iterator_get (_file_mon_it);
			g_signal_handlers_disconnect_matched (file_mon, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("changed", G_TYPE_FILE_MONITOR, &_tmp0_, NULL, FALSE), _tmp0_), 0, NULL, (GCallback) _vbf_project_on_project_file_changed_g_file_monitor_changed, self);
			g_file_monitor_cancel (file_mon);
			_g_object_unref0 (file_mon);
		}
		_gee_collection_object_unref0 (_file_mon_it);
	}
	gee_collection_clear ((GeeCollection*) self->priv->file_mons);
}


static void vbf_project_on_project_file_changed (VbfProject* self, GFileMonitor* sender, GFile* file, GFile* other_file, GFileMonitorEvent event_type) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (file != NULL);
	vbf_project_update (self);
}


void vbf_project_update (VbfProject* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->in_refresh) {
		return;
	}
	self->priv->in_refresh = TRUE;
	vbf_iproject_manager_refresh (self->backend, self);
	g_signal_emit_by_name (self, "updated");
	self->priv->in_refresh = FALSE;
}


static void vbf_project_class_init (VbfProjectClass * klass) {
	vbf_project_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VbfProjectPrivate));
	VBF_CONFIG_NODE_CLASS (klass)->to_string = vbf_project_real_to_string;
	G_OBJECT_CLASS (klass)->finalize = vbf_project_finalize;
	g_signal_new ("updated", VBF_TYPE_PROJECT, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vbf_project_instance_init (VbfProject * self) {
	self->priv = VBF_PROJECT_GET_PRIVATE (self);
	self->priv->groups = (GeeList*) gee_array_list_new (VBF_TYPE_GROUP, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->modules = (GeeList*) gee_array_list_new (VBF_TYPE_MODULE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->variables = (GeeList*) gee_array_list_new (VBF_TYPE_VARIABLE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->file_mons = (GeeList*) gee_array_list_new (G_TYPE_FILE_MONITOR, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->in_refresh = FALSE;
	self->backend = NULL;
}


static void vbf_project_finalize (GObject* obj) {
	VbfProject * self;
	self = VBF_PROJECT (obj);
	_g_free0 (self->id);
	_g_free0 (self->name);
	_g_free0 (self->url);
	_g_free0 (self->version);
	_g_free0 (self->working_dir);
	_gee_collection_object_unref0 (self->priv->groups);
	_gee_collection_object_unref0 (self->priv->modules);
	_gee_collection_object_unref0 (self->priv->variables);
	_gee_collection_object_unref0 (self->priv->file_mons);
	_g_object_unref0 (self->backend);
	G_OBJECT_CLASS (vbf_project_parent_class)->finalize (obj);
}


GType vbf_project_get_type (void) {
	static GType vbf_project_type_id = 0;
	if (vbf_project_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VbfProjectClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vbf_project_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VbfProject), 0, (GInstanceInitFunc) vbf_project_instance_init, NULL };
		vbf_project_type_id = g_type_register_static (VBF_TYPE_CONFIG_NODE, "VbfProject", &g_define_type_info, 0);
	}
	return vbf_project_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




