/*
 *  vscclasslistvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2009 - Levi Bard <taktaktaktaktaktaktaktaktaktak@gmail.com>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <gee.h>


#define VSC_TYPE_CLASS_LIST (vsc_class_list_get_type ())
#define VSC_CLASS_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_CLASS_LIST, VscClassList))
#define VSC_CLASS_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_CLASS_LIST, VscClassListClass))
#define VSC_IS_CLASS_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_CLASS_LIST))
#define VSC_IS_CLASS_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_CLASS_LIST))
#define VSC_CLASS_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_CLASS_LIST, VscClassListClass))

typedef struct _VscClassList VscClassList;
typedef struct _VscClassListClass VscClassListClass;
typedef struct _VscClassListPrivate VscClassListPrivate;

#define VSC_TYPE_SYMBOL_COMPLETION_ITEM (vsc_symbol_completion_item_get_type ())
#define VSC_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItem))
#define VSC_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))
#define VSC_IS_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_IS_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_SYMBOL_COMPLETION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))

typedef struct _VscSymbolCompletionItem VscSymbolCompletionItem;
typedef struct _VscSymbolCompletionItemClass VscSymbolCompletionItemClass;
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _VscClassList {
	ValaCodeVisitor parent_instance;
	VscClassListPrivate * priv;
};

struct _VscClassListClass {
	ValaCodeVisitorClass parent_class;
};

struct _VscClassListPrivate {
	GeeList* _classes;
};


static gpointer vsc_class_list_parent_class = NULL;

GType vsc_class_list_get_type (void);
GType vsc_symbol_completion_item_get_type (void);
#define VSC_CLASS_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_CLASS_LIST, VscClassListPrivate))
enum  {
	VSC_CLASS_LIST_DUMMY_PROPERTY
};
VscClassList* vsc_class_list_new (GeeList* classes);
VscClassList* vsc_class_list_construct (GType object_type, GeeList* classes);
static void vsc_class_list_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vsc_class_list_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_class (ValaClass* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_class (GType object_type, ValaClass* item);
static void vsc_class_list_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_struct (ValaStruct* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_struct (GType object_type, ValaStruct* item);
static void vsc_class_list_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_interface (ValaInterface* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_interface (GType object_type, ValaInterface* item);
static void vsc_class_list_real_visit_interface (ValaCodeVisitor* base, ValaInterface* cl);
static void vsc_class_list_finalize (ValaCodeVisitor* obj);



static gpointer _gee_collection_object_ref0 (gpointer self) {
	return self ? gee_collection_object_ref (self) : NULL;
}


VscClassList* vsc_class_list_construct (GType object_type, GeeList* classes) {
	VscClassList* self;
	GeeList* _tmp0_;
	g_return_val_if_fail (classes != NULL, NULL);
	self = (VscClassList*) vala_code_visitor_construct (object_type);
	self->priv->_classes = (_tmp0_ = _gee_collection_object_ref0 (classes), _gee_collection_object_unref0 (self->priv->_classes), _tmp0_);
	return self;
}


VscClassList* vsc_class_list_new (GeeList* classes) {
	return vsc_class_list_construct (VSC_TYPE_CLASS_LIST, classes);
}


static void vsc_class_list_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* file) {
	VscClassList * self;
	self = (VscClassList*) base;
	g_return_if_fail (file != NULL);
	vala_source_file_accept_children (file, (ValaCodeVisitor*) self);
}


static void vsc_class_list_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns) {
	VscClassList * self;
	self = (VscClassList*) base;
	g_return_if_fail (ns != NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
		_item_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_namespace_get_namespaces (ns))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaNamespace* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaNamespace*) gee_iterator_get (_item_it);
			vala_code_node_accept ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _cl_it;
		_cl_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_namespace_get_classes (ns))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaClass* cl;
			if (!gee_iterator_next (_cl_it)) {
				break;
			}
			cl = (ValaClass*) gee_iterator_get (_cl_it);
			vala_code_node_accept_children ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (cl);
		}
		_gee_collection_object_unref0 (_cl_it);
	}
	{
		GeeList* _tmp4_;
		GeeIterator* _tmp5_;
		GeeIterator* _item_it;
		_item_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vala_namespace_get_structs (ns))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
		while (TRUE) {
			ValaStruct* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaStruct*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	{
		GeeList* _tmp6_;
		GeeIterator* _tmp7_;
		GeeIterator* _item_it;
		_item_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vala_namespace_get_interfaces (ns))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
		while (TRUE) {
			ValaInterface* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaInterface*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
}


static void vsc_class_list_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	VscClassList * self;
	VscSymbolCompletionItem* _tmp4_;
	self = (VscClassList*) base;
	g_return_if_fail (cl != NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
		_item_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_class_get_classes (cl))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaClass* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaClass*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _item_it;
		_item_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_class_get_structs (cl))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaStruct* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaStruct*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	gee_collection_add ((GeeCollection*) self->priv->_classes, _tmp4_ = vsc_symbol_completion_item_new_with_class (cl));
	_g_object_unref0 (_tmp4_);
}


static void vsc_class_list_real_visit_struct (ValaCodeVisitor* base, ValaStruct* st) {
	VscClassList * self;
	VscSymbolCompletionItem* _tmp0_;
	self = (VscClassList*) base;
	g_return_if_fail (st != NULL);
	gee_collection_add ((GeeCollection*) self->priv->_classes, _tmp0_ = vsc_symbol_completion_item_new_with_struct (st));
	_g_object_unref0 (_tmp0_);
}


static void vsc_class_list_real_visit_interface (ValaCodeVisitor* base, ValaInterface* cl) {
	VscClassList * self;
	VscSymbolCompletionItem* _tmp0_;
	self = (VscClassList*) base;
	g_return_if_fail (cl != NULL);
	gee_collection_add ((GeeCollection*) self->priv->_classes, _tmp0_ = vsc_symbol_completion_item_new_with_interface (cl));
	_g_object_unref0 (_tmp0_);
}


static void vsc_class_list_class_init (VscClassListClass * klass) {
	vsc_class_list_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = vsc_class_list_finalize;
	g_type_class_add_private (klass, sizeof (VscClassListPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_source_file = vsc_class_list_real_visit_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace = vsc_class_list_real_visit_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vsc_class_list_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_struct = vsc_class_list_real_visit_struct;
	VALA_CODE_VISITOR_CLASS (klass)->visit_interface = vsc_class_list_real_visit_interface;
}


static void vsc_class_list_instance_init (VscClassList * self) {
	self->priv = VSC_CLASS_LIST_GET_PRIVATE (self);
}


static void vsc_class_list_finalize (ValaCodeVisitor* obj) {
	VscClassList * self;
	self = VSC_CLASS_LIST (obj);
	_gee_collection_object_unref0 (self->priv->_classes);
	VALA_CODE_VISITOR_CLASS (vsc_class_list_parent_class)->finalize (obj);
}


GType vsc_class_list_get_type (void) {
	static GType vsc_class_list_type_id = 0;
	if (vsc_class_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscClassListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_class_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscClassList), 0, (GInstanceInitFunc) vsc_class_list_instance_init, NULL };
		vsc_class_list_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "VscClassList", &g_define_type_info, 0);
	}
	return vsc_class_list_type_id;
}




