/*
 *  vscmethodlistvisitor.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <vala.h>
#include <gee.h>


#define VSC_TYPE_METHOD_LIST (vsc_method_list_get_type ())
#define VSC_METHOD_LIST(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_METHOD_LIST, VscMethodList))
#define VSC_METHOD_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_METHOD_LIST, VscMethodListClass))
#define VSC_IS_METHOD_LIST(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_METHOD_LIST))
#define VSC_IS_METHOD_LIST_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_METHOD_LIST))
#define VSC_METHOD_LIST_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_METHOD_LIST, VscMethodListClass))

typedef struct _VscMethodList VscMethodList;
typedef struct _VscMethodListClass VscMethodListClass;
typedef struct _VscMethodListPrivate VscMethodListPrivate;

#define VSC_TYPE_SYMBOL_COMPLETION_ITEM (vsc_symbol_completion_item_get_type ())
#define VSC_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItem))
#define VSC_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))
#define VSC_IS_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_IS_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_SYMBOL_COMPLETION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))

typedef struct _VscSymbolCompletionItem VscSymbolCompletionItem;
typedef struct _VscSymbolCompletionItemClass VscSymbolCompletionItemClass;
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _VscMethodList {
	ValaCodeVisitor parent_instance;
	VscMethodListPrivate * priv;
};

struct _VscMethodListClass {
	ValaCodeVisitorClass parent_class;
};

struct _VscMethodListPrivate {
	GeeList* _methods;
};


static gpointer vsc_method_list_parent_class = NULL;

GType vsc_method_list_get_type (void);
GType vsc_symbol_completion_item_get_type (void);
#define VSC_METHOD_LIST_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VSC_TYPE_METHOD_LIST, VscMethodListPrivate))
enum  {
	VSC_METHOD_LIST_DUMMY_PROPERTY
};
VscMethodList* vsc_method_list_new (GeeList* methods);
VscMethodList* vsc_method_list_construct (GType object_type, GeeList* methods);
static void vsc_method_list_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* file);
static void vsc_method_list_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns);
static void vsc_method_list_real_visit_class (ValaCodeVisitor* base, ValaClass* cl);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_method (ValaMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_method (GType object_type, ValaMethod* item);
static void vsc_method_list_real_visit_method (ValaCodeVisitor* base, ValaMethod* m);
static void vsc_method_list_finalize (ValaCodeVisitor* obj);



static gpointer _gee_collection_object_ref0 (gpointer self) {
	return self ? gee_collection_object_ref (self) : NULL;
}


VscMethodList* vsc_method_list_construct (GType object_type, GeeList* methods) {
	VscMethodList* self;
	GeeList* _tmp0_;
	g_return_val_if_fail (methods != NULL, NULL);
	self = (VscMethodList*) vala_code_visitor_construct (object_type);
	self->priv->_methods = (_tmp0_ = _gee_collection_object_ref0 (methods), _gee_collection_object_unref0 (self->priv->_methods), _tmp0_);
	return self;
}


VscMethodList* vsc_method_list_new (GeeList* methods) {
	return vsc_method_list_construct (VSC_TYPE_METHOD_LIST, methods);
}


static void vsc_method_list_real_visit_source_file (ValaCodeVisitor* base, ValaSourceFile* file) {
	VscMethodList * self;
	self = (VscMethodList*) base;
	g_return_if_fail (file != NULL);
	vala_source_file_accept_children (file, (ValaCodeVisitor*) self);
}


static void vsc_method_list_real_visit_namespace (ValaCodeVisitor* base, ValaNamespace* ns) {
	VscMethodList * self;
	self = (VscMethodList*) base;
	g_return_if_fail (ns != NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
		_item_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_namespace_get_namespaces (ns))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaNamespace* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaNamespace*) gee_iterator_get (_item_it);
			vala_code_node_accept ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _m_it;
		_m_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_namespace_get_methods (ns))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaMethod* m;
			if (!gee_iterator_next (_m_it)) {
				break;
			}
			m = (ValaMethod*) gee_iterator_get (_m_it);
			vala_code_node_accept ((ValaCodeNode*) m, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (m);
		}
		_gee_collection_object_unref0 (_m_it);
	}
	{
		GeeList* _tmp4_;
		GeeIterator* _tmp5_;
		GeeIterator* _cl_it;
		_cl_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vala_namespace_get_classes (ns))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
		while (TRUE) {
			ValaClass* cl;
			if (!gee_iterator_next (_cl_it)) {
				break;
			}
			cl = (ValaClass*) gee_iterator_get (_cl_it);
			vala_code_node_accept_children ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (cl);
		}
		_gee_collection_object_unref0 (_cl_it);
	}
	{
		GeeList* _tmp6_;
		GeeIterator* _tmp7_;
		GeeIterator* _st_it;
		_st_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vala_namespace_get_structs (ns))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
		while (TRUE) {
			ValaStruct* st;
			if (!gee_iterator_next (_st_it)) {
				break;
			}
			st = (ValaStruct*) gee_iterator_get (_st_it);
			vala_code_node_accept_children ((ValaCodeNode*) st, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (st);
		}
		_gee_collection_object_unref0 (_st_it);
	}
	{
		GeeList* _tmp8_;
		GeeIterator* _tmp9_;
		GeeIterator* _cl_it;
		_cl_it = (_tmp9_ = gee_iterable_iterator ((GeeIterable*) (_tmp8_ = vala_namespace_get_interfaces (ns))), _gee_collection_object_unref0 (_tmp8_), _tmp9_);
		while (TRUE) {
			ValaInterface* cl;
			if (!gee_iterator_next (_cl_it)) {
				break;
			}
			cl = (ValaInterface*) gee_iterator_get (_cl_it);
			vala_code_node_accept_children ((ValaCodeNode*) cl, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (cl);
		}
		_gee_collection_object_unref0 (_cl_it);
	}
}


static void vsc_method_list_real_visit_class (ValaCodeVisitor* base, ValaClass* cl) {
	VscMethodList * self;
	self = (VscMethodList*) base;
	g_return_if_fail (cl != NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _item_it;
		_item_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vala_class_get_classes (cl))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			ValaClass* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaClass*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _item_it;
		_item_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vala_class_get_structs (cl))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			ValaStruct* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (ValaStruct*) gee_iterator_get (_item_it);
			vala_code_node_accept_children ((ValaCodeNode*) item, (ValaCodeVisitor*) self);
			_vala_code_node_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
}


static void vsc_method_list_real_visit_method (ValaCodeVisitor* base, ValaMethod* m) {
	VscMethodList * self;
	VscSymbolCompletionItem* _tmp0_;
	self = (VscMethodList*) base;
	g_return_if_fail (m != NULL);
	gee_collection_add ((GeeCollection*) self->priv->_methods, _tmp0_ = vsc_symbol_completion_item_new_with_method (m));
	_g_object_unref0 (_tmp0_);
}


static void vsc_method_list_class_init (VscMethodListClass * klass) {
	vsc_method_list_parent_class = g_type_class_peek_parent (klass);
	VALA_CODE_VISITOR_CLASS (klass)->finalize = vsc_method_list_finalize;
	g_type_class_add_private (klass, sizeof (VscMethodListPrivate));
	VALA_CODE_VISITOR_CLASS (klass)->visit_source_file = vsc_method_list_real_visit_source_file;
	VALA_CODE_VISITOR_CLASS (klass)->visit_namespace = vsc_method_list_real_visit_namespace;
	VALA_CODE_VISITOR_CLASS (klass)->visit_class = vsc_method_list_real_visit_class;
	VALA_CODE_VISITOR_CLASS (klass)->visit_method = vsc_method_list_real_visit_method;
}


static void vsc_method_list_instance_init (VscMethodList * self) {
	self->priv = VSC_METHOD_LIST_GET_PRIVATE (self);
}


static void vsc_method_list_finalize (ValaCodeVisitor* obj) {
	VscMethodList * self;
	self = VSC_METHOD_LIST (obj);
	_gee_collection_object_unref0 (self->priv->_methods);
	VALA_CODE_VISITOR_CLASS (vsc_method_list_parent_class)->finalize (obj);
}


GType vsc_method_list_get_type (void) {
	static GType vsc_method_list_type_id = 0;
	if (vsc_method_list_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscMethodListClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_method_list_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscMethodList), 0, (GInstanceInitFunc) vsc_method_list_instance_init, NULL };
		vsc_method_list_type_id = g_type_register_static (VALA_TYPE_CODE_VISITOR, "VscMethodList", &g_define_type_info, 0);
	}
	return vsc_method_list_type_id;
}




