/*
 *  vscsymbolcompletionfilteroptions.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gobject/gvaluecollector.h>


#define VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS (vsc_symbol_completion_filter_options_get_type ())
#define VSC_SYMBOL_COMPLETION_FILTER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS, VscSymbolCompletionFilterOptions))
#define VSC_SYMBOL_COMPLETION_FILTER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS, VscSymbolCompletionFilterOptionsClass))
#define VSC_IS_SYMBOL_COMPLETION_FILTER_OPTIONS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS))
#define VSC_IS_SYMBOL_COMPLETION_FILTER_OPTIONS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS))
#define VSC_SYMBOL_COMPLETION_FILTER_OPTIONS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS, VscSymbolCompletionFilterOptionsClass))

typedef struct _VscSymbolCompletionFilterOptions VscSymbolCompletionFilterOptions;
typedef struct _VscSymbolCompletionFilterOptionsClass VscSymbolCompletionFilterOptionsClass;
typedef struct _VscSymbolCompletionFilterOptionsPrivate VscSymbolCompletionFilterOptionsPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _VscParamSpecSymbolCompletionFilterOptions VscParamSpecSymbolCompletionFilterOptions;

struct _VscSymbolCompletionFilterOptions {
	GTypeInstance parent_instance;
	volatile int ref_count;
	VscSymbolCompletionFilterOptionsPrivate * priv;
	gboolean static_symbols;
	gboolean interface_symbols;
	gboolean constructors;
	gboolean instance_symbols;
	gboolean imported_namespaces;
	gboolean local_variables;
	gboolean error_domains;
	gboolean error_base;
	gboolean private_symbols;
	gboolean public_symbols;
	gboolean protected_symbols;
	gboolean internal_symbols;
	char* exclude_type;
};

struct _VscSymbolCompletionFilterOptionsClass {
	GTypeClass parent_class;
	void (*finalize) (VscSymbolCompletionFilterOptions *self);
};

struct _VscParamSpecSymbolCompletionFilterOptions {
	GParamSpec parent_instance;
};


static gpointer vsc_symbol_completion_filter_options_parent_class = NULL;

gpointer vsc_symbol_completion_filter_options_ref (gpointer instance);
void vsc_symbol_completion_filter_options_unref (gpointer instance);
GParamSpec* vsc_param_spec_symbol_completion_filter_options (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vsc_value_set_symbol_completion_filter_options (GValue* value, gpointer v_object);
gpointer vsc_value_get_symbol_completion_filter_options (const GValue* value);
GType vsc_symbol_completion_filter_options_get_type (void);
enum  {
	VSC_SYMBOL_COMPLETION_FILTER_OPTIONS_DUMMY_PROPERTY
};
void vsc_symbol_completion_filter_options_defaults (VscSymbolCompletionFilterOptions* self);
VscSymbolCompletionFilterOptions* vsc_symbol_completion_filter_options_new (void);
VscSymbolCompletionFilterOptions* vsc_symbol_completion_filter_options_construct (GType object_type);
void vsc_symbol_completion_filter_options_public_only (VscSymbolCompletionFilterOptions* self);
static void vsc_symbol_completion_filter_options_finalize (VscSymbolCompletionFilterOptions* obj);



VscSymbolCompletionFilterOptions* vsc_symbol_completion_filter_options_construct (GType object_type) {
	VscSymbolCompletionFilterOptions* self;
	self = (VscSymbolCompletionFilterOptions*) g_type_create_instance (object_type);
	vsc_symbol_completion_filter_options_defaults (self);
	return self;
}


VscSymbolCompletionFilterOptions* vsc_symbol_completion_filter_options_new (void) {
	return vsc_symbol_completion_filter_options_construct (VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS);
}


void vsc_symbol_completion_filter_options_defaults (VscSymbolCompletionFilterOptions* self) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	self->static_symbols = TRUE;
	self->interface_symbols = TRUE;
	self->instance_symbols = TRUE;
	self->constructors = FALSE;
	self->imported_namespaces = FALSE;
	self->local_variables = FALSE;
	self->error_domains = FALSE;
	self->error_base = FALSE;
	self->private_symbols = TRUE;
	self->public_symbols = TRUE;
	self->protected_symbols = TRUE;
	self->internal_symbols = TRUE;
	self->exclude_type = (_tmp0_ = NULL, _g_free0 (self->exclude_type), _tmp0_);
}


void vsc_symbol_completion_filter_options_public_only (VscSymbolCompletionFilterOptions* self) {
	g_return_if_fail (self != NULL);
	self->public_symbols = TRUE;
	self->private_symbols = FALSE;
	self->protected_symbols = FALSE;
	self->internal_symbols = FALSE;
}


static void vsc_value_symbol_completion_filter_options_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vsc_value_symbol_completion_filter_options_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vsc_symbol_completion_filter_options_unref (value->data[0].v_pointer);
	}
}


static void vsc_value_symbol_completion_filter_options_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vsc_symbol_completion_filter_options_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vsc_value_symbol_completion_filter_options_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vsc_value_symbol_completion_filter_options_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VscSymbolCompletionFilterOptions* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vsc_symbol_completion_filter_options_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vsc_value_symbol_completion_filter_options_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VscSymbolCompletionFilterOptions** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vsc_symbol_completion_filter_options_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vsc_param_spec_symbol_completion_filter_options (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VscParamSpecSymbolCompletionFilterOptions* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vsc_value_get_symbol_completion_filter_options (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS), NULL);
	return value->data[0].v_pointer;
}


void vsc_value_set_symbol_completion_filter_options (GValue* value, gpointer v_object) {
	VscSymbolCompletionFilterOptions* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VSC_TYPE_SYMBOL_COMPLETION_FILTER_OPTIONS));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vsc_symbol_completion_filter_options_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vsc_symbol_completion_filter_options_unref (old);
	}
}


static void vsc_symbol_completion_filter_options_class_init (VscSymbolCompletionFilterOptionsClass * klass) {
	vsc_symbol_completion_filter_options_parent_class = g_type_class_peek_parent (klass);
	VSC_SYMBOL_COMPLETION_FILTER_OPTIONS_CLASS (klass)->finalize = vsc_symbol_completion_filter_options_finalize;
}


static void vsc_symbol_completion_filter_options_instance_init (VscSymbolCompletionFilterOptions * self) {
	self->ref_count = 1;
}


static void vsc_symbol_completion_filter_options_finalize (VscSymbolCompletionFilterOptions* obj) {
	VscSymbolCompletionFilterOptions * self;
	self = VSC_SYMBOL_COMPLETION_FILTER_OPTIONS (obj);
	_g_free0 (self->exclude_type);
}


GType vsc_symbol_completion_filter_options_get_type (void) {
	static GType vsc_symbol_completion_filter_options_type_id = 0;
	if (vsc_symbol_completion_filter_options_type_id == 0) {
		static const GTypeValueTable g_define_type_value_table = { vsc_value_symbol_completion_filter_options_init, vsc_value_symbol_completion_filter_options_free_value, vsc_value_symbol_completion_filter_options_copy_value, vsc_value_symbol_completion_filter_options_peek_pointer, "p", vsc_value_symbol_completion_filter_options_collect_value, "p", vsc_value_symbol_completion_filter_options_lcopy_value };
		static const GTypeInfo g_define_type_info = { sizeof (VscSymbolCompletionFilterOptionsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_symbol_completion_filter_options_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSymbolCompletionFilterOptions), 0, (GInstanceInitFunc) vsc_symbol_completion_filter_options_instance_init, &g_define_type_value_table };
		static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
		vsc_symbol_completion_filter_options_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VscSymbolCompletionFilterOptions", &g_define_type_info, &g_define_type_fundamental_info, 0);
	}
	return vsc_symbol_completion_filter_options_type_id;
}


gpointer vsc_symbol_completion_filter_options_ref (gpointer instance) {
	VscSymbolCompletionFilterOptions* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vsc_symbol_completion_filter_options_unref (gpointer instance) {
	VscSymbolCompletionFilterOptions* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VSC_SYMBOL_COMPLETION_FILTER_OPTIONS_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}




