/*
 *  vscsymbolcompletionitem.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <vala.h>
#include <gee.h>


#define VSC_TYPE_SYMBOL_COMPLETION_ITEM (vsc_symbol_completion_item_get_type ())
#define VSC_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItem))
#define VSC_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))
#define VSC_IS_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_IS_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_SYMBOL_COMPLETION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))

typedef struct _VscSymbolCompletionItem VscSymbolCompletionItem;
typedef struct _VscSymbolCompletionItemClass VscSymbolCompletionItemClass;
typedef struct _VscSymbolCompletionItemPrivate VscSymbolCompletionItemPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _vala_code_node_unref0(var) ((var == NULL) ? NULL : (var = (vala_code_node_unref (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

struct _VscSymbolCompletionItem {
	GObject parent_instance;
	VscSymbolCompletionItemPrivate * priv;
	char* name;
	char* type_name;
	char* info;
	char* file;
	gint first_line;
	gint last_line;
	ValaSymbol* symbol;
};

struct _VscSymbolCompletionItemClass {
	GObjectClass parent_class;
};


static gpointer vsc_symbol_completion_item_parent_class = NULL;

GType vsc_symbol_completion_item_get_type (void);
enum  {
	VSC_SYMBOL_COMPLETION_ITEM_DUMMY_PROPERTY
};
VscSymbolCompletionItem* vsc_symbol_completion_item_new (const char* name);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct (GType object_type, const char* name);
static char* vsc_symbol_completion_item_data_type_to_string (VscSymbolCompletionItem* self, ValaDataType* type);
static char* vsc_symbol_completion_item_formal_parameters_to_string (VscSymbolCompletionItem* self, GeeList* parameters);
static void vsc_symbol_completion_item_update_source_reference_info (VscSymbolCompletionItem* self, ValaSymbol* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_method (ValaMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_method (GType object_type, ValaMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_creation_method (ValaCreationMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_creation_method (GType object_type, ValaCreationMethod* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_field (ValaField* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_field (GType object_type, ValaField* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_property (ValaProperty* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_property (GType object_type, ValaProperty* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_struct (ValaStruct* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_struct (GType object_type, ValaStruct* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_class (ValaClass* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_class (GType object_type, ValaClass* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_interface (ValaInterface* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_interface (GType object_type, ValaInterface* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_signal (ValaSignal* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_signal (GType object_type, ValaSignal* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_namespace (ValaNamespace* item);
VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_namespace (GType object_type, ValaNamespace* item);
static void vsc_symbol_completion_item_finalize (GObject* obj);
static int _vala_strcmp0 (const char * str1, const char * str2);



VscSymbolCompletionItem* vsc_symbol_completion_item_construct (GType object_type, const char* name) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp1_;
	g_return_val_if_fail (name != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (name), _g_free0 (self->name), _tmp0_);
	self->info = (_tmp1_ = g_strdup (name), _g_free0 (self->info), _tmp1_);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new (const char* name) {
	return vsc_symbol_completion_item_construct (VSC_TYPE_SYMBOL_COMPLETION_ITEM, name);
}


static char* vsc_symbol_completion_item_data_type_to_string (VscSymbolCompletionItem* self, ValaDataType* type) {
	char* result;
	char* _result_;
	char* _tmp0_;
	GeeList* type_args;
	gboolean _tmp2_;
	gboolean _tmp9_;
	gboolean _tmp10_;
	char* _tmp13_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (type != NULL, NULL);
	_result_ = NULL;
	_result_ = (_tmp0_ = vala_data_type_to_qualified_string (type, NULL), _g_free0 (_result_), _tmp0_);
	if (_result_ == NULL) {
		char* _tmp1_;
		_result_ = (_tmp1_ = g_strdup ("void"), _g_free0 (_result_), _tmp1_);
	}
	type_args = vala_data_type_get_type_arguments (type);
	_tmp2_ = FALSE;
	if (gee_collection_get_size ((GeeCollection*) type_args) > 0) {
		_tmp2_ = strstr (_result_, "<") == NULL;
	} else {
		_tmp2_ = FALSE;
	}
	if (_tmp2_) {
		char* _tmp3_;
		gboolean first;
		char* _tmp8_;
		_result_ = (_tmp3_ = g_strconcat (_result_, "<", NULL), _g_free0 (_result_), _tmp3_);
		first = TRUE;
		{
			GeeIterator* _type_arg_it;
			_type_arg_it = gee_iterable_iterator ((GeeIterable*) type_args);
			while (TRUE) {
				ValaDataType* type_arg;
				char* _tmp7_;
				char* _tmp6_;
				if (!gee_iterator_next (_type_arg_it)) {
					break;
				}
				type_arg = (ValaDataType*) gee_iterator_get (_type_arg_it);
				if (!first) {
					char* _tmp4_;
					_result_ = (_tmp4_ = g_strconcat (_result_, ",", NULL), _g_free0 (_result_), _tmp4_);
				} else {
					first = FALSE;
				}
				if (!vala_data_type_get_value_owned (type_arg)) {
					char* _tmp5_;
					_result_ = (_tmp5_ = g_strconcat (_result_, "weak ", NULL), _g_free0 (_result_), _tmp5_);
				}
				_result_ = (_tmp7_ = g_strconcat (_result_, _tmp6_ = vsc_symbol_completion_item_data_type_to_string (self, type_arg), NULL), _g_free0 (_result_), _tmp7_);
				_g_free0 (_tmp6_);
				_vala_code_node_unref0 (type_arg);
			}
			_gee_collection_object_unref0 (_type_arg_it);
		}
		_result_ = (_tmp8_ = g_strconcat (_result_, ">", NULL), _g_free0 (_result_), _tmp8_);
	}
	_tmp9_ = FALSE;
	_tmp10_ = FALSE;
	if (vala_data_type_get_nullable (type)) {
		_tmp10_ = !g_str_has_suffix (_result_, "?");
	} else {
		_tmp10_ = FALSE;
	}
	if (_tmp10_) {
		_tmp9_ = !g_str_has_suffix (_result_, "*");
	} else {
		_tmp9_ = FALSE;
	}
	if (_tmp9_) {
		char* _tmp11_;
		_result_ = (_tmp11_ = g_strconcat (_result_, "?", NULL), _g_free0 (_result_), _tmp11_);
	}
	if (vala_data_type_get_is_dynamic (type)) {
		char* _tmp12_;
		_result_ = (_tmp12_ = g_strconcat ("dynamic ", _result_, NULL), _g_free0 (_result_), _tmp12_);
	}
	_result_ = (_tmp13_ = g_markup_escape_text (_result_, -1), _g_free0 (_result_), _tmp13_);
	result = _result_;
	_gee_collection_object_unref0 (type_args);
	return result;
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static char* vsc_symbol_completion_item_formal_parameters_to_string (VscSymbolCompletionItem* self, GeeList* parameters) {
	char* result;
	char* params;
	char* param_sep;
	char* _tmp11_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (parameters != NULL, NULL);
	params = g_strdup ("");
	param_sep = g_strdup (" ");
	if (gee_collection_get_size ((GeeCollection*) parameters) > 2) {
		char* _tmp0_;
		param_sep = (_tmp0_ = g_strdup ("\n\t\t"), _g_free0 (param_sep), _tmp0_);
	}
	{
		GeeIterator* _parameter_it;
		_parameter_it = gee_iterable_iterator ((GeeIterable*) parameters);
		while (TRUE) {
			ValaFormalParameter* parameter;
			char* direction;
			char* default_expr;
			char* parameter_type;
			if (!gee_iterator_next (_parameter_it)) {
				break;
			}
			parameter = (ValaFormalParameter*) gee_iterator_get (_parameter_it);
			direction = g_strdup ("");
			default_expr = g_strdup ("");
			parameter_type = g_strdup ("");
			if (vala_formal_parameter_get_ellipsis (parameter)) {
				char* _tmp1_;
				params = (_tmp1_ = g_strdup_printf ("%s, %s", params, "..."), _g_free0 (params), _tmp1_);
			} else {
				gboolean _tmp4_;
				char* _tmp9_;
				if (vala_formal_parameter_get_direction (parameter) == VALA_PARAMETER_DIRECTION_OUT) {
					char* _tmp2_;
					direction = (_tmp2_ = g_strdup ("out "), _g_free0 (direction), _tmp2_);
				} else {
					if (vala_formal_parameter_get_direction (parameter) == VALA_PARAMETER_DIRECTION_REF) {
						char* _tmp3_;
						direction = (_tmp3_ = g_strdup ("ref "), _g_free0 (direction), _tmp3_);
					}
				}
				_tmp4_ = FALSE;
				if (vala_formal_parameter_get_default_expression (parameter) != NULL) {
					_tmp4_ = vala_expression_get_symbol_reference (vala_formal_parameter_get_default_expression (parameter)) != NULL;
				} else {
					_tmp4_ = FALSE;
				}
				if (_tmp4_) {
					char* _tmp6_;
					char* _tmp5_;
					default_expr = (_tmp6_ = g_strconcat (" = ", _tmp5_ = vala_code_node_to_string ((ValaCodeNode*) vala_formal_parameter_get_default_expression (parameter)), NULL), _g_free0 (default_expr), _tmp6_);
					_g_free0 (_tmp5_);
				}
				if (vala_formal_parameter_get_parameter_type (parameter) != NULL) {
					char* _tmp7_;
					parameter_type = (_tmp7_ = vsc_symbol_completion_item_data_type_to_string (self, vala_formal_parameter_get_parameter_type (parameter)), _g_free0 (parameter_type), _tmp7_);
				} else {
					char* _tmp8_;
					parameter_type = (_tmp8_ = g_strdup ("unknown"), _g_free0 (parameter_type), _tmp8_);
				}
				params = (_tmp9_ = g_strdup_printf ("%s,%s%s%s %s%s", params, param_sep, direction, parameter_type, vala_symbol_get_name ((ValaSymbol*) parameter), default_expr), _g_free0 (params), _tmp9_);
			}
			_vala_code_node_unref0 (parameter);
			_g_free0 (direction);
			_g_free0 (default_expr);
			_g_free0 (parameter_type);
		}
		_gee_collection_object_unref0 (_parameter_it);
	}
	if (_vala_strcmp0 (params, "") != 0) {
		char* _tmp10_;
		params = (_tmp10_ = string_substring (params, (glong) 2, string_get_length (params) - 2), _g_free0 (params), _tmp10_);
	}
	params = (_tmp11_ = g_markup_escape_text (params, -1), _g_free0 (params), _tmp11_);
	result = params;
	_g_free0 (param_sep);
	return result;
}


static gpointer _vala_code_node_ref0 (gpointer self) {
	return self ? vala_code_node_ref (self) : NULL;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_method (GType object_type, ValaMethod* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	gboolean _tmp1_;
	ValaSymbol* _tmp3_;
	GeeList* _tmp7_;
	gint _tmp8_;
	gint param_count;
	GeeList* _tmp9_;
	char* _tmp10_;
	char* params;
	const char* _tmp11_;
	const char* _tmp12_;
	char* _tmp14_;
	char* _tmp13_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	_tmp1_ = FALSE;
	if (vala_method_get_body (item) != NULL) {
		_tmp1_ = vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_body (item)) != NULL;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		gint _tmp2_;
		_tmp2_ = 0;
		if (vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_body (item))) == 0) {
			_tmp2_ = self->first_line;
		} else {
			_tmp2_ = vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_body (item)));
		}
		self->last_line = _tmp2_;
	}
	self->symbol = (_tmp3_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp3_);
	if (g_str_has_prefix (self->name, "new")) {
		char* _tmp4_;
		self->name = (_tmp4_ = string_substring (self->name, (glong) 3, string_get_length (self->name) - 3), _g_free0 (self->name), _tmp4_);
		if (_vala_strcmp0 (self->name, "") == 0) {
			char* _tmp5_;
			self->name = (_tmp5_ = g_strdup (vala_symbol_get_name (vala_symbol_get_parent_symbol ((ValaSymbol*) item))), _g_free0 (self->name), _tmp5_);
		} else {
			if (g_str_has_prefix (self->name, ".")) {
				char* _tmp6_;
				self->name = (_tmp6_ = string_substring (self->name, (glong) 1, string_get_length (self->name) - 1), _g_free0 (self->name), _tmp6_);
			}
		}
	}
	param_count = (_tmp8_ = gee_collection_get_size ((GeeCollection*) (_tmp7_ = vala_method_get_parameters (item))), _gee_collection_object_unref0 (_tmp7_), _tmp8_);
	params = (_tmp10_ = vsc_symbol_completion_item_formal_parameters_to_string (self, _tmp9_ = vala_method_get_parameters (item)), _gee_collection_object_unref0 (_tmp9_), _tmp10_);
	_tmp11_ = NULL;
	if (param_count > 2) {
		_tmp11_ = "\n";
	} else {
		_tmp11_ = " ";
	}
	_tmp12_ = NULL;
	if (param_count > 2) {
		_tmp12_ = "\n";
	} else {
		_tmp12_ = "";
	}
	self->info = (_tmp14_ = g_strdup_printf ("Method: %s\n\n%s%s<b>%s</b> (%s%s)", self->name, _tmp13_ = vsc_symbol_completion_item_data_type_to_string (self, vala_method_get_return_type (item)), _tmp11_, self->name, _tmp12_, params), _g_free0 (self->info), _tmp14_);
	_g_free0 (_tmp13_);
	_g_free0 (params);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_method (ValaMethod* item) {
	return vsc_symbol_completion_item_construct_with_method (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_creation_method (GType object_type, ValaCreationMethod* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp3_;
	gboolean _tmp4_;
	ValaSymbol* _tmp6_;
	GeeList* _tmp7_;
	gint _tmp8_;
	gint param_count;
	GeeList* _tmp9_;
	char* _tmp10_;
	char* params;
	const char* _tmp11_;
	const char* _tmp12_;
	char* _tmp14_;
	char* _tmp13_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	_tmp0_ = NULL;
	if (_vala_strcmp0 ("new", vala_symbol_get_name ((ValaSymbol*) item)) == 0) {
		char* _tmp1_;
		_tmp0_ = (_tmp1_ = g_strdup (vala_symbol_get_name (vala_symbol_get_parent_symbol ((ValaSymbol*) item))), _g_free0 (_tmp0_), _tmp1_);
	} else {
		char* _tmp2_;
		_tmp0_ = (_tmp2_ = g_strdup_printf ("%s.%s", vala_symbol_get_name (vala_symbol_get_parent_symbol ((ValaSymbol*) item)), vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (_tmp0_), _tmp2_);
	}
	self->name = (_tmp3_ = g_strdup (_tmp0_), _g_free0 (self->name), _tmp3_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	_tmp4_ = FALSE;
	if (vala_method_get_body ((ValaMethod*) item) != NULL) {
		_tmp4_ = vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_body ((ValaMethod*) item)) != NULL;
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		gint _tmp5_;
		_tmp5_ = 0;
		if (vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_body ((ValaMethod*) item))) == 0) {
			_tmp5_ = self->first_line;
		} else {
			_tmp5_ = vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_method_get_body ((ValaMethod*) item)));
		}
		self->last_line = _tmp5_;
	}
	self->symbol = (_tmp6_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp6_);
	param_count = (_tmp8_ = gee_collection_get_size ((GeeCollection*) (_tmp7_ = vala_method_get_parameters ((ValaMethod*) item))), _gee_collection_object_unref0 (_tmp7_), _tmp8_);
	params = (_tmp10_ = vsc_symbol_completion_item_formal_parameters_to_string (self, _tmp9_ = vala_method_get_parameters ((ValaMethod*) item)), _gee_collection_object_unref0 (_tmp9_), _tmp10_);
	_tmp11_ = NULL;
	if (param_count > 2) {
		_tmp11_ = "\n";
	} else {
		_tmp11_ = " ";
	}
	_tmp12_ = NULL;
	if (param_count > 2) {
		_tmp12_ = "\n";
	} else {
		_tmp12_ = "";
	}
	self->info = (_tmp14_ = g_strdup_printf ("CreationMethod: %s\n\n%s%s<b>%s</b> (%s%s)", self->name, _tmp13_ = vsc_symbol_completion_item_data_type_to_string (self, vala_method_get_return_type ((ValaMethod*) item)), _tmp11_, self->name, _tmp12_, params), _g_free0 (self->info), _tmp14_);
	_g_free0 (_tmp13_);
	_g_free0 (_tmp0_);
	_g_free0 (params);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_creation_method (ValaCreationMethod* item) {
	return vsc_symbol_completion_item_construct_with_creation_method (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_field (GType object_type, ValaField* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	ValaSymbol* _tmp1_;
	char* default_expr;
	char* _tmp5_;
	char* _tmp4_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp1_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp1_);
	default_expr = g_strdup ("");
	if (vala_field_get_initializer (item) != NULL) {
		char* _tmp3_;
		char* _tmp2_;
		default_expr = (_tmp3_ = g_strconcat (" = ", _tmp2_ = vala_code_node_to_string ((ValaCodeNode*) vala_field_get_initializer (item)), NULL), _g_free0 (default_expr), _tmp3_);
		_g_free0 (_tmp2_);
	}
	self->info = (_tmp5_ = g_strdup_printf ("Field: %s\n\n%s <b>%s</b>%s", self->name, _tmp4_ = vsc_symbol_completion_item_data_type_to_string (self, vala_field_get_field_type (item)), self->name, default_expr), _g_free0 (self->info), _tmp5_);
	_g_free0 (_tmp4_);
	_g_free0 (default_expr);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_field (ValaField* item) {
	return vsc_symbol_completion_item_construct_with_field (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_property (GType object_type, ValaProperty* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	ValaSymbol* _tmp1_;
	char* default_expr;
	char* _tmp6_;
	char* _tmp5_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp1_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp1_);
	if (NULL != vala_property_get_get_accessor (item)) {
		gboolean _tmp2_;
		self->last_line = vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_accessor_get_body (vala_property_get_get_accessor (item))));
		_tmp2_ = FALSE;
		if (NULL != vala_property_get_set_accessor (item)) {
			_tmp2_ = vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_accessor_get_body (vala_property_get_set_accessor (item)))) > self->last_line;
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			self->last_line = vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) vala_property_accessor_get_body (vala_property_get_set_accessor (item))));
		}
	}
	default_expr = g_strdup ("");
	if (vala_property_get_default_expression (item) != NULL) {
		char* _tmp4_;
		char* _tmp3_;
		default_expr = (_tmp4_ = g_strconcat (" = ", _tmp3_ = vala_code_node_to_string ((ValaCodeNode*) vala_property_get_default_expression (item)), NULL), _g_free0 (default_expr), _tmp4_);
		_g_free0 (_tmp3_);
	}
	self->info = (_tmp6_ = g_strdup_printf ("Property: %s\n\n%s <b>%s</b>%s", self->name, _tmp5_ = vsc_symbol_completion_item_data_type_to_string (self, vala_property_get_property_type (item)), self->name, default_expr), _g_free0 (self->info), _tmp6_);
	_g_free0 (_tmp5_);
	_g_free0 (default_expr);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_property (ValaProperty* item) {
	return vsc_symbol_completion_item_construct_with_property (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_struct (GType object_type, ValaStruct* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp1_;
	ValaSymbol* _tmp2_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	self->info = (_tmp1_ = g_strdup_printf ("Struct: %s", vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->info), _tmp1_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp2_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp2_);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_struct (ValaStruct* item) {
	return vsc_symbol_completion_item_construct_with_struct (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_class (GType object_type, ValaClass* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp1_;
	ValaSymbol* _tmp2_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	self->info = (_tmp1_ = g_strdup_printf ("Class: %s", vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->info), _tmp1_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp2_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp2_);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_class (ValaClass* item) {
	return vsc_symbol_completion_item_construct_with_class (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_interface (GType object_type, ValaInterface* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp1_;
	ValaSymbol* _tmp2_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	self->info = (_tmp1_ = g_strdup_printf ("Interface: %s", vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->info), _tmp1_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp2_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp2_);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_interface (ValaInterface* item) {
	return vsc_symbol_completion_item_construct_with_interface (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_signal (GType object_type, ValaSignal* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp1_;
	ValaSymbol* _tmp2_;
	GeeList* _tmp3_;
	gint _tmp4_;
	gint param_count;
	GeeList* _tmp5_;
	char* _tmp6_;
	char* params;
	const char* _tmp7_;
	const char* _tmp8_;
	char* _tmp10_;
	char* _tmp9_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	self->info = (_tmp1_ = g_strdup_printf ("Signal: %s", vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->info), _tmp1_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp2_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp2_);
	param_count = (_tmp4_ = gee_collection_get_size ((GeeCollection*) (_tmp3_ = vala_signal_get_parameters (item))), _gee_collection_object_unref0 (_tmp3_), _tmp4_);
	params = (_tmp6_ = vsc_symbol_completion_item_formal_parameters_to_string (self, _tmp5_ = vala_signal_get_parameters (item)), _gee_collection_object_unref0 (_tmp5_), _tmp6_);
	_tmp7_ = NULL;
	if (param_count > 2) {
		_tmp7_ = "\n";
	} else {
		_tmp7_ = " ";
	}
	_tmp8_ = NULL;
	if (param_count > 2) {
		_tmp8_ = "\n";
	} else {
		_tmp8_ = "";
	}
	self->info = (_tmp10_ = g_strdup_printf ("Signal: %s\n\n%s%s<b>%s</b> (%s%s)", self->name, _tmp9_ = vsc_symbol_completion_item_data_type_to_string (self, vala_signal_get_return_type (item)), _tmp7_, self->name, _tmp8_, params), _g_free0 (self->info), _tmp10_);
	_g_free0 (_tmp9_);
	_g_free0 (params);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_signal (ValaSignal* item) {
	return vsc_symbol_completion_item_construct_with_signal (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


VscSymbolCompletionItem* vsc_symbol_completion_item_construct_with_namespace (GType object_type, ValaNamespace* item) {
	VscSymbolCompletionItem * self;
	char* _tmp0_;
	char* _tmp1_;
	ValaSymbol* _tmp2_;
	g_return_val_if_fail (item != NULL, NULL);
	self = (VscSymbolCompletionItem*) g_object_new (object_type, NULL);
	self->name = (_tmp0_ = g_strdup (vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->name), _tmp0_);
	self->info = (_tmp1_ = g_strdup_printf ("Namespace: %s", vala_symbol_get_name ((ValaSymbol*) item)), _g_free0 (self->info), _tmp1_);
	vsc_symbol_completion_item_update_source_reference_info (self, (ValaSymbol*) item);
	self->symbol = (_tmp2_ = _vala_code_node_ref0 ((ValaSymbol*) item), _vala_code_node_unref0 (self->symbol), _tmp2_);
	return self;
}


VscSymbolCompletionItem* vsc_symbol_completion_item_new_with_namespace (ValaNamespace* item) {
	return vsc_symbol_completion_item_construct_with_namespace (VSC_TYPE_SYMBOL_COMPLETION_ITEM, item);
}


static void vsc_symbol_completion_item_update_source_reference_info (VscSymbolCompletionItem* self, ValaSymbol* item) {
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (item != NULL);
	self->file = (_tmp0_ = g_strdup (""), _g_free0 (self->file), _tmp0_);
	self->first_line = 0;
	self->last_line = 0;
	if (vala_code_node_get_source_reference ((ValaCodeNode*) item) != NULL) {
		if (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) item)) != NULL) {
			char* _tmp1_;
			self->file = (_tmp1_ = g_strdup (vala_source_file_get_filename (vala_source_reference_get_file (vala_code_node_get_source_reference ((ValaCodeNode*) item)))), _g_free0 (self->file), _tmp1_);
		}
		self->first_line = vala_source_reference_get_first_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
		self->last_line = vala_source_reference_get_last_line (vala_code_node_get_source_reference ((ValaCodeNode*) item));
	}
}


static void vsc_symbol_completion_item_class_init (VscSymbolCompletionItemClass * klass) {
	vsc_symbol_completion_item_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vsc_symbol_completion_item_finalize;
}


static void vsc_symbol_completion_item_instance_init (VscSymbolCompletionItem * self) {
	self->type_name = NULL;
	self->first_line = 0;
	self->last_line = 0;
}


static void vsc_symbol_completion_item_finalize (GObject* obj) {
	VscSymbolCompletionItem * self;
	self = VSC_SYMBOL_COMPLETION_ITEM (obj);
	_g_free0 (self->name);
	_g_free0 (self->type_name);
	_g_free0 (self->info);
	_g_free0 (self->file);
	_vala_code_node_unref0 (self->symbol);
	G_OBJECT_CLASS (vsc_symbol_completion_item_parent_class)->finalize (obj);
}


GType vsc_symbol_completion_item_get_type (void) {
	static GType vsc_symbol_completion_item_type_id = 0;
	if (vsc_symbol_completion_item_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscSymbolCompletionItemClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_symbol_completion_item_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSymbolCompletionItem), 0, (GInstanceInitFunc) vsc_symbol_completion_item_instance_init, NULL };
		vsc_symbol_completion_item_type_id = g_type_register_static (G_TYPE_OBJECT, "VscSymbolCompletionItem", &g_define_type_info, 0);
	}
	return vsc_symbol_completion_item_type_id;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




