/*
 *  vscsymbolcompletionresult.vala - Vala symbol completion library
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This library is free software; you can redistribute it and/or
 *  modify it under the terms of the GNU Lesser General Public
 *  License as published by the Free Software Foundation; either
 *  version 2.1 of the License, or (at your option) any later version.
 *
 *  This library is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *  Lesser General Public License for more details.
 *
 *  You should have received a copy of the GNU Lesser General Public
 *  License along with this library; if not, write to the Free Software
 *  Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301  USA
 */

#include <glib.h>
#include <glib-object.h>
#include <gee.h>
#include <stdlib.h>
#include <string.h>
#include <vala.h>


#define VSC_TYPE_SYMBOL_COMPLETION_RESULT (vsc_symbol_completion_result_get_type ())
#define VSC_SYMBOL_COMPLETION_RESULT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_RESULT, VscSymbolCompletionResult))
#define VSC_SYMBOL_COMPLETION_RESULT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_RESULT, VscSymbolCompletionResultClass))
#define VSC_IS_SYMBOL_COMPLETION_RESULT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_RESULT))
#define VSC_IS_SYMBOL_COMPLETION_RESULT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_RESULT))
#define VSC_SYMBOL_COMPLETION_RESULT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_RESULT, VscSymbolCompletionResultClass))

typedef struct _VscSymbolCompletionResult VscSymbolCompletionResult;
typedef struct _VscSymbolCompletionResultClass VscSymbolCompletionResultClass;
typedef struct _VscSymbolCompletionResultPrivate VscSymbolCompletionResultPrivate;

#define VSC_TYPE_SYMBOL_COMPLETION_ITEM (vsc_symbol_completion_item_get_type ())
#define VSC_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItem))
#define VSC_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))
#define VSC_IS_SYMBOL_COMPLETION_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_IS_SYMBOL_COMPLETION_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VSC_TYPE_SYMBOL_COMPLETION_ITEM))
#define VSC_SYMBOL_COMPLETION_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VSC_TYPE_SYMBOL_COMPLETION_ITEM, VscSymbolCompletionItemClass))

typedef struct _VscSymbolCompletionItem VscSymbolCompletionItem;
typedef struct _VscSymbolCompletionItemClass VscSymbolCompletionItemClass;
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
typedef struct _VscSymbolCompletionItemPrivate VscSymbolCompletionItemPrivate;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _VscSymbolCompletionResult {
	GObject parent_instance;
	VscSymbolCompletionResultPrivate * priv;
	GeeList* properties;
	GeeList* classes;
	GeeList* interfaces;
	GeeList* structs;
	GeeList* methods;
	GeeList* fields;
	GeeList* signals;
	GeeList* others;
	GeeList* namespaces;
	GeeList* enums;
	GeeList* constants;
	GeeList* error_domains;
};

struct _VscSymbolCompletionResultClass {
	GObjectClass parent_class;
};

struct _VscSymbolCompletionItem {
	GObject parent_instance;
	VscSymbolCompletionItemPrivate * priv;
	char* name;
	char* type_name;
	char* info;
	char* file;
	gint first_line;
	gint last_line;
	ValaSymbol* symbol;
};

struct _VscSymbolCompletionItemClass {
	GObjectClass parent_class;
};


static gpointer vsc_symbol_completion_result_parent_class = NULL;

GType vsc_symbol_completion_result_get_type (void);
GType vsc_symbol_completion_item_get_type (void);
enum  {
	VSC_SYMBOL_COMPLETION_RESULT_DUMMY_PROPERTY,
	VSC_SYMBOL_COMPLETION_RESULT_IS_EMPTY,
	VSC_SYMBOL_COMPLETION_RESULT_COUNT
};
static gboolean vsc_symbol_completion_result_symbols_contain (VscSymbolCompletionResult* self, GeeList* data, const char* name);
gboolean vsc_symbol_completion_result_namespaces_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_classes_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_interfaces_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_structs_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_constants_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_enums_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_methods_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_delegates_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_signals_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_fields_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_properties_contain (VscSymbolCompletionResult* self, const char* name);
gboolean vsc_symbol_completion_result_error_domains_contain (VscSymbolCompletionResult* self, const char* name);
VscSymbolCompletionResult* vsc_symbol_completion_result_new (void);
VscSymbolCompletionResult* vsc_symbol_completion_result_construct (GType object_type);
gint vsc_symbol_completion_result_get_count (VscSymbolCompletionResult* self);
gboolean vsc_symbol_completion_result_get_is_empty (VscSymbolCompletionResult* self);
static void vsc_symbol_completion_result_finalize (GObject* obj);
static void vsc_symbol_completion_result_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



gboolean vsc_symbol_completion_result_namespaces_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->namespaces, name);
	return result;
}


gboolean vsc_symbol_completion_result_classes_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->classes, name);
	return result;
}


gboolean vsc_symbol_completion_result_interfaces_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->interfaces, name);
	return result;
}


gboolean vsc_symbol_completion_result_structs_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->structs, name);
	return result;
}


gboolean vsc_symbol_completion_result_constants_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->constants, name);
	return result;
}


gboolean vsc_symbol_completion_result_enums_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->enums, name);
	return result;
}


gboolean vsc_symbol_completion_result_methods_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->methods, name);
	return result;
}


gboolean vsc_symbol_completion_result_delegates_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->others, name);
	return result;
}


gboolean vsc_symbol_completion_result_signals_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->signals, name);
	return result;
}


gboolean vsc_symbol_completion_result_fields_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->fields, name);
	return result;
}


gboolean vsc_symbol_completion_result_properties_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->properties, name);
	return result;
}


gboolean vsc_symbol_completion_result_error_domains_contain (VscSymbolCompletionResult* self, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	result = vsc_symbol_completion_result_symbols_contain (self, self->error_domains, name);
	return result;
}


static gboolean vsc_symbol_completion_result_symbols_contain (VscSymbolCompletionResult* self, GeeList* data, const char* name) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (data != NULL, FALSE);
	g_return_val_if_fail (name != NULL, FALSE);
	if (gee_collection_get_size ((GeeCollection*) data) == 0) {
		result = FALSE;
		return result;
	}
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) data);
		while (TRUE) {
			VscSymbolCompletionItem* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (VscSymbolCompletionItem*) gee_iterator_get (_item_it);
			if (_vala_strcmp0 (item->name, name) == 0) {
				result = TRUE;
				_g_object_unref0 (item);
				_gee_collection_object_unref0 (_item_it);
				return result;
			}
			_g_object_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	result = FALSE;
	return result;
}


VscSymbolCompletionResult* vsc_symbol_completion_result_construct (GType object_type) {
	VscSymbolCompletionResult * self;
	self = (VscSymbolCompletionResult*) g_object_new (object_type, NULL);
	return self;
}


VscSymbolCompletionResult* vsc_symbol_completion_result_new (void) {
	return vsc_symbol_completion_result_construct (VSC_TYPE_SYMBOL_COMPLETION_RESULT);
}


gboolean vsc_symbol_completion_result_get_is_empty (VscSymbolCompletionResult* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	result = vsc_symbol_completion_result_get_count (self) == 0;
	return result;
}


gint vsc_symbol_completion_result_get_count (VscSymbolCompletionResult* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = ((((((((((gee_collection_get_size ((GeeCollection*) self->properties) + gee_collection_get_size ((GeeCollection*) self->classes)) + gee_collection_get_size ((GeeCollection*) self->interfaces)) + gee_collection_get_size ((GeeCollection*) self->structs)) + gee_collection_get_size ((GeeCollection*) self->methods)) + gee_collection_get_size ((GeeCollection*) self->fields)) + gee_collection_get_size ((GeeCollection*) self->signals)) + gee_collection_get_size ((GeeCollection*) self->namespaces)) + gee_collection_get_size ((GeeCollection*) self->enums)) + gee_collection_get_size ((GeeCollection*) self->error_domains)) + gee_collection_get_size ((GeeCollection*) self->constants)) + gee_collection_get_size ((GeeCollection*) self->others);
	return result;
}


static void vsc_symbol_completion_result_class_init (VscSymbolCompletionResultClass * klass) {
	vsc_symbol_completion_result_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->get_property = vsc_symbol_completion_result_get_property;
	G_OBJECT_CLASS (klass)->finalize = vsc_symbol_completion_result_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SYMBOL_COMPLETION_RESULT_IS_EMPTY, g_param_spec_boolean ("is-empty", "is-empty", "is-empty", FALSE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VSC_SYMBOL_COMPLETION_RESULT_COUNT, g_param_spec_int ("count", "count", "count", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vsc_symbol_completion_result_instance_init (VscSymbolCompletionResult * self) {
	self->properties = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->classes = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->interfaces = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->structs = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->methods = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->fields = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->signals = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->others = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->namespaces = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->enums = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->constants = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->error_domains = (GeeList*) gee_array_list_new (VSC_TYPE_SYMBOL_COMPLETION_ITEM, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
}


static void vsc_symbol_completion_result_finalize (GObject* obj) {
	VscSymbolCompletionResult * self;
	self = VSC_SYMBOL_COMPLETION_RESULT (obj);
	_gee_collection_object_unref0 (self->properties);
	_gee_collection_object_unref0 (self->classes);
	_gee_collection_object_unref0 (self->interfaces);
	_gee_collection_object_unref0 (self->structs);
	_gee_collection_object_unref0 (self->methods);
	_gee_collection_object_unref0 (self->fields);
	_gee_collection_object_unref0 (self->signals);
	_gee_collection_object_unref0 (self->others);
	_gee_collection_object_unref0 (self->namespaces);
	_gee_collection_object_unref0 (self->enums);
	_gee_collection_object_unref0 (self->constants);
	_gee_collection_object_unref0 (self->error_domains);
	G_OBJECT_CLASS (vsc_symbol_completion_result_parent_class)->finalize (obj);
}


GType vsc_symbol_completion_result_get_type (void) {
	static GType vsc_symbol_completion_result_type_id = 0;
	if (vsc_symbol_completion_result_type_id == 0) {
		static const GTypeInfo g_define_type_info = { sizeof (VscSymbolCompletionResultClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vsc_symbol_completion_result_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VscSymbolCompletionResult), 0, (GInstanceInitFunc) vsc_symbol_completion_result_instance_init, NULL };
		vsc_symbol_completion_result_type_id = g_type_register_static (G_TYPE_OBJECT, "VscSymbolCompletionResult", &g_define_type_info, 0);
	}
	return vsc_symbol_completion_result_type_id;
}


static void vsc_symbol_completion_result_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VscSymbolCompletionResult * self;
	self = VSC_SYMBOL_COMPLETION_RESULT (object);
	switch (property_id) {
		case VSC_SYMBOL_COMPLETION_RESULT_IS_EMPTY:
		g_value_set_boolean (value, vsc_symbol_completion_result_get_is_empty (self));
		break;
		case VSC_SYMBOL_COMPLETION_RESULT_COUNT:
		g_value_set_int (value, vsc_symbol_completion_result_get_count (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




