/*
 *  vtgbracketcompletion.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gedit/gedit-view.h>
#include <stdlib.h>
#include <string.h>
#include <gdk/gdk.h>
#include <gtk/gtk.h>
#include <float.h>
#include <math.h>
#include <gdk/gdkkeysyms.h>
#include <gedit/gedit-prefs-manager.h>


#define VTG_TYPE_BRACKET_COMPLETION (vtg_bracket_completion_get_type ())
#define VTG_BRACKET_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_BRACKET_COMPLETION, VtgBracketCompletion))
#define VTG_BRACKET_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_BRACKET_COMPLETION, VtgBracketCompletionClass))
#define VTG_IS_BRACKET_COMPLETION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_BRACKET_COMPLETION))
#define VTG_IS_BRACKET_COMPLETION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_BRACKET_COMPLETION))
#define VTG_BRACKET_COMPLETION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_BRACKET_COMPLETION, VtgBracketCompletionClass))

typedef struct _VtgBracketCompletion VtgBracketCompletion;
typedef struct _VtgBracketCompletionClass VtgBracketCompletionClass;
typedef struct _VtgBracketCompletionPrivate VtgBracketCompletionPrivate;

#define VTG_TYPE_PLUGIN_INSTANCE (vtg_plugin_instance_get_type ())
#define VTG_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstance))
#define VTG_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))
#define VTG_IS_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_IS_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_PLUGIN_INSTANCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))

typedef struct _VtgPluginInstance VtgPluginInstance;
typedef struct _VtgPluginInstanceClass VtgPluginInstanceClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))

struct _VtgBracketCompletion {
	GObject parent_instance;
	VtgBracketCompletionPrivate * priv;
};

struct _VtgBracketCompletionClass {
	GObjectClass parent_class;
};

struct _VtgBracketCompletionPrivate {
	VtgPluginInstance* _plugin_instance;
	GeditView* _view;
	char* tab_chars;
};


static gpointer vtg_bracket_completion_parent_class = NULL;
static GType vtg_bracket_completion_type_id = 0;

GType vtg_bracket_completion_get_type (void);
GType vtg_bracket_completion_register_type (GTypeModule * module);
GType vtg_plugin_instance_get_type (void);
GType vtg_plugin_instance_register_type (GTypeModule * module);
#define VTG_BRACKET_COMPLETION_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_BRACKET_COMPLETION, VtgBracketCompletionPrivate))
enum  {
	VTG_BRACKET_COMPLETION_DUMMY_PROPERTY,
	VTG_BRACKET_COMPLETION_PLUGIN_INSTANCE,
	VTG_BRACKET_COMPLETION_VIEW
};
static void vtg_bracket_completion_set_plugin_instance (VtgBracketCompletion* self, VtgPluginInstance* value);
static void vtg_bracket_completion_set_view (VtgBracketCompletion* self, GeditView* value);
VtgBracketCompletion* vtg_bracket_completion_new (VtgPluginInstance* plugin_instance, GeditView* view);
VtgBracketCompletion* vtg_bracket_completion_construct (GType object_type, VtgPluginInstance* plugin_instance, GeditView* view);
static void vtg_bracket_completion_disconnect_view (VtgBracketCompletion* self, GeditView* view);
void vtg_bracket_completion_deactivate (VtgBracketCompletion* self);
static gboolean vtg_bracket_completion_on_view_key_press (GeditView* sender, const GdkEventKey* evt, VtgBracketCompletion* instance);
static void vtg_bracket_completion_connect_view (VtgBracketCompletion* self, GeditView* view);
static gboolean vtg_bracket_completion_enclose_selection_with_delimiters (VtgBracketCompletion* self, GtkTextBuffer* src, const char* start_delimiter, const char* end_delimiter);
static void vtg_bracket_completion_insert_chars (VtgBracketCompletion* self, GtkTextBuffer* src, const char* chars);
static void vtg_bracket_completion_move_backwards (VtgBracketCompletion* self, GtkTextBuffer* src, gint count);
static const char* vtg_bracket_completion_current_indentation_text (VtgBracketCompletion* self, GtkTextBuffer* src);
static gboolean vtg_bracket_completion_find_char (VtgBracketCompletion* self, const GtkTextIter* start, gunichar char_to_find, gunichar complementary_char, gunichar stop_to_char);
VtgPluginInstance* vtg_bracket_completion_get_plugin_instance (VtgBracketCompletion* self);
GeditView* vtg_bracket_completion_get_view (VtgBracketCompletion* self);
static GObject * vtg_bracket_completion_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void vtg_bracket_completion_finalize (GObject* obj);
static void vtg_bracket_completion_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void vtg_bracket_completion_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



VtgBracketCompletion* vtg_bracket_completion_construct (GType object_type, VtgPluginInstance* plugin_instance, GeditView* view) {
	GParameter * __params;
	GParameter * __params_it;
	VtgBracketCompletion * self;
	g_return_val_if_fail (plugin_instance != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	__params = g_new0 (GParameter, 2);
	__params_it = __params;
	__params_it->name = "plugin-instance";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN_INSTANCE);
	g_value_set_object (&__params_it->value, plugin_instance);
	__params_it++;
	__params_it->name = "view";
	g_value_init (&__params_it->value, GEDIT_TYPE_VIEW);
	g_value_set_object (&__params_it->value, view);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgBracketCompletion* vtg_bracket_completion_new (VtgPluginInstance* plugin_instance, GeditView* view) {
	return vtg_bracket_completion_construct (VTG_TYPE_BRACKET_COMPLETION, plugin_instance, view);
}


void vtg_bracket_completion_deactivate (VtgBracketCompletion* self) {
	g_return_if_fail (self != NULL);
	vtg_bracket_completion_disconnect_view (self, self->priv->_view);
}


static void vtg_bracket_completion_connect_view (VtgBracketCompletion* self, GeditView* view) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	g_signal_connect (view, "key-press-event", (GCallback) vtg_bracket_completion_on_view_key_press, self);
}


static void vtg_bracket_completion_disconnect_view (VtgBracketCompletion* self, GeditView* view) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (view != NULL);
	g_signal_handlers_disconnect_by_func (view, (void*) vtg_bracket_completion_on_view_key_press, self);
}


static gboolean vtg_bracket_completion_enclose_selection_with_delimiters (VtgBracketCompletion* self, GtkTextBuffer* src, const char* start_delimiter, const char* end_delimiter) {
	gboolean result;
	GtkTextIter sel_start = {0};
	GtkTextIter sel_end = {0};
	char* text;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (src != NULL, FALSE);
	g_return_val_if_fail (start_delimiter != NULL, FALSE);
	gtk_text_buffer_get_selection_bounds (src, &sel_start, &sel_end);
	text = g_strdup (gtk_text_buffer_get_text (src, &sel_start, &sel_end, TRUE));
	if (end_delimiter == NULL) {
		end_delimiter = start_delimiter;
	}
	_tmp0_ = FALSE;
	if (g_str_has_prefix (text, start_delimiter) == FALSE) {
		_tmp0_ = g_str_has_suffix (text, end_delimiter) == FALSE;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		GtkTextMark* mark;
		GtkTextIter pos = {0};
		char* _tmp1_;
		mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
		text = (_tmp1_ = g_strdup_printf ("%s%s%s", start_delimiter, text, end_delimiter), _g_free0 (text), _tmp1_);
		gtk_text_buffer_begin_user_action (src);
		gtk_text_buffer_delete_selection (src, TRUE, TRUE);
		gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
		gtk_text_buffer_insert (src, &pos, text, (gint) g_utf8_strlen (text, -1));
		gtk_text_buffer_end_user_action (src);
		result = TRUE;
		_g_free0 (text);
		return result;
	}
	result = FALSE;
	_g_free0 (text);
	return result;
}


static void vtg_bracket_completion_insert_chars (VtgBracketCompletion* self, GtkTextBuffer* src, const char* chars) {
	GtkTextMark* mark;
	GtkTextIter pos = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (src != NULL);
	g_return_if_fail (chars != NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
	gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
	gtk_text_buffer_begin_user_action (src);
	gtk_text_buffer_insert (src, &pos, chars, (gint) g_utf8_strlen (chars, -1));
	gtk_text_buffer_end_user_action (src);
}


static void vtg_bracket_completion_move_backwards (VtgBracketCompletion* self, GtkTextBuffer* src, gint count) {
	GtkTextMark* mark;
	GtkTextIter pos = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (src != NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
	gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
	gtk_text_iter_backward_chars (&pos, count);
	gtk_text_buffer_place_cursor (src, &pos);
}


static const char* vtg_bracket_completion_current_indentation_text (VtgBracketCompletion* self, GtkTextBuffer* src) {
	const char* result;
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	const char* text;
	gint col;
	gunichar ch;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (src != NULL, NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
	text = "";
	col = 0;
	gtk_text_buffer_get_iter_at_mark (src, &end, mark);
	col = gtk_text_iter_get_line_offset (&end);
	gtk_text_iter_set_line_offset (&end, 0);
	start = end;
	ch = gtk_text_iter_get_char (&end);
	if (g_unichar_isspace (ch)) {
		while (TRUE) {
			if (!gtk_text_iter_forward_char (&end)) {
				break;
			}
			if (gtk_text_iter_starts_word (&end)) {
				break;
			} else {
				if (gtk_text_iter_get_line_offset (&end) >= col) {
					break;
				} else {
					ch = gtk_text_iter_get_char (&end);
					if (g_unichar_isspace (ch) == FALSE) {
						break;
					}
				}
			}
		}
	}
	if (!gtk_text_iter_equal (&start, &end)) {
		text = gtk_text_iter_get_text (&start, &end);
	}
	result = text;
	return result;
}


static gboolean vtg_bracket_completion_find_char (VtgBracketCompletion* self, const GtkTextIter* start, gunichar char_to_find, gunichar complementary_char, gunichar stop_to_char) {
	gboolean result;
	gboolean _result_;
	gint level;
	GtkTextIter curr;
	g_return_val_if_fail (self != NULL, FALSE);
	_result_ = FALSE;
	level = 0;
	curr = (*start);
	{
		gboolean _tmp0_;
		_tmp0_ = TRUE;
		while (TRUE) {
			gunichar ch;
			if (!_tmp0_) {
				if (!gtk_text_iter_forward_char (&curr)) {
					break;
				}
			}
			_tmp0_ = FALSE;
			ch = gtk_text_iter_get_char (&curr);
			if (ch == stop_to_char) {
				break;
			} else {
				if (ch == char_to_find) {
					if (level == 0) {
						_result_ = TRUE;
						break;
					} else {
						level--;
					}
				} else {
					if (ch == complementary_char) {
						level++;
					}
				}
			}
		}
	}
	result = _result_;
	return result;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static gboolean vtg_bracket_completion_on_view_key_press (GeditView* sender, const GdkEventKey* evt, VtgBracketCompletion* instance) {
	gboolean result;
	gboolean _result_;
	g_return_val_if_fail (sender != NULL, FALSE);
	g_return_val_if_fail (instance != NULL, FALSE);
	_result_ = FALSE;
	if (((*evt).state & GDK_MOD1_MASK) == 0) {
		GtkTextBuffer* src;
		GtkTextMark* mark;
		GtkTextIter pos = {0};
		const char* indent;
		char* buffer;
		gunichar ch;
		src = _g_object_ref0 (gtk_text_view_get_buffer ((GtkTextView*) sender));
		mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert (src));
		indent = NULL;
		buffer = NULL;
		ch = (gunichar) gdk_keyval_to_unicode ((*evt).keyval);
		gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
		if (ch == '(') {
			if (gtk_text_buffer_get_has_selection (src)) {
				if (vtg_bracket_completion_enclose_selection_with_delimiters (instance, src, "(", ")")) {
					gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
					gtk_text_buffer_place_cursor (src, &pos);
					_result_ = TRUE;
				}
			} else {
				if (!vtg_bracket_completion_find_char (instance, &pos, (gunichar) ')', (gunichar) '(', (gunichar) ';')) {
					vtg_bracket_completion_insert_chars (instance, src, ")");
					vtg_bracket_completion_move_backwards (instance, src, 1);
				}
			}
		} else {
			if (ch == '[') {
				if (!vtg_bracket_completion_find_char (instance, &pos, (gunichar) ']', (gunichar) '[', (gunichar) ';')) {
					vtg_bracket_completion_insert_chars (instance, src, "]");
					vtg_bracket_completion_move_backwards (instance, src, 1);
				}
			} else {
				if (ch == '*') {
					indent = vtg_bracket_completion_current_indentation_text (instance, src);
					if (gtk_text_buffer_get_has_selection (src)) {
						GtkTextIter sel_start = {0};
						GtkTextIter sel_end = {0};
						gboolean _tmp0_;
						gtk_text_buffer_get_selection_bounds (src, &sel_start, &sel_end);
						gtk_text_iter_backward_char (&sel_start);
						_tmp0_ = FALSE;
						if (gtk_text_iter_get_char (&sel_start) == '/') {
							_tmp0_ = vtg_bracket_completion_enclose_selection_with_delimiters (instance, src, "*", "*/");
						} else {
							_tmp0_ = FALSE;
						}
						if (_tmp0_) {
							gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
							gtk_text_buffer_place_cursor (src, &pos);
							_result_ = TRUE;
						}
					} else {
						gtk_text_iter_backward_char (&pos);
						if (gtk_text_iter_get_char (&pos) == '/') {
							char* _tmp1_;
							buffer = (_tmp1_ = g_strdup_printf ("*\n%s \n%s*/", indent, indent), _g_free0 (buffer), _tmp1_);
							vtg_bracket_completion_insert_chars (instance, src, buffer);
							gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
							vtg_bracket_completion_move_backwards (instance, src, (2 + ((gint) string_get_length (indent))) + 1);
							_result_ = TRUE;
						}
					}
				} else {
					if (ch == '{') {
						indent = vtg_bracket_completion_current_indentation_text (instance, src);
						if (gtk_text_buffer_get_has_selection (src)) {
							if (vtg_bracket_completion_enclose_selection_with_delimiters (instance, src, "{", "} ")) {
								gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
								gtk_text_buffer_place_cursor (src, &pos);
							}
						} else {
							char* _tmp2_;
							buffer = (_tmp2_ = g_strdup_printf ("{\n%s%s\n%s}", indent, instance->priv->tab_chars, indent), _g_free0 (buffer), _tmp2_);
							vtg_bracket_completion_insert_chars (instance, src, buffer);
							gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
							vtg_bracket_completion_move_backwards (instance, src, 2 + ((gint) string_get_length (indent)));
						}
						_result_ = TRUE;
					} else {
						if ((*evt).keyval == GDK_Return) {
							indent = vtg_bracket_completion_current_indentation_text (instance, src);
							if (((*evt).state & GDK_SHIFT_MASK) != 0) {
								if (!gtk_text_iter_ends_line (&pos)) {
									gtk_text_iter_forward_to_line_end (&pos);
									gtk_text_buffer_place_cursor (src, &pos);
								}
								if (((*evt).state & GDK_CONTROL_MASK) == 0) {
									GtkTextIter tmp;
									tmp = pos;
									while (TRUE) {
										if (!gtk_text_iter_backward_char (&pos)) {
											break;
										}
										ch = gtk_text_iter_get_char (&pos);
										if (!g_unichar_isspace (ch)) {
											gtk_text_iter_forward_char (&pos);
											break;
										}
									}
									if (gtk_text_iter_equal (&tmp, &pos)) {
										char* _tmp3_;
										vtg_bracket_completion_insert_chars (instance, src, _tmp3_ = g_strdup_printf (";\n%s", indent));
										_g_free0 (_tmp3_);
									} else {
										char* _tmp4_;
										gtk_text_buffer_place_cursor (src, &pos);
										vtg_bracket_completion_insert_chars (instance, src, ";");
										gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
										gtk_text_iter_forward_to_line_end (&pos);
										gtk_text_buffer_place_cursor (src, &pos);
										vtg_bracket_completion_insert_chars (instance, src, _tmp4_ = g_strdup_printf ("\n%s", indent));
										_g_free0 (_tmp4_);
									}
									gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
									gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
									gtk_text_buffer_place_cursor (src, &pos);
								} else {
									char* _tmp5_;
									buffer = (_tmp5_ = g_strdup_printf ("\n%s{\n%s%s\n%s}", indent, indent, instance->priv->tab_chars, indent), _g_free0 (buffer), _tmp5_);
									vtg_bracket_completion_insert_chars (instance, src, buffer);
									gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
									vtg_bracket_completion_move_backwards (instance, src, 2 + ((gint) string_get_length (indent)));
								}
								_result_ = TRUE;
							} else {
								gboolean _tmp6_;
								_tmp6_ = FALSE;
								if (indent != NULL) {
									_tmp6_ = _vala_strcmp0 (indent, "") != 0;
								} else {
									_tmp6_ = FALSE;
								}
								if (_tmp6_) {
									char* _tmp7_;
									vtg_bracket_completion_insert_chars (instance, src, _tmp7_ = g_strdup_printf ("\n%s", indent));
									_g_free0 (_tmp7_);
									_result_ = TRUE;
									gtk_text_view_scroll_to_mark ((GtkTextView*) sender, mark, (double) 0, FALSE, (double) 0, (double) 0);
								}
							}
						} else {
							if (ch == '\"') {
								if (gtk_text_buffer_get_has_selection (src)) {
									if (vtg_bracket_completion_enclose_selection_with_delimiters (instance, src, "\"", NULL)) {
										gtk_text_buffer_get_iter_at_mark (src, &pos, mark);
										gtk_text_buffer_place_cursor (src, &pos);
										_result_ = TRUE;
									}
								} else {
									if (!vtg_bracket_completion_find_char (instance, &pos, (gunichar) '\"', (gunichar) '\"', (gunichar) ';')) {
										vtg_bracket_completion_insert_chars (instance, src, "\"");
										vtg_bracket_completion_move_backwards (instance, src, 1);
									}
								}
							}
						}
					}
				}
			}
		}
		_g_object_unref0 (src);
		_g_free0 (buffer);
	}
	result = _result_;
	return result;
}


VtgPluginInstance* vtg_bracket_completion_get_plugin_instance (VtgBracketCompletion* self) {
	VtgPluginInstance* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_plugin_instance;
	return result;
}


static void vtg_bracket_completion_set_plugin_instance (VtgBracketCompletion* self, VtgPluginInstance* value) {
	VtgPluginInstance* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_plugin_instance = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_plugin_instance), _tmp0_);
	g_object_notify ((GObject *) self, "plugin-instance");
}


GeditView* vtg_bracket_completion_get_view (VtgBracketCompletion* self) {
	GeditView* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_view;
	return result;
}


static void vtg_bracket_completion_set_view (VtgBracketCompletion* self, GeditView* value) {
	GeditView* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_view = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_view), _tmp0_);
	g_object_notify ((GObject *) self, "view");
}


static GObject * vtg_bracket_completion_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgBracketCompletionClass * klass;
	GObjectClass * parent_class;
	VtgBracketCompletion * self;
	klass = VTG_BRACKET_COMPLETION_CLASS (g_type_class_peek (VTG_TYPE_BRACKET_COMPLETION));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_BRACKET_COMPLETION (obj);
	{
		if (gedit_prefs_manager_get_insert_spaces ()) {
			char* _tmp0_;
			self->priv->tab_chars = (_tmp0_ = g_strnfill ((gsize) gedit_prefs_manager_get_tabs_size (), ' '), _g_free0 (self->priv->tab_chars), _tmp0_);
		} else {
			char* _tmp1_;
			self->priv->tab_chars = (_tmp1_ = g_strdup ("\t"), _g_free0 (self->priv->tab_chars), _tmp1_);
		}
		vtg_bracket_completion_connect_view (self, self->priv->_view);
	}
	return obj;
}


static void vtg_bracket_completion_class_init (VtgBracketCompletionClass * klass) {
	vtg_bracket_completion_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgBracketCompletionPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_bracket_completion_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_bracket_completion_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_bracket_completion_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_bracket_completion_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BRACKET_COMPLETION_PLUGIN_INSTANCE, g_param_spec_object ("plugin-instance", "plugin-instance", "plugin-instance", VTG_TYPE_PLUGIN_INSTANCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BRACKET_COMPLETION_VIEW, g_param_spec_object ("view", "view", "view", GEDIT_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_bracket_completion_instance_init (VtgBracketCompletion * self) {
	self->priv = VTG_BRACKET_COMPLETION_GET_PRIVATE (self);
	self->priv->tab_chars = g_strdup ("");
}


static void vtg_bracket_completion_finalize (GObject* obj) {
	VtgBracketCompletion * self;
	self = VTG_BRACKET_COMPLETION (obj);
	_g_object_unref0 (self->priv->_plugin_instance);
	_g_object_unref0 (self->priv->_view);
	_g_free0 (self->priv->tab_chars);
	G_OBJECT_CLASS (vtg_bracket_completion_parent_class)->finalize (obj);
}


GType vtg_bracket_completion_get_type (void) {
	return vtg_bracket_completion_type_id;
}


GType vtg_bracket_completion_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgBracketCompletionClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_bracket_completion_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgBracketCompletion), 0, (GInstanceInitFunc) vtg_bracket_completion_instance_init, NULL };
	vtg_bracket_completion_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgBracketCompletion", &g_define_type_info, 0);
	return vtg_bracket_completion_type_id;
}


static void vtg_bracket_completion_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgBracketCompletion * self;
	self = VTG_BRACKET_COMPLETION (object);
	switch (property_id) {
		case VTG_BRACKET_COMPLETION_PLUGIN_INSTANCE:
		g_value_set_object (value, vtg_bracket_completion_get_plugin_instance (self));
		break;
		case VTG_BRACKET_COMPLETION_VIEW:
		g_value_set_object (value, vtg_bracket_completion_get_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_bracket_completion_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgBracketCompletion * self;
	self = VTG_BRACKET_COMPLETION (object);
	switch (property_id) {
		case VTG_BRACKET_COMPLETION_PLUGIN_INSTANCE:
		vtg_bracket_completion_set_plugin_instance (self, g_value_get_object (value));
		break;
		case VTG_BRACKET_COMPLETION_VIEW:
		vtg_bracket_completion_set_view (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




