/*
 *  vtgbuildlogview.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-panel.h>
#include <gedit/gedit-view.h>
#include <stdlib.h>
#include <string.h>
#include <gedit/gedit-tab.h>
#include <float.h>
#include <math.h>
#include <glib/gi18n-lib.h>


#define VTG_TYPE_BUILD_LOG_VIEW (vtg_build_log_view_get_type ())
#define VTG_BUILD_LOG_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogView))
#define VTG_BUILD_LOG_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogViewClass))
#define VTG_IS_BUILD_LOG_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_BUILD_LOG_VIEW))
#define VTG_IS_BUILD_LOG_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_BUILD_LOG_VIEW))
#define VTG_BUILD_LOG_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogViewClass))

typedef struct _VtgBuildLogView VtgBuildLogView;
typedef struct _VtgBuildLogViewClass VtgBuildLogViewClass;
typedef struct _VtgBuildLogViewPrivate VtgBuildLogViewPrivate;

#define VTG_TYPE_PLUGIN_INSTANCE (vtg_plugin_instance_get_type ())
#define VTG_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstance))
#define VTG_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))
#define VTG_IS_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_IS_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_PLUGIN_INSTANCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))

typedef struct _VtgPluginInstance VtgPluginInstance;
typedef struct _VtgPluginInstanceClass VtgPluginInstanceClass;

#define VTG_TYPE_PROJECT_MANAGER (vtg_project_manager_get_type ())
#define VTG_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManager))
#define VTG_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))
#define VTG_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER))
#define VTG_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER))
#define VTG_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))

typedef struct _VtgProjectManager VtgProjectManager;
typedef struct _VtgProjectManagerClass VtgProjectManagerClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define VTG_TYPE_OUTPUT_TYPES (vtg_output_types_get_type ())

#define VTG_TYPE_OUTPUT_VIEW (vtg_output_view_get_type ())
#define VTG_OUTPUT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_OUTPUT_VIEW, VtgOutputView))
#define VTG_OUTPUT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_OUTPUT_VIEW, VtgOutputViewClass))
#define VTG_IS_OUTPUT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_OUTPUT_VIEW))
#define VTG_IS_OUTPUT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_OUTPUT_VIEW))
#define VTG_OUTPUT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_OUTPUT_VIEW, VtgOutputViewClass))

typedef struct _VtgOutputView VtgOutputView;
typedef struct _VtgOutputViewClass VtgOutputViewClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gtk_tree_path_free0(var) ((var == NULL) ? NULL : (var = (gtk_tree_path_free (var), NULL)))

struct _VtgBuildLogView {
	GObject parent_instance;
	VtgBuildLogViewPrivate * priv;
};

struct _VtgBuildLogViewClass {
	GObjectClass parent_class;
};

struct _VtgBuildLogViewPrivate {
	GtkVBox* _ui;
	GtkListStore* _child_model;
	GtkTreeModelFilter* _model;
	GtkTreeView* _build_view;
	gint current_error_row;
	gint _error_count;
	gint _warning_count;
	VtgPluginInstance* _plugin_instance;
	VtgProjectManager* _project;
	gboolean show_warnings;
	gboolean show_errors;
};

typedef enum  {
	VTG_OUTPUT_TYPES_MESSAGE,
	VTG_OUTPUT_TYPES_ERROR,
	VTG_OUTPUT_TYPES_CHILD_PROCESS,
	VTG_OUTPUT_TYPES_BUILD,
	VTG_OUTPUT_TYPES_SEARCH
} VtgOutputTypes;


static gpointer vtg_build_log_view_parent_class = NULL;
static GType vtg_build_log_view_type_id = 0;

GType vtg_build_log_view_get_type (void);
GType vtg_build_log_view_register_type (GTypeModule * module);
GType vtg_plugin_instance_get_type (void);
GType vtg_plugin_instance_register_type (GTypeModule * module);
GType vtg_project_manager_get_type (void);
GType vtg_project_manager_register_type (GTypeModule * module);
#define VTG_BUILD_LOG_VIEW_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogViewPrivate))
enum  {
	VTG_BUILD_LOG_VIEW_DUMMY_PROPERTY,
	VTG_BUILD_LOG_VIEW_PLUGIN_INSTANCE,
	VTG_BUILD_LOG_VIEW_ERROR_COUNT,
	VTG_BUILD_LOG_VIEW_WARNING_COUNT
};
static void vtg_build_log_view_set_plugin_instance (VtgBuildLogView* self, VtgPluginInstance* value);
VtgBuildLogView* vtg_build_log_view_new (VtgPluginInstance* plugin_instance);
VtgBuildLogView* vtg_build_log_view_construct (GType object_type, VtgPluginInstance* plugin_instance);
void vtg_build_log_view_initialize (VtgBuildLogView* self, VtgProjectManager* project);
GeditWindow* vtg_plugin_instance_get_window (VtgPluginInstance* self);
void vtg_build_log_view_activate (VtgBuildLogView* self);
static void vtg_build_log_view_on_toggle_warnings_toggled (VtgBuildLogView* self, GtkToggleToolButton* sender);
static void vtg_build_log_view_on_toggle_errors_toggled (VtgBuildLogView* self, GtkToggleToolButton* sender);
GType vtg_output_types_get_type (void);
static void vtg_build_log_view_add_message (VtgBuildLogView* self, const char* file, const char* message);
GType vtg_output_view_get_type (void);
GType vtg_output_view_register_type (GTypeModule * module);
void vtg_build_log_view_on_message_added (VtgBuildLogView* self, VtgOutputView* sender, VtgOutputTypes output_type, const char* message);
static void vtg_build_log_view_activate_path (VtgBuildLogView* self, const GtkTreePath* path);
void vtg_build_log_view_on_build_view_row_activated (VtgBuildLogView* self, GtkWidget* sender, const GtkTreePath* path, GtkTreeViewColumn* column);
char* vtg_project_manager_source_uri_for_name (VtgProjectManager* self, const char* name);
GeditTab* vtg_plugin_instance_activate_uri (VtgPluginInstance* self, const char* uri, gint line, gint col);
void vtg_plugin_instance_activate_display_name (VtgPluginInstance* self, const char* display_name, gint line, gint col);
void vtg_build_log_view_next_error (VtgBuildLogView* self);
void vtg_build_log_view_previous_error (VtgBuildLogView* self);
static gboolean vtg_build_log_view_filter_model (VtgBuildLogView* self, GtkTreeModel* model, GtkTreeIter* iter);
VtgPluginInstance* vtg_build_log_view_get_plugin_instance (VtgBuildLogView* self);
gint vtg_build_log_view_get_error_count (VtgBuildLogView* self);
gint vtg_build_log_view_get_warning_count (VtgBuildLogView* self);
static void _vtg_build_log_view_on_toggle_warnings_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self);
static void _vtg_build_log_view_on_toggle_errors_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self);
static gboolean _vtg_build_log_view_filter_model_gtk_tree_model_filter_visible_func (GtkTreeModel* model, GtkTreeIter* iter, gpointer self);
static void _vtg_build_log_view_on_build_view_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self);
static void _vtg_build_log_view_on_message_added_vtg_output_view_message_added (VtgOutputView* _sender, VtgOutputTypes output_type, const char* message, gpointer self);
VtgOutputView* vtg_plugin_instance_get_output_view (VtgPluginInstance* self);
static GObject * vtg_build_log_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void vtg_build_log_view_finalize (GObject* obj);
static void vtg_build_log_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void vtg_build_log_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



VtgBuildLogView* vtg_build_log_view_construct (GType object_type, VtgPluginInstance* plugin_instance) {
	GParameter * __params;
	GParameter * __params_it;
	VtgBuildLogView * self;
	g_return_val_if_fail (plugin_instance != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin-instance";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN_INSTANCE);
	g_value_set_object (&__params_it->value, plugin_instance);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgBuildLogView* vtg_build_log_view_new (VtgPluginInstance* plugin_instance) {
	return vtg_build_log_view_construct (VTG_TYPE_BUILD_LOG_VIEW, plugin_instance);
}


void vtg_build_log_view_initialize (VtgBuildLogView* self, VtgProjectManager* project) {
	g_return_if_fail (self != NULL);
	self->priv->_project = project;
	self->priv->current_error_row = 0;
	self->priv->_error_count = 0;
	self->priv->_warning_count = 0;
	gtk_list_store_clear (self->priv->_child_model);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


void vtg_build_log_view_activate (VtgBuildLogView* self) {
	GeditPanel* panel;
	GeditView* view;
	gboolean _tmp0_;
	g_return_if_fail (self != NULL);
	panel = _g_object_ref0 (gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	gedit_panel_activate_item (panel, (GtkWidget*) self->priv->_ui);
	view = _g_object_ref0 (gedit_window_get_active_view (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	_tmp0_ = FALSE;
	if (view != NULL) {
		gboolean _tmp1_;
		_tmp0_ = !(g_object_get ((GtkWidget*) view, "is-focus", &_tmp1_, NULL), _tmp1_);
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		gtk_widget_grab_focus ((GtkWidget*) view);
	}
	_g_object_unref0 (panel);
	_g_object_unref0 (view);
}


static void vtg_build_log_view_on_toggle_warnings_toggled (VtgBuildLogView* self, GtkToggleToolButton* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	self->priv->show_warnings = gtk_toggle_tool_button_get_active (sender);
	if (self->priv->_model != NULL) {
		gtk_tree_model_filter_refilter (self->priv->_model);
	}
}


static void vtg_build_log_view_on_toggle_errors_toggled (VtgBuildLogView* self, GtkToggleToolButton* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	self->priv->show_errors = gtk_toggle_tool_button_get_active (sender);
	if (self->priv->_model != NULL) {
		gtk_tree_model_filter_refilter (self->priv->_model);
	}
}


void vtg_build_log_view_on_message_added (VtgBuildLogView* self, VtgOutputView* sender, VtgOutputTypes output_type, const char* message) {
	char** _tmp1_;
	gint lines_size;
	gint lines_length1;
	char** _tmp0_;
	char** lines;
	gint idx;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (message != NULL);
	if (output_type != VTG_OUTPUT_TYPES_BUILD) {
		return;
	}
	lines = (_tmp1_ = _tmp0_ = g_strsplit (message, "\n", 0), lines_length1 = _vala_array_length (_tmp0_), lines_size = lines_length1, _tmp1_);
	idx = 0;
	while (TRUE) {
		char** _tmp3_;
		gint tmp_size;
		gint tmp_length1;
		char** _tmp2_;
		char** tmp;
		gboolean _tmp4_;
		if (!(lines[idx] != NULL)) {
			break;
		}
		tmp = (_tmp3_ = _tmp2_ = g_strsplit (lines[idx], ":", 2), tmp_length1 = _vala_array_length (_tmp2_), tmp_size = tmp_length1, _tmp3_);
		_tmp4_ = FALSE;
		if (tmp[0] != NULL) {
			gboolean _tmp5_;
			_tmp5_ = FALSE;
			if (g_str_has_suffix (tmp[0], ".vala")) {
				_tmp5_ = TRUE;
			} else {
				_tmp5_ = g_str_has_suffix (tmp[0], ".vapi");
			}
			_tmp4_ = _tmp5_;
		} else {
			_tmp4_ = FALSE;
		}
		if (_tmp4_) {
			vtg_build_log_view_add_message (self, tmp[0], tmp[1]);
		}
		idx++;
		tmp = (_vala_array_free (tmp, tmp_length1, (GDestroyNotify) g_free), NULL);
	}
	lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
}


void vtg_build_log_view_on_build_view_row_activated (VtgBuildLogView* self, GtkWidget* sender, const GtkTreePath* path, GtkTreeViewColumn* column) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (path != NULL);
	g_return_if_fail (column != NULL);
	vtg_build_log_view_activate_path (self, path);
}


static void vtg_build_log_view_activate_path (VtgBuildLogView* self, const GtkTreePath* path) {
	GtkTreeIter iter = {0};
	g_return_if_fail (self != NULL);
	g_return_if_fail (path != NULL);
	if (gtk_tree_model_get_iter ((GtkTreeModel*) self->priv->_child_model, &iter, path)) {
		char* name;
		gint line;
		gint col;
		VtgProjectManager* proj;
		name = NULL;
		line = 0;
		col = 0;
		proj = NULL;
		gtk_tree_model_get ((GtkTreeModel*) self->priv->_child_model, &iter, 2, &name, 3, &line, 4, &col, 6, &proj, -1);
		if (proj != NULL) {
			char* uri;
			uri = vtg_project_manager_source_uri_for_name (proj, name);
			if (uri != NULL) {
				GeditTab* _tmp0_;
				_tmp0_ = vtg_plugin_instance_activate_uri (self->priv->_plugin_instance, uri, line, col);
				_g_object_unref0 (_tmp0_);
			} else {
				g_warning ("vtgbuildlogview.vala:211: Couldn't find uri for source: %s", name);
			}
			_g_free0 (uri);
		} else {
			vtg_plugin_instance_activate_display_name (self->priv->_plugin_instance, name, line, col);
		}
		_g_free0 (name);
		_g_object_unref0 (proj);
	}
}


void vtg_build_log_view_next_error (VtgBuildLogView* self) {
	char* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* path;
	g_return_if_fail (self != NULL);
	path = (_tmp1_ = gtk_tree_path_new_from_string (_tmp0_ = g_strdup_printf ("%i", self->priv->current_error_row)), _g_free0 (_tmp0_), _tmp1_);
	if (path != NULL) {
		vtg_build_log_view_activate_path (self, path);
		gtk_tree_view_scroll_to_cell (self->priv->_build_view, path, NULL, FALSE, (float) 0, (float) 0);
		gtk_tree_selection_select_path (gtk_tree_view_get_selection (self->priv->_build_view), path);
		if (self->priv->current_error_row < ((self->priv->_error_count + self->priv->_warning_count) - 1)) {
			self->priv->current_error_row++;
		} else {
			self->priv->current_error_row = 0;
		}
	}
	_gtk_tree_path_free0 (path);
}


void vtg_build_log_view_previous_error (VtgBuildLogView* self) {
	char* _tmp0_;
	GtkTreePath* _tmp1_;
	GtkTreePath* path;
	g_return_if_fail (self != NULL);
	path = (_tmp1_ = gtk_tree_path_new_from_string (_tmp0_ = g_strdup_printf ("%i", self->priv->current_error_row)), _g_free0 (_tmp0_), _tmp1_);
	if (path != NULL) {
		vtg_build_log_view_activate_path (self, path);
		gtk_tree_view_scroll_to_cell (self->priv->_build_view, path, NULL, FALSE, (float) 0, (float) 0);
		gtk_tree_selection_select_path (gtk_tree_view_get_selection (self->priv->_build_view), path);
		if (self->priv->current_error_row > 0) {
			self->priv->current_error_row--;
		} else {
			self->priv->current_error_row = (self->priv->_error_count + self->priv->_warning_count) - 1;
		}
	}
	_gtk_tree_path_free0 (path);
}


static void vtg_build_log_view_add_message (VtgBuildLogView* self, const char* file, const char* message) {
	char** _tmp1_;
	gint parts_size;
	gint parts_length1;
	char** _tmp0_;
	char** parts;
	char** _tmp6_;
	gint src_ref_size;
	gint src_ref_length1;
	char** _tmp4_;
	gint _tmp3__length1;
	char** _tmp3_;
	char** _tmp2_;
	char** _tmp5_;
	char** src_ref;
	gint line;
	gint col;
	char* stock_id;
	g_return_if_fail (self != NULL);
	g_return_if_fail (file != NULL);
	g_return_if_fail (message != NULL);
	parts = (_tmp1_ = _tmp0_ = g_strsplit (message, ":", 3), parts_length1 = _vala_array_length (_tmp0_), parts_size = parts_length1, _tmp1_);
	src_ref = (_tmp6_ = (_tmp5_ = _tmp4_ = g_strsplit ((_tmp3_ = _tmp2_ = g_strsplit (parts[0], "-", 0), _tmp3__length1 = _vala_array_length (_tmp2_), _tmp3_)[0], ".", 0), _tmp3_ = (_vala_array_free (_tmp3_, _tmp3__length1, (GDestroyNotify) g_free), NULL), _tmp5_), src_ref_length1 = _vala_array_length (_tmp4_), src_ref_size = src_ref_length1, _tmp6_);
	line = atoi (src_ref[0]);
	col = 0;
	if (src_ref[1] != NULL) {
		col = atoi (src_ref[1]);
	}
	stock_id = NULL;
	if (parts[1] != NULL) {
		gint sort_id;
		sort_id = 0;
		if (g_str_has_suffix (parts[1], "error")) {
			char* _tmp7_;
			stock_id = (_tmp7_ = g_strdup (GTK_STOCK_DIALOG_ERROR), _g_free0 (stock_id), _tmp7_);
			self->priv->_error_count++;
			sort_id = 0;
		} else {
			if (g_str_has_suffix (parts[1], "warning")) {
				char* _tmp8_;
				stock_id = (_tmp8_ = g_strdup (GTK_STOCK_DIALOG_WARNING), _g_free0 (stock_id), _tmp8_);
				self->priv->_warning_count++;
				sort_id = 1;
			} else {
				self->priv->_error_count++;
				sort_id = 0;
			}
		}
		if (parts[2] != NULL) {
			GtkTreeIter iter = {0};
			gtk_list_store_append (self->priv->_child_model, &iter);
			gtk_list_store_set (self->priv->_child_model, &iter, 0, stock_id, 1, parts[2], 2, file, 3, line, 4, col, 5, sort_id, 6, self->priv->_project, -1);
		}
	}
	parts = (_vala_array_free (parts, parts_length1, (GDestroyNotify) g_free), NULL);
	src_ref = (_vala_array_free (src_ref, src_ref_length1, (GDestroyNotify) g_free), NULL);
	_g_free0 (stock_id);
}


static gboolean vtg_build_log_view_filter_model (VtgBuildLogView* self, GtkTreeModel* model, GtkTreeIter* iter) {
	gboolean result;
	gboolean _tmp0_;
	gint val;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (model != NULL, FALSE);
	_tmp0_ = FALSE;
	if (self->priv->show_warnings) {
		_tmp0_ = self->priv->show_errors;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		result = TRUE;
		return result;
	}
	val = 0;
	gtk_tree_model_get (model, &(*iter), 5, &val, -1);
	_tmp1_ = FALSE;
	if (val == 0) {
		_tmp1_ = self->priv->show_errors;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		result = TRUE;
		return result;
	} else {
		gboolean _tmp2_;
		_tmp2_ = FALSE;
		if (val == 1) {
			_tmp2_ = self->priv->show_warnings;
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			result = TRUE;
			return result;
		}
	}
	result = FALSE;
	return result;
}


VtgPluginInstance* vtg_build_log_view_get_plugin_instance (VtgBuildLogView* self) {
	VtgPluginInstance* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_plugin_instance;
	return result;
}


static void vtg_build_log_view_set_plugin_instance (VtgBuildLogView* self, VtgPluginInstance* value) {
	VtgPluginInstance* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_plugin_instance = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_plugin_instance), _tmp0_);
	g_object_notify ((GObject *) self, "plugin-instance");
}


gint vtg_build_log_view_get_error_count (VtgBuildLogView* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_error_count;
	return result;
}


gint vtg_build_log_view_get_warning_count (VtgBuildLogView* self) {
	gint result;
	g_return_val_if_fail (self != NULL, 0);
	result = self->priv->_warning_count;
	return result;
}


static void _vtg_build_log_view_on_toggle_warnings_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self) {
	vtg_build_log_view_on_toggle_warnings_toggled (self, _sender);
}


static void _vtg_build_log_view_on_toggle_errors_toggled_gtk_toggle_tool_button_toggled (GtkToggleToolButton* _sender, gpointer self) {
	vtg_build_log_view_on_toggle_errors_toggled (self, _sender);
}


static gboolean _vtg_build_log_view_filter_model_gtk_tree_model_filter_visible_func (GtkTreeModel* model, GtkTreeIter* iter, gpointer self) {
	return vtg_build_log_view_filter_model (self, model, iter);
}


static void _vtg_build_log_view_on_build_view_row_activated_gtk_tree_view_row_activated (GtkTreeView* _sender, const GtkTreePath* path, GtkTreeViewColumn* column, gpointer self) {
	vtg_build_log_view_on_build_view_row_activated (self, _sender, path, column);
}


static void _vtg_build_log_view_on_message_added_vtg_output_view_message_added (VtgOutputView* _sender, VtgOutputTypes output_type, const char* message, gpointer self) {
	vtg_build_log_view_on_message_added (self, _sender, output_type, message);
}


static GObject * vtg_build_log_view_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgBuildLogViewClass * klass;
	GObjectClass * parent_class;
	VtgBuildLogView * self;
	klass = VTG_BUILD_LOG_VIEW_CLASS (g_type_class_peek (VTG_TYPE_BUILD_LOG_VIEW));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_BUILD_LOG_VIEW (obj);
	{
		GeditPanel* panel;
		GtkVBox* _tmp0_;
		GtkToolbar* toolbar;
		GtkToggleToolButton* toggle;
		GtkToggleToolButton* _tmp1_;
		GtkListStore* _tmp2_;
		GtkTreeModelFilter* _tmp3_;
		GtkTreeView* _tmp4_;
		GtkCellRenderer* renderer;
		GtkTreeViewColumn* column;
		GtkCellRenderer* _tmp5_;
		GtkCellRenderer* _tmp6_;
		GtkTreeViewColumn* _tmp7_;
		GtkCellRenderer* _tmp8_;
		GtkTreeViewColumn* _tmp9_;
		GtkCellRenderer* _tmp10_;
		GtkTreeViewColumn* _tmp11_;
		GtkScrolledWindow* scroll;
		panel = _g_object_ref0 (gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
		self->priv->_ui = (_tmp0_ = g_object_ref_sink ((GtkVBox*) gtk_vbox_new (FALSE, 8)), _g_object_unref0 (self->priv->_ui), _tmp0_);
		toolbar = g_object_ref_sink ((GtkToolbar*) gtk_toolbar_new ());
		gtk_toolbar_set_style (toolbar, GTK_TOOLBAR_BOTH_HORIZ);
		gtk_toolbar_set_icon_size (toolbar, GTK_ICON_SIZE_SMALL_TOOLBAR);
		toggle = g_object_ref_sink ((GtkToggleToolButton*) gtk_toggle_tool_button_new ());
		gtk_tool_button_set_label ((GtkToolButton*) toggle, _ ("Warnings"));
		gtk_tool_item_set_is_important ((GtkToolItem*) toggle, TRUE);
		gtk_tool_button_set_icon_name ((GtkToolButton*) toggle, GTK_STOCK_DIALOG_WARNING);
		gtk_toggle_tool_button_set_active (toggle, TRUE);
		g_signal_connect_object (toggle, "toggled", (GCallback) _vtg_build_log_view_on_toggle_warnings_toggled_gtk_toggle_tool_button_toggled, self, 0);
		gtk_tool_item_set_tooltip_text ((GtkToolItem*) toggle, _ ("Show or hide the warnings from the build result view"));
		gtk_toolbar_insert (toolbar, (GtkToolItem*) toggle, -1);
		toggle = (_tmp1_ = g_object_ref_sink ((GtkToggleToolButton*) gtk_toggle_tool_button_new ()), _g_object_unref0 (toggle), _tmp1_);
		gtk_tool_button_set_label ((GtkToolButton*) toggle, _ ("Errors"));
		gtk_tool_item_set_is_important ((GtkToolItem*) toggle, TRUE);
		gtk_tool_button_set_icon_name ((GtkToolButton*) toggle, GTK_STOCK_DIALOG_ERROR);
		g_signal_connect_object (toggle, "toggled", (GCallback) _vtg_build_log_view_on_toggle_errors_toggled_gtk_toggle_tool_button_toggled, self, 0);
		gtk_tool_item_set_tooltip_text ((GtkToolItem*) toggle, _ ("Show or hide the errors from the build result view"));
		gtk_toggle_tool_button_set_active (toggle, TRUE);
		gtk_toolbar_insert (toolbar, (GtkToolItem*) toggle, -1);
		gtk_box_pack_start ((GtkBox*) self->priv->_ui, (GtkWidget*) toolbar, FALSE, TRUE, (guint) 0);
		self->priv->_child_model = (_tmp2_ = gtk_list_store_new (7, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INT, G_TYPE_INT, G_TYPE_INT, G_TYPE_OBJECT, NULL), _g_object_unref0 (self->priv->_child_model), _tmp2_);
		self->priv->_model = (_tmp3_ = (GtkTreeModelFilter*) gtk_tree_model_filter_new ((GtkTreeModel*) self->priv->_child_model, NULL), _g_object_unref0 (self->priv->_model), _tmp3_);
		gtk_tree_model_filter_set_visible_func (self->priv->_model, _vtg_build_log_view_filter_model_gtk_tree_model_filter_visible_func, g_object_ref (self), g_object_unref);
		self->priv->_build_view = (_tmp4_ = g_object_ref_sink ((GtkTreeView*) gtk_tree_view_new_with_model ((GtkTreeModel*) self->priv->_model)), _g_object_unref0 (self->priv->_build_view), _tmp4_);
		renderer = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererPixbuf*) gtk_cell_renderer_pixbuf_new ());
		column = g_object_ref_sink (gtk_tree_view_column_new ());
		gtk_tree_view_column_set_title (column, _ ("Message"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "stock-id", 0);
		renderer = (_tmp5_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), _g_object_unref0 (renderer), _tmp5_);
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, TRUE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 1);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		renderer = (_tmp6_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), _g_object_unref0 (renderer), _tmp6_);
		column = (_tmp7_ = g_object_ref_sink (gtk_tree_view_column_new ()), _g_object_unref0 (column), _tmp7_);
		gtk_tree_view_column_set_title (column, _ ("File"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 2);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		renderer = (_tmp8_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), _g_object_unref0 (renderer), _tmp8_);
		column = (_tmp9_ = g_object_ref_sink (gtk_tree_view_column_new ()), _g_object_unref0 (column), _tmp9_);
		gtk_tree_view_column_set_title (column, _ ("Line"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 3);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		renderer = (_tmp10_ = (GtkCellRenderer*) g_object_ref_sink ((GtkCellRendererText*) gtk_cell_renderer_text_new ()), _g_object_unref0 (renderer), _tmp10_);
		column = (_tmp11_ = g_object_ref_sink (gtk_tree_view_column_new ()), _g_object_unref0 (column), _tmp11_);
		gtk_tree_view_column_set_title (column, _ ("Column"));
		gtk_cell_layout_pack_start ((GtkCellLayout*) column, renderer, FALSE);
		gtk_cell_layout_add_attribute ((GtkCellLayout*) column, renderer, "text", 4);
		gtk_tree_view_append_column (self->priv->_build_view, column);
		g_signal_connect_object (self->priv->_build_view, "row-activated", (GCallback) _vtg_build_log_view_on_build_view_row_activated_gtk_tree_view_row_activated, self, 0);
		gtk_tree_view_set_rules_hint (self->priv->_build_view, TRUE);
		scroll = g_object_ref_sink ((GtkScrolledWindow*) gtk_scrolled_window_new (NULL, NULL));
		gtk_container_add ((GtkContainer*) scroll, (GtkWidget*) self->priv->_build_view);
		gtk_box_pack_start ((GtkBox*) self->priv->_ui, (GtkWidget*) scroll, TRUE, TRUE, (guint) 0);
		gtk_widget_show_all ((GtkWidget*) self->priv->_ui);
		gedit_panel_add_item_with_stock_icon (panel, (GtkWidget*) self->priv->_ui, _ ("Build results"), GTK_STOCK_EXECUTE);
		g_signal_connect_object (vtg_plugin_instance_get_output_view (self->priv->_plugin_instance), "message-added", (GCallback) _vtg_build_log_view_on_message_added_vtg_output_view_message_added, self, 0);
		gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_child_model, 5, GTK_SORT_ASCENDING);
		_g_object_unref0 (panel);
		_g_object_unref0 (toolbar);
		_g_object_unref0 (toggle);
		_g_object_unref0 (renderer);
		_g_object_unref0 (column);
		_g_object_unref0 (scroll);
	}
	return obj;
}


static void vtg_build_log_view_class_init (VtgBuildLogViewClass * klass) {
	vtg_build_log_view_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgBuildLogViewPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_build_log_view_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_build_log_view_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_build_log_view_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_build_log_view_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BUILD_LOG_VIEW_PLUGIN_INSTANCE, g_param_spec_object ("plugin-instance", "plugin-instance", "plugin-instance", VTG_TYPE_PLUGIN_INSTANCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BUILD_LOG_VIEW_ERROR_COUNT, g_param_spec_int ("error-count", "error-count", "error-count", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_BUILD_LOG_VIEW_WARNING_COUNT, g_param_spec_int ("warning-count", "warning-count", "warning-count", G_MININT, G_MAXINT, 0, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_build_log_view_instance_init (VtgBuildLogView * self) {
	self->priv = VTG_BUILD_LOG_VIEW_GET_PRIVATE (self);
	self->priv->_child_model = NULL;
	self->priv->_build_view = NULL;
	self->priv->current_error_row = 0;
	self->priv->_error_count = 0;
	self->priv->_warning_count = 0;
	self->priv->show_warnings = TRUE;
	self->priv->show_errors = TRUE;
}


static void vtg_build_log_view_finalize (GObject* obj) {
	VtgBuildLogView * self;
	self = VTG_BUILD_LOG_VIEW (obj);
	{
		GeditPanel* panel;
		panel = _g_object_ref0 (gedit_window_get_bottom_panel (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
		gedit_panel_remove_item (panel, (GtkWidget*) self->priv->_ui);
		_g_object_unref0 (panel);
	}
	_g_object_unref0 (self->priv->_ui);
	_g_object_unref0 (self->priv->_child_model);
	_g_object_unref0 (self->priv->_model);
	_g_object_unref0 (self->priv->_build_view);
	_g_object_unref0 (self->priv->_plugin_instance);
	G_OBJECT_CLASS (vtg_build_log_view_parent_class)->finalize (obj);
}


GType vtg_build_log_view_get_type (void) {
	return vtg_build_log_view_type_id;
}


GType vtg_build_log_view_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgBuildLogViewClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_build_log_view_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgBuildLogView), 0, (GInstanceInitFunc) vtg_build_log_view_instance_init, NULL };
	vtg_build_log_view_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgBuildLogView", &g_define_type_info, 0);
	return vtg_build_log_view_type_id;
}


static void vtg_build_log_view_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgBuildLogView * self;
	self = VTG_BUILD_LOG_VIEW (object);
	switch (property_id) {
		case VTG_BUILD_LOG_VIEW_PLUGIN_INSTANCE:
		g_value_set_object (value, vtg_build_log_view_get_plugin_instance (self));
		break;
		case VTG_BUILD_LOG_VIEW_ERROR_COUNT:
		g_value_set_int (value, vtg_build_log_view_get_error_count (self));
		break;
		case VTG_BUILD_LOG_VIEW_WARNING_COUNT:
		g_value_set_int (value, vtg_build_log_view_get_warning_count (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_build_log_view_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgBuildLogView * self;
	self = VTG_BUILD_LOG_VIEW (object);
	switch (property_id) {
		case VTG_BUILD_LOG_VIEW_PLUGIN_INSTANCE:
		vtg_build_log_view_set_plugin_instance (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




