/*
 *  vtgplugin.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gedit/gedit-plugin.h>
#include <gee.h>
#include <gedit/gedit-window.h>
#include <gtk/gtk.h>
#include <gedit/gedit-document.h>
#include <vsc.h>
#include <gtksourceview/gtksourceview.h>
#include <stdlib.h>
#include <string.h>
#include <gedit/gedit-tab.h>
#include <vbf.h>
#include <glib/gi18n-lib.h>
#include <config.h>


#define VTG_TYPE_PLUGIN (vtg_plugin_get_type ())
#define VTG_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN, VtgPlugin))
#define VTG_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN, VtgPluginClass))
#define VTG_IS_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN))
#define VTG_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN))
#define VTG_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN, VtgPluginClass))

typedef struct _VtgPlugin VtgPlugin;
typedef struct _VtgPluginClass VtgPluginClass;
typedef struct _VtgPluginPrivate VtgPluginPrivate;

#define VTG_TYPE_PLUGIN_INSTANCE (vtg_plugin_instance_get_type ())
#define VTG_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstance))
#define VTG_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))
#define VTG_IS_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_IS_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_PLUGIN_INSTANCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))

typedef struct _VtgPluginInstance VtgPluginInstance;
typedef struct _VtgPluginInstanceClass VtgPluginInstanceClass;

#define VTG_TYPE_PROJECT_DESCRIPTOR (vtg_project_descriptor_get_type ())
#define VTG_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptor))
#define VTG_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))
#define VTG_IS_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_IS_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_PROJECT_DESCRIPTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))

typedef struct _VtgProjectDescriptor VtgProjectDescriptor;
typedef struct _VtgProjectDescriptorClass VtgProjectDescriptorClass;

#define VTG_TYPE_CONFIGURATION (vtg_configuration_get_type ())
#define VTG_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_CONFIGURATION, VtgConfiguration))
#define VTG_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_CONFIGURATION, VtgConfigurationClass))
#define VTG_IS_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_CONFIGURATION))
#define VTG_IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_CONFIGURATION))
#define VTG_CONFIGURATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_CONFIGURATION, VtgConfigurationClass))

typedef struct _VtgConfiguration VtgConfiguration;
typedef struct _VtgConfigurationClass VtgConfigurationClass;

#define VTG_PLUGIN_TYPE_DEACTIVATE_MODULE_OPTIONS (vtg_plugin_deactivate_module_options_get_type ())
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
typedef struct _VtgProjectDescriptorPrivate VtgProjectDescriptorPrivate;

#define VTG_TYPE_PROJECT_MANAGER (vtg_project_manager_get_type ())
#define VTG_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManager))
#define VTG_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))
#define VTG_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER))
#define VTG_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER))
#define VTG_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))

typedef struct _VtgProjectManager VtgProjectManager;
typedef struct _VtgProjectManagerClass VtgProjectManagerClass;

#define VTG_TYPE_PROJECT_VIEW (vtg_project_view_get_type ())
#define VTG_PROJECT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_VIEW, VtgProjectView))
#define VTG_PROJECT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_VIEW, VtgProjectViewClass))
#define VTG_IS_PROJECT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_VIEW))
#define VTG_IS_PROJECT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_VIEW))
#define VTG_PROJECT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_VIEW, VtgProjectViewClass))

typedef struct _VtgProjectView VtgProjectView;
typedef struct _VtgProjectViewClass VtgProjectViewClass;

#define VTG_TYPE_PROJECT_MANAGER_UI (vtg_project_manager_ui_get_type ())
#define VTG_PROJECT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUi))
#define VTG_PROJECT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUiClass))
#define VTG_IS_PROJECT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER_UI))
#define VTG_IS_PROJECT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER_UI))
#define VTG_PROJECT_MANAGER_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUiClass))

typedef struct _VtgProjectManagerUi VtgProjectManagerUi;
typedef struct _VtgProjectManagerUiClass VtgProjectManagerUiClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

struct _VtgPlugin {
	GeditPlugin parent_instance;
	VtgPluginPrivate * priv;
};

struct _VtgPluginClass {
	GeditPluginClass parent_class;
};

struct _VtgPluginPrivate {
	GeeList* _instances;
	GeeList* _projects;
	VtgConfiguration* _config;
	VtgProjectDescriptor* _default_project;
};

typedef enum  {
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL,
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET,
	VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL
} VtgPluginDeactivateModuleOptions;

struct _VtgProjectDescriptor {
	GObject parent_instance;
	VtgProjectDescriptorPrivate * priv;
	VscSymbolCompletion* completion;
	VtgProjectManager* project;
};

struct _VtgProjectDescriptorClass {
	GObjectClass parent_class;
};


static gpointer vtg_plugin_parent_class = NULL;
static GType vtg_plugin_type_id = 0;

GType vtg_plugin_get_type (void);
GType vtg_plugin_register_type (GTypeModule * module);
GType vtg_plugin_instance_get_type (void);
GType vtg_plugin_instance_register_type (GTypeModule * module);
GType vtg_project_descriptor_get_type (void);
GType vtg_project_descriptor_register_type (GTypeModule * module);
GType vtg_configuration_get_type (void);
GType vtg_configuration_register_type (GTypeModule * module);
#define VTG_PLUGIN_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PLUGIN, VtgPluginPrivate))
enum  {
	VTG_PLUGIN_DUMMY_PROPERTY,
	VTG_PLUGIN_PROJECTS,
	VTG_PLUGIN_CONFIG
};
static GType vtg_plugin_deactivate_module_options_get_type (void);
GeditWindow* vtg_plugin_instance_get_window (VtgPluginInstance* self);
static VtgPluginInstance* vtg_plugin_get_plugin_instance_for_window (VtgPlugin* self, GeditWindow* window);
VtgPluginInstance* vtg_plugin_instance_new (VtgPlugin* plugin, GeditWindow* window);
VtgPluginInstance* vtg_plugin_instance_construct (GType object_type, VtgPlugin* plugin, GeditWindow* window);
static void vtg_plugin_real_activate (GeditPlugin* base, GeditWindow* window);
static void vtg_plugin_deactivate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options);
static void vtg_plugin_real_deactivate (GeditPlugin* base, GeditWindow* window);
static gboolean vtg_plugin_real_is_configurable (GeditPlugin* base);
GtkWidget* vtg_configuration_get_configuration_dialog (VtgConfiguration* self);
static GtkWidget* vtg_plugin_real_create_configure_dialog (GeditPlugin* base);
VtgProjectDescriptor* vtg_plugin_project_descriptor_find_from_document (VtgPlugin* self, GeditDocument* document);
GType vtg_project_manager_get_type (void);
GType vtg_project_manager_register_type (GTypeModule * module);
GType vtg_project_view_get_type (void);
GType vtg_project_view_register_type (GTypeModule * module);
void vtg_project_view_set_current_project (VtgProjectView* self, VtgProjectManager* value);
GType vtg_project_manager_ui_get_type (void);
GType vtg_project_manager_ui_register_type (GTypeModule * module);
VtgProjectManagerUi* vtg_plugin_instance_get_project_manager_ui (VtgPluginInstance* self);
VtgProjectView* vtg_project_manager_ui_get_project_view (VtgProjectManagerUi* self);
static void vtg_plugin_real_update_ui (GeditPlugin* base, GeditWindow* window);
gboolean vtg_configuration_get_bracket_enabled (VtgConfiguration* self);
static void vtg_plugin_activate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options);
static void vtg_plugin_on_configuration_property_changed (VtgPlugin* self, GObject* sender, GParamSpec* param);
void vtg_plugin_instance_deactivate_symbols (VtgPluginInstance* self);
void vtg_plugin_instance_deactivate_brackets (VtgPluginInstance* self);
void vtg_plugin_instance_initialize_views (VtgPluginInstance* self);
gboolean vtg_project_manager_contains_file (VtgProjectManager* self, const char* uri);
VtgProjectDescriptor* vtg_project_descriptor_new (void);
VtgProjectDescriptor* vtg_project_descriptor_construct (GType object_type);
void vtg_project_view_remove_project (VtgProjectView* self, VbfProject* project);
VbfProject* vtg_project_manager_get_project (VtgProjectManager* self);
void vtg_plugin_on_project_closed (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project);
gboolean vtg_plugin_project_need_save (VtgPlugin* self, VtgProjectManager* project);
gboolean vtg_string_utils_is_null_or_empty (const char* data);
void vtg_plugin_project_save_all (VtgPlugin* self, VtgProjectManager* project);
void vtg_project_view_add_project (VtgProjectView* self, VbfProject* project);
static char** _vala_array_dup1 (char** self, int length);
static void gtk_recent_data_destroy (GtkRecentData* self);
void vtg_plugin_on_project_loaded (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project_manager);
VtgPlugin* vtg_plugin_new (void);
VtgPlugin* vtg_plugin_construct (GType object_type);
GeeList* vtg_plugin_get_projects (VtgPlugin* self);
VtgConfiguration* vtg_plugin_get_config (VtgPlugin* self);
VtgConfiguration* vtg_configuration_new (void);
VtgConfiguration* vtg_configuration_construct (GType object_type);
static void _vtg_plugin_on_configuration_property_changed_g_object_notify (VtgConfiguration* _sender, GParamSpec* pspec, gpointer self);
static GObject * vtg_plugin_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void vtg_plugin_finalize (GObject* obj);
static void vtg_plugin_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
GType register_gedit_plugin (GTypeModule* module);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);




static GType vtg_plugin_deactivate_module_options_get_type (void) {
	static GType vtg_plugin_deactivate_module_options_type_id = 0;
	if (G_UNLIKELY (vtg_plugin_deactivate_module_options_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL", "all"}, {VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET", "bracket"}, {VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL, "VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL", "symbol"}, {0, NULL, NULL}};
		vtg_plugin_deactivate_module_options_type_id = g_enum_register_static ("VtgPluginDeactivateModuleOptions", values);
	}
	return vtg_plugin_deactivate_module_options_type_id;
}


static VtgPluginInstance* vtg_plugin_get_plugin_instance_for_window (VtgPlugin* self, GeditWindow* window) {
	VtgPluginInstance* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (window != NULL, NULL);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (TRUE) {
			VtgPluginInstance* instance;
			if (!gee_iterator_next (_instance_it)) {
				break;
			}
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			if (vtg_plugin_instance_get_window (instance) == window) {
				result = instance;
				_gee_collection_object_unref0 (_instance_it);
				return result;
			}
			_g_object_unref0 (instance);
		}
		_gee_collection_object_unref0 (_instance_it);
	}
	result = NULL;
	return result;
}


static void vtg_plugin_real_activate (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	VtgPluginInstance* _tmp0_;
	gboolean _tmp1_;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	if ((_tmp1_ = (_tmp0_ = vtg_plugin_get_plugin_instance_for_window (self, window)) == NULL, _g_object_unref0 (_tmp0_), _tmp1_)) {
		VtgPluginInstance* instance;
		instance = vtg_plugin_instance_new (self, window);
		gee_collection_add ((GeeCollection*) self->priv->_instances, instance);
		_g_object_unref0 (instance);
	}
}


static void vtg_plugin_real_deactivate (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL);
	gee_collection_clear ((GeeCollection*) self->priv->_instances);
}


static gboolean vtg_plugin_real_is_configurable (GeditPlugin* base) {
	VtgPlugin * self;
	gboolean result;
	self = (VtgPlugin*) base;
	result = TRUE;
	return result;
}


static GtkWidget* vtg_plugin_real_create_configure_dialog (GeditPlugin* base) {
	VtgPlugin * self;
	GtkWidget* result;
	self = (VtgPlugin*) base;
	result = vtg_configuration_get_configuration_dialog (self->priv->_config);
	return result;
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static void vtg_plugin_real_update_ui (GeditPlugin* base, GeditWindow* window) {
	VtgPlugin * self;
	GeditDocument* doc;
	VtgPluginInstance* instance;
	self = (VtgPlugin*) base;
	g_return_if_fail (window != NULL);
	doc = _g_object_ref0 (gedit_window_get_active_document (window));
	instance = vtg_plugin_get_plugin_instance_for_window (self, window);
	if (doc != NULL) {
		VtgProjectDescriptor* prj;
		gboolean _tmp0_;
		prj = vtg_plugin_project_descriptor_find_from_document (self, doc);
		_tmp0_ = FALSE;
		if (prj->project == NULL) {
			gboolean _tmp1_;
			_tmp1_ = FALSE;
			if (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) == NULL) {
				_tmp1_ = TRUE;
			} else {
				_tmp1_ = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") != 0;
			}
			_tmp0_ = _tmp1_;
		} else {
			_tmp0_ = FALSE;
		}
		if (!_tmp0_) {
			vtg_project_view_set_current_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance)), prj->project);
		}
		_g_object_unref0 (prj);
	}
	_g_object_unref0 (doc);
	_g_object_unref0 (instance);
}


static void vtg_plugin_on_configuration_property_changed (VtgPlugin* self, GObject* sender, GParamSpec* param) {
	char* name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (param != NULL);
	name = g_strdup (g_param_spec_get_name (param));
	if (_vala_strcmp0 (name, "bracket-enabled") == 0) {
		if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
			vtg_plugin_activate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET);
		} else {
			vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET);
		}
	} else {
		if (_vala_strcmp0 (name, "symbol-enabled") == 0) {
			if (vtg_configuration_get_bracket_enabled (self->priv->_config)) {
				vtg_plugin_activate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL);
			} else {
				vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL);
			}
		}
	}
	_g_free0 (name);
}


static void vtg_plugin_deactivate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options) {
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (self != NULL);
	_tmp0_ = FALSE;
	if (options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_SYMBOL;
	}
	if (_tmp0_) {
		{
			GeeIterator* _instance_it;
			_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
			while (TRUE) {
				VtgPluginInstance* instance;
				if (!gee_iterator_next (_instance_it)) {
					break;
				}
				instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
				vtg_plugin_instance_deactivate_symbols (instance);
				_g_object_unref0 (instance);
			}
			_gee_collection_object_unref0 (_instance_it);
		}
	}
	_tmp1_ = FALSE;
	if (options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = options == VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_BRACKET;
	}
	if (_tmp1_) {
		{
			GeeIterator* _instance_it;
			_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
			while (TRUE) {
				VtgPluginInstance* instance;
				if (!gee_iterator_next (_instance_it)) {
					break;
				}
				instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
				vtg_plugin_instance_deactivate_brackets (instance);
				_g_object_unref0 (instance);
			}
			_gee_collection_object_unref0 (_instance_it);
		}
	}
}


static void vtg_plugin_activate_modules (VtgPlugin* self, VtgPluginDeactivateModuleOptions options) {
	g_return_if_fail (self != NULL);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (TRUE) {
			VtgPluginInstance* instance;
			if (!gee_iterator_next (_instance_it)) {
				break;
			}
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			vtg_plugin_instance_initialize_views (instance);
			_g_object_unref0 (instance);
		}
		_gee_collection_object_unref0 (_instance_it);
	}
}


VtgProjectDescriptor* vtg_plugin_project_descriptor_find_from_document (VtgPlugin* self, GeditDocument* document) {
	VtgProjectDescriptor* result;
	char* file;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (document != NULL, NULL);
	file = g_strdup (gedit_document_get_uri (document));
	if (file == NULL) {
		char* _tmp0_;
		file = (_tmp0_ = g_strdup (gedit_document_get_short_name_for_display (document)), _g_free0 (file), _tmp0_);
	}
	{
		GeeIterator* _project_it;
		_project_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (TRUE) {
			VtgProjectDescriptor* project;
			if (!gee_iterator_next (_project_it)) {
				break;
			}
			project = (VtgProjectDescriptor*) gee_iterator_get (_project_it);
			if (vtg_project_manager_contains_file (project->project, file)) {
				result = project;
				_gee_collection_object_unref0 (_project_it);
				_g_free0 (file);
				return result;
			}
			_g_object_unref0 (project);
		}
		_gee_collection_object_unref0 (_project_it);
	}
	if (self->priv->_default_project == NULL) {
		VtgProjectDescriptor* _tmp1_;
		VscSymbolCompletion* _tmp2_;
		self->priv->_default_project = (_tmp1_ = vtg_project_descriptor_new (), _g_object_unref0 (self->priv->_default_project), _tmp1_);
		self->priv->_default_project->completion = (_tmp2_ = vsc_symbol_completion_new (), _g_object_unref0 (self->priv->_default_project->completion), _tmp2_);
		vsc_parser_manager_resume_parsing (vsc_symbol_completion_get_parser (self->priv->_default_project->completion));
	}
	result = _g_object_ref0 (self->priv->_default_project);
	_g_free0 (file);
	return result;
}


void vtg_plugin_on_project_closed (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (project != NULL);
	g_return_if_fail (project != self->priv->_default_project->project);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (TRUE) {
			VtgPluginInstance* instance;
			if (!gee_iterator_next (_instance_it)) {
				break;
			}
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			{
				GList* doc_collection;
				GList* doc_it;
				doc_collection = gedit_window_get_documents (vtg_plugin_instance_get_window (instance));
				for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
					GeditDocument* doc;
					doc = _g_object_ref0 ((GeditDocument*) doc_it->data);
					{
						if (vtg_project_manager_contains_file (project, gedit_document_get_uri (doc))) {
							GeditTab* tab;
							tab = _g_object_ref0 (gedit_tab_get_from_document (doc));
							gedit_window_close_tab (vtg_plugin_instance_get_window (instance), tab);
							_g_object_unref0 (tab);
						}
						_g_object_unref0 (doc);
					}
				}
			}
			if (vtg_plugin_instance_get_project_manager_ui (instance) != sender) {
				vtg_project_view_remove_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance)), vtg_project_manager_get_project (project));
			}
			_g_object_unref0 (instance);
		}
		_gee_collection_object_unref0 (_instance_it);
	}
	{
		GeeIterator* _descriptor_it;
		_descriptor_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (TRUE) {
			VtgProjectDescriptor* descriptor;
			if (!gee_iterator_next (_descriptor_it)) {
				break;
			}
			descriptor = (VtgProjectDescriptor*) gee_iterator_get (_descriptor_it);
			if (descriptor->project == project) {
				gee_collection_remove ((GeeCollection*) self->priv->_projects, descriptor);
				_g_object_unref0 (descriptor);
				break;
			}
			_g_object_unref0 (descriptor);
		}
		_gee_collection_object_unref0 (_descriptor_it);
	}
}


gboolean vtg_plugin_project_need_save (VtgPlugin* self, VtgProjectManager* project) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project != NULL, FALSE);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (TRUE) {
			VtgPluginInstance* instance;
			if (!gee_iterator_next (_instance_it)) {
				break;
			}
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			{
				GList* doc_collection;
				GList* doc_it;
				doc_collection = gedit_window_get_unsaved_documents (vtg_plugin_instance_get_window (instance));
				for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
					GeditDocument* doc;
					doc = _g_object_ref0 ((GeditDocument*) doc_it->data);
					{
						if (vtg_project_manager_contains_file (project, gedit_document_get_uri (doc))) {
							result = TRUE;
							_g_object_unref0 (doc);
							_g_object_unref0 (instance);
							_gee_collection_object_unref0 (_instance_it);
							return result;
						}
						_g_object_unref0 (doc);
					}
				}
			}
			_g_object_unref0 (instance);
		}
		_gee_collection_object_unref0 (_instance_it);
	}
	result = FALSE;
	return result;
}


void vtg_plugin_project_save_all (VtgPlugin* self, VtgProjectManager* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (TRUE) {
			VtgPluginInstance* instance;
			if (!gee_iterator_next (_instance_it)) {
				break;
			}
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			{
				GList* doc_collection;
				GList* doc_it;
				doc_collection = gedit_window_get_unsaved_documents (vtg_plugin_instance_get_window (instance));
				for (doc_it = doc_collection; doc_it != NULL; doc_it = doc_it->next) {
					GeditDocument* doc;
					doc = _g_object_ref0 ((GeditDocument*) doc_it->data);
					{
						char* uri;
						gboolean _tmp0_;
						uri = g_strdup (gedit_document_get_uri (doc));
						_tmp0_ = FALSE;
						if (!vtg_string_utils_is_null_or_empty (uri)) {
							_tmp0_ = vtg_project_manager_contains_file (project, uri);
						} else {
							_tmp0_ = FALSE;
						}
						if (_tmp0_) {
							gedit_document_save (doc, GEDIT_DOCUMENT_SAVE_IGNORE_MTIME);
						}
						_g_object_unref0 (doc);
						_g_free0 (uri);
					}
				}
			}
			_g_object_unref0 (instance);
		}
		_gee_collection_object_unref0 (_instance_it);
	}
}


static char** _vala_array_dup1 (char** self, int length) {
	char** result;
	int i;
	result = g_new0 (char*, length);
	for (i = 0; i < length; i++) {
		result[i] = g_strdup (self[i]);
	}
	return result;
}


static void gtk_recent_data_destroy (GtkRecentData* self) {
	_g_free0 (self->display_name);
	_g_free0 (self->description);
	_g_free0 (self->mime_type);
	_g_free0 (self->app_name);
	_g_free0 (self->app_exec);
	self->groups = (_vala_array_free (self->groups, _vala_array_length ((*self).groups), (GDestroyNotify) g_free), NULL);
}


void vtg_plugin_on_project_loaded (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project_manager) {
	GError * _inner_error_;
	VtgProjectDescriptor* prj;
	VscSymbolCompletion* completion;
	VbfProject* project;
	VscSymbolCompletion* _tmp30_;
	VtgProjectManager* _tmp31_;
	GtkRecentManager* recent;
	GtkRecentData _tmp32_ = {0};
	GtkRecentData recent_data;
	char* name;
	char** _tmp34_;
	gint groups_size;
	gint groups_length1;
	char** _tmp33_;
	char** groups;
	char* _tmp35_;
	char** _tmp37_;
	char** _tmp36_;
	char* _tmp38_;
	char* _tmp39_;
	char* _tmp40_;
	char* _tmp41_;
	char* _tmp42_;
	char* _tmp43_;
	char* _tmp44_;
	gboolean _tmp45_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (project_manager != NULL);
	_inner_error_ = NULL;
	prj = vtg_project_descriptor_new ();
	completion = vsc_symbol_completion_new ();
	project = _g_object_ref0 (vtg_project_manager_get_project (project_manager));
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _module_it;
		_module_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vbf_project_get_modules (project))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			VbfModule* module;
			if (!gee_iterator_next (_module_it)) {
				break;
			}
			module = (VbfModule*) gee_iterator_get (_module_it);
			{
				GeeList* _tmp2_;
				GeeIterator* _tmp3_;
				GeeIterator* _package_it;
				_package_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vbf_module_get_packages (module))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
				while (TRUE) {
					VbfPackage* package;
					if (!gee_iterator_next (_package_it)) {
						break;
					}
					package = (VbfPackage*) gee_iterator_get (_package_it);
					vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->name, TRUE);
					_g_object_unref0 (package);
				}
				_gee_collection_object_unref0 (_package_it);
			}
			_g_object_unref0 (module);
		}
		_gee_collection_object_unref0 (_module_it);
	}
	{
		GeeList* _tmp4_;
		GeeIterator* _tmp5_;
		GeeIterator* _group_it;
		_group_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vbf_project_get_groups (project))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp6_;
				GeeIterator* _tmp7_;
				GeeIterator* _package_it;
				_package_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vbf_group_get_built_libraries (group))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
				while (TRUE) {
					char* package;
					if (!gee_iterator_next (_package_it)) {
						break;
					}
					package = (char*) gee_iterator_get (_package_it);
					vsc_parser_manager_add_built_package (vsc_symbol_completion_get_parser (completion), package);
					_g_free0 (package);
				}
				_gee_collection_object_unref0 (_package_it);
			}
			{
				GeeList* _tmp8_;
				GeeIterator* _tmp9_;
				GeeIterator* _target_it;
				_target_it = (_tmp9_ = gee_iterable_iterator ((GeeIterable*) (_tmp8_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp8_), _tmp9_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp10_;
						GeeIterator* _tmp11_;
						GeeIterator* _package_it;
						_package_it = (_tmp11_ = gee_iterable_iterator ((GeeIterable*) (_tmp10_ = vbf_target_get_built_libraries (target))), _gee_collection_object_unref0 (_tmp10_), _tmp11_);
						while (TRUE) {
							char* package;
							if (!gee_iterator_next (_package_it)) {
								break;
							}
							package = (char*) gee_iterator_get (_package_it);
							vsc_parser_manager_add_built_package (vsc_symbol_completion_get_parser (completion), package);
							_g_free0 (package);
						}
						_gee_collection_object_unref0 (_package_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	{
		GeeList* _tmp12_;
		GeeIterator* _tmp13_;
		GeeIterator* _group_it;
		_group_it = (_tmp13_ = gee_iterable_iterator ((GeeIterable*) (_tmp12_ = vbf_project_get_groups (project))), _gee_collection_object_unref0 (_tmp12_), _tmp13_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp14_;
				GeeIterator* _tmp15_;
				GeeIterator* _path_it;
				_path_it = (_tmp15_ = gee_iterable_iterator ((GeeIterable*) (_tmp14_ = vbf_group_get_include_dirs (group))), _gee_collection_object_unref0 (_tmp14_), _tmp15_);
				while (TRUE) {
					char* path;
					if (!gee_iterator_next (_path_it)) {
						break;
					}
					path = (char*) gee_iterator_get (_path_it);
					vsc_parser_manager_add_path_to_vapi_search_dir (vsc_symbol_completion_get_parser (completion), path);
					_g_free0 (path);
				}
				_gee_collection_object_unref0 (_path_it);
			}
			{
				GeeList* _tmp16_;
				GeeIterator* _tmp17_;
				GeeIterator* _package_it;
				_package_it = (_tmp17_ = gee_iterable_iterator ((GeeIterable*) (_tmp16_ = vbf_group_get_packages (group))), _gee_collection_object_unref0 (_tmp16_), _tmp17_);
				while (TRUE) {
					VbfPackage* package;
					if (!gee_iterator_next (_package_it)) {
						break;
					}
					package = (VbfPackage*) gee_iterator_get (_package_it);
					vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->id, TRUE);
					_g_object_unref0 (package);
				}
				_gee_collection_object_unref0 (_package_it);
			}
			{
				GeeList* _tmp18_;
				GeeIterator* _tmp19_;
				GeeIterator* _target_it;
				_target_it = (_tmp19_ = gee_iterable_iterator ((GeeIterable*) (_tmp18_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp18_), _tmp19_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp20_;
						GeeIterator* _tmp21_;
						GeeIterator* _path_it;
						_path_it = (_tmp21_ = gee_iterable_iterator ((GeeIterable*) (_tmp20_ = vbf_target_get_include_dirs (target))), _gee_collection_object_unref0 (_tmp20_), _tmp21_);
						while (TRUE) {
							char* path;
							if (!gee_iterator_next (_path_it)) {
								break;
							}
							path = (char*) gee_iterator_get (_path_it);
							vsc_parser_manager_add_path_to_vapi_search_dir (vsc_symbol_completion_get_parser (completion), path);
							_g_free0 (path);
						}
						_gee_collection_object_unref0 (_path_it);
					}
					{
						GeeList* _tmp22_;
						GeeIterator* _tmp23_;
						GeeIterator* _package_it;
						_package_it = (_tmp23_ = gee_iterable_iterator ((GeeIterable*) (_tmp22_ = vbf_target_get_packages (target))), _gee_collection_object_unref0 (_tmp22_), _tmp23_);
						while (TRUE) {
							VbfPackage* package;
							if (!gee_iterator_next (_package_it)) {
								break;
							}
							package = (VbfPackage*) gee_iterator_get (_package_it);
							vsc_parser_manager_try_add_package (vsc_symbol_completion_get_parser (completion), package->id, TRUE);
							_g_object_unref0 (package);
						}
						_gee_collection_object_unref0 (_package_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	{
		GeeList* _tmp24_;
		GeeIterator* _tmp25_;
		GeeIterator* _group_it;
		_group_it = (_tmp25_ = gee_iterable_iterator ((GeeIterable*) (_tmp24_ = vbf_project_get_groups (project))), _gee_collection_object_unref0 (_tmp24_), _tmp25_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp26_;
				GeeIterator* _tmp27_;
				GeeIterator* _target_it;
				_target_it = (_tmp27_ = gee_iterable_iterator ((GeeIterable*) (_tmp26_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp26_), _tmp27_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp28_;
						GeeIterator* _tmp29_;
						GeeIterator* _source_it;
						_source_it = (_tmp29_ = gee_iterable_iterator ((GeeIterable*) (_tmp28_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp28_), _tmp29_);
						while (TRUE) {
							VbfSource* source;
							if (!gee_iterator_next (_source_it)) {
								break;
							}
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (((VbfFile*) source)->type == VBF_FILE_TYPES_VALA_SOURCE) {
								{
									vsc_parser_manager_add_source (vsc_symbol_completion_get_parser (completion), ((VbfFile*) source)->filename, &_inner_error_);
									if (_inner_error_ != NULL) {
										goto __catch36_g_error;
										goto __finally36;
									}
								}
								goto __finally36;
								__catch36_g_error:
								{
									GError * err;
									err = _inner_error_;
									_inner_error_ = NULL;
									{
										g_warning ("vtgplugin.vala:274: Error adding source %s: %s", ((VbfFile*) source)->filename, err->message);
										_g_error_free0 (err);
									}
								}
								__finally36:
								if (_inner_error_ != NULL) {
									_g_object_unref0 (source);
									_gee_collection_object_unref0 (_source_it);
									_g_object_unref0 (target);
									_gee_collection_object_unref0 (_target_it);
									_g_object_unref0 (group);
									_gee_collection_object_unref0 (_group_it);
									_g_object_unref0 (prj);
									_g_object_unref0 (completion);
									_g_object_unref0 (project);
									g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
									g_clear_error (&_inner_error_);
									return;
								}
							}
							_g_object_unref0 (source);
						}
						_gee_collection_object_unref0 (_source_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	{
		GeeIterator* _instance_it;
		_instance_it = gee_iterable_iterator ((GeeIterable*) self->priv->_instances);
		while (TRUE) {
			VtgPluginInstance* instance;
			if (!gee_iterator_next (_instance_it)) {
				break;
			}
			instance = (VtgPluginInstance*) gee_iterator_get (_instance_it);
			if (vtg_plugin_instance_get_project_manager_ui (instance) != sender) {
				vtg_project_view_add_project (vtg_project_manager_ui_get_project_view (vtg_plugin_instance_get_project_manager_ui (instance)), project);
			}
			_g_object_unref0 (instance);
		}
		_gee_collection_object_unref0 (_instance_it);
	}
	prj->completion = (_tmp30_ = _g_object_ref0 (completion), _g_object_unref0 (prj->completion), _tmp30_);
	prj->project = (_tmp31_ = _g_object_ref0 (project_manager), _g_object_unref0 (prj->project), _tmp31_);
	gee_collection_add ((GeeCollection*) self->priv->_projects, prj);
	vsc_parser_manager_resume_parsing (vsc_symbol_completion_get_parser (completion));
	recent = gtk_recent_manager_get_default ();
	recent_data = (memset (&_tmp32_, 0, sizeof (GtkRecentData)), _tmp32_);
	name = g_strdup (project->name);
	_tmp33_ = NULL;
	groups = (_tmp34_ = (_tmp33_ = g_new0 (char*, 1 + 1), _tmp33_[0] = g_strdup ("vtg"), _tmp33_), groups_length1 = 1, groups_size = groups_length1, _tmp34_);
	recent_data.display_name = (_tmp35_ = g_strdup (name), _g_free0 (recent_data.display_name), _tmp35_);
	recent_data.groups = (_tmp37_ = (_tmp36_ = groups, (_tmp36_ == NULL) ? ((gpointer) _tmp36_) : _vala_array_dup1 (_tmp36_, groups_length1)), recent_data.groups = (_vala_array_free (recent_data.groups, _vala_array_length (recent_data.groups), (GDestroyNotify) g_free), NULL), _tmp37_);
	recent_data.is_private = TRUE;
	recent_data.mime_type = (_tmp38_ = g_strdup ("text/plain"), _g_free0 (recent_data.mime_type), _tmp38_);
	recent_data.app_name = (_tmp39_ = g_strdup ("vtg"), _g_free0 (recent_data.app_name), _tmp39_);
	recent_data.app_exec = (_tmp40_ = g_strdup ("gedit %u"), _g_free0 (recent_data.app_exec), _tmp40_);
	_tmp43_ = (_tmp42_ = g_filename_to_uri (_tmp41_ = g_strconcat (project->id, "/configure.ac", NULL), NULL, &_inner_error_), _g_free0 (_tmp41_), _tmp42_);
	if (_inner_error_ != NULL) {
		_g_object_unref0 (prj);
		_g_object_unref0 (completion);
		_g_object_unref0 (project);
		gtk_recent_data_destroy (&recent_data);
		_g_free0 (name);
		groups = (_vala_array_free (groups, groups_length1, (GDestroyNotify) g_free), NULL);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	if ((_tmp45_ = !gtk_recent_manager_add_full (recent, _tmp44_ = _tmp43_, &recent_data), _g_free0 (_tmp44_), _tmp45_)) {
		g_warning ("vtgplugin.vala:304: cannot add project %s to recently used list", project->id);
	}
	_g_object_unref0 (prj);
	_g_object_unref0 (completion);
	_g_object_unref0 (project);
	gtk_recent_data_destroy (&recent_data);
	_g_free0 (name);
	groups = (_vala_array_free (groups, groups_length1, (GDestroyNotify) g_free), NULL);
}


VtgPlugin* vtg_plugin_construct (GType object_type) {
	VtgPlugin * self;
	self = g_object_newv (object_type, 0, NULL);
	return self;
}


VtgPlugin* vtg_plugin_new (void) {
	return vtg_plugin_construct (VTG_TYPE_PLUGIN);
}


GeeList* vtg_plugin_get_projects (VtgPlugin* self) {
	GeeList* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_projects;
	return result;
}


VtgConfiguration* vtg_plugin_get_config (VtgPlugin* self) {
	VtgConfiguration* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_config;
	return result;
}


static void _vtg_plugin_on_configuration_property_changed_g_object_notify (VtgConfiguration* _sender, GParamSpec* pspec, gpointer self) {
	vtg_plugin_on_configuration_property_changed (self, _sender, pspec);
}


static GObject * vtg_plugin_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgPluginClass * klass;
	GObjectClass * parent_class;
	VtgPlugin * self;
	klass = VTG_PLUGIN_CLASS (g_type_class_peek (VTG_TYPE_PLUGIN));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PLUGIN (obj);
	{
		VtgConfiguration* _tmp0_;
		self->priv->_config = (_tmp0_ = vtg_configuration_new (), _g_object_unref0 (self->priv->_config), _tmp0_);
		g_signal_connect_object ((GObject*) self->priv->_config, "notify", (GCallback) _vtg_plugin_on_configuration_property_changed_g_object_notify, self, 0);
		bindtextdomain (GETTEXT_PACKAGE, NULL);
	}
	return obj;
}


static void vtg_plugin_class_init (VtgPluginClass * klass) {
	vtg_plugin_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgPluginPrivate));
	GEDIT_PLUGIN_CLASS (klass)->activate = vtg_plugin_real_activate;
	GEDIT_PLUGIN_CLASS (klass)->deactivate = vtg_plugin_real_deactivate;
	GEDIT_PLUGIN_CLASS (klass)->is_configurable = vtg_plugin_real_is_configurable;
	GEDIT_PLUGIN_CLASS (klass)->create_configure_dialog = vtg_plugin_real_create_configure_dialog;
	GEDIT_PLUGIN_CLASS (klass)->update_ui = vtg_plugin_real_update_ui;
	G_OBJECT_CLASS (klass)->get_property = vtg_plugin_get_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_plugin_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_plugin_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_PROJECTS, gee_param_spec_collection_object ("projects", "projects", "projects", GEE_TYPE_LIST, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PLUGIN_CONFIG, g_param_spec_object ("config", "config", "config", VTG_TYPE_CONFIGURATION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_plugin_instance_init (VtgPlugin * self) {
	self->priv = VTG_PLUGIN_GET_PRIVATE (self);
	self->priv->_instances = (GeeList*) gee_array_list_new (VTG_TYPE_PLUGIN_INSTANCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_projects = (GeeList*) gee_array_list_new (VTG_TYPE_PROJECT_DESCRIPTOR, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_config = NULL;
	self->priv->_default_project = NULL;
}


static void vtg_plugin_finalize (GObject* obj) {
	VtgPlugin * self;
	self = VTG_PLUGIN (obj);
	{
		vtg_plugin_deactivate_modules (self, VTG_PLUGIN_DEACTIVATE_MODULE_OPTIONS_ALL);
	}
	_gee_collection_object_unref0 (self->priv->_instances);
	_gee_collection_object_unref0 (self->priv->_projects);
	_g_object_unref0 (self->priv->_config);
	_g_object_unref0 (self->priv->_default_project);
	G_OBJECT_CLASS (vtg_plugin_parent_class)->finalize (obj);
}


GType vtg_plugin_get_type (void) {
	return vtg_plugin_type_id;
}


GType vtg_plugin_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgPluginClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_plugin_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgPlugin), 0, (GInstanceInitFunc) vtg_plugin_instance_init, NULL };
	vtg_plugin_type_id = g_type_module_register_type (module, GEDIT_TYPE_PLUGIN, "VtgPlugin", &g_define_type_info, 0);
	return vtg_plugin_type_id;
}


static void vtg_plugin_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgPlugin * self;
	self = VTG_PLUGIN (object);
	switch (property_id) {
		case VTG_PLUGIN_PROJECTS:
		g_value_set_object (value, vtg_plugin_get_projects (self));
		break;
		case VTG_PLUGIN_CONFIG:
		g_value_set_object (value, vtg_plugin_get_config (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


GType register_gedit_plugin (GTypeModule* module) {
	GType result;
	g_return_val_if_fail (module != NULL, 0UL);
	vtg_bracket_completion_register_type (module);
	vtg_symbol_completion_provider_register_type (module);
	vtg_symbol_completion_trigger_register_type (module);
	vtg_symbol_completion_helper_register_type (module);
	vtg_project_manager_ui_register_type (module);
	vtg_project_descriptor_register_type (module);
	vtg_project_view_register_type (module);
	vtg_project_builder_register_type (module);
	vtg_project_executer_register_type (module);
	vtg_project_executer_dialog_register_type (module);
	vtg_project_search_register_type (module);
	vtg_project_search_results_view_register_type (module);
	vtg_project_search_dialog_register_type (module);
	vtg_process_watch_info_register_type (module);
	vtg_output_view_register_type (module);
	vtg_build_log_view_register_type (module);
	vtg_project_manager_register_type (module);
	vtg_filtered_list_dialog_register_type (module);
	vtg_vcs_backends_item_register_type (module);
	vtg_vcs_backends_git_register_type (module);
	vtg_vcs_backends_bzr_register_type (module);
	vtg_vcs_backends_svn_register_type (module);
	vtg_change_log_register_type (module);
	vtg_interaction_parameters_dialog_register_type (module);
	vtg_configuration_register_type (module);
	vtg_source_bookmark_register_type (module);
	vtg_source_bookmarks_register_type (module);
	vtg_plugin_instance_register_type (module);
	vtg_plugin_register_type (module);
	result = VTG_TYPE_PLUGIN;
	return result;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




