/*
 *  vtgprojectdescriptor.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <vsc.h>


#define VTG_TYPE_PROJECT_DESCRIPTOR (vtg_project_descriptor_get_type ())
#define VTG_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptor))
#define VTG_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))
#define VTG_IS_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_IS_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_PROJECT_DESCRIPTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))

typedef struct _VtgProjectDescriptor VtgProjectDescriptor;
typedef struct _VtgProjectDescriptorClass VtgProjectDescriptorClass;
typedef struct _VtgProjectDescriptorPrivate VtgProjectDescriptorPrivate;

#define VTG_TYPE_PROJECT_MANAGER (vtg_project_manager_get_type ())
#define VTG_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManager))
#define VTG_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))
#define VTG_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER))
#define VTG_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER))
#define VTG_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))

typedef struct _VtgProjectManager VtgProjectManager;
typedef struct _VtgProjectManagerClass VtgProjectManagerClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

struct _VtgProjectDescriptor {
	GObject parent_instance;
	VtgProjectDescriptorPrivate * priv;
	VscSymbolCompletion* completion;
	VtgProjectManager* project;
};

struct _VtgProjectDescriptorClass {
	GObjectClass parent_class;
};


static gpointer vtg_project_descriptor_parent_class = NULL;
static GType vtg_project_descriptor_type_id = 0;

GType vtg_project_descriptor_get_type (void);
GType vtg_project_descriptor_register_type (GTypeModule * module);
GType vtg_project_manager_get_type (void);
GType vtg_project_manager_register_type (GTypeModule * module);
enum  {
	VTG_PROJECT_DESCRIPTOR_DUMMY_PROPERTY
};
VtgProjectDescriptor* vtg_project_descriptor_new (void);
VtgProjectDescriptor* vtg_project_descriptor_construct (GType object_type);
static void vtg_project_descriptor_finalize (GObject* obj);



VtgProjectDescriptor* vtg_project_descriptor_construct (GType object_type) {
	VtgProjectDescriptor * self;
	self = (VtgProjectDescriptor*) g_object_new (object_type, NULL);
	return self;
}


VtgProjectDescriptor* vtg_project_descriptor_new (void) {
	return vtg_project_descriptor_construct (VTG_TYPE_PROJECT_DESCRIPTOR);
}


static void vtg_project_descriptor_class_init (VtgProjectDescriptorClass * klass) {
	vtg_project_descriptor_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vtg_project_descriptor_finalize;
}


static void vtg_project_descriptor_instance_init (VtgProjectDescriptor * self) {
}


static void vtg_project_descriptor_finalize (GObject* obj) {
	VtgProjectDescriptor * self;
	self = VTG_PROJECT_DESCRIPTOR (obj);
	{
		if (self->completion != NULL) {
			vsc_symbol_completion_cleanup (self->completion);
		}
	}
	_g_object_unref0 (self->completion);
	_g_object_unref0 (self->project);
	G_OBJECT_CLASS (vtg_project_descriptor_parent_class)->finalize (obj);
}


GType vtg_project_descriptor_get_type (void) {
	return vtg_project_descriptor_type_id;
}


GType vtg_project_descriptor_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectDescriptorClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_descriptor_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectDescriptor), 0, (GInstanceInitFunc) vtg_project_descriptor_instance_init, NULL };
	vtg_project_descriptor_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectDescriptor", &g_define_type_info, 0);
	return vtg_project_descriptor_type_id;
}




