/*
 *  vtgprojectmanager.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <vbf.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>


#define VTG_TYPE_PROJECT_MANAGER (vtg_project_manager_get_type ())
#define VTG_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManager))
#define VTG_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))
#define VTG_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER))
#define VTG_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER))
#define VTG_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))

typedef struct _VtgProjectManager VtgProjectManager;
typedef struct _VtgProjectManagerClass VtgProjectManagerClass;
typedef struct _VtgProjectManagerPrivate VtgProjectManagerPrivate;

#define VTG_TYPE_VCS_TYPES (vtg_vcs_types_get_type ())
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

#define VTG_VCS_BACKENDS_TYPE_GIT (vtg_vcs_backends_git_get_type ())
#define VTG_VCS_BACKENDS_GIT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_GIT, VtgVcsBackendsGit))
#define VTG_VCS_BACKENDS_GIT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_GIT, VtgVcsBackendsGitClass))
#define VTG_VCS_BACKENDS_IS_GIT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_GIT))
#define VTG_VCS_BACKENDS_IS_GIT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_GIT))
#define VTG_VCS_BACKENDS_GIT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_GIT, VtgVcsBackendsGitClass))

typedef struct _VtgVcsBackendsGit VtgVcsBackendsGit;
typedef struct _VtgVcsBackendsGitClass VtgVcsBackendsGitClass;

#define VTG_VCS_BACKENDS_TYPE_IVCS (vtg_vcs_backends_ivcs_get_type ())
#define VTG_VCS_BACKENDS_IVCS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_IVCS, VtgVcsBackendsIVcs))
#define VTG_VCS_BACKENDS_IS_IVCS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_IVCS))
#define VTG_VCS_BACKENDS_IVCS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VTG_VCS_BACKENDS_TYPE_IVCS, VtgVcsBackendsIVcsIface))

typedef struct _VtgVcsBackendsIVcs VtgVcsBackendsIVcs;
typedef struct _VtgVcsBackendsIVcsIface VtgVcsBackendsIVcsIface;

#define VTG_VCS_BACKENDS_TYPE_ITEM (vtg_vcs_backends_item_get_type ())
#define VTG_VCS_BACKENDS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItem))
#define VTG_VCS_BACKENDS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItemClass))
#define VTG_VCS_BACKENDS_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_ITEM))
#define VTG_VCS_BACKENDS_IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_ITEM))
#define VTG_VCS_BACKENDS_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItemClass))

typedef struct _VtgVcsBackendsItem VtgVcsBackendsItem;
typedef struct _VtgVcsBackendsItemClass VtgVcsBackendsItemClass;

#define VTG_VCS_BACKENDS_TYPE_BZR (vtg_vcs_backends_bzr_get_type ())
#define VTG_VCS_BACKENDS_BZR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_BZR, VtgVcsBackendsBzr))
#define VTG_VCS_BACKENDS_BZR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_BZR, VtgVcsBackendsBzrClass))
#define VTG_VCS_BACKENDS_IS_BZR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_BZR))
#define VTG_VCS_BACKENDS_IS_BZR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_BZR))
#define VTG_VCS_BACKENDS_BZR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_BZR, VtgVcsBackendsBzrClass))

typedef struct _VtgVcsBackendsBzr VtgVcsBackendsBzr;
typedef struct _VtgVcsBackendsBzrClass VtgVcsBackendsBzrClass;

#define VTG_VCS_BACKENDS_TYPE_SVN (vtg_vcs_backends_svn_get_type ())
#define VTG_VCS_BACKENDS_SVN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_SVN, VtgVcsBackendsSvn))
#define VTG_VCS_BACKENDS_SVN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_SVN, VtgVcsBackendsSvnClass))
#define VTG_VCS_BACKENDS_IS_SVN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_SVN))
#define VTG_VCS_BACKENDS_IS_SVN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_SVN))
#define VTG_VCS_BACKENDS_SVN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_SVN, VtgVcsBackendsSvnClass))

typedef struct _VtgVcsBackendsSvn VtgVcsBackendsSvn;
typedef struct _VtgVcsBackendsSvnClass VtgVcsBackendsSvnClass;

typedef enum  {
	VTG_VCS_TYPES_NONE,
	VTG_VCS_TYPES_GIT,
	VTG_VCS_TYPES_BZR,
	VTG_VCS_TYPES_SVN
} VtgVcsTypes;

struct _VtgProjectManager {
	GObject parent_instance;
	VtgProjectManagerPrivate * priv;
	char* filename;
	GeeList* exec_targets;
	GeeList* all_vala_sources;
	VtgVcsTypes vcs_type;
	char* changelog_uri;
};

struct _VtgProjectManagerClass {
	GObjectClass parent_class;
};

struct _VtgProjectManagerPrivate {
	VbfProject* _project;
	GtkTreeStore* _model;
	gboolean in_update;
};

typedef enum  {
	VTG_PROJECT_MANAGER_ERROR_NO_BACKEND
} VtgProjectManagerError;
#define VTG_PROJECT_MANAGER_ERROR vtg_project_manager_error_quark ()
struct _VtgVcsBackendsIVcsIface {
	GTypeInterface parent_iface;
	GeeList* (*get_items) (VtgVcsBackendsIVcs* self, const char* path, GError** error);
	gboolean (*test) (VtgVcsBackendsIVcs* self, const char* path);
};


static gpointer vtg_project_manager_parent_class = NULL;
static GType vtg_project_manager_type_id = 0;

GType vtg_project_manager_get_type (void);
GType vtg_project_manager_register_type (GTypeModule * module);
GType vtg_vcs_types_get_type (void);
#define VTG_PROJECT_MANAGER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerPrivate))
enum  {
	VTG_PROJECT_MANAGER_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_MODEL,
	VTG_PROJECT_MANAGER_PROJECT
};
gboolean vtg_project_manager_contains_file (VtgProjectManager* self, const char* uri);
VbfSource* vtg_project_manager_get_source_file_from_uri (VtgProjectManager* self, const char* uri);
gboolean vtg_project_manager_contains_vala_source_file (VtgProjectManager* self, const char* uri);
char* vtg_project_manager_source_uri_for_name (VtgProjectManager* self, const char* name);
static void vtg_project_manager_parse_project (VtgProjectManager* self);
static void vtg_project_manager_build_tree_model (VtgProjectManager* self);
static void vtg_project_manager_vcs_test (VtgProjectManager* self, const char* filename);
static void vtg_project_manager_on_project_updated (VtgProjectManager* self, VbfProject* sender);
static void _vtg_project_manager_on_project_updated_vbf_project_updated (VbfProject* _sender, gpointer self);
GQuark vtg_project_manager_error_quark (void);
gboolean vtg_project_manager_open (VtgProjectManager* self, const char* project_filename, GError** error);
VtgVcsBackendsGit* vtg_vcs_backends_git_new (void);
VtgVcsBackendsGit* vtg_vcs_backends_git_construct (GType object_type);
GType vtg_vcs_backends_git_get_type (void);
GType vtg_vcs_backends_git_register_type (GTypeModule * module);
gpointer vtg_vcs_backends_item_ref (gpointer instance);
void vtg_vcs_backends_item_unref (gpointer instance);
GParamSpec* vtg_vcs_backends_param_spec_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_vcs_backends_value_set_item (GValue* value, gpointer v_object);
gpointer vtg_vcs_backends_value_get_item (const GValue* value);
GType vtg_vcs_backends_item_get_type (void);
GType vtg_vcs_backends_item_register_type (GTypeModule * module);
GType vtg_vcs_backends_ivcs_get_type (void);
gboolean vtg_vcs_backends_ivcs_test (VtgVcsBackendsIVcs* self, const char* path);
VtgVcsBackendsBzr* vtg_vcs_backends_bzr_new (void);
VtgVcsBackendsBzr* vtg_vcs_backends_bzr_construct (GType object_type);
GType vtg_vcs_backends_bzr_get_type (void);
GType vtg_vcs_backends_bzr_register_type (GTypeModule * module);
VtgVcsBackendsSvn* vtg_vcs_backends_svn_new (void);
VtgVcsBackendsSvn* vtg_vcs_backends_svn_construct (GType object_type);
GType vtg_vcs_backends_svn_get_type (void);
GType vtg_vcs_backends_svn_register_type (GTypeModule * module);
void vtg_project_manager_close (VtgProjectManager* self);
static gint vtg_project_manager_sort_model (VtgProjectManager* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b);
static gint _vtg_project_manager_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self);
gint vtg_path_utils_compare_vala_filenames (const char* filea, const char* fileb);
VtgProjectManager* vtg_project_manager_new (void);
VtgProjectManager* vtg_project_manager_construct (GType object_type);
GtkTreeModel* vtg_project_manager_get_model (VtgProjectManager* self);
VbfProject* vtg_project_manager_get_project (VtgProjectManager* self);
static void vtg_project_manager_finalize (GObject* obj);
static void vtg_project_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



gboolean vtg_project_manager_contains_file (VtgProjectManager* self, const char* uri) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (uri != NULL, FALSE);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _group_it;
		_group_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vbf_project_get_groups (self->priv->_project))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp2_;
				GeeIterator* _tmp3_;
				GeeIterator* _target_it;
				_target_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp4_;
						GeeIterator* _tmp5_;
						GeeIterator* _source_it;
						_source_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
						while (TRUE) {
							VbfSource* source;
							if (!gee_iterator_next (_source_it)) {
								break;
							}
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (_vala_strcmp0 (((VbfFile*) source)->uri, uri) == 0) {
								result = TRUE;
								_g_object_unref0 (source);
								_gee_collection_object_unref0 (_source_it);
								_g_object_unref0 (target);
								_gee_collection_object_unref0 (_target_it);
								_g_object_unref0 (group);
								_gee_collection_object_unref0 (_group_it);
								return result;
							}
							_g_object_unref0 (source);
						}
						_gee_collection_object_unref0 (_source_it);
					}
					{
						GeeList* _tmp6_;
						GeeIterator* _tmp7_;
						GeeIterator* _file_it;
						_file_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vbf_target_get_files (target))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
						while (TRUE) {
							VbfFile* file;
							if (!gee_iterator_next (_file_it)) {
								break;
							}
							file = (VbfFile*) gee_iterator_get (_file_it);
							if (_vala_strcmp0 (file->uri, uri) == 0) {
								result = TRUE;
								_g_object_unref0 (file);
								_gee_collection_object_unref0 (_file_it);
								_g_object_unref0 (target);
								_gee_collection_object_unref0 (_target_it);
								_g_object_unref0 (group);
								_gee_collection_object_unref0 (_group_it);
								return result;
							}
							_g_object_unref0 (file);
						}
						_gee_collection_object_unref0 (_file_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	result = FALSE;
	return result;
}


VbfSource* vtg_project_manager_get_source_file_from_uri (VtgProjectManager* self, const char* uri) {
	VbfSource* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (uri != NULL, NULL);
	{
		GeeList* _tmp0_;
		GeeIterator* _tmp1_;
		GeeIterator* _group_it;
		_group_it = (_tmp1_ = gee_iterable_iterator ((GeeIterable*) (_tmp0_ = vbf_project_get_groups (self->priv->_project))), _gee_collection_object_unref0 (_tmp0_), _tmp1_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp2_;
				GeeIterator* _tmp3_;
				GeeIterator* _target_it;
				_target_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp4_;
						GeeIterator* _tmp5_;
						GeeIterator* _source_it;
						_source_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
						while (TRUE) {
							VbfSource* source;
							if (!gee_iterator_next (_source_it)) {
								break;
							}
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (_vala_strcmp0 (((VbfFile*) source)->uri, uri) == 0) {
								result = source;
								_gee_collection_object_unref0 (_source_it);
								_g_object_unref0 (target);
								_gee_collection_object_unref0 (_target_it);
								_g_object_unref0 (group);
								_gee_collection_object_unref0 (_group_it);
								return result;
							}
							_g_object_unref0 (source);
						}
						_gee_collection_object_unref0 (_source_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	result = NULL;
	return result;
}


gboolean vtg_project_manager_contains_vala_source_file (VtgProjectManager* self, const char* uri) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (uri != NULL, FALSE);
	{
		GeeIterator* _source_it;
		_source_it = gee_iterable_iterator ((GeeIterable*) self->all_vala_sources);
		while (TRUE) {
			VbfSource* source;
			if (!gee_iterator_next (_source_it)) {
				break;
			}
			source = (VbfSource*) gee_iterator_get (_source_it);
			if (_vala_strcmp0 (((VbfFile*) source)->uri, uri) == 0) {
				result = TRUE;
				_g_object_unref0 (source);
				_gee_collection_object_unref0 (_source_it);
				return result;
			}
			_g_object_unref0 (source);
		}
		_gee_collection_object_unref0 (_source_it);
	}
	result = FALSE;
	return result;
}


char* vtg_project_manager_source_uri_for_name (VtgProjectManager* self, const char* name) {
	char* result;
	char** _tmp1_;
	gint name_parts_size;
	gint name_parts_length1;
	char** _tmp0_;
	char** name_parts;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	name_parts = (_tmp1_ = _tmp0_ = g_strsplit (name, "/", 0), name_parts_length1 = _vala_array_length (_tmp0_), name_parts_size = name_parts_length1, _tmp1_);
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _group_it;
		_group_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vbf_project_get_groups (self->priv->_project))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp4_;
				GeeIterator* _tmp5_;
				GeeIterator* _target_it;
				_target_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
				while (TRUE) {
					VbfTarget* target;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					{
						GeeList* _tmp6_;
						GeeIterator* _tmp7_;
						GeeIterator* _source_it;
						_source_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
						while (TRUE) {
							VbfSource* source;
							if (!gee_iterator_next (_source_it)) {
								break;
							}
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (name_parts_length1 == 1) {
								if (_vala_strcmp0 (((VbfFile*) source)->name, name) == 0) {
									result = g_strdup (((VbfFile*) source)->uri);
									_g_object_unref0 (source);
									_gee_collection_object_unref0 (_source_it);
									_g_object_unref0 (target);
									_gee_collection_object_unref0 (_target_it);
									_g_object_unref0 (group);
									_gee_collection_object_unref0 (_group_it);
									name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL);
									return result;
								}
							} else {
								if (((VbfFile*) source)->uri != NULL) {
									char** _tmp9_;
									gint src_parts_size;
									gint src_parts_length1;
									char** _tmp8_;
									char** src_parts;
									src_parts = (_tmp9_ = _tmp8_ = g_strsplit (((VbfFile*) source)->uri, "/", 0), src_parts_length1 = _vala_array_length (_tmp8_), src_parts_size = src_parts_length1, _tmp9_);
									if (name_parts_length1 <= src_parts_length1) {
										gboolean equals;
										equals = TRUE;
										{
											gint idx;
											idx = 0;
											{
												gboolean _tmp10_;
												_tmp10_ = TRUE;
												while (TRUE) {
													if (!_tmp10_) {
														idx++;
													}
													_tmp10_ = FALSE;
													if (!(idx < name_parts_length1)) {
														break;
													}
													if (_vala_strcmp0 (src_parts[src_parts_length1 - idx], name_parts[name_parts_length1 - idx]) != 0) {
														equals = FALSE;
														break;
													}
												}
											}
										}
										if (equals) {
											result = g_strdup (((VbfFile*) source)->uri);
											src_parts = (_vala_array_free (src_parts, src_parts_length1, (GDestroyNotify) g_free), NULL);
											_g_object_unref0 (source);
											_gee_collection_object_unref0 (_source_it);
											_g_object_unref0 (target);
											_gee_collection_object_unref0 (_target_it);
											_g_object_unref0 (group);
											_gee_collection_object_unref0 (_group_it);
											name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL);
											return result;
										}
									}
									src_parts = (_vala_array_free (src_parts, src_parts_length1, (GDestroyNotify) g_free), NULL);
								}
							}
							_g_object_unref0 (source);
						}
						_gee_collection_object_unref0 (_source_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	result = NULL;
	name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL);
	return result;
}


static void _vtg_project_manager_on_project_updated_vbf_project_updated (VbfProject* _sender, gpointer self) {
	vtg_project_manager_on_project_updated (self, _sender);
}


gboolean vtg_project_manager_open (VtgProjectManager* self, const char* project_filename, GError** error) {
	gboolean result;
	GError * _inner_error_;
	VbfIProjectManager* pm;
	gboolean res;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (project_filename != NULL, FALSE);
	_inner_error_ = NULL;
	pm = (VbfIProjectManager*) vbf_am_project_manager_new ();
	res = vbf_iproject_manager_probe (pm, project_filename);
	if (res) {
		VbfProject* _tmp0_;
		self->priv->_project = (_tmp0_ = vbf_iproject_manager_open (pm, project_filename), _g_object_unref0 (self->priv->_project), _tmp0_);
		if (self->priv->_project == NULL) {
			result = FALSE;
			_g_object_unref0 (pm);
			return result;
		}
		vtg_project_manager_parse_project (self);
		vtg_project_manager_build_tree_model (self);
		vtg_project_manager_vcs_test (self, project_filename);
		g_signal_connect_object (self->priv->_project, "updated", (GCallback) _vtg_project_manager_on_project_updated_vbf_project_updated, self, 0);
		result = TRUE;
		_g_object_unref0 (pm);
		return result;
	} else {
		_inner_error_ = g_error_new_literal (VTG_PROJECT_MANAGER_ERROR, VTG_PROJECT_MANAGER_ERROR_NO_BACKEND, _ ("Can't load project, no suitable backend found"));
		if (_inner_error_ != NULL) {
			g_propagate_error (error, _inner_error_);
			_g_object_unref0 (pm);
			return FALSE;
		}
	}
	_g_object_unref0 (pm);
}


static void vtg_project_manager_vcs_test (VtgProjectManager* self, const char* filename) {
	VtgVcsBackendsIVcs* backend;
	g_return_if_fail (self != NULL);
	g_return_if_fail (filename != NULL);
	backend = (VtgVcsBackendsIVcs*) vtg_vcs_backends_git_new ();
	self->vcs_type = VTG_VCS_TYPES_NONE;
	if (vtg_vcs_backends_ivcs_test (backend, filename)) {
		self->vcs_type = VTG_VCS_TYPES_GIT;
	} else {
		VtgVcsBackendsIVcs* _tmp0_;
		backend = (_tmp0_ = (VtgVcsBackendsIVcs*) vtg_vcs_backends_bzr_new (), _g_object_unref0 (backend), _tmp0_);
		if (vtg_vcs_backends_ivcs_test (backend, filename)) {
			self->vcs_type = VTG_VCS_TYPES_BZR;
		} else {
			VtgVcsBackendsIVcs* _tmp1_;
			backend = (_tmp1_ = (VtgVcsBackendsIVcs*) vtg_vcs_backends_svn_new (), _g_object_unref0 (backend), _tmp1_);
			if (vtg_vcs_backends_ivcs_test (backend, filename)) {
				self->vcs_type = VTG_VCS_TYPES_SVN;
			}
		}
	}
	_g_object_unref0 (backend);
}


void vtg_project_manager_close (VtgProjectManager* self) {
	GtkTreeStore* _tmp0_;
	VbfProject* _tmp1_;
	g_return_if_fail (self != NULL);
	gee_collection_clear ((GeeCollection*) self->exec_targets);
	gee_collection_clear ((GeeCollection*) self->all_vala_sources);
	self->priv->_model = (_tmp0_ = NULL, _g_object_unref0 (self->priv->_model), _tmp0_);
	self->priv->_project = (_tmp1_ = NULL, _g_object_unref0 (self->priv->_project), _tmp1_);
}


static void vtg_project_manager_on_project_updated (VtgProjectManager* self, VbfProject* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (self->priv->in_update) {
		return;
	}
	self->priv->in_update = TRUE;
	vtg_project_manager_parse_project (self);
	vtg_project_manager_build_tree_model (self);
	g_signal_emit_by_name (self, "updated");
	self->priv->in_update = FALSE;
}


static gint _vtg_project_manager_sort_model_gtk_tree_iter_compare_func (GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b, gpointer self) {
	return vtg_project_manager_sort_model (self, model, a, b);
}


static void vtg_project_manager_build_tree_model (VtgProjectManager* self) {
	GtkTreeIter project_iter = {0};
	GtkTreeIter modules_iter = {0};
	GtkTreeIter groups_iter = {0};
	GtkTreeStore* _tmp0_;
	char* _tmp1_;
	g_return_if_fail (self != NULL);
	self->priv->_model = (_tmp0_ = gtk_tree_store_new (5, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_OBJECT, G_TYPE_STRING, NULL), _g_object_unref0 (self->priv->_model), _tmp0_);
	gtk_tree_store_append (self->priv->_model, &project_iter, NULL);
	gtk_tree_store_set (self->priv->_model, &project_iter, 0, GTK_STOCK_DIRECTORY, 1, _tmp1_ = g_strdup_printf ("%s - %s", self->priv->_project->name, self->priv->_project->version), 2, "project-root", 4, "", -1);
	_g_free0 (_tmp1_);
	gtk_tree_store_append (self->priv->_model, &modules_iter, &project_iter);
	gtk_tree_store_set (self->priv->_model, &modules_iter, 0, GTK_STOCK_DIRECTORY, 1, _ ("References"), 2, "project-reference", 4, "1", -1);
	{
		GeeList* _tmp2_;
		GeeIterator* _tmp3_;
		GeeIterator* _module_it;
		_module_it = (_tmp3_ = gee_iterable_iterator ((GeeIterable*) (_tmp2_ = vbf_project_get_modules (self->priv->_project))), _gee_collection_object_unref0 (_tmp2_), _tmp3_);
		while (TRUE) {
			VbfModule* module;
			GtkTreeIter module_iter = {0};
			if (!gee_iterator_next (_module_it)) {
				break;
			}
			module = (VbfModule*) gee_iterator_get (_module_it);
			gtk_tree_store_append (self->priv->_model, &module_iter, &modules_iter);
			gtk_tree_store_set (self->priv->_model, &module_iter, 0, GTK_STOCK_DIRECTORY, 1, module->name, 2, module->id, 3, module, 4, module->name, -1);
			{
				GeeList* _tmp4_;
				GeeIterator* _tmp5_;
				GeeIterator* _package_it;
				_package_it = (_tmp5_ = gee_iterable_iterator ((GeeIterable*) (_tmp4_ = vbf_module_get_packages (module))), _gee_collection_object_unref0 (_tmp4_), _tmp5_);
				while (TRUE) {
					VbfPackage* package;
					GtkTreeIter package_iter = {0};
					if (!gee_iterator_next (_package_it)) {
						break;
					}
					package = (VbfPackage*) gee_iterator_get (_package_it);
					gtk_tree_store_append (self->priv->_model, &package_iter, &module_iter);
					gtk_tree_store_set (self->priv->_model, &package_iter, 0, GTK_STOCK_FILE, 1, package->name, 2, package->id, 3, package, 4, package->name, -1);
					_g_object_unref0 (package);
				}
				_gee_collection_object_unref0 (_package_it);
			}
			_g_object_unref0 (module);
		}
		_gee_collection_object_unref0 (_module_it);
	}
	gtk_tree_store_append (self->priv->_model, &groups_iter, &project_iter);
	gtk_tree_store_set (self->priv->_model, &groups_iter, 0, GTK_STOCK_DIRECTORY, 1, _ ("Files"), 2, "project-files", 4, "2", -1);
	{
		GeeList* _tmp6_;
		GeeIterator* _tmp7_;
		GeeIterator* _group_it;
		_group_it = (_tmp7_ = gee_iterable_iterator ((GeeIterable*) (_tmp6_ = vbf_project_get_groups (self->priv->_project))), _gee_collection_object_unref0 (_tmp6_), _tmp7_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp8_;
				GeeIterator* _tmp9_;
				GeeIterator* _target_it;
				_target_it = (_tmp9_ = gee_iterable_iterator ((GeeIterable*) (_tmp8_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp8_), _tmp9_);
				while (TRUE) {
					VbfTarget* target;
					gboolean _tmp10_;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					_tmp10_ = FALSE;
					if (vbf_target_has_sources_of_type (target, VBF_FILE_TYPES_VALA_SOURCE)) {
						_tmp10_ = TRUE;
					} else {
						GeeList* _tmp11_;
						_tmp10_ = gee_collection_get_size ((GeeCollection*) (_tmp11_ = vbf_target_get_files (target))) > 0;
						_gee_collection_object_unref0 (_tmp11_);
					}
					if (_tmp10_) {
						GtkTreeIter target_iter;
						gboolean target_added;
						target_iter = groups_iter;
						target_added = FALSE;
						{
							GeeList* _tmp12_;
							GeeIterator* _tmp13_;
							GeeIterator* _source_it;
							_source_it = (_tmp13_ = gee_iterable_iterator ((GeeIterable*) (_tmp12_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp12_), _tmp13_);
							while (TRUE) {
								VbfSource* source;
								gboolean _tmp14_;
								gboolean _tmp15_;
								gboolean _tmp16_;
								GtkTreeIter source_iter = {0};
								if (!gee_iterator_next (_source_it)) {
									break;
								}
								source = (VbfSource*) gee_iterator_get (_source_it);
								_tmp14_ = FALSE;
								_tmp15_ = FALSE;
								_tmp16_ = FALSE;
								if (g_str_has_prefix (((VbfFile*) source)->name, ".")) {
									_tmp16_ = TRUE;
								} else {
									_tmp16_ = g_str_has_suffix (((VbfFile*) source)->name, ".c");
								}
								if (_tmp16_) {
									_tmp15_ = TRUE;
								} else {
									_tmp15_ = g_str_has_suffix (((VbfFile*) source)->name, ".h");
								}
								if (_tmp15_) {
									_tmp14_ = TRUE;
								} else {
									_tmp14_ = g_str_has_suffix (((VbfFile*) source)->name, ".stamp");
								}
								if (_tmp14_) {
									_g_object_unref0 (source);
									continue;
								}
								if (!target_added) {
									gtk_tree_store_append (self->priv->_model, &target_iter, &groups_iter);
									gtk_tree_store_set (self->priv->_model, &target_iter, 0, GTK_STOCK_DIRECTORY, 1, group->name, 2, target->id, 3, target, 4, group->name, -1);
									target_added = TRUE;
								}
								gtk_tree_store_append (self->priv->_model, &source_iter, &target_iter);
								gtk_tree_store_set (self->priv->_model, &source_iter, 0, GTK_STOCK_FILE, 1, ((VbfFile*) source)->name, 2, ((VbfFile*) source)->uri, 3, source, 4, ((VbfFile*) source)->name, -1);
								_g_object_unref0 (source);
							}
							_gee_collection_object_unref0 (_source_it);
						}
						{
							GeeList* _tmp17_;
							GeeIterator* _tmp18_;
							GeeIterator* _file_it;
							_file_it = (_tmp18_ = gee_iterable_iterator ((GeeIterable*) (_tmp17_ = vbf_target_get_files (target))), _gee_collection_object_unref0 (_tmp17_), _tmp18_);
							while (TRUE) {
								VbfFile* file;
								GtkTreeIter file_iter = {0};
								if (!gee_iterator_next (_file_it)) {
									break;
								}
								file = (VbfFile*) gee_iterator_get (_file_it);
								if (!target_added) {
									gtk_tree_store_append (self->priv->_model, &target_iter, &groups_iter);
									gtk_tree_store_set (self->priv->_model, &target_iter, 0, GTK_STOCK_DIRECTORY, 1, group->name, 2, target->id, 3, target, 4, group->name, -1);
									target_added = TRUE;
								}
								gtk_tree_store_append (self->priv->_model, &file_iter, &target_iter);
								gtk_tree_store_set (self->priv->_model, &file_iter, 0, GTK_STOCK_FILE, 1, file->name, 2, file->uri, 3, file, 4, file->name, -1);
								_g_object_unref0 (file);
							}
							_gee_collection_object_unref0 (_file_it);
						}
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	gtk_tree_sortable_set_sort_column_id ((GtkTreeSortable*) self->priv->_model, 4, GTK_SORT_ASCENDING);
	gtk_tree_sortable_set_sort_func ((GtkTreeSortable*) self->priv->_model, 4, _vtg_project_manager_sort_model_gtk_tree_iter_compare_func, g_object_ref (self), g_object_unref);
}


static void vtg_project_manager_parse_project (VtgProjectManager* self) {
	char* _tmp0_;
	char* _tmp9_;
	gboolean _tmp10_;
	g_return_if_fail (self != NULL);
	gee_collection_clear ((GeeCollection*) self->exec_targets);
	gee_collection_clear ((GeeCollection*) self->all_vala_sources);
	self->changelog_uri = (_tmp0_ = NULL, _g_free0 (self->changelog_uri), _tmp0_);
	{
		GeeList* _tmp1_;
		GeeIterator* _tmp2_;
		GeeIterator* _group_it;
		_group_it = (_tmp2_ = gee_iterable_iterator ((GeeIterable*) (_tmp1_ = vbf_project_get_groups (self->priv->_project))), _gee_collection_object_unref0 (_tmp1_), _tmp2_);
		while (TRUE) {
			VbfGroup* group;
			if (!gee_iterator_next (_group_it)) {
				break;
			}
			group = (VbfGroup*) gee_iterator_get (_group_it);
			{
				GeeList* _tmp3_;
				GeeIterator* _tmp4_;
				GeeIterator* _target_it;
				_target_it = (_tmp4_ = gee_iterable_iterator ((GeeIterable*) (_tmp3_ = vbf_group_get_targets (group))), _gee_collection_object_unref0 (_tmp3_), _tmp4_);
				while (TRUE) {
					VbfTarget* target;
					gboolean _tmp5_;
					if (!gee_iterator_next (_target_it)) {
						break;
					}
					target = (VbfTarget*) gee_iterator_get (_target_it);
					_tmp5_ = FALSE;
					if (target->type == VBF_TARGET_TYPES_PROGRAM) {
						_tmp5_ = TRUE;
					} else {
						gboolean _tmp6_;
						_tmp6_ = FALSE;
						if (target->type == VBF_TARGET_TYPES_BUILT_SOURCES) {
							_tmp6_ = !g_str_has_prefix (target->name, "lib");
						} else {
							_tmp6_ = FALSE;
						}
						_tmp5_ = _tmp6_;
					}
					if (_tmp5_) {
						gee_collection_add ((GeeCollection*) self->exec_targets, target);
					}
					{
						GeeList* _tmp7_;
						GeeIterator* _tmp8_;
						GeeIterator* _source_it;
						_source_it = (_tmp8_ = gee_iterable_iterator ((GeeIterable*) (_tmp7_ = vbf_target_get_sources (target))), _gee_collection_object_unref0 (_tmp7_), _tmp8_);
						while (TRUE) {
							VbfSource* source;
							if (!gee_iterator_next (_source_it)) {
								break;
							}
							source = (VbfSource*) gee_iterator_get (_source_it);
							if (((VbfFile*) source)->type == VBF_FILE_TYPES_VALA_SOURCE) {
								gee_collection_add ((GeeCollection*) self->all_vala_sources, source);
							}
							_g_object_unref0 (source);
						}
						_gee_collection_object_unref0 (_source_it);
					}
					_g_object_unref0 (target);
				}
				_gee_collection_object_unref0 (_target_it);
			}
			_g_object_unref0 (group);
		}
		_gee_collection_object_unref0 (_group_it);
	}
	if ((_tmp10_ = g_file_test (_tmp9_ = g_build_filename (self->priv->_project->working_dir, "changelog", NULL), G_FILE_TEST_EXISTS), _g_free0 (_tmp9_), _tmp10_)) {
		char* _tmp12_;
		char* _tmp11_;
		self->changelog_uri = (_tmp12_ = g_strdup_printf ("file://%s", _tmp11_ = g_build_filename (self->priv->_project->working_dir, "changelog", NULL)), _g_free0 (self->changelog_uri), _tmp12_);
		_g_free0 (_tmp11_);
	} else {
		char* _tmp13_;
		gboolean _tmp14_;
		if ((_tmp14_ = g_file_test (_tmp13_ = g_build_filename (self->priv->_project->working_dir, "ChangeLog", NULL), G_FILE_TEST_EXISTS), _g_free0 (_tmp13_), _tmp14_)) {
			char* _tmp16_;
			char* _tmp15_;
			self->changelog_uri = (_tmp16_ = g_strdup_printf ("file://%s", _tmp15_ = g_build_filename (self->priv->_project->working_dir, "ChangeLog", NULL)), _g_free0 (self->changelog_uri), _tmp16_);
			_g_free0 (_tmp15_);
		}
	}
}


static gint vtg_project_manager_sort_model (VtgProjectManager* self, GtkTreeModel* model, GtkTreeIter* a, GtkTreeIter* b) {
	gint result;
	char* vala;
	char* valb;
	g_return_val_if_fail (self != NULL, 0);
	g_return_val_if_fail (model != NULL, 0);
	vala = NULL;
	valb = NULL;
	gtk_tree_model_get (model, &(*a), 4, &vala, -1);
	gtk_tree_model_get (model, &(*b), 4, &valb, -1);
	result = vtg_path_utils_compare_vala_filenames (vala, valb);
	_g_free0 (vala);
	_g_free0 (valb);
	return result;
}


VtgProjectManager* vtg_project_manager_construct (GType object_type) {
	VtgProjectManager * self;
	self = (VtgProjectManager*) g_object_new (object_type, NULL);
	return self;
}


VtgProjectManager* vtg_project_manager_new (void) {
	return vtg_project_manager_construct (VTG_TYPE_PROJECT_MANAGER);
}


GtkTreeModel* vtg_project_manager_get_model (VtgProjectManager* self) {
	GtkTreeModel* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = (GtkTreeModel*) self->priv->_model;
	return result;
}


VbfProject* vtg_project_manager_get_project (VtgProjectManager* self) {
	VbfProject* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_project;
	return result;
}


static void vtg_project_manager_class_init (VtgProjectManagerClass * klass) {
	vtg_project_manager_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_get_property;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_MODEL, g_param_spec_object ("model", "model", "model", GTK_TYPE_TREE_MODEL, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_PROJECT, g_param_spec_object ("project", "project", "project", VBF_TYPE_PROJECT, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
	g_signal_new ("updated", VTG_TYPE_PROJECT_MANAGER, G_SIGNAL_RUN_LAST, 0, NULL, NULL, g_cclosure_marshal_VOID__VOID, G_TYPE_NONE, 0);
}


static void vtg_project_manager_instance_init (VtgProjectManager * self) {
	self->priv = VTG_PROJECT_MANAGER_GET_PRIVATE (self);
	self->priv->_project = NULL;
	self->priv->in_update = FALSE;
	self->filename = NULL;
	self->exec_targets = (GeeList*) gee_array_list_new (VBF_TYPE_TARGET, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->all_vala_sources = (GeeList*) gee_array_list_new (VBF_TYPE_SOURCE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->vcs_type = VTG_VCS_TYPES_NONE;
	self->changelog_uri = NULL;
}


static void vtg_project_manager_finalize (GObject* obj) {
	VtgProjectManager * self;
	self = VTG_PROJECT_MANAGER (obj);
	{
	}
	_g_object_unref0 (self->priv->_project);
	_g_object_unref0 (self->priv->_model);
	_g_free0 (self->filename);
	_gee_collection_object_unref0 (self->exec_targets);
	_gee_collection_object_unref0 (self->all_vala_sources);
	_g_free0 (self->changelog_uri);
	G_OBJECT_CLASS (vtg_project_manager_parent_class)->finalize (obj);
}


GType vtg_project_manager_get_type (void) {
	return vtg_project_manager_type_id;
}


GType vtg_project_manager_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManager), 0, (GInstanceInitFunc) vtg_project_manager_instance_init, NULL };
	vtg_project_manager_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManager", &g_define_type_info, 0);
	return vtg_project_manager_type_id;
}


static void vtg_project_manager_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManager * self;
	self = VTG_PROJECT_MANAGER (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_MODEL:
		g_value_set_object (value, vtg_project_manager_get_model (self));
		break;
		case VTG_PROJECT_MANAGER_PROJECT:
		g_value_set_object (value, vtg_project_manager_get_project (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




