/*
 *  vtgprojectmanagerui.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <gtk/gtk.h>
#include <glib/gi18n-lib.h>
#include <config.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-tab.h>
#include <gedit/gedit-document.h>
#include <vbf.h>
#include <gedit/gedit-view.h>
#include <gedit/gedit-plugin.h>
#include <vsc.h>
#include <gtksourceview/gtksourceview.h>
#include <sys/wait.h>


#define VTG_TYPE_PROJECT_MANAGER_UI (vtg_project_manager_ui_get_type ())
#define VTG_PROJECT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUi))
#define VTG_PROJECT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUiClass))
#define VTG_IS_PROJECT_MANAGER_UI(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER_UI))
#define VTG_IS_PROJECT_MANAGER_UI_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER_UI))
#define VTG_PROJECT_MANAGER_UI_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUiClass))

typedef struct _VtgProjectManagerUi VtgProjectManagerUi;
typedef struct _VtgProjectManagerUiClass VtgProjectManagerUiClass;
typedef struct _VtgProjectManagerUiPrivate VtgProjectManagerUiPrivate;

#define VTG_TYPE_PROJECT_MANAGER (vtg_project_manager_get_type ())
#define VTG_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManager))
#define VTG_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))
#define VTG_IS_PROJECT_MANAGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_MANAGER))
#define VTG_IS_PROJECT_MANAGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_MANAGER))
#define VTG_PROJECT_MANAGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_MANAGER, VtgProjectManagerClass))

typedef struct _VtgProjectManager VtgProjectManager;
typedef struct _VtgProjectManagerClass VtgProjectManagerClass;

#define VTG_TYPE_PLUGIN_INSTANCE (vtg_plugin_instance_get_type ())
#define VTG_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstance))
#define VTG_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))
#define VTG_IS_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_IS_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_PLUGIN_INSTANCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))

typedef struct _VtgPluginInstance VtgPluginInstance;
typedef struct _VtgPluginInstanceClass VtgPluginInstanceClass;

#define VTG_TYPE_PROJECT_VIEW (vtg_project_view_get_type ())
#define VTG_PROJECT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_VIEW, VtgProjectView))
#define VTG_PROJECT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_VIEW, VtgProjectViewClass))
#define VTG_IS_PROJECT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_VIEW))
#define VTG_IS_PROJECT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_VIEW))
#define VTG_PROJECT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_VIEW, VtgProjectViewClass))

typedef struct _VtgProjectView VtgProjectView;
typedef struct _VtgProjectViewClass VtgProjectViewClass;

#define VTG_TYPE_PROJECT_BUILDER (vtg_project_builder_get_type ())
#define VTG_PROJECT_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilder))
#define VTG_PROJECT_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilderClass))
#define VTG_IS_PROJECT_BUILDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_BUILDER))
#define VTG_IS_PROJECT_BUILDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_BUILDER))
#define VTG_PROJECT_BUILDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_BUILDER, VtgProjectBuilderClass))

typedef struct _VtgProjectBuilder VtgProjectBuilder;
typedef struct _VtgProjectBuilderClass VtgProjectBuilderClass;

#define VTG_TYPE_PROJECT_EXECUTER (vtg_project_executer_get_type ())
#define VTG_PROJECT_EXECUTER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_EXECUTER, VtgProjectExecuter))
#define VTG_PROJECT_EXECUTER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_EXECUTER, VtgProjectExecuterClass))
#define VTG_IS_PROJECT_EXECUTER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_EXECUTER))
#define VTG_IS_PROJECT_EXECUTER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_EXECUTER))
#define VTG_PROJECT_EXECUTER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_EXECUTER, VtgProjectExecuterClass))

typedef struct _VtgProjectExecuter VtgProjectExecuter;
typedef struct _VtgProjectExecuterClass VtgProjectExecuterClass;

#define VTG_TYPE_PROJECT_SEARCH (vtg_project_search_get_type ())
#define VTG_PROJECT_SEARCH(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_SEARCH, VtgProjectSearch))
#define VTG_PROJECT_SEARCH_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_SEARCH, VtgProjectSearchClass))
#define VTG_IS_PROJECT_SEARCH(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_SEARCH))
#define VTG_IS_PROJECT_SEARCH_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_SEARCH))
#define VTG_PROJECT_SEARCH_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_SEARCH, VtgProjectSearchClass))

typedef struct _VtgProjectSearch VtgProjectSearch;
typedef struct _VtgProjectSearchClass VtgProjectSearchClass;

#define VTG_TYPE_CHANGE_LOG (vtg_change_log_get_type ())
#define VTG_CHANGE_LOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_CHANGE_LOG, VtgChangeLog))
#define VTG_CHANGE_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_CHANGE_LOG, VtgChangeLogClass))
#define VTG_IS_CHANGE_LOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_CHANGE_LOG))
#define VTG_IS_CHANGE_LOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_CHANGE_LOG))
#define VTG_CHANGE_LOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_CHANGE_LOG, VtgChangeLogClass))

typedef struct _VtgChangeLog VtgChangeLog;
typedef struct _VtgChangeLogClass VtgChangeLogClass;

#define VTG_TYPE_SOURCE_BOOKMARKS (vtg_source_bookmarks_get_type ())
#define VTG_SOURCE_BOOKMARKS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_SOURCE_BOOKMARKS, VtgSourceBookmarks))
#define VTG_SOURCE_BOOKMARKS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_SOURCE_BOOKMARKS, VtgSourceBookmarksClass))
#define VTG_IS_SOURCE_BOOKMARKS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_SOURCE_BOOKMARKS))
#define VTG_IS_SOURCE_BOOKMARKS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_SOURCE_BOOKMARKS))
#define VTG_SOURCE_BOOKMARKS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_SOURCE_BOOKMARKS, VtgSourceBookmarksClass))

typedef struct _VtgSourceBookmarks VtgSourceBookmarks;
typedef struct _VtgSourceBookmarksClass VtgSourceBookmarksClass;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))

#define VTG_TYPE_SOURCE_BOOKMARK (vtg_source_bookmark_get_type ())
#define VTG_SOURCE_BOOKMARK(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_SOURCE_BOOKMARK, VtgSourceBookmark))
#define VTG_SOURCE_BOOKMARK_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_SOURCE_BOOKMARK, VtgSourceBookmarkClass))
#define VTG_IS_SOURCE_BOOKMARK(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_SOURCE_BOOKMARK))
#define VTG_IS_SOURCE_BOOKMARK_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_SOURCE_BOOKMARK))
#define VTG_SOURCE_BOOKMARK_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_SOURCE_BOOKMARK, VtgSourceBookmarkClass))

typedef struct _VtgSourceBookmark VtgSourceBookmark;
typedef struct _VtgSourceBookmarkClass VtgSourceBookmarkClass;
typedef struct _VtgSourceBookmarkPrivate VtgSourceBookmarkPrivate;
#define _vtg_source_bookmark_unref0(var) ((var == NULL) ? NULL : (var = (vtg_source_bookmark_unref (var), NULL)))

#define VTG_TYPE_SYMBOL_COMPLETION_HELPER (vtg_symbol_completion_helper_get_type ())
#define VTG_SYMBOL_COMPLETION_HELPER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_SYMBOL_COMPLETION_HELPER, VtgSymbolCompletionHelper))
#define VTG_SYMBOL_COMPLETION_HELPER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_SYMBOL_COMPLETION_HELPER, VtgSymbolCompletionHelperClass))
#define VTG_IS_SYMBOL_COMPLETION_HELPER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_SYMBOL_COMPLETION_HELPER))
#define VTG_IS_SYMBOL_COMPLETION_HELPER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_SYMBOL_COMPLETION_HELPER))
#define VTG_SYMBOL_COMPLETION_HELPER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_SYMBOL_COMPLETION_HELPER, VtgSymbolCompletionHelperClass))

typedef struct _VtgSymbolCompletionHelper VtgSymbolCompletionHelper;
typedef struct _VtgSymbolCompletionHelperClass VtgSymbolCompletionHelperClass;

#define VTG_TYPE_SYMBOL_COMPLETION_TRIGGER (vtg_symbol_completion_trigger_get_type ())
#define VTG_SYMBOL_COMPLETION_TRIGGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTrigger))
#define VTG_SYMBOL_COMPLETION_TRIGGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTriggerClass))
#define VTG_IS_SYMBOL_COMPLETION_TRIGGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER))
#define VTG_IS_SYMBOL_COMPLETION_TRIGGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER))
#define VTG_SYMBOL_COMPLETION_TRIGGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTriggerClass))

typedef struct _VtgSymbolCompletionTrigger VtgSymbolCompletionTrigger;
typedef struct _VtgSymbolCompletionTriggerClass VtgSymbolCompletionTriggerClass;
typedef struct _VtgPluginInstancePrivate VtgPluginInstancePrivate;

#define VTG_TYPE_PLUGIN (vtg_plugin_get_type ())
#define VTG_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN, VtgPlugin))
#define VTG_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN, VtgPluginClass))
#define VTG_IS_PLUGIN(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN))
#define VTG_IS_PLUGIN_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN))
#define VTG_PLUGIN_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN, VtgPluginClass))

typedef struct _VtgPlugin VtgPlugin;
typedef struct _VtgPluginClass VtgPluginClass;

#define VTG_TYPE_PROJECT_SEARCH_DIALOG (vtg_project_search_dialog_get_type ())
#define VTG_PROJECT_SEARCH_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_SEARCH_DIALOG, VtgProjectSearchDialog))
#define VTG_PROJECT_SEARCH_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_SEARCH_DIALOG, VtgProjectSearchDialogClass))
#define VTG_IS_PROJECT_SEARCH_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_SEARCH_DIALOG))
#define VTG_IS_PROJECT_SEARCH_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_SEARCH_DIALOG))
#define VTG_PROJECT_SEARCH_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_SEARCH_DIALOG, VtgProjectSearchDialogClass))

typedef struct _VtgProjectSearchDialog VtgProjectSearchDialog;
typedef struct _VtgProjectSearchDialogClass VtgProjectSearchDialogClass;
typedef struct _VtgProjectSearchDialogPrivate VtgProjectSearchDialogPrivate;
typedef struct _VtgProjectManagerPrivate VtgProjectManagerPrivate;

#define VTG_TYPE_VCS_TYPES (vtg_vcs_types_get_type ())

#define VTG_TYPE_FILTERED_LIST_DIALOG (vtg_filtered_list_dialog_get_type ())
#define VTG_FILTERED_LIST_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_FILTERED_LIST_DIALOG, VtgFilteredListDialog))
#define VTG_FILTERED_LIST_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_FILTERED_LIST_DIALOG, VtgFilteredListDialogClass))
#define VTG_IS_FILTERED_LIST_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_FILTERED_LIST_DIALOG))
#define VTG_IS_FILTERED_LIST_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_FILTERED_LIST_DIALOG))
#define VTG_FILTERED_LIST_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_FILTERED_LIST_DIALOG, VtgFilteredListDialogClass))

typedef struct _VtgFilteredListDialog VtgFilteredListDialog;
typedef struct _VtgFilteredListDialogClass VtgFilteredListDialogClass;
typedef struct _VtgFilteredListDialogPrivate VtgFilteredListDialogPrivate;

#define VTG_TYPE_PROJECT_DESCRIPTOR (vtg_project_descriptor_get_type ())
#define VTG_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptor))
#define VTG_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))
#define VTG_IS_PROJECT_DESCRIPTOR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_IS_PROJECT_DESCRIPTOR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_DESCRIPTOR))
#define VTG_PROJECT_DESCRIPTOR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_DESCRIPTOR, VtgProjectDescriptorClass))

typedef struct _VtgProjectDescriptor VtgProjectDescriptor;
typedef struct _VtgProjectDescriptorClass VtgProjectDescriptorClass;
typedef struct _VtgProjectDescriptorPrivate VtgProjectDescriptorPrivate;

#define VTG_INTERACTION_TYPE_PARAMETERS_DIALOG (vtg_interaction_parameters_dialog_get_type ())
#define VTG_INTERACTION_PARAMETERS_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_INTERACTION_TYPE_PARAMETERS_DIALOG, VtgInteractionParametersDialog))
#define VTG_INTERACTION_PARAMETERS_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_INTERACTION_TYPE_PARAMETERS_DIALOG, VtgInteractionParametersDialogClass))
#define VTG_INTERACTION_IS_PARAMETERS_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_INTERACTION_TYPE_PARAMETERS_DIALOG))
#define VTG_INTERACTION_IS_PARAMETERS_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_INTERACTION_TYPE_PARAMETERS_DIALOG))
#define VTG_INTERACTION_PARAMETERS_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_INTERACTION_TYPE_PARAMETERS_DIALOG, VtgInteractionParametersDialogClass))

typedef struct _VtgInteractionParametersDialog VtgInteractionParametersDialog;
typedef struct _VtgInteractionParametersDialogClass VtgInteractionParametersDialogClass;
#define _vtg_interaction_parameters_dialog_unref0(var) ((var == NULL) ? NULL : (var = (vtg_interaction_parameters_dialog_unref (var), NULL)))

#define VTG_TYPE_CONFIGURATION (vtg_configuration_get_type ())
#define VTG_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_CONFIGURATION, VtgConfiguration))
#define VTG_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_CONFIGURATION, VtgConfigurationClass))
#define VTG_IS_CONFIGURATION(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_CONFIGURATION))
#define VTG_IS_CONFIGURATION_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_CONFIGURATION))
#define VTG_CONFIGURATION_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_CONFIGURATION, VtgConfigurationClass))

typedef struct _VtgConfiguration VtgConfiguration;
typedef struct _VtgConfigurationClass VtgConfigurationClass;

#define VTG_TYPE_PROJECT_EXECUTER_DIALOG (vtg_project_executer_dialog_get_type ())
#define VTG_PROJECT_EXECUTER_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PROJECT_EXECUTER_DIALOG, VtgProjectExecuterDialog))
#define VTG_PROJECT_EXECUTER_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PROJECT_EXECUTER_DIALOG, VtgProjectExecuterDialogClass))
#define VTG_IS_PROJECT_EXECUTER_DIALOG(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PROJECT_EXECUTER_DIALOG))
#define VTG_IS_PROJECT_EXECUTER_DIALOG_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PROJECT_EXECUTER_DIALOG))
#define VTG_PROJECT_EXECUTER_DIALOG_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PROJECT_EXECUTER_DIALOG, VtgProjectExecuterDialogClass))

typedef struct _VtgProjectExecuterDialog VtgProjectExecuterDialog;
typedef struct _VtgProjectExecuterDialogClass VtgProjectExecuterDialogClass;
#define _vtg_project_executer_dialog_unref0(var) ((var == NULL) ? NULL : (var = (vtg_project_executer_dialog_unref (var), NULL)))

#define VTG_TYPE_BUILD_LOG_VIEW (vtg_build_log_view_get_type ())
#define VTG_BUILD_LOG_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogView))
#define VTG_BUILD_LOG_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogViewClass))
#define VTG_IS_BUILD_LOG_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_BUILD_LOG_VIEW))
#define VTG_IS_BUILD_LOG_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_BUILD_LOG_VIEW))
#define VTG_BUILD_LOG_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_BUILD_LOG_VIEW, VtgBuildLogViewClass))

typedef struct _VtgBuildLogView VtgBuildLogView;
typedef struct _VtgBuildLogViewClass VtgBuildLogViewClass;

#define VTG_TYPE_OUTPUT_VIEW (vtg_output_view_get_type ())
#define VTG_OUTPUT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_OUTPUT_VIEW, VtgOutputView))
#define VTG_OUTPUT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_OUTPUT_VIEW, VtgOutputViewClass))
#define VTG_IS_OUTPUT_VIEW(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_OUTPUT_VIEW))
#define VTG_IS_OUTPUT_VIEW_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_OUTPUT_VIEW))
#define VTG_OUTPUT_VIEW_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_OUTPUT_VIEW, VtgOutputViewClass))

typedef struct _VtgOutputView VtgOutputView;
typedef struct _VtgOutputViewClass VtgOutputViewClass;

#define VTG_TYPE_OUTPUT_TYPES (vtg_output_types_get_type ())
#define _g_dir_close0(var) ((var == NULL) ? NULL : (var = (g_dir_close (var), NULL)))

struct _VtgProjectManagerUi {
	GObject parent_instance;
	VtgProjectManagerUiPrivate * priv;
};

struct _VtgProjectManagerUiClass {
	GObjectClass parent_class;
};

struct _VtgProjectManagerUiPrivate {
	char* _ui_def;
	guint _ui_id;
	GeeList* _projects;
	GtkActionGroup* _actions;
	VtgPluginInstance* _plugin_instance;
	VtgProjectView* _prj_view;
	VtgProjectBuilder* _prj_builder;
	VtgProjectExecuter* _prj_executer;
	VtgProjectSearch* _prj_search;
	VtgChangeLog* _changelog;
	VtgSourceBookmarks* _bookmarks;
};

struct _VtgSourceBookmark {
	GTypeInstance parent_instance;
	volatile int ref_count;
	VtgSourceBookmarkPrivate * priv;
	char* uri;
	gint line;
	gint column;
};

struct _VtgSourceBookmarkClass {
	GTypeClass parent_class;
	void (*finalize) (VtgSourceBookmark *self);
};

struct _VtgPluginInstance {
	GObject parent_instance;
	VtgPluginInstancePrivate * priv;
	VtgPlugin* plugin;
};

struct _VtgPluginInstanceClass {
	GObjectClass parent_class;
};

struct _VtgProjectSearchDialog {
	GObject parent_instance;
	VtgProjectSearchDialogPrivate * priv;
	char* search_text;
	gboolean match_case;
};

struct _VtgProjectSearchDialogClass {
	GObjectClass parent_class;
};

typedef enum  {
	VTG_VCS_TYPES_NONE,
	VTG_VCS_TYPES_GIT,
	VTG_VCS_TYPES_BZR,
	VTG_VCS_TYPES_SVN
} VtgVcsTypes;

struct _VtgProjectManager {
	GObject parent_instance;
	VtgProjectManagerPrivate * priv;
	char* filename;
	GeeList* exec_targets;
	GeeList* all_vala_sources;
	VtgVcsTypes vcs_type;
	char* changelog_uri;
};

struct _VtgProjectManagerClass {
	GObjectClass parent_class;
};

struct _VtgFilteredListDialog {
	GObject parent_instance;
	VtgFilteredListDialogPrivate * priv;
	GtkTreeIter selected_iter;
};

struct _VtgFilteredListDialogClass {
	GObjectClass parent_class;
};

struct _VtgProjectDescriptor {
	GObject parent_instance;
	VtgProjectDescriptorPrivate * priv;
	VscSymbolCompletion* completion;
	VtgProjectManager* project;
};

struct _VtgProjectDescriptorClass {
	GObjectClass parent_class;
};

typedef enum  {
	VTG_OUTPUT_TYPES_MESSAGE,
	VTG_OUTPUT_TYPES_ERROR,
	VTG_OUTPUT_TYPES_CHILD_PROCESS,
	VTG_OUTPUT_TYPES_BUILD,
	VTG_OUTPUT_TYPES_SEARCH
} VtgOutputTypes;


static gpointer vtg_project_manager_ui_parent_class = NULL;
static GType vtg_project_manager_ui_type_id = 0;

GType vtg_project_manager_ui_get_type (void);
GType vtg_project_manager_ui_register_type (GTypeModule * module);
GType vtg_project_manager_get_type (void);
GType vtg_project_manager_register_type (GTypeModule * module);
GType vtg_plugin_instance_get_type (void);
GType vtg_plugin_instance_register_type (GTypeModule * module);
GType vtg_project_view_get_type (void);
GType vtg_project_view_register_type (GTypeModule * module);
GType vtg_project_builder_get_type (void);
GType vtg_project_builder_register_type (GTypeModule * module);
GType vtg_project_executer_get_type (void);
GType vtg_project_executer_register_type (GTypeModule * module);
GType vtg_project_search_get_type (void);
GType vtg_project_search_register_type (GTypeModule * module);
GType vtg_change_log_get_type (void);
GType vtg_change_log_register_type (GTypeModule * module);
GType vtg_source_bookmarks_get_type (void);
GType vtg_source_bookmarks_register_type (GTypeModule * module);
#define VTG_PROJECT_MANAGER_UI_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_PROJECT_MANAGER_UI, VtgProjectManagerUiPrivate))
enum  {
	VTG_PROJECT_MANAGER_UI_DUMMY_PROPERTY,
	VTG_PROJECT_MANAGER_UI_PLUGIN_INSTANCE,
	VTG_PROJECT_MANAGER_UI_PROJECT_VIEW
};
static void vtg_project_manager_ui_on_project_new (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_new_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_open (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_open_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_close (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_close_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_build (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_build_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_clean (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_clean_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_clean_stamps (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_clean_stamps_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_configure (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_configure_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_standalone_file_compile (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_standalone_file_compile_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_error_next (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_error_next_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_error_previuos (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_error_previuos_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_execute_process (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_execute_process_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_kill_process (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_kill_process_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_search (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_search_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_search_result_next (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_search_result_next_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_search_result_previous (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_search_result_previous_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_goto_document (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_goto_document_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_goto_next_position (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_goto_next_position_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_goto_prev_position (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_goto_prev_position_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_goto_method (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_goto_method_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_project_goto_definition (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_project_goto_definition_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_complete_word (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_complete_word_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_prepare_changelog (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_prepare_changelog_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_on_prepare_single_file_changelog (VtgProjectManagerUi* self, GtkAction* action);
static void _vtg_project_manager_ui_on_prepare_single_file_changelog_gtk_action_callback (GtkAction* action, gpointer self);
static void vtg_project_manager_ui_set_plugin_instance (VtgProjectManagerUi* self, VtgPluginInstance* value);
VtgProjectManagerUi* vtg_project_manager_ui_new (VtgPluginInstance* plugin_instance);
VtgProjectManagerUi* vtg_project_manager_ui_construct (GType object_type, VtgPluginInstance* plugin_instance);
static void vtg_project_manager_ui_on_project_open_recent (VtgProjectManagerUi* self, GtkRecentChooser* sender);
static void _vtg_project_manager_ui_on_project_open_recent_gtk_recent_chooser_item_activated (GtkRecentChooser* _sender, gpointer self);
GeditWindow* vtg_plugin_instance_get_window (VtgPluginInstance* self);
static void vtg_project_manager_ui_initialize_ui (VtgProjectManagerUi* self);
static void vtg_project_manager_ui_open_project (VtgProjectManagerUi* self, const char* name);
gpointer vtg_source_bookmark_ref (gpointer instance);
void vtg_source_bookmark_unref (gpointer instance);
GParamSpec* vtg_param_spec_source_bookmark (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_value_set_source_bookmark (GValue* value, gpointer v_object);
gpointer vtg_value_get_source_bookmark (const GValue* value);
GType vtg_source_bookmark_get_type (void);
GType vtg_source_bookmark_register_type (GTypeModule * module);
VtgSourceBookmark* vtg_source_bookmarks_get_current_bookmark (VtgSourceBookmarks* self);
GeditTab* vtg_plugin_instance_activate_uri (VtgPluginInstance* self, const char* uri, gint line, gint col);
static void vtg_project_manager_ui_on_current_bookmark_changed (VtgProjectManagerUi* self, VtgSourceBookmarks* sender);
gboolean vtg_change_log_prepare (VtgChangeLog* self, const char* file, GError** error);
void vtg_interaction_error_message (const char* message, GError* err);
VtgProjectManager* vtg_project_view_get_current_project (VtgProjectView* self);
VbfSource* vtg_project_manager_get_source_file_from_uri (VtgProjectManager* self, const char* uri);
GType vtg_symbol_completion_helper_get_type (void);
GType vtg_symbol_completion_helper_register_type (GTypeModule * module);
VtgSymbolCompletionHelper* vtg_plugin_instance_scs_find_from_view (VtgPluginInstance* self, GeditView* view);
GType vtg_symbol_completion_trigger_get_type (void);
GType vtg_symbol_completion_trigger_register_type (GTypeModule * module);
VtgSymbolCompletionTrigger* vtg_symbol_completion_helper_get_trigger (VtgSymbolCompletionHelper* self);
void vtg_symbol_completion_trigger_complete_word (VtgSymbolCompletionTrigger* self);
void vtg_symbol_completion_helper_goto_definition (VtgSymbolCompletionHelper* self);
GType vtg_plugin_get_type (void);
GType vtg_plugin_register_type (GTypeModule * module);
gboolean vtg_plugin_project_need_save (VtgPlugin* self, VtgProjectManager* project);
void vtg_plugin_project_save_all (VtgPlugin* self, VtgProjectManager* project);
void vtg_project_manager_ui_close_project (VtgProjectManagerUi* self, VtgProjectManager* project);
static void vtg_project_manager_ui_create_project (VtgProjectManagerUi* self, const char* project_path);
void vtg_project_search_next_match (VtgProjectSearch* self);
void vtg_project_search_previous_match (VtgProjectSearch* self);
VtgProjectSearchDialog* vtg_project_search_dialog_new (GtkWindow* parent);
VtgProjectSearchDialog* vtg_project_search_dialog_construct (GType object_type, GtkWindow* parent);
GType vtg_project_search_dialog_get_type (void);
GType vtg_project_search_dialog_register_type (GTypeModule * module);
gint vtg_project_search_dialog_run (VtgProjectSearchDialog* self);
gboolean vtg_project_search_search (VtgProjectSearch* self, VtgProjectManager* project_manager, const char* text, gboolean match_case);
void vtg_source_bookmarks_move_next (VtgSourceBookmarks* self);
void vtg_source_bookmarks_move_previous (VtgSourceBookmarks* self);
GType vtg_vcs_types_get_type (void);
VtgFilteredListDialog* vtg_filtered_list_dialog_new (GtkListStore* model);
VtgFilteredListDialog* vtg_filtered_list_dialog_construct (GType object_type, GtkListStore* model);
GType vtg_filtered_list_dialog_get_type (void);
GType vtg_filtered_list_dialog_register_type (GTypeModule * module);
void vtg_filtered_list_dialog_set_transient_for (VtgFilteredListDialog* self, GtkWindow* parent);
gboolean vtg_filtered_list_dialog_run (VtgFilteredListDialog* self);
GType vtg_project_descriptor_get_type (void);
GType vtg_project_descriptor_register_type (GTypeModule * module);
static VtgProjectDescriptor* vtg_project_manager_ui_get_projectdescriptor_for_project (VtgProjectManagerUi* self, VtgProjectManager* project);
GeeList* vtg_plugin_get_projects (VtgPlugin* self);
gboolean vtg_project_manager_contains_vala_source_file (VtgProjectManager* self, const char* uri);
GtkListStore* vtg_caches_get_compile_cache (void);
VtgInteractionParametersDialog* vtg_interaction_parameters_dialog_new (const char* title, GtkWindow* parent, GtkListStore* completions);
VtgInteractionParametersDialog* vtg_interaction_parameters_dialog_construct (GType object_type, const char* title, GtkWindow* parent, GtkListStore* completions);
gpointer vtg_interaction_parameters_dialog_ref (gpointer instance);
void vtg_interaction_parameters_dialog_unref (gpointer instance);
GParamSpec* vtg_interaction_param_spec_parameters_dialog (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_interaction_value_set_parameters_dialog (GValue* value, gpointer v_object);
gpointer vtg_interaction_value_get_parameters_dialog (const GValue* value);
GType vtg_interaction_parameters_dialog_get_type (void);
GType vtg_interaction_parameters_dialog_register_type (GTypeModule * module);
gint vtg_interaction_parameters_dialog_run (VtgInteractionParametersDialog* self);
const char* vtg_interaction_parameters_dialog_get_parameters (VtgInteractionParametersDialog* self);
gboolean vtg_caches_cache_contains (GtkListStore* cache, const char* data);
void vtg_caches_cache_add (GtkListStore* cache, const char* data);
GType vtg_configuration_get_type (void);
GType vtg_configuration_register_type (GTypeModule * module);
VtgConfiguration* vtg_plugin_get_config (VtgPlugin* self);
gboolean vtg_configuration_get_save_before_build (VtgConfiguration* self);
gboolean vtg_project_builder_compile_file (VtgProjectBuilder* self, const char* filename, const char* params);
GtkListStore* vtg_caches_get_build_cache (void);
gboolean vtg_project_builder_get_is_building (VtgProjectBuilder* self);
void vtg_project_builder_stop_build (VtgProjectBuilder* self);
gboolean vtg_project_builder_build (VtgProjectBuilder* self, VtgProjectManager* project_manager, const char* params);
GtkListStore* vtg_caches_get_configure_cache (void);
gboolean vtg_project_builder_configure (VtgProjectBuilder* self, VtgProjectManager* project_manager, const char* params);
static void vtg_project_manager_ui_clean_project (VtgProjectManagerUi* self, gboolean stamps);
VtgProjectExecuterDialog* vtg_project_executer_dialog_new (GtkWindow* parent, VtgProjectManager* project);
VtgProjectExecuterDialog* vtg_project_executer_dialog_construct (GType object_type, GtkWindow* parent, VtgProjectManager* project);
gpointer vtg_project_executer_dialog_ref (gpointer instance);
void vtg_project_executer_dialog_unref (gpointer instance);
GParamSpec* vtg_param_spec_project_executer_dialog (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_value_set_project_executer_dialog (GValue* value, gpointer v_object);
gpointer vtg_value_get_project_executer_dialog (const GValue* value);
GType vtg_project_executer_dialog_get_type (void);
GType vtg_project_executer_dialog_register_type (GTypeModule * module);
gint vtg_project_executer_dialog_run (VtgProjectExecuterDialog* self);
const char* vtg_project_executer_dialog_get_command_line (VtgProjectExecuterDialog* self);
gboolean vtg_project_executer_execute (VtgProjectExecuter* self, VbfProject* project, const char* command_line);
VbfProject* vtg_project_manager_get_project (VtgProjectManager* self);
void vtg_project_executer_kill_last (VtgProjectExecuter* self);
gboolean vtg_project_builder_clean (VtgProjectBuilder* self, VtgProjectManager* project_manager, gboolean vala_stamp);
void vtg_project_builder_next_error (VtgProjectBuilder* self);
void vtg_project_builder_previous_error (VtgProjectBuilder* self);
static void vtg_project_manager_ui_update_ui (VtgProjectManagerUi* self, gboolean default_project);
static void vtg_project_manager_ui_on_current_project_changed (VtgProjectManagerUi* self, GObject* sender, GParamSpec* pspec);
GType vtg_build_log_view_get_type (void);
GType vtg_build_log_view_register_type (GTypeModule * module);
VtgBuildLogView* vtg_project_builder_get_error_pane (VtgProjectBuilder* self);
gint vtg_build_log_view_get_error_count (VtgBuildLogView* self);
gboolean vtg_project_executer_get_is_executing (VtgProjectExecuter* self);
gboolean vtg_project_search_get_is_searching (VtgProjectSearch* self);
gboolean vtg_source_bookmarks_get_is_empty (VtgSourceBookmarks* self);
VtgProjectManager* vtg_project_manager_ui_find_project_for_id (VtgProjectManagerUi* self, const char* id);
void vtg_project_view_set_current_project (VtgProjectView* self, VtgProjectManager* value);
VtgProjectManager* vtg_project_manager_new (void);
VtgProjectManager* vtg_project_manager_construct (GType object_type);
gboolean vtg_project_manager_open (VtgProjectManager* self, const char* project_filename, GError** error);
void vtg_plugin_on_project_loaded (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project_manager);
void vtg_project_view_add_project (VtgProjectView* self, VbfProject* project);
void vtg_project_view_remove_project (VtgProjectView* self, VbfProject* project);
void vtg_plugin_on_project_closed (VtgPlugin* self, VtgProjectManagerUi* sender, VtgProjectManager* project);
void vtg_project_manager_close (VtgProjectManager* self);
GType vtg_output_view_get_type (void);
GType vtg_output_view_register_type (GTypeModule * module);
VtgOutputView* vtg_plugin_instance_get_output_view (VtgPluginInstance* self);
static gboolean vtg_project_manager_ui_is_dir_empty (VtgProjectManagerUi* self, const char* dir_path);
GType vtg_output_types_get_type (void);
void vtg_output_view_log_message (VtgOutputView* self, VtgOutputTypes output_type, const char* message);
static void vtg_project_manager_ui_on_child_watch (VtgProjectManagerUi* self, GPid pid, gint status);
static void _vtg_project_manager_ui_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self);
void vtg_output_view_start_watch (VtgOutputView* self, VtgOutputTypes output_type, guint id, gint stdo, gint stde, gint stdi);
void vtg_output_view_activate (VtgOutputView* self);
void vtg_output_view_stop_watch (VtgOutputView* self, guint id);
VtgPluginInstance* vtg_project_manager_ui_get_plugin_instance (VtgProjectManagerUi* self);
VtgProjectView* vtg_project_manager_ui_get_project_view (VtgProjectManagerUi* self);
VtgProjectView* vtg_project_view_new (VtgPluginInstance* plugin_instance);
VtgProjectView* vtg_project_view_construct (GType object_type, VtgPluginInstance* plugin_instance);
static void _vtg_project_manager_ui_on_current_project_changed_g_object_notify (VtgProjectView* _sender, GParamSpec* pspec, gpointer self);
VtgProjectBuilder* vtg_project_builder_new (VtgPluginInstance* plugin_instance);
VtgProjectBuilder* vtg_project_builder_construct (GType object_type, VtgPluginInstance* plugin_instance);
VtgProjectExecuter* vtg_project_executer_new (VtgPluginInstance* plugin_instance);
VtgProjectExecuter* vtg_project_executer_construct (GType object_type, VtgPluginInstance* plugin_instance);
VtgProjectSearch* vtg_project_search_new (VtgPluginInstance* plugin_instance);
VtgProjectSearch* vtg_project_search_construct (GType object_type, VtgPluginInstance* plugin_instance);
static void _lambda0_ (VtgProjectExecuter* sender, VtgProjectManagerUi* self);
static void __lambda0__vtg_project_executer_process_start (VtgProjectExecuter* _sender, gpointer self);
static void _lambda1_ (VtgProjectExecuter* sender, gint exit_status, VtgProjectManagerUi* self);
static void __lambda1__vtg_project_executer_process_exit (VtgProjectExecuter* _sender, gint exit_status, gpointer self);
static void _lambda2_ (VtgProjectBuilder* sender, VtgProjectManagerUi* self);
static void __lambda2__vtg_project_builder_build_start (VtgProjectBuilder* _sender, gpointer self);
static void _lambda3_ (VtgProjectBuilder* sender, gint exit_status, VtgProjectManagerUi* self);
static void __lambda3__vtg_project_builder_build_exit (VtgProjectBuilder* _sender, gint exit_status, gpointer self);
static void _lambda4_ (VtgProjectSearch* sender, VtgProjectManagerUi* self);
static void __lambda4__vtg_project_search_search_start (VtgProjectSearch* _sender, gpointer self);
static void _lambda5_ (VtgProjectSearch* sender, gint exit_status, VtgProjectManagerUi* self);
static void __lambda5__vtg_project_search_search_exit (VtgProjectSearch* _sender, gint exit_status, gpointer self);
VtgChangeLog* vtg_change_log_new (VtgPluginInstance* plugin_instance);
VtgChangeLog* vtg_change_log_construct (GType object_type, VtgPluginInstance* plugin_instance);
VtgSourceBookmarks* vtg_source_bookmarks_new (VtgPluginInstance* plugin_instance);
VtgSourceBookmarks* vtg_source_bookmarks_construct (GType object_type, VtgPluginInstance* plugin_instance);
static void _vtg_project_manager_ui_on_current_bookmark_changed_vtg_source_bookmarks_current_bookmark_changed (VtgSourceBookmarks* _sender, gpointer self);
static GObject * vtg_project_manager_ui_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void vtg_project_manager_ui_finalize (GObject* obj);
static void vtg_project_manager_ui_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec);
static void vtg_project_manager_ui_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static int _vala_strcmp0 (const char * str1, const char * str2);

static const GtkActionEntry VTG_PROJECT_MANAGER_UI__action_entries[] = {{"ProjectNew", NULL, N_ ("_New Project..."), NULL, N_ ("Create a new project"), (GCallback) _vtg_project_manager_ui_on_project_new_gtk_action_callback}, {"ProjectOpen", NULL, N_ ("Op_en Project..."), "<control><alt>O", N_ ("Open an existing project"), (GCallback) _vtg_project_manager_ui_on_project_open_gtk_action_callback}, {"ProjectClose", NULL, N_ ("_Close Current Project"), NULL, N_ ("Close current selected project"), (GCallback) _vtg_project_manager_ui_on_project_close_gtk_action_callback}, {"ProjectBuildMenuAction", NULL, N_ ("Build"), NULL, N_ ("Build menu"), (GCallback) NULL}, {"ProjectBuild", GTK_STOCK_EXECUTE, N_ ("_Build Project"), "<control><shift>B", N_ ("Build the current project using 'make'"), (GCallback) _vtg_project_manager_ui_on_project_build_gtk_action_callback}, {"ProjectBuildClean", GTK_STOCK_CLEAR, N_ ("_Clean Project"), NULL, N_ ("Clean the current project using 'make clean'"), (GCallback) _vtg_project_manager_ui_on_project_clean_gtk_action_callback}, {"ProjectBuildCleanStamps", NULL, N_ ("_Clean Project and Vala 'Stamp' Files"), NULL, N_ ("Clean the current project stamp files"), (GCallback) _vtg_project_manager_ui_on_project_clean_stamps_gtk_action_callback}, {"ProjectBuildConfigure", NULL, N_ ("C_onfigure Project"), NULL, N_ ("Configure or reconfigure the current project"), (GCallback) _vtg_project_manager_ui_on_project_configure_gtk_action_callback}, {"ProjectBuildCompileFile", NULL, N_ ("_Compile File"), "<control>B", N_ ("Compile the current file with the vala compiler"), (GCallback) _vtg_project_manager_ui_on_standalone_file_compile_gtk_action_callback}, {"ProjectBuildNextError", GTK_STOCK_GO_FORWARD, N_ ("_Next Error"), "<control><shift>F12", N_ ("Go to next error source line"), (GCallback) _vtg_project_manager_ui_on_project_error_next_gtk_action_callback}, {"ProjectBuildPreviousError", GTK_STOCK_GO_BACK, N_ ("_Previuos Error"), NULL, N_ ("Go to previous error source line"), (GCallback) _vtg_project_manager_ui_on_project_error_previuos_gtk_action_callback}, {"ProjectBuildExecute", GTK_STOCK_EXECUTE, N_ ("_Execute"), "F5", N_ ("Excute target program"), (GCallback) _vtg_project_manager_ui_on_project_execute_process_gtk_action_callback}, {"ProjectBuildKill", GTK_STOCK_STOP, N_ ("_Stop process"), NULL, N_ ("Stop (kill) executing program"), (GCallback) _vtg_project_manager_ui_on_project_kill_process_gtk_action_callback}, {"ProjectSearch", GTK_STOCK_FIND, N_ ("Find In _Project..."), "<control><shift>F", N_ ("Search for text in all the project files"), (GCallback) _vtg_project_manager_ui_on_project_search_gtk_action_callback}, {"ProjectSearchNextResult", NULL, N_ ("Find N_ext In Project"), NULL, N_ ("Search forward for the same text in all the project files"), (GCallback) _vtg_project_manager_ui_on_project_search_result_next_gtk_action_callback}, {"ProjectSearchPrevResult", NULL, N_ ("Find Previ_ous In Project"), NULL, N_ ("Search backward for the same text in all the project files"), (GCallback) _vtg_project_manager_ui_on_project_search_result_previous_gtk_action_callback}, {"ProjectGotoDocument", GTK_STOCK_JUMP_TO, N_ ("_Go To Document..."), "<control>J", N_ ("Open a document that belong to this project"), (GCallback) _vtg_project_manager_ui_on_project_goto_document_gtk_action_callback}, {"ProjectGotoNextPosition", GTK_STOCK_GO_FORWARD, N_ ("_Go To Next Source Position"), NULL, N_ ("Go to the next source position"), (GCallback) _vtg_project_manager_ui_on_project_goto_next_position_gtk_action_callback}, {"ProjectGotoPrevPosition", GTK_STOCK_GO_BACK, N_ ("_Go To Previous Source Position"), "<alt>Left", N_ ("Go to the previous or last saved source position"), (GCallback) _vtg_project_manager_ui_on_project_goto_prev_position_gtk_action_callback}, {"ProjectGotoMethod", NULL, N_ ("_Go To Method..."), "<control>M", N_ ("Goto to a specific method in the current source document"), (GCallback) _vtg_project_manager_ui_on_project_goto_method_gtk_action_callback}, {"ProjectGotoDefinition", NULL, N_ ("_Go To Definition"), "F12", N_ ("Goto to a current symbol definition"), (GCallback) _vtg_project_manager_ui_on_project_goto_definition_gtk_action_callback}, {"ProjectCompleteWord", NULL, N_ ("Complete _Word"), "<control>space", N_ ("Try to complete the word in the current source document"), (GCallback) _vtg_project_manager_ui_on_complete_word_gtk_action_callback}, {"ProjectPrepareChangeLog", NULL, N_ ("_Prepare ChangeLog"), NULL, N_ ("Add an entry to the ChangeLog with all added/modified files"), (GCallback) _vtg_project_manager_ui_on_prepare_changelog_gtk_action_callback}, {"ProjectPrepareSingleFileChangeLog", NULL, N_ ("_Add Current File To ChangeLog"), NULL, N_ ("Add the current file to the ChangeLog"), (GCallback) _vtg_project_manager_ui_on_prepare_single_file_changelog_gtk_action_callback}};


static void _vtg_project_manager_ui_on_project_new_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_new (self, action);
}


static void _vtg_project_manager_ui_on_project_open_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_open (self, action);
}


static void _vtg_project_manager_ui_on_project_close_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_close (self, action);
}


static void _vtg_project_manager_ui_on_project_build_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_build (self, action);
}


static void _vtg_project_manager_ui_on_project_clean_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_clean (self, action);
}


static void _vtg_project_manager_ui_on_project_clean_stamps_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_clean_stamps (self, action);
}


static void _vtg_project_manager_ui_on_project_configure_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_configure (self, action);
}


static void _vtg_project_manager_ui_on_standalone_file_compile_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_standalone_file_compile (self, action);
}


static void _vtg_project_manager_ui_on_project_error_next_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_error_next (self, action);
}


static void _vtg_project_manager_ui_on_project_error_previuos_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_error_previuos (self, action);
}


static void _vtg_project_manager_ui_on_project_execute_process_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_execute_process (self, action);
}


static void _vtg_project_manager_ui_on_project_kill_process_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_kill_process (self, action);
}


static void _vtg_project_manager_ui_on_project_search_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_search (self, action);
}


static void _vtg_project_manager_ui_on_project_search_result_next_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_search_result_next (self, action);
}


static void _vtg_project_manager_ui_on_project_search_result_previous_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_search_result_previous (self, action);
}


static void _vtg_project_manager_ui_on_project_goto_document_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_goto_document (self, action);
}


static void _vtg_project_manager_ui_on_project_goto_next_position_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_goto_next_position (self, action);
}


static void _vtg_project_manager_ui_on_project_goto_prev_position_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_goto_prev_position (self, action);
}


static void _vtg_project_manager_ui_on_project_goto_method_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_goto_method (self, action);
}


static void _vtg_project_manager_ui_on_project_goto_definition_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_project_goto_definition (self, action);
}


static void _vtg_project_manager_ui_on_complete_word_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_complete_word (self, action);
}


static void _vtg_project_manager_ui_on_prepare_changelog_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_prepare_changelog (self, action);
}


static void _vtg_project_manager_ui_on_prepare_single_file_changelog_gtk_action_callback (GtkAction* action, gpointer self) {
	vtg_project_manager_ui_on_prepare_single_file_changelog (self, action);
}


VtgProjectManagerUi* vtg_project_manager_ui_construct (GType object_type, VtgPluginInstance* plugin_instance) {
	GParameter * __params;
	GParameter * __params_it;
	VtgProjectManagerUi * self;
	g_return_val_if_fail (plugin_instance != NULL, NULL);
	__params = g_new0 (GParameter, 1);
	__params_it = __params;
	__params_it->name = "plugin-instance";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN_INSTANCE);
	g_value_set_object (&__params_it->value, plugin_instance);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgProjectManagerUi* vtg_project_manager_ui_new (VtgPluginInstance* plugin_instance) {
	return vtg_project_manager_ui_construct (VTG_TYPE_PROJECT_MANAGER_UI, plugin_instance);
}


static void _vtg_project_manager_ui_on_project_open_recent_gtk_recent_chooser_item_activated (GtkRecentChooser* _sender, gpointer self) {
	vtg_project_manager_ui_on_project_open_recent (self, _sender);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static void vtg_project_manager_ui_initialize_ui (VtgProjectManagerUi* self) {
	GError * _inner_error_;
	GtkActionGroup* _tmp0_;
	GtkRecentAction* recent_action;
	GtkRecentFilter* recent_filter;
	GtkUIManager* manager;
	g_return_if_fail (self != NULL);
	_inner_error_ = NULL;
	self->priv->_actions = (_tmp0_ = gtk_action_group_new ("ProjectManagerActionGroup"), _g_object_unref0 (self->priv->_actions), _tmp0_);
	gtk_action_group_set_translation_domain (self->priv->_actions, GETTEXT_PACKAGE);
	gtk_action_group_add_actions (self->priv->_actions, VTG_PROJECT_MANAGER_UI__action_entries, G_N_ELEMENTS (VTG_PROJECT_MANAGER_UI__action_entries), self);
	recent_action = (GtkRecentAction*) gtk_recent_action_new ("ProjectRecent", "Open Recent Project", "", "");
	gtk_recent_chooser_set_show_private ((GtkRecentChooser*) recent_action, TRUE);
	recent_filter = g_object_ref_sink (gtk_recent_filter_new ());
	gtk_recent_filter_add_application (recent_filter, "vtg");
	gtk_recent_chooser_add_filter ((GtkRecentChooser*) recent_action, recent_filter);
	g_signal_connect_object ((GtkRecentChooser*) recent_action, "item-activated", (GCallback) _vtg_project_manager_ui_on_project_open_recent_gtk_recent_chooser_item_activated, self, 0);
	gtk_action_group_add_action (self->priv->_actions, (GtkAction*) recent_action);
	manager = _g_object_ref0 (gedit_window_get_ui_manager (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	gtk_ui_manager_insert_action_group (manager, self->priv->_actions, -1);
	{
		guint _tmp1_;
		_tmp1_ = gtk_ui_manager_add_ui_from_string (manager, self->priv->_ui_def, (gssize) (-1), &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch3_g_error;
			goto __finally3;
		}
		self->priv->_ui_id = _tmp1_;
	}
	goto __finally3;
	__catch3_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("vtgprojectmanagerui.vala:245: Error %s", err->message);
			_g_error_free0 (err);
		}
	}
	__finally3:
	if (_inner_error_ != NULL) {
		_g_object_unref0 (recent_action);
		_g_object_unref0 (recent_filter);
		_g_object_unref0 (manager);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	_g_object_unref0 (recent_action);
	_g_object_unref0 (recent_filter);
	_g_object_unref0 (manager);
}


static char* string_replace (const char* self, const char* old, const char* replacement) {
	char* result;
	GError * _inner_error_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _g_free0 (_tmp0_), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch4_g_regex_error;
			}
			goto __finally4;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			_g_regex_unref0 (regex);
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch4_g_regex_error;
			}
			goto __finally4;
		}
		result = _tmp2_;
		_g_regex_unref0 (regex);
		return result;
	}
	goto __finally4;
	__catch4_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_assert_not_reached ();
			_g_error_free0 (e);
		}
	}
	__finally4:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


static void vtg_project_manager_ui_on_project_open_recent (VtgProjectManagerUi* self, GtkRecentChooser* sender) {
	GError * _inner_error_;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* project_name;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	_inner_error_ = NULL;
	_tmp0_ = g_filename_from_uri (gtk_recent_chooser_get_current_uri (sender), NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	project_name = (_tmp2_ = string_replace (_tmp1_ = _tmp0_, "/configure.ac", ""), _g_free0 (_tmp1_), _tmp2_);
	vtg_project_manager_ui_open_project (self, project_name);
	_g_free0 (project_name);
}


static void vtg_project_manager_ui_on_current_bookmark_changed (VtgProjectManagerUi* self, VtgSourceBookmarks* sender) {
	VtgSourceBookmark* book;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	book = vtg_source_bookmarks_get_current_bookmark (sender);
	if (book != NULL) {
		GeditTab* _tmp0_;
		_tmp0_ = vtg_plugin_instance_activate_uri (self->priv->_plugin_instance, book->uri, book->line, book->column);
		_g_object_unref0 (_tmp0_);
	}
	_vtg_source_bookmark_unref0 (book);
}


static void vtg_project_manager_ui_on_prepare_changelog (VtgProjectManagerUi* self, GtkAction* action) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_inner_error_ = NULL;
	{
		vtg_change_log_prepare (self->priv->_changelog, NULL, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch5_g_error;
			goto __finally5;
		}
	}
	goto __finally5;
	__catch5_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			vtg_interaction_error_message (_ ("Can't prepare the ChangeLog entry"), err);
			_g_error_free0 (err);
		}
	}
	__finally5:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


static void vtg_project_manager_ui_on_prepare_single_file_changelog (VtgProjectManagerUi* self, GtkAction* action) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_inner_error_ = NULL;
	{
		GeditDocument* doc;
		doc = _g_object_ref0 (gedit_window_get_active_document (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
		if (doc != NULL) {
			VtgProjectManager* prj;
			char* uri;
			char* file;
			prj = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
			uri = g_strdup (gedit_document_get_uri (doc));
			file = g_strdup (gedit_document_get_short_name_for_display (doc));
			if (prj != NULL) {
				VbfSource* src;
				src = vtg_project_manager_get_source_file_from_uri (prj, uri);
				if (src != NULL) {
					char* _tmp0_;
					file = (_tmp0_ = g_strdup (((VbfFile*) src)->name), _g_free0 (file), _tmp0_);
				}
				_g_object_unref0 (src);
			}
			vtg_change_log_prepare (self->priv->_changelog, file, &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (prj);
				_g_free0 (uri);
				_g_free0 (file);
				_g_object_unref0 (doc);
				goto __catch6_g_error;
				goto __finally6;
			}
			_g_object_unref0 (prj);
			_g_free0 (uri);
			_g_free0 (file);
		}
		_g_object_unref0 (doc);
	}
	goto __finally6;
	__catch6_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			vtg_interaction_error_message (_ ("Can't prepare the ChangeLog entry"), err);
			_g_error_free0 (err);
		}
	}
	__finally6:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


static void vtg_project_manager_ui_on_complete_word (VtgProjectManagerUi* self, GtkAction* action) {
	VtgProjectManager* project;
	GeditView* view;
	VtgSymbolCompletionHelper* sch;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
	g_return_if_fail (project != NULL);
	view = _g_object_ref0 (gedit_window_get_active_view (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	if (view == NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (view);
		return;
	}
	sch = vtg_plugin_instance_scs_find_from_view (self->priv->_plugin_instance, view);
	if (sch == NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (view);
		_g_object_unref0 (sch);
		return;
	}
	vtg_symbol_completion_trigger_complete_word (vtg_symbol_completion_helper_get_trigger (sch));
	_g_object_unref0 (project);
	_g_object_unref0 (view);
	_g_object_unref0 (sch);
}


static void vtg_project_manager_ui_on_project_goto_definition (VtgProjectManagerUi* self, GtkAction* action) {
	VtgProjectManager* project;
	GeditView* view;
	VtgSymbolCompletionHelper* sch;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
	g_return_if_fail (project != NULL);
	view = _g_object_ref0 (gedit_window_get_active_view (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	if (view == NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (view);
		return;
	}
	sch = vtg_plugin_instance_scs_find_from_view (self->priv->_plugin_instance, view);
	if (sch == NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (view);
		_g_object_unref0 (sch);
		return;
	}
	vtg_symbol_completion_helper_goto_definition (sch);
	_g_object_unref0 (project);
	_g_object_unref0 (view);
	_g_object_unref0 (sch);
}


static void vtg_project_manager_ui_on_project_open (VtgProjectManagerUi* self, GtkAction* action) {
	GtkFileChooserDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Open Project"), (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_OPEN, GTK_RESPONSE_ACCEPT, NULL, NULL));
	if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
		char* foldername;
		gtk_widget_hide_all ((GtkWidget*) dialog);
		foldername = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog);
		vtg_project_manager_ui_open_project (self, foldername);
		_g_free0 (foldername);
	}
	gtk_object_destroy ((GtkObject*) dialog);
	_g_object_unref0 (dialog);
}


static void vtg_project_manager_ui_on_project_close (VtgProjectManagerUi* self, GtkAction* action) {
	VtgProjectManager* project;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
	g_return_if_fail (project != NULL);
	if (vtg_plugin_project_need_save (self->priv->_plugin_instance->plugin, project)) {
		GtkMessageDialog* dialog;
		gint response;
		dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_QUESTION, GTK_BUTTONS_NONE, _ ("Project files need to be saved")));
		gtk_dialog_add_buttons ((GtkDialog*) dialog, GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL);
		response = gtk_dialog_run ((GtkDialog*) dialog);
		gtk_object_destroy ((GtkObject*) dialog);
		if (response == GTK_RESPONSE_CANCEL) {
			_g_object_unref0 (dialog);
			_g_object_unref0 (project);
			return;
		} else {
			if (response == GTK_RESPONSE_ACCEPT) {
				vtg_plugin_project_save_all (self->priv->_plugin_instance->plugin, project);
			}
		}
		_g_object_unref0 (dialog);
	}
	vtg_project_manager_ui_close_project (self, project);
	_g_object_unref0 (project);
}


static void vtg_project_manager_ui_on_project_new (VtgProjectManagerUi* self, GtkAction* action) {
	GtkFileChooserDialog* dialog;
	char* foldername;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	dialog = g_object_ref_sink ((GtkFileChooserDialog*) gtk_file_chooser_dialog_new (_ ("Save Project"), (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), GTK_FILE_CHOOSER_ACTION_SELECT_FOLDER, GTK_STOCK_CANCEL, GTK_RESPONSE_CANCEL, GTK_STOCK_SAVE, GTK_RESPONSE_ACCEPT, NULL, NULL));
	foldername = NULL;
	if (gtk_dialog_run ((GtkDialog*) dialog) == GTK_RESPONSE_ACCEPT) {
		char* _tmp0_;
		foldername = (_tmp0_ = gtk_file_chooser_get_filename ((GtkFileChooser*) dialog), _g_free0 (foldername), _tmp0_);
	}
	gtk_object_destroy ((GtkObject*) dialog);
	while (TRUE) {
		if (!g_main_context_pending (g_main_context_default ())) {
			break;
		}
		g_main_context_iteration (g_main_context_default (), FALSE);
	}
	if (foldername != NULL) {
		vtg_project_manager_ui_create_project (self, foldername);
		vtg_project_manager_ui_open_project (self, foldername);
	}
	_g_object_unref0 (dialog);
	_g_free0 (foldername);
}


static void vtg_project_manager_ui_on_project_search_result_next (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_search_next_match (self->priv->_prj_search);
}


static void vtg_project_manager_ui_on_project_search_result_previous (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_search_previous_match (self->priv->_prj_search);
}


static void vtg_project_manager_ui_on_project_search (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_view_get_current_project (self->priv->_prj_view) != NULL) {
		VtgProjectManager* project;
		VtgProjectSearchDialog* exec_dialog;
		project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
		exec_dialog = vtg_project_search_dialog_new ((GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance));
		if (vtg_project_search_dialog_run (exec_dialog) == GTK_RESPONSE_OK) {
			vtg_project_search_search (self->priv->_prj_search, project, exec_dialog->search_text, exec_dialog->match_case);
		}
		_g_object_unref0 (project);
		_g_object_unref0 (exec_dialog);
	}
}


static void vtg_project_manager_ui_on_project_goto_next_position (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_source_bookmarks_move_next (self->priv->_bookmarks);
}


static void vtg_project_manager_ui_on_project_goto_prev_position (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_source_bookmarks_move_previous (self->priv->_bookmarks);
}


static void vtg_project_manager_ui_on_project_goto_document (VtgProjectManagerUi* self, GtkAction* action) {
	VtgProjectManager* project;
	GtkTreeIter iter = {0};
	GtkListStore* model;
	VtgFilteredListDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
	g_return_if_fail (project != NULL);
	model = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, G_TYPE_OBJECT, NULL);
	{
		GeeIterator* _src_it;
		_src_it = gee_iterable_iterator ((GeeIterable*) project->all_vala_sources);
		while (TRUE) {
			VbfSource* src;
			if (!gee_iterator_next (_src_it)) {
				break;
			}
			src = (VbfSource*) gee_iterator_get (_src_it);
			gtk_list_store_append (model, &iter);
			gtk_list_store_set (model, &iter, 0, ((VbfFile*) src)->name, 1, ((VbfFile*) src)->name, 2, TRUE, 3, src, -1);
			_g_object_unref0 (src);
		}
		_gee_collection_object_unref0 (_src_it);
	}
	dialog = vtg_filtered_list_dialog_new (model);
	vtg_filtered_list_dialog_set_transient_for (dialog, (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance));
	if (vtg_filtered_list_dialog_run (dialog)) {
		VbfSource* src;
		GeditTab* _tmp0_;
		src = NULL;
		gtk_tree_model_get ((GtkTreeModel*) model, &dialog->selected_iter, 3, &src, -1);
		_tmp0_ = vtg_plugin_instance_activate_uri (self->priv->_plugin_instance, ((VbfFile*) src)->uri, 0, 0);
		_g_object_unref0 (_tmp0_);
		_g_object_unref0 (src);
	}
	_g_object_unref0 (project);
	_g_object_unref0 (model);
	_g_object_unref0 (dialog);
}


static void vtg_project_manager_ui_on_project_goto_method (VtgProjectManagerUi* self, GtkAction* action) {
	GError * _inner_error_;
	VtgProjectManager* project;
	VtgProjectDescriptor* pdes;
	GeditView* view;
	GeditDocument* doc;
	char* uri;
	char* _tmp0_;
	char* _tmp1_;
	GeeList* methods;
	GtkTreeIter iter = {0};
	GtkListStore* model;
	VtgFilteredListDialog* dialog;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_inner_error_ = NULL;
	project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
	g_return_if_fail (project != NULL);
	pdes = vtg_project_manager_ui_get_projectdescriptor_for_project (self, project);
	g_return_if_fail (pdes != NULL);
	view = _g_object_ref0 (gedit_window_get_active_view (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	if (view == NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (pdes);
		_g_object_unref0 (view);
		return;
	}
	doc = _g_object_ref0 (GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) view)));
	g_return_if_fail (doc != NULL);
	uri = g_strdup (gedit_document_get_uri (doc));
	if (uri == NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (pdes);
		_g_object_unref0 (view);
		_g_object_unref0 (doc);
		_g_free0 (uri);
		return;
	}
	_tmp0_ = g_filename_from_uri (uri, NULL, &_inner_error_);
	if (_inner_error_ != NULL) {
		_g_object_unref0 (project);
		_g_object_unref0 (pdes);
		_g_object_unref0 (view);
		_g_object_unref0 (doc);
		_g_free0 (uri);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
	uri = (_tmp1_ = _tmp0_, _g_free0 (uri), _tmp1_);
	methods = vsc_symbol_completion_get_methods_for_source (pdes->completion, uri);
	if (gee_collection_get_size ((GeeCollection*) methods) <= 0) {
		_g_object_unref0 (project);
		_g_object_unref0 (pdes);
		_g_object_unref0 (view);
		_g_object_unref0 (doc);
		_g_free0 (uri);
		_gee_collection_object_unref0 (methods);
		return;
	}
	model = gtk_list_store_new (4, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_BOOLEAN, VSC_TYPE_SYMBOL_COMPLETION_ITEM, NULL);
	{
		GeeIterator* _method_it;
		_method_it = gee_iterable_iterator ((GeeIterable*) methods);
		while (TRUE) {
			VscSymbolCompletionItem* method;
			if (!gee_iterator_next (_method_it)) {
				break;
			}
			method = (VscSymbolCompletionItem*) gee_iterator_get (_method_it);
			gtk_list_store_append (model, &iter);
			gtk_list_store_set (model, &iter, 0, method->name, 1, method->name, 2, TRUE, 3, method, -1);
			_g_object_unref0 (method);
		}
		_gee_collection_object_unref0 (_method_it);
	}
	dialog = vtg_filtered_list_dialog_new (model);
	vtg_filtered_list_dialog_set_transient_for (dialog, (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance));
	if (vtg_filtered_list_dialog_run (dialog)) {
		VscSymbolCompletionItem* method;
		method = NULL;
		gtk_tree_model_get ((GtkTreeModel*) model, &dialog->selected_iter, 3, &method, -1);
		gedit_document_goto_line (doc, method->first_line - 1);
		gedit_view_scroll_to_cursor (view);
		_g_object_unref0 (method);
	}
	_g_object_unref0 (project);
	_g_object_unref0 (pdes);
	_g_object_unref0 (view);
	_g_object_unref0 (doc);
	_g_free0 (uri);
	_gee_collection_object_unref0 (methods);
	_g_object_unref0 (model);
	_g_object_unref0 (dialog);
}


static VtgProjectDescriptor* vtg_project_manager_ui_get_projectdescriptor_for_project (VtgProjectManagerUi* self, VtgProjectManager* project) {
	VtgProjectDescriptor* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (project != NULL, NULL);
	{
		GeeIterator* _current_it;
		_current_it = gee_iterable_iterator ((GeeIterable*) vtg_plugin_get_projects (self->priv->_plugin_instance->plugin));
		while (TRUE) {
			VtgProjectDescriptor* current;
			if (!gee_iterator_next (_current_it)) {
				break;
			}
			current = (VtgProjectDescriptor*) gee_iterator_get (_current_it);
			if (current->project == project) {
				result = current;
				_gee_collection_object_unref0 (_current_it);
				return result;
			}
			_g_object_unref0 (current);
		}
		_gee_collection_object_unref0 (_current_it);
	}
	result = NULL;
	return result;
}


static void vtg_project_manager_ui_on_standalone_file_compile (VtgProjectManagerUi* self, GtkAction* action) {
	GError * _inner_error_;
	GeditDocument* doc;
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	_inner_error_ = NULL;
	doc = _g_object_ref0 (gedit_window_get_active_document (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	if (doc != NULL) {
		char* file;
		VtgProjectManager* project;
		GtkListStore* cache;
		VtgInteractionParametersDialog* params_dialog;
		file = g_strdup (gedit_document_get_uri (doc));
		project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
		if (project != NULL) {
			if (vtg_project_manager_contains_vala_source_file (project, file)) {
				g_warning ("vtgprojectmanagerui.vala:509: Can't compile a project file (for now)");
				_g_free0 (file);
				_g_object_unref0 (project);
				_g_object_unref0 (doc);
				return;
			}
		}
		cache = vtg_caches_get_compile_cache ();
		params_dialog = vtg_interaction_parameters_dialog_new (_ ("Compile File"), (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), cache);
		if (vtg_interaction_parameters_dialog_run (params_dialog) == GTK_RESPONSE_OK) {
			char* params;
			char* _tmp0_;
			char* _tmp1_;
			gboolean _tmp2_;
			params = g_strdup (vtg_interaction_parameters_dialog_get_parameters (params_dialog));
			if (!vtg_caches_cache_contains (cache, params)) {
				vtg_caches_cache_add (cache, params);
			}
			_tmp0_ = g_filename_from_uri (file, NULL, &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_free0 (params);
				_g_free0 (file);
				_g_object_unref0 (project);
				_g_object_unref0 (cache);
				_vtg_interaction_parameters_dialog_unref0 (params_dialog);
				_g_object_unref0 (doc);
				g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
				g_clear_error (&_inner_error_);
				return;
			}
			file = (_tmp1_ = _tmp0_, _g_free0 (file), _tmp1_);
			_tmp2_ = FALSE;
			if (!gedit_document_is_untouched (doc)) {
				_tmp2_ = vtg_configuration_get_save_before_build (vtg_plugin_get_config (self->priv->_plugin_instance->plugin));
			} else {
				_tmp2_ = FALSE;
			}
			if (_tmp2_) {
				gedit_document_save (doc, GEDIT_DOCUMENT_SAVE_IGNORE_MTIME);
			}
			vtg_project_builder_compile_file (self->priv->_prj_builder, file, params);
			_g_free0 (params);
		}
		_g_free0 (file);
		_g_object_unref0 (project);
		_g_object_unref0 (cache);
		_vtg_interaction_parameters_dialog_unref0 (params_dialog);
	}
	_g_object_unref0 (doc);
}


static void vtg_project_manager_ui_on_project_build (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_view_get_current_project (self->priv->_prj_view) != NULL) {
		char* pars;
		GtkListStore* cache;
		VtgProjectManager* project;
		pars = NULL;
		cache = vtg_caches_get_build_cache ();
		if (vtg_project_builder_get_is_building (self->priv->_prj_builder)) {
			GtkMessageDialog* dialog;
			gint res;
			dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new ((GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), GTK_DIALOG_MODAL, GTK_MESSAGE_QUESTION, GTK_BUTTONS_YES_NO, _ ("Stop the current build process and restart a new one?")));
			g_object_set (dialog, "secondary-text", _ ("Stop the current build process and start a new one with the same command line parameters"), NULL);
			res = gtk_dialog_run ((GtkDialog*) dialog);
			gtk_object_destroy ((GtkObject*) dialog);
			if (res == GTK_RESPONSE_YES) {
				GtkTreeIter iter = {0};
				vtg_project_builder_stop_build (self->priv->_prj_builder);
				if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
					gtk_tree_model_get ((GtkTreeModel*) cache, &iter, 0, &pars, -1);
				}
			} else {
				_g_object_unref0 (dialog);
				_g_free0 (pars);
				_g_object_unref0 (cache);
				return;
			}
			_g_object_unref0 (dialog);
		}
		if (pars == NULL) {
			VtgInteractionParametersDialog* params_dialog;
			char* _tmp0_;
			params_dialog = vtg_interaction_parameters_dialog_new (_ ("Build Project"), (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), cache);
			if (vtg_interaction_parameters_dialog_run (params_dialog) != GTK_RESPONSE_OK) {
				_vtg_interaction_parameters_dialog_unref0 (params_dialog);
				_g_free0 (pars);
				_g_object_unref0 (cache);
				return;
			}
			pars = (_tmp0_ = g_strdup (vtg_interaction_parameters_dialog_get_parameters (params_dialog)), _g_free0 (pars), _tmp0_);
			if (!vtg_caches_cache_contains (cache, pars)) {
				vtg_caches_cache_add (cache, pars);
			}
			_vtg_interaction_parameters_dialog_unref0 (params_dialog);
		}
		project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
		vtg_plugin_project_save_all (self->priv->_plugin_instance->plugin, project);
		vtg_project_builder_build (self->priv->_prj_builder, project, pars);
		_g_free0 (pars);
		_g_object_unref0 (cache);
		_g_object_unref0 (project);
	}
}


static void vtg_project_manager_ui_on_project_configure (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_view_get_current_project (self->priv->_prj_view) != NULL) {
		GtkListStore* cache;
		VtgInteractionParametersDialog* params_dialog;
		cache = vtg_caches_get_configure_cache ();
		params_dialog = vtg_interaction_parameters_dialog_new (_ ("Configure Project"), (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), cache);
		if (vtg_interaction_parameters_dialog_run (params_dialog) == GTK_RESPONSE_OK) {
			VtgProjectManager* project;
			char* params;
			project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
			params = g_strdup (vtg_interaction_parameters_dialog_get_parameters (params_dialog));
			if (!vtg_caches_cache_contains (cache, params)) {
				vtg_caches_cache_add (cache, params);
			}
			vtg_plugin_project_save_all (self->priv->_plugin_instance->plugin, project);
			vtg_project_builder_configure (self->priv->_prj_builder, project, params);
			_g_object_unref0 (project);
			_g_free0 (params);
		}
		_g_object_unref0 (cache);
		_vtg_interaction_parameters_dialog_unref0 (params_dialog);
	}
}


static void vtg_project_manager_ui_on_project_clean (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_ui_clean_project (self, FALSE);
}


static void vtg_project_manager_ui_on_project_clean_stamps (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_manager_ui_clean_project (self, TRUE);
}


static void vtg_project_manager_ui_on_project_execute_process (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	if (vtg_project_view_get_current_project (self->priv->_prj_view) != NULL) {
		VtgProjectManager* project;
		VtgProjectExecuterDialog* exec_dialog;
		project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
		exec_dialog = vtg_project_executer_dialog_new ((GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance), project);
		if (vtg_project_executer_dialog_run (exec_dialog) == GTK_RESPONSE_OK) {
			char* command_line;
			command_line = g_strdup (vtg_project_executer_dialog_get_command_line (exec_dialog));
			vtg_project_executer_execute (self->priv->_prj_executer, vtg_project_manager_get_project (project), command_line);
			_g_free0 (command_line);
		}
		_g_object_unref0 (project);
		_vtg_project_executer_dialog_unref0 (exec_dialog);
	}
}


static void vtg_project_manager_ui_on_project_kill_process (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_executer_kill_last (self->priv->_prj_executer);
}


static void vtg_project_manager_ui_clean_project (VtgProjectManagerUi* self, gboolean stamps) {
	g_return_if_fail (self != NULL);
	if (vtg_project_view_get_current_project (self->priv->_prj_view) != NULL) {
		VtgProjectManager* project;
		project = _g_object_ref0 (vtg_project_view_get_current_project (self->priv->_prj_view));
		vtg_project_builder_clean (self->priv->_prj_builder, project, stamps);
		_g_object_unref0 (project);
	}
}


static void vtg_project_manager_ui_on_project_error_next (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_builder_next_error (self->priv->_prj_builder);
}


static void vtg_project_manager_ui_on_project_error_previuos (VtgProjectManagerUi* self, GtkAction* action) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (action != NULL);
	vtg_project_builder_previous_error (self->priv->_prj_builder);
}


static void vtg_project_manager_ui_on_current_project_changed (VtgProjectManagerUi* self, GObject* sender, GParamSpec* pspec) {
	VtgProjectView* view;
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	g_return_if_fail (pspec != NULL);
	view = _g_object_ref0 (VTG_PROJECT_VIEW (sender));
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (view) == NULL);
	_g_object_unref0 (view);
}


static void vtg_project_manager_ui_update_ui (VtgProjectManagerUi* self, gboolean default_project) {
	GtkAction* action;
	GtkAction* _tmp0_;
	GtkAction* _tmp1_;
	GtkAction* _tmp2_;
	GeditDocument* doc;
	gboolean _tmp3_;
	gboolean _tmp4_;
	gboolean is_vala_source;
	GtkAction* _tmp5_;
	gboolean _tmp6_;
	GtkAction* _tmp7_;
	GtkAction* _tmp8_;
	gboolean has_errors;
	GtkAction* _tmp9_;
	GtkAction* _tmp10_;
	GtkAction* _tmp11_;
	gboolean _tmp12_;
	GtkAction* _tmp13_;
	gboolean _tmp14_;
	GtkAction* _tmp15_;
	gboolean can_complete;
	GeditView* view;
	GtkAction* _tmp16_;
	gboolean has_changelog;
	gboolean has_vcs_backend;
	gboolean _tmp17_;
	GtkAction* _tmp18_;
	gboolean _tmp19_;
	GtkAction* _tmp20_;
	GtkAction* _tmp21_;
	GtkAction* _tmp22_;
	g_return_if_fail (self != NULL);
	action = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectClose"));
	gtk_action_set_sensitive (action, !default_project);
	action = (_tmp0_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuild")), _g_object_unref0 (action), _tmp0_);
	gtk_action_set_sensitive (action, !default_project);
	action = (_tmp1_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildClean")), _g_object_unref0 (action), _tmp1_);
	gtk_action_set_sensitive (action, !default_project);
	action = (_tmp2_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildCleanStamps")), _g_object_unref0 (action), _tmp2_);
	gtk_action_set_sensitive (action, !default_project);
	doc = _g_object_ref0 (gedit_window_get_active_document (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	_tmp3_ = FALSE;
	_tmp4_ = FALSE;
	if (doc != NULL) {
		_tmp4_ = gtk_source_buffer_get_language ((GtkSourceBuffer*) doc) != NULL;
	} else {
		_tmp4_ = FALSE;
	}
	if (_tmp4_) {
		_tmp3_ = _vala_strcmp0 (gtk_source_language_get_id (gtk_source_buffer_get_language ((GtkSourceBuffer*) doc)), "vala") == 0;
	} else {
		_tmp3_ = FALSE;
	}
	is_vala_source = _tmp3_;
	action = (_tmp5_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildCompileFile")), _g_object_unref0 (action), _tmp5_);
	_tmp6_ = FALSE;
	if (default_project) {
		_tmp6_ = is_vala_source;
	} else {
		_tmp6_ = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp6_);
	action = (_tmp7_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectGotoMethod")), _g_object_unref0 (action), _tmp7_);
	gtk_action_set_sensitive (action, is_vala_source);
	action = (_tmp8_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectGotoDocument")), _g_object_unref0 (action), _tmp8_);
	gtk_action_set_sensitive (action, !default_project);
	has_errors = vtg_build_log_view_get_error_count (vtg_project_builder_get_error_pane (self->priv->_prj_builder)) > 0;
	action = (_tmp9_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildNextError")), _g_object_unref0 (action), _tmp9_);
	gtk_action_set_sensitive (action, has_errors);
	action = (_tmp10_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildPreviousError")), _g_object_unref0 (action), _tmp10_);
	gtk_action_set_sensitive (action, has_errors);
	action = (_tmp11_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildExecute")), _g_object_unref0 (action), _tmp11_);
	_tmp12_ = FALSE;
	if (!vtg_project_executer_get_is_executing (self->priv->_prj_executer)) {
		_tmp12_ = !default_project;
	} else {
		_tmp12_ = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp12_);
	action = (_tmp13_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectBuildKill")), _g_object_unref0 (action), _tmp13_);
	_tmp14_ = FALSE;
	if (vtg_project_executer_get_is_executing (self->priv->_prj_executer)) {
		_tmp14_ = !default_project;
	} else {
		_tmp14_ = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp14_);
	action = (_tmp15_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectSearch")), _g_object_unref0 (action), _tmp15_);
	gtk_action_set_sensitive (action, !vtg_project_search_get_is_searching (self->priv->_prj_search));
	can_complete = FALSE;
	view = _g_object_ref0 (gedit_window_get_active_view (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
	if (view != NULL) {
		VtgSymbolCompletionHelper* sch;
		sch = vtg_plugin_instance_scs_find_from_view (self->priv->_plugin_instance, view);
		can_complete = sch != NULL;
		_g_object_unref0 (sch);
	}
	action = (_tmp16_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectCompleteWord")), _g_object_unref0 (action), _tmp16_);
	gtk_action_set_sensitive (action, can_complete);
	has_changelog = FALSE;
	has_vcs_backend = FALSE;
	_tmp17_ = FALSE;
	if (vtg_project_view_get_current_project (self->priv->_prj_view) != NULL) {
		_tmp17_ = vtg_project_view_get_current_project (self->priv->_prj_view)->changelog_uri != NULL;
	} else {
		_tmp17_ = FALSE;
	}
	if (_tmp17_) {
		has_changelog = TRUE;
		if (vtg_project_view_get_current_project (self->priv->_prj_view)->vcs_type != VTG_VCS_TYPES_NONE) {
			has_vcs_backend = TRUE;
		}
	}
	action = (_tmp18_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectPrepareChangeLog")), _g_object_unref0 (action), _tmp18_);
	_tmp19_ = FALSE;
	if (has_changelog) {
		_tmp19_ = has_vcs_backend;
	} else {
		_tmp19_ = FALSE;
	}
	gtk_action_set_sensitive (action, _tmp19_);
	action = (_tmp20_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectPrepareSingleFileChangeLog")), _g_object_unref0 (action), _tmp20_);
	gtk_action_set_sensitive (action, has_changelog);
	action = (_tmp21_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectGotoNextPosition")), _g_object_unref0 (action), _tmp21_);
	gtk_action_set_sensitive (action, !vtg_source_bookmarks_get_is_empty (self->priv->_bookmarks));
	action = (_tmp22_ = _g_object_ref0 (gtk_action_group_get_action (self->priv->_actions, "ProjectGotoPrevPosition")), _g_object_unref0 (action), _tmp22_);
	gtk_action_set_sensitive (action, !vtg_source_bookmarks_get_is_empty (self->priv->_bookmarks));
	_g_object_unref0 (action);
	_g_object_unref0 (doc);
	_g_object_unref0 (view);
}


static void vtg_project_manager_ui_open_project (VtgProjectManagerUi* self, const char* name) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (name != NULL);
	_inner_error_ = NULL;
	{
		VtgProjectManager* project;
		project = vtg_project_manager_ui_find_project_for_id (self, name);
		if (project != NULL) {
			vtg_project_view_set_current_project (self->priv->_prj_view, project);
		} else {
			VtgProjectManager* _tmp0_;
			gboolean _tmp1_;
			project = (_tmp0_ = vtg_project_manager_new (), _g_object_unref0 (project), _tmp0_);
			_tmp1_ = vtg_project_manager_open (project, name, &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (project);
				goto __catch7_g_error;
				goto __finally7;
			}
			if (_tmp1_) {
				gee_collection_add ((GeeCollection*) self->priv->_projects, project);
				vtg_plugin_on_project_loaded (self->priv->_plugin_instance->plugin, self, project);
				vtg_project_view_add_project (self->priv->_prj_view, vtg_project_manager_get_project (project));
			}
		}
		_g_object_unref0 (project);
	}
	goto __finally7;
	__catch7_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			char* _tmp2_;
			vtg_interaction_error_message (_tmp2_ = g_strdup_printf (_ ("Error opening project %s"), name), err);
			_g_free0 (_tmp2_);
			_g_error_free0 (err);
		}
	}
	__finally7:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


void vtg_project_manager_ui_close_project (VtgProjectManagerUi* self, VtgProjectManager* project) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (project != NULL);
	vtg_project_view_remove_project (self->priv->_prj_view, vtg_project_manager_get_project (project));
	vtg_plugin_on_project_closed (self->priv->_plugin_instance->plugin, self, project);
	vtg_project_manager_close (project);
	gee_collection_remove ((GeeCollection*) self->priv->_projects, project);
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static void _vtg_project_manager_ui_on_child_watch_gchild_watch_func (GPid pid, gint status, gpointer self) {
	vtg_project_manager_ui_on_child_watch (self, pid, status);
}


static void vtg_project_manager_ui_create_project (VtgProjectManagerUi* self, const char* project_path) {
	GError * _inner_error_;
	g_return_if_fail (self != NULL);
	g_return_if_fail (project_path != NULL);
	_inner_error_ = NULL;
	{
		VtgOutputView* log;
		char* process_file;
		gint status;
		gint stdo;
		gint stde;
		GPid child_pid;
		gint _tmp2__length1;
		char** _tmp2_;
		char** _tmp1_;
		gboolean _tmp3_;
		gboolean _tmp4_;
		log = _g_object_ref0 (vtg_plugin_instance_get_output_view (self->priv->_plugin_instance));
		if (!vtg_project_manager_ui_is_dir_empty (self, project_path)) {
			char* _tmp0_;
			vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_MESSAGE, _tmp0_ = g_strdup_printf ("project directory %s not empty\n", project_path));
			_g_free0 (_tmp0_);
			_g_object_unref0 (log);
			return;
		}
		process_file = g_strdup ("vala-gen-project");
		status = 0;
		stdo = 0;
		stde = 0;
		child_pid = 0;
		_tmp1_ = NULL;
		_tmp4_ = (_tmp3_ = g_spawn_sync (project_path, (_tmp2_ = (_tmp1_ = g_new0 (char*, 3 + 1), _tmp1_[0] = g_strdup (process_file), _tmp1_[1] = g_strdup ("--projectdir"), _tmp1_[2] = g_strdup (project_path), _tmp1_), _tmp2__length1 = 3, _tmp2_), NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, NULL, NULL, &status, &_inner_error_), _tmp2_ = (_vala_array_free (_tmp2_, _tmp2__length1, (GDestroyNotify) g_free), NULL), _tmp3_);
		if (_inner_error_ != NULL) {
			_g_object_unref0 (log);
			_g_free0 (process_file);
			goto __catch8_g_error;
			goto __finally8;
		}
		if (_tmp4_) {
			if (WEXITSTATUS (status) == 0) {
				char* start_message;
				char* _tmp6_;
				char* _tmp5_;
				gint _tmp8__length1;
				char** _tmp8_;
				char** _tmp7_;
				start_message = g_strdup_printf (_ ("Autogenerating project: %s\n"), project_path);
				vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_MESSAGE, start_message);
				vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_MESSAGE, _tmp6_ = g_strdup_printf ("%s\n\n", _tmp5_ = g_strnfill ((gsize) (string_get_length (start_message) - 1), '-')));
				_g_free0 (_tmp6_);
				_g_free0 (_tmp5_);
				_tmp7_ = NULL;
				g_spawn_async_with_pipes (project_path, (_tmp8_ = (_tmp7_ = g_new0 (char*, 1 + 1), _tmp7_[0] = g_strdup ("./autogen.sh"), _tmp7_), _tmp8__length1 = 1, _tmp8_), NULL, G_SPAWN_SEARCH_PATH | G_SPAWN_DO_NOT_REAP_CHILD, NULL, NULL, &child_pid, NULL, &stdo, &stde, &_inner_error_);
				if (_inner_error_ != NULL) {
					_g_free0 (start_message);
					_g_object_unref0 (log);
					_g_free0 (process_file);
					goto __catch8_g_error;
					goto __finally8;
				}
				_tmp8_ = (_vala_array_free (_tmp8_, _tmp8__length1, (GDestroyNotify) g_free), NULL);
				if (child_pid != ((GPid) NULL)) {
					g_child_watch_add (child_pid, _vtg_project_manager_ui_on_child_watch_gchild_watch_func, self);
					vtg_output_view_start_watch (log, VTG_OUTPUT_TYPES_CHILD_PROCESS, (guint) child_pid, stdo, stde, -1);
					vtg_output_view_activate (log);
				} else {
					vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_ERROR, "error spawning ./autogen.sh process\n");
				}
				_g_free0 (start_message);
			} else {
				vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_ERROR, "error executing vala-gen-project process\n");
			}
		} else {
			vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_ERROR, "error spawning vala-gen-project process\n");
		}
		_g_object_unref0 (log);
		_g_free0 (process_file);
	}
	goto __finally8;
	__catch8_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("vtgprojectmanagerui.vala:776: error creating project: %s", err->message);
			_g_error_free0 (err);
		}
	}
	__finally8:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


static gboolean vtg_project_manager_ui_is_dir_empty (VtgProjectManagerUi* self, const char* dir_path) {
	gboolean result;
	GError * _inner_error_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (dir_path != NULL, FALSE);
	_inner_error_ = NULL;
	{
		GDir* dir;
		dir = g_dir_open (dir_path, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			goto __catch9_g_error;
			goto __finally9;
		}
		result = g_dir_read_name (dir) == NULL;
		_g_dir_close0 (dir);
		return result;
	}
	goto __finally9;
	__catch9_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("vtgprojectmanagerui.vala:786: cannot open directort %s", dir_path);
			result = FALSE;
			_g_error_free0 (err);
			return result;
		}
	}
	__finally9:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return FALSE;
	}
}


static void vtg_project_manager_ui_on_child_watch (VtgProjectManagerUi* self, GPid pid, gint status) {
	VtgOutputView* log;
	char* _tmp0_;
	g_return_if_fail (self != NULL);
	log = _g_object_ref0 (vtg_plugin_instance_get_output_view (self->priv->_plugin_instance));
	g_spawn_close_pid (pid);
	vtg_output_view_stop_watch (log, (guint) pid);
	vtg_output_view_log_message (log, VTG_OUTPUT_TYPES_MESSAGE, _tmp0_ = g_strdup_printf (_ ("\nautogeneration end with exit status %d\n"), status));
	_g_free0 (_tmp0_);
	_g_object_unref0 (log);
}


VtgProjectManager* vtg_project_manager_ui_find_project_for_id (VtgProjectManagerUi* self, const char* id) {
	VtgProjectManager* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (id != NULL, NULL);
	{
		GeeIterator* _project_manager_it;
		_project_manager_it = gee_iterable_iterator ((GeeIterable*) self->priv->_projects);
		while (TRUE) {
			VtgProjectManager* project_manager;
			if (!gee_iterator_next (_project_manager_it)) {
				break;
			}
			project_manager = (VtgProjectManager*) gee_iterator_get (_project_manager_it);
			if (_vala_strcmp0 (vtg_project_manager_get_project (project_manager)->id, id) == 0) {
				result = project_manager;
				_gee_collection_object_unref0 (_project_manager_it);
				return result;
			}
			_g_object_unref0 (project_manager);
		}
		_gee_collection_object_unref0 (_project_manager_it);
	}
	result = NULL;
	return result;
}


VtgPluginInstance* vtg_project_manager_ui_get_plugin_instance (VtgProjectManagerUi* self) {
	VtgPluginInstance* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_plugin_instance;
	return result;
}


static void vtg_project_manager_ui_set_plugin_instance (VtgProjectManagerUi* self, VtgPluginInstance* value) {
	VtgPluginInstance* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_plugin_instance = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_plugin_instance), _tmp0_);
	g_object_notify ((GObject *) self, "plugin-instance");
}


VtgProjectView* vtg_project_manager_ui_get_project_view (VtgProjectManagerUi* self) {
	VtgProjectView* result;
	g_return_val_if_fail (self != NULL, NULL);
	result = self->priv->_prj_view;
	return result;
}


static void _vtg_project_manager_ui_on_current_project_changed_g_object_notify (VtgProjectView* _sender, GParamSpec* pspec, gpointer self) {
	vtg_project_manager_ui_on_current_project_changed (self, _sender, pspec);
}


static void _lambda0_ (VtgProjectExecuter* sender, VtgProjectManagerUi* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void __lambda0__vtg_project_executer_process_start (VtgProjectExecuter* _sender, gpointer self) {
	_lambda0_ (_sender, self);
}


static void _lambda1_ (VtgProjectExecuter* sender, gint exit_status, VtgProjectManagerUi* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void __lambda1__vtg_project_executer_process_exit (VtgProjectExecuter* _sender, gint exit_status, gpointer self) {
	_lambda1_ (_sender, exit_status, self);
}


static void _lambda2_ (VtgProjectBuilder* sender, VtgProjectManagerUi* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void __lambda2__vtg_project_builder_build_start (VtgProjectBuilder* _sender, gpointer self) {
	_lambda2_ (_sender, self);
}


static void _lambda3_ (VtgProjectBuilder* sender, gint exit_status, VtgProjectManagerUi* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void __lambda3__vtg_project_builder_build_exit (VtgProjectBuilder* _sender, gint exit_status, gpointer self) {
	_lambda3_ (_sender, exit_status, self);
}


static void _lambda4_ (VtgProjectSearch* sender, VtgProjectManagerUi* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void __lambda4__vtg_project_search_search_start (VtgProjectSearch* _sender, gpointer self) {
	_lambda4_ (_sender, self);
}


static void _lambda5_ (VtgProjectSearch* sender, gint exit_status, VtgProjectManagerUi* self) {
	g_return_if_fail (sender != NULL);
	vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
}


static void __lambda5__vtg_project_search_search_exit (VtgProjectSearch* _sender, gint exit_status, gpointer self) {
	_lambda5_ (_sender, exit_status, self);
}


static void _vtg_project_manager_ui_on_current_bookmark_changed_vtg_source_bookmarks_current_bookmark_changed (VtgSourceBookmarks* _sender, gpointer self) {
	vtg_project_manager_ui_on_current_bookmark_changed (self, _sender);
}


static GObject * vtg_project_manager_ui_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgProjectManagerUiClass * klass;
	GObjectClass * parent_class;
	VtgProjectManagerUi * self;
	klass = VTG_PROJECT_MANAGER_UI_CLASS (g_type_class_peek (VTG_TYPE_PROJECT_MANAGER_UI));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_PROJECT_MANAGER_UI (obj);
	{
		VtgProjectView* _tmp0_;
		VtgProjectBuilder* _tmp1_;
		VtgProjectExecuter* _tmp2_;
		VtgProjectSearch* _tmp3_;
		VtgChangeLog* _tmp4_;
		VtgSourceBookmarks* _tmp5_;
		self->priv->_prj_view = (_tmp0_ = vtg_project_view_new (self->priv->_plugin_instance), _g_object_unref0 (self->priv->_prj_view), _tmp0_);
		{
			GeeIterator* _prj_it;
			_prj_it = gee_iterable_iterator ((GeeIterable*) vtg_plugin_get_projects (self->priv->_plugin_instance->plugin));
			while (TRUE) {
				VtgProjectDescriptor* prj;
				if (!gee_iterator_next (_prj_it)) {
					break;
				}
				prj = (VtgProjectDescriptor*) gee_iterator_get (_prj_it);
				vtg_project_view_add_project (self->priv->_prj_view, vtg_project_manager_get_project (prj->project));
				_g_object_unref0 (prj);
			}
			_gee_collection_object_unref0 (_prj_it);
		}
		g_signal_connect_object ((GObject*) self->priv->_prj_view, "notify::current-project", (GCallback) _vtg_project_manager_ui_on_current_project_changed_g_object_notify, self, 0);
		self->priv->_prj_builder = (_tmp1_ = vtg_project_builder_new (self->priv->_plugin_instance), _g_object_unref0 (self->priv->_prj_builder), _tmp1_);
		self->priv->_prj_executer = (_tmp2_ = vtg_project_executer_new (self->priv->_plugin_instance), _g_object_unref0 (self->priv->_prj_executer), _tmp2_);
		self->priv->_prj_search = (_tmp3_ = vtg_project_search_new (self->priv->_plugin_instance), _g_object_unref0 (self->priv->_prj_search), _tmp3_);
		g_signal_connect_object (self->priv->_prj_executer, "process-start", (GCallback) __lambda0__vtg_project_executer_process_start, self, 0);
		g_signal_connect_object (self->priv->_prj_executer, "process-exit", (GCallback) __lambda1__vtg_project_executer_process_exit, self, 0);
		g_signal_connect_object (self->priv->_prj_builder, "build-start", (GCallback) __lambda2__vtg_project_builder_build_start, self, 0);
		g_signal_connect_object (self->priv->_prj_builder, "build-exit", (GCallback) __lambda3__vtg_project_builder_build_exit, self, 0);
		g_signal_connect_object (self->priv->_prj_search, "search-start", (GCallback) __lambda4__vtg_project_search_search_start, self, 0);
		g_signal_connect_object (self->priv->_prj_search, "search-exit", (GCallback) __lambda5__vtg_project_search_search_exit, self, 0);
		vtg_project_manager_ui_initialize_ui (self);
		self->priv->_changelog = (_tmp4_ = vtg_change_log_new (self->priv->_plugin_instance), _g_object_unref0 (self->priv->_changelog), _tmp4_);
		self->priv->_bookmarks = (_tmp5_ = vtg_source_bookmarks_new (self->priv->_plugin_instance), _g_object_unref0 (self->priv->_bookmarks), _tmp5_);
		g_signal_connect_object (self->priv->_bookmarks, "current-bookmark-changed", (GCallback) _vtg_project_manager_ui_on_current_bookmark_changed_vtg_source_bookmarks_current_bookmark_changed, self, 0);
		vtg_project_manager_ui_update_ui (self, vtg_project_view_get_current_project (self->priv->_prj_view) == NULL);
	}
	return obj;
}


static void vtg_project_manager_ui_class_init (VtgProjectManagerUiClass * klass) {
	vtg_project_manager_ui_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgProjectManagerUiPrivate));
	G_OBJECT_CLASS (klass)->get_property = vtg_project_manager_ui_get_property;
	G_OBJECT_CLASS (klass)->set_property = vtg_project_manager_ui_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_project_manager_ui_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_project_manager_ui_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_UI_PLUGIN_INSTANCE, g_param_spec_object ("plugin-instance", "plugin-instance", "plugin-instance", VTG_TYPE_PLUGIN_INSTANCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_PROJECT_MANAGER_UI_PROJECT_VIEW, g_param_spec_object ("project-view", "project-view", "project-view", VTG_TYPE_PROJECT_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_READABLE));
}


static void vtg_project_manager_ui_instance_init (VtgProjectManagerUi * self) {
	self->priv = VTG_PROJECT_MANAGER_UI_GET_PRIVATE (self);
	self->priv->_ui_def = g_strdup ("<ui>\n                                            <menubar name=\"MenuBar\">\n                                                <menu name=\"FileMenu\" action=\"File\">\n                                                    <placeholder name=\"FileOps_2\">\n                                                        <separator />\n                                                        <menuitem name=\"ProjectNew\" action=\"ProjectNew\"/>\n                                                        <menuitem name=\"ProjectOpen\" action=\"ProjectOpen\"/>\n                                                        <menuitem name=\"ProjectRecent\" action=\"ProjectRecent\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectClose\" action=\"ProjectClose\"/>\n                                                        <separator />\n                                                    </placeholder>\n                                                </menu>\n                                            </menubar>\n\n                                            <menubar name=\"MenuBar\">\n                                              <menu name=\"EditMenu\" action=\"Edit\">\n                                                 <placeholder name=\"EditOps_3\">\n                                                     <separator />\n                                                     <menuitem name=\"ProjectCompleteWord\" action=\"ProjectCompleteWord\"/>\n                                                 </placeholder>\n                                              </menu>\n                                            </menubar>\n\n                                            <menubar name=\"MenuBar\">\n                                              <placeholder name=\"ExtraMenu_1\">\n                                                <menu name=\"BuildMenu\" action=\"ProjectBuildMenuAction\">\n                                                    <placeholder name=\"BuildMenuOps_1\">\n                                                        <menuitem name=\"ProjectBuild\" action=\"ProjectBuild\"/>\n                                                        <menuitem name=\"ProjectBuildClean\" action=\"ProjectBuildClean\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectBuildCleanStamps\" action=\"ProjectBuildCleanStamps\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectBuildConfigure\" action=\"ProjectBuildConfigure\"/>\n                                                        <separator />\n                                                        <menuitem name=\"ProjectBuildCompileFile\" action=\"ProjectBuildCompileFile\"/>\n                                                    </placeholder>\n                                                    <placeholder name=\"BuildMenuOps_2\">\n                                                        <separator />\n                                                        <menuitem name=\"NextError\" action=\"ProjectBuildNextError\"/>\n                                                        <menuitem name=\"PreviousError\" action=\"ProjectBuildPreviousError\"/>\n                                                    </placeholder>\n                                                    <placeholder name=\"BuildMenuOps_3\">\n                                                        <separator />\n                                                        <menuitem name=\"Execute\" action=\"ProjectBuildExecute\"/>\n                                                        <menuitem name=\"KillProcess\" action=\"ProjectBuildKill\"/>\n                                                    </placeholder>\n                                                </menu>\n                                              </placeholder>                                              \n                                            </menubar>\n                                            \n                                            <menubar name=\"MenuBar\">\n                                                <menu name=\"DocumentsMenu\" action=\"Documents\">\n                                                    <placeholder name=\"DocumentsOps_3\">\n                                                        <menuitem name=\"GotoDocument\" action=\"ProjectGotoDocument\"/>\n                                                        <separator />\n                                                        <menuitem name=\"GotoNextPosition\" action=\"ProjectGotoNextPosition\"/>\n                                                        <menuitem name=\"GotoPrevPosition\" action=\"ProjectGotoPrevPosition\"/>\n                                                        <separator />\n                                                    </placeholder>\n                                                </menu>\n                                            </menubar>\n\n                                             <menubar name=\"MenuBar\">\n                                                <menu name=\"SearchMenu\" action=\"Search\">\n                                                    <placeholder name=\"SearchOps_1\">\n                                                    \t<separator />                                                    \n                                                        <menuitem name=\"ProjectSearch\" action=\"ProjectSearch\"/>\n\t\t\t\t\t\t\t<menuitem name=\"ProjectSearchPrevReult\" action=\"ProjectSearchPrevResult\"/>\n\t\t\t\t\t\t\t<menuitem name=\"ProjectSearchNextResult\" action=\"ProjectSearchNextResult\"/>\n                                                    </placeholder>\n                                                    <placeholder name=\"SearchOps_8\">\n                                                    \t<separator />\n                                                        <menuitem name=\"GotoMethod\" action=\"ProjectGotoMethod\"/>\n                                                    \t<separator />\n                                                        <menuitem name=\"GotoDefinition\" action=\"ProjectGotoDefinition\"/>\n                                                    </placeholder>\n                                                </menu>\n                                            </menubar>\n                                            \n                                            <menubar name=\"MenuBar\">\n                                            \t <menu name=\"ToolsMenu\" action=\"Tools\">\n\t\t\t\t\t\t \t<placeholder name=\"ToolsOps_1\"> \n\t\t\t\t\t\t \t\t<separator />\n\t\t\t\t\t\t\t \t<menuitem name=\"PrepareSingleFileChangeLog\" action=\"ProjectPrepareSingleFileChangeLog\"/>\n\t\t\t\t\t\t\t \t<menuitem name=\"PrepareChangeLog\" action=\"ProjectPrepareChangeLog\"/>\n\t\t\t\t\t\t \t\t<separator />\t\t\t\t\t\t\t \t\n                                                \t</placeholder>\n                                                 </menu>\n                                            </menubar>\n                                        </ui>");
	self->priv->_projects = (GeeList*) gee_array_list_new (VTG_TYPE_PROJECT_MANAGER, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal);
	self->priv->_actions = NULL;
	self->priv->_prj_view = NULL;
	self->priv->_prj_builder = NULL;
	self->priv->_prj_executer = NULL;
	self->priv->_prj_search = NULL;
	self->priv->_changelog = NULL;
	self->priv->_bookmarks = NULL;
}


static void vtg_project_manager_ui_finalize (GObject* obj) {
	VtgProjectManagerUi * self;
	self = VTG_PROJECT_MANAGER_UI (obj);
	{
		GtkUIManager* manager;
		manager = _g_object_ref0 (gedit_window_get_ui_manager (vtg_plugin_instance_get_window (self->priv->_plugin_instance)));
		gtk_ui_manager_remove_ui (manager, self->priv->_ui_id);
		gtk_ui_manager_remove_action_group (manager, self->priv->_actions);
		_g_object_unref0 (manager);
	}
	_g_free0 (self->priv->_ui_def);
	_gee_collection_object_unref0 (self->priv->_projects);
	_g_object_unref0 (self->priv->_actions);
	_g_object_unref0 (self->priv->_plugin_instance);
	_g_object_unref0 (self->priv->_prj_view);
	_g_object_unref0 (self->priv->_prj_builder);
	_g_object_unref0 (self->priv->_prj_executer);
	_g_object_unref0 (self->priv->_prj_search);
	_g_object_unref0 (self->priv->_changelog);
	_g_object_unref0 (self->priv->_bookmarks);
	G_OBJECT_CLASS (vtg_project_manager_ui_parent_class)->finalize (obj);
}


GType vtg_project_manager_ui_get_type (void) {
	return vtg_project_manager_ui_type_id;
}


GType vtg_project_manager_ui_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgProjectManagerUiClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_project_manager_ui_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgProjectManagerUi), 0, (GInstanceInitFunc) vtg_project_manager_ui_instance_init, NULL };
	vtg_project_manager_ui_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgProjectManagerUi", &g_define_type_info, 0);
	return vtg_project_manager_ui_type_id;
}


static void vtg_project_manager_ui_get_property (GObject * object, guint property_id, GValue * value, GParamSpec * pspec) {
	VtgProjectManagerUi * self;
	self = VTG_PROJECT_MANAGER_UI (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_UI_PLUGIN_INSTANCE:
		g_value_set_object (value, vtg_project_manager_ui_get_plugin_instance (self));
		break;
		case VTG_PROJECT_MANAGER_UI_PROJECT_VIEW:
		g_value_set_object (value, vtg_project_manager_ui_get_project_view (self));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void vtg_project_manager_ui_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgProjectManagerUi * self;
	self = VTG_PROJECT_MANAGER_UI (object);
	switch (property_id) {
		case VTG_PROJECT_MANAGER_UI_PLUGIN_INSTANCE:
		vtg_project_manager_ui_set_plugin_instance (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




