/*
 *  vtgsymbolcompletionprovider.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtksourcecompletion/gsc-provider.h>
#include <vsc.h>
#include <gedit/gedit-view.h>
#include <gtksourcecompletion/gsc-proposal.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gtksourcecompletion/gsc-info.h>
#include <gedit/gedit-window.h>
#include <gedit/gedit-statusbar.h>
#include <gtk/gtk.h>
#include <gtksourcecompletion/gsc-trigger.h>
#include <gedit/gedit-document.h>
#include <gdk/gdk.h>
#include <gdk/gdkkeysyms.h>
#include <stdlib.h>
#include <string.h>
#include <float.h>
#include <math.h>
#include <gee.h>


#define VTG_TYPE_SYMBOL_COMPLETION_PROVIDER (vtg_symbol_completion_provider_get_type ())
#define VTG_SYMBOL_COMPLETION_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, VtgSymbolCompletionProvider))
#define VTG_SYMBOL_COMPLETION_PROVIDER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, VtgSymbolCompletionProviderClass))
#define VTG_IS_SYMBOL_COMPLETION_PROVIDER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER))
#define VTG_IS_SYMBOL_COMPLETION_PROVIDER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER))
#define VTG_SYMBOL_COMPLETION_PROVIDER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, VtgSymbolCompletionProviderClass))

typedef struct _VtgSymbolCompletionProvider VtgSymbolCompletionProvider;
typedef struct _VtgSymbolCompletionProviderClass VtgSymbolCompletionProviderClass;
typedef struct _VtgSymbolCompletionProviderPrivate VtgSymbolCompletionProviderPrivate;

#define VTG_TYPE_SYMBOL_COMPLETION_TRIGGER (vtg_symbol_completion_trigger_get_type ())
#define VTG_SYMBOL_COMPLETION_TRIGGER(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTrigger))
#define VTG_SYMBOL_COMPLETION_TRIGGER_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTriggerClass))
#define VTG_IS_SYMBOL_COMPLETION_TRIGGER(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER))
#define VTG_IS_SYMBOL_COMPLETION_TRIGGER_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER))
#define VTG_SYMBOL_COMPLETION_TRIGGER_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_SYMBOL_COMPLETION_TRIGGER, VtgSymbolCompletionTriggerClass))

typedef struct _VtgSymbolCompletionTrigger VtgSymbolCompletionTrigger;
typedef struct _VtgSymbolCompletionTriggerClass VtgSymbolCompletionTriggerClass;

#define VTG_TYPE_PLUGIN_INSTANCE (vtg_plugin_instance_get_type ())
#define VTG_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstance))
#define VTG_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))
#define VTG_IS_PLUGIN_INSTANCE(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_IS_PLUGIN_INSTANCE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_PLUGIN_INSTANCE))
#define VTG_PLUGIN_INSTANCE_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_PLUGIN_INSTANCE, VtgPluginInstanceClass))

typedef struct _VtgPluginInstance VtgPluginInstance;
typedef struct _VtgPluginInstanceClass VtgPluginInstanceClass;
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))
#define __g_list_free_g_object_unref0(var) ((var == NULL) ? NULL : (var = (_g_list_free_g_object_unref (var), NULL)))
typedef struct _VtgSymbolCompletionTriggerPrivate VtgSymbolCompletionTriggerPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _g_timer_destroy0(var) ((var == NULL) ? NULL : (var = (g_timer_destroy (var), NULL)))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _g_string_free0(var) ((var == NULL) ? NULL : (var = (g_string_free (var, TRUE), NULL)))
#define _vsc_symbol_completion_filter_options_unref0(var) ((var == NULL) ? NULL : (var = (vsc_symbol_completion_filter_options_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

struct _VtgSymbolCompletionProvider {
	GObject parent_instance;
	VtgSymbolCompletionProviderPrivate * priv;
};

struct _VtgSymbolCompletionProviderClass {
	GObjectClass parent_class;
};

struct _VtgSymbolCompletionProviderPrivate {
	VscSourceBuffer* _sb;
	VscSymbolCompletion* _completion;
	GeditView* _view;
	GList* list;
	GdkPixbuf* _icon_generic;
	GdkPixbuf* _icon_field;
	GdkPixbuf* _icon_method;
	GdkPixbuf* _icon_class;
	GdkPixbuf* _icon_struct;
	GdkPixbuf* _icon_property;
	GdkPixbuf* _icon_signal;
	GdkPixbuf* _icon_iface;
	GdkPixbuf* _icon_const;
	GdkPixbuf* _icon_enum;
	GdkPixbuf* _icon_namespace;
	guint timeout_id;
	guint idle_id;
	gboolean all_doc;
	gint prealloc_index;
	gint counter;
	gboolean cache_building;
	gboolean prev_cache_building;
	gboolean tooltip_is_visible;
	VtgSymbolCompletionTrigger* _last_trigger;
	guint sb_msg_id;
	guint sb_context_id;
	VtgPluginInstance* _plugin_instance;
	GscInfo* _calltip_window;
	gboolean need_parse;
	gint current_edited_line;
};

struct _VtgSymbolCompletionTrigger {
	GObject parent_instance;
	VtgSymbolCompletionTriggerPrivate * priv;
	gboolean shortcut_triggered;
};

struct _VtgSymbolCompletionTriggerClass {
	GObjectClass parent_class;
};


static gpointer vtg_symbol_completion_provider_parent_class = NULL;
static GscProviderIface* vtg_symbol_completion_provider_gsc_provider_parent_iface = NULL;
static GType vtg_symbol_completion_provider_type_id = 0;

GType vtg_symbol_completion_provider_get_type (void);
GType vtg_symbol_completion_provider_register_type (GTypeModule * module);
GType vtg_symbol_completion_trigger_get_type (void);
GType vtg_symbol_completion_trigger_register_type (GTypeModule * module);
GType vtg_plugin_instance_get_type (void);
GType vtg_plugin_instance_register_type (GTypeModule * module);
#define VTG_SYMBOL_COMPLETION_PROVIDER_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, VtgSymbolCompletionProviderPrivate))
enum  {
	VTG_SYMBOL_COMPLETION_PROVIDER_DUMMY_PROPERTY,
	VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION,
	VTG_SYMBOL_COMPLETION_PROVIDER_VIEW,
	VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN_INSTANCE
};
static void _g_list_free_g_object_unref (GList* self);
static void vtg_symbol_completion_provider_set_plugin_instance (VtgSymbolCompletionProvider* self, VtgPluginInstance* value);
static void vtg_symbol_completion_provider_set_completion (VtgSymbolCompletionProvider* self, VscSymbolCompletion* value);
static void vtg_symbol_completion_provider_set_view (VtgSymbolCompletionProvider* self, GeditView* value);
VtgSymbolCompletionProvider* vtg_symbol_completion_provider_new (VtgPluginInstance* plugin_instance, GeditView* view, VscSymbolCompletion* completion);
VtgSymbolCompletionProvider* vtg_symbol_completion_provider_construct (GType object_type, VtgPluginInstance* plugin_instance, GeditView* view, VscSymbolCompletion* completion);
static gboolean vtg_symbol_completion_provider_on_idle (VtgSymbolCompletionProvider* self);
static gboolean _vtg_symbol_completion_provider_on_idle_gsource_func (gpointer self);
static void vtg_symbol_completion_provider_on_cache_building (VtgSymbolCompletionProvider* self, VscParserManager* sender);
static void vtg_symbol_completion_provider_on_cache_builded (VtgSymbolCompletionProvider* self, VscParserManager* sender);
GeditWindow* vtg_plugin_instance_get_window (VtgPluginInstance* self);
void vtg_symbol_completion_trigger_trigger_event (VtgSymbolCompletionTrigger* self, gboolean trigger_reason);
static void vtg_symbol_completion_provider_parse (VtgSymbolCompletionProvider* self, GeditDocument* doc);
static gboolean vtg_symbol_completion_provider_on_timeout_parse (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_show_calltip (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_hide_calltip (VtgSymbolCompletionProvider* self);
static gboolean _vtg_symbol_completion_provider_on_timeout_parse_gsource_func (gpointer self);
static gboolean vtg_symbol_completion_provider_on_view_key_press (VtgSymbolCompletionProvider* self, GtkTextView* view, const GdkEventKey* evt);
VscSymbolCompletionItem* vtg_symbol_completion_provider_get_current_symbol_completion_item (VtgSymbolCompletionProvider* self);
static void vtg_symbol_completion_provider_initialize_calltip_window (VtgSymbolCompletionProvider* self);
static char* vtg_symbol_completion_provider_get_document_text (VtgSymbolCompletionProvider* self, GeditDocument* doc, gboolean all_doc);
static void vtg_symbol_completion_provider_real_finish (GscProvider* base);
static const char* vtg_symbol_completion_provider_real_get_name (GscProvider* base);
static void vtg_symbol_completion_provider_transform_result (VtgSymbolCompletionProvider* self, VscSymbolCompletionResult* _result_);
static VscSymbolCompletionResult* vtg_symbol_completion_provider_get_completions (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger);
static GList* vtg_symbol_completion_provider_real_get_proposals (GscProvider* base, GscTrigger* trigger);
GscProposal** vtg_utils_get_proposal_cache (int* result_length1);
#define VTG_UTILS_prealloc_count 500
static gint vtg_symbol_completion_provider_proposal_sort (void* a, void* b);
static gint _vtg_symbol_completion_provider_proposal_sort_gcompare_func (void* a, void* b);
static void vtg_symbol_completion_provider_append_symbols (VtgSymbolCompletionProvider* self, GeeList* symbols, GdkPixbuf* icon);
static void vtg_symbol_completion_provider_parse_current_line (VtgSymbolCompletionProvider* self, gboolean skip_leading_spaces, char** symbolname, char** last_symbolname, char** line, gint* lineno, gint* colno);
static gboolean vtg_symbol_completion_provider_is_vala_keyword (VtgSymbolCompletionProvider* self, const char* keyword);
static VscSymbolCompletionResult* vtg_symbol_completion_provider_complete (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger, const char* line, const char* word, gint lineno, gint colno);
static VscSymbolCompletionItem* vtg_symbol_completion_provider_search_completion_item_by_name (VtgSymbolCompletionProvider* self, const char* name, GeeList* items);
static gboolean _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event (GeditView* _sender, const GdkEventKey* event, gpointer self);
char* vtg_utils_get_image_path (const char* id);
static void _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building (VscParserManager* _sender, gpointer self);
static void _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded (VscParserManager* _sender, gpointer self);
static GObject * vtg_symbol_completion_provider_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties);
static void vtg_symbol_completion_provider_finalize (GObject* obj);
static void vtg_symbol_completion_provider_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec);
static int _vala_strcmp0 (const char * str1, const char * str2);



static void _g_list_free_g_object_unref (GList* self) {
	g_list_foreach (self, (GFunc) g_object_unref, NULL);
	g_list_free (self);
}


VtgSymbolCompletionProvider* vtg_symbol_completion_provider_construct (GType object_type, VtgPluginInstance* plugin_instance, GeditView* view, VscSymbolCompletion* completion) {
	GParameter * __params;
	GParameter * __params_it;
	VtgSymbolCompletionProvider * self;
	g_return_val_if_fail (plugin_instance != NULL, NULL);
	g_return_val_if_fail (view != NULL, NULL);
	g_return_val_if_fail (completion != NULL, NULL);
	__params = g_new0 (GParameter, 3);
	__params_it = __params;
	__params_it->name = "plugin-instance";
	g_value_init (&__params_it->value, VTG_TYPE_PLUGIN_INSTANCE);
	g_value_set_object (&__params_it->value, plugin_instance);
	__params_it++;
	__params_it->name = "completion";
	g_value_init (&__params_it->value, VSC_TYPE_SYMBOL_COMPLETION);
	g_value_set_object (&__params_it->value, completion);
	__params_it++;
	__params_it->name = "view";
	g_value_init (&__params_it->value, GEDIT_TYPE_VIEW);
	g_value_set_object (&__params_it->value, view);
	__params_it++;
	self = g_object_newv (object_type, __params_it - __params, __params);
	while (__params_it > __params) {
		--__params_it;
		g_value_unset (&__params_it->value);
	}
	g_free (__params);
	return self;
}


VtgSymbolCompletionProvider* vtg_symbol_completion_provider_new (VtgPluginInstance* plugin_instance, GeditView* view, VscSymbolCompletion* completion) {
	return vtg_symbol_completion_provider_construct (VTG_TYPE_SYMBOL_COMPLETION_PROVIDER, plugin_instance, view, completion);
}


static gboolean _vtg_symbol_completion_provider_on_idle_gsource_func (gpointer self) {
	return vtg_symbol_completion_provider_on_idle (self);
}


static void vtg_symbol_completion_provider_on_cache_building (VtgSymbolCompletionProvider* self, VscParserManager* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (self->priv->cache_building == FALSE) {
		self->priv->cache_building = TRUE;
		self->priv->idle_id = g_idle_add_full (0, _vtg_symbol_completion_provider_on_idle_gsource_func, g_object_ref (self), g_object_unref);
	}
}


static void vtg_symbol_completion_provider_on_cache_builded (VtgSymbolCompletionProvider* self, VscParserManager* sender) {
	g_return_if_fail (self != NULL);
	g_return_if_fail (sender != NULL);
	if (self->priv->cache_building == TRUE) {
		self->priv->cache_building = FALSE;
		if (self->priv->idle_id == 0) {
			self->priv->idle_id = g_idle_add_full (0, _vtg_symbol_completion_provider_on_idle_gsource_func, g_object_ref (self), g_object_unref);
		}
	}
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


static gboolean vtg_symbol_completion_provider_on_idle (VtgSymbolCompletionProvider* self) {
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->_plugin_instance == NULL) {
		result = FALSE;
		return result;
	}
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	if (self->priv->cache_building) {
		_tmp1_ = !self->priv->tooltip_is_visible;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		_tmp0_ = self->priv->prev_cache_building == FALSE;
	} else {
		_tmp0_ = FALSE;
	}
	if (_tmp0_) {
		GeditStatusbar* status_bar;
		self->priv->prev_cache_building = self->priv->cache_building;
		status_bar = _g_object_ref0 (GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_instance_get_window (self->priv->_plugin_instance))));
		if (self->priv->sb_msg_id != 0) {
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
		}
		self->priv->sb_msg_id = gtk_statusbar_push ((GtkStatusbar*) status_bar, self->priv->sb_context_id, "rebuilding symbol cache...");
		_g_object_unref0 (status_bar);
	} else {
		gboolean _tmp2_;
		_tmp2_ = FALSE;
		if (self->priv->cache_building == FALSE) {
			_tmp2_ = self->priv->prev_cache_building == TRUE;
		} else {
			_tmp2_ = FALSE;
		}
		if (_tmp2_) {
			GeditStatusbar* status_bar;
			self->priv->prev_cache_building = FALSE;
			status_bar = _g_object_ref0 (GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_instance_get_window (self->priv->_plugin_instance))));
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
			self->priv->sb_msg_id = (guint) 0;
			if (self->priv->_last_trigger != NULL) {
				VtgSymbolCompletionTrigger* trigger;
				trigger = _g_object_ref0 (VTG_SYMBOL_COMPLETION_TRIGGER (self->priv->_last_trigger));
				vtg_symbol_completion_trigger_trigger_event (trigger, trigger->shortcut_triggered);
				_g_object_unref0 (trigger);
			}
			_g_object_unref0 (status_bar);
		}
	}
	self->priv->idle_id = (guint) 0;
	result = FALSE;
	return result;
}


static gboolean vtg_symbol_completion_provider_on_timeout_parse (VtgSymbolCompletionProvider* self) {
	gboolean result;
	g_return_val_if_fail (self != NULL, FALSE);
	if (self->priv->counter == 0) {
		vtg_symbol_completion_provider_parse (self, GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view)));
		self->priv->timeout_id = (guint) 0;
		result = FALSE;
		return result;
	} else {
		self->priv->counter--;
		result = TRUE;
		return result;
	}
}


static gboolean _vtg_symbol_completion_provider_on_timeout_parse_gsource_func (gpointer self) {
	return vtg_symbol_completion_provider_on_timeout_parse (self);
}


static gboolean vtg_symbol_completion_provider_on_view_key_press (VtgSymbolCompletionProvider* self, GtkTextView* view, const GdkEventKey* evt) {
	gboolean result;
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter start = {0};
	gint line;
	gunichar ch;
	gboolean _tmp9_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (view != NULL, FALSE);
	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	line = gtk_text_iter_get_line (&start);
	ch = (gunichar) gdk_keyval_to_unicode ((*evt).keyval);
	if (ch == '(') {
		vtg_symbol_completion_provider_show_calltip (self);
	} else {
		gboolean _tmp0_;
		gboolean _tmp1_;
		gboolean _tmp2_;
		_tmp0_ = FALSE;
		_tmp1_ = FALSE;
		_tmp2_ = FALSE;
		if ((*evt).keyval == GDK_Escape) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = ch == ')';
		}
		if (_tmp2_) {
			_tmp1_ = TRUE;
		} else {
			_tmp1_ = ch == ';';
		}
		if (_tmp1_) {
			_tmp0_ = TRUE;
		} else {
			gboolean _tmp3_;
			_tmp3_ = FALSE;
			if ((*evt).keyval == GDK_Return) {
				_tmp3_ = ((*evt).state & GDK_SHIFT_MASK) != 0;
			} else {
				_tmp3_ = FALSE;
			}
			_tmp0_ = _tmp3_;
		}
		if (_tmp0_) {
			vtg_symbol_completion_provider_hide_calltip (self);
		}
	}
	if (self->priv->counter <= 0) {
		gboolean _tmp4_;
		_tmp4_ = FALSE;
		if ((*evt).keyval == GDK_Return) {
			_tmp4_ = TRUE;
		} else {
			_tmp4_ = ch == ';';
		}
		if (_tmp4_) {
			self->priv->all_doc = TRUE;
			self->priv->counter = 0;
			self->priv->current_edited_line = -1;
		} else {
			gboolean _tmp5_;
			gboolean _tmp6_;
			_tmp5_ = FALSE;
			_tmp6_ = FALSE;
			if (g_unichar_isprint (ch)) {
				_tmp6_ = TRUE;
			} else {
				_tmp6_ = (*evt).keyval == GDK_Delete;
			}
			if (_tmp6_) {
				_tmp5_ = TRUE;
			} else {
				_tmp5_ = (*evt).keyval == GDK_BackSpace;
			}
			if (_tmp5_) {
				self->priv->need_parse = TRUE;
			} else {
				gboolean _tmp7_;
				_tmp7_ = FALSE;
				if ((*evt).keyval == GDK_Up) {
					_tmp7_ = TRUE;
				} else {
					_tmp7_ = (*evt).keyval == GDK_Down;
				}
				if (_tmp7_) {
					self->priv->current_edited_line = -1;
				}
			}
		}
	} else {
		gboolean _tmp8_;
		_tmp8_ = FALSE;
		if ((*evt).keyval == GDK_Return) {
			_tmp8_ = TRUE;
		} else {
			_tmp8_ = ch == ';';
		}
		if (_tmp8_) {
			self->priv->all_doc = TRUE;
			self->priv->counter = 0;
		} else {
			self->priv->all_doc = FALSE;
			self->priv->counter = 5;
		}
	}
	_tmp9_ = FALSE;
	if (self->priv->need_parse) {
		_tmp9_ = self->priv->current_edited_line != line;
	} else {
		_tmp9_ = FALSE;
	}
	if (_tmp9_) {
		self->priv->need_parse = FALSE;
		self->priv->current_edited_line = line;
		self->priv->timeout_id = g_timeout_add_full (0, (guint) 25, _vtg_symbol_completion_provider_on_timeout_parse_gsource_func, g_object_ref (self), g_object_unref);
	}
	result = FALSE;
	return result;
}


static void vtg_symbol_completion_provider_show_calltip (VtgSymbolCompletionProvider* self) {
	VscSymbolCompletionItem* completion_result;
	g_return_if_fail (self != NULL);
	completion_result = vtg_symbol_completion_provider_get_current_symbol_completion_item (self);
	if (completion_result != NULL) {
		char* calltip_text;
		if (self->priv->_calltip_window == NULL) {
			vtg_symbol_completion_provider_initialize_calltip_window (self);
		}
		calltip_text = g_strdup (completion_result->info);
		if (calltip_text != NULL) {
			gsc_info_set_markup (self->priv->_calltip_window, calltip_text);
			gsc_info_move_to_cursor (self->priv->_calltip_window, (GtkTextView*) self->priv->_view);
			gtk_widget_show_all ((GtkWidget*) self->priv->_calltip_window);
		}
		_g_free0 (calltip_text);
	}
	_g_object_unref0 (completion_result);
}


static void vtg_symbol_completion_provider_hide_calltip (VtgSymbolCompletionProvider* self) {
	g_return_if_fail (self != NULL);
	if (self->priv->_calltip_window == NULL) {
		return;
	}
	gtk_widget_hide ((GtkWidget*) self->priv->_calltip_window);
}


static void vtg_symbol_completion_provider_initialize_calltip_window (VtgSymbolCompletionProvider* self) {
	GscInfo* _tmp0_;
	GtkWidget* child;
	g_return_if_fail (self != NULL);
	self->priv->_calltip_window = (_tmp0_ = g_object_ref_sink (gsc_info_new ()), _g_object_unref0 (self->priv->_calltip_window), _tmp0_);
	gtk_window_set_transient_for ((GtkWindow*) self->priv->_calltip_window, (GtkWindow*) vtg_plugin_instance_get_window (self->priv->_plugin_instance));
	gsc_info_set_adjust_width (self->priv->_calltip_window, TRUE, 800);
	gsc_info_set_adjust_height (self->priv->_calltip_window, TRUE, 600);
	g_object_set ((GtkWindow*) self->priv->_calltip_window, "allow-grow", TRUE, NULL);
	g_object_set ((GtkWindow*) self->priv->_calltip_window, "allow-shrink", TRUE, NULL);
	child = _g_object_ref0 (gtk_bin_get_child ((GtkBin*) self->priv->_calltip_window));
	while (TRUE) {
		gboolean _tmp1_;
		_tmp1_ = FALSE;
		if (child != NULL) {
			_tmp1_ = !GTK_IS_LABEL (child);
		} else {
			_tmp1_ = FALSE;
		}
		if (!_tmp1_) {
			break;
		}
		if (GTK_IS_BIN (child)) {
			GtkWidget* _tmp2_;
			child = (_tmp2_ = _g_object_ref0 (gtk_bin_get_child (GTK_BIN (child))), _g_object_unref0 (child), _tmp2_);
		} else {
			GtkWidget* _tmp3_;
			child = (_tmp3_ = NULL, _g_object_unref0 (child), _tmp3_);
		}
	}
	if (GTK_IS_LABEL (child)) {
		GtkLabel* l;
		l = _g_object_ref0 (GTK_LABEL (child));
		g_object_set ((GtkMisc*) l, "xalign", (float) 0, NULL);
		g_object_set ((GtkMisc*) l, "xpad", 8, NULL);
		_g_object_unref0 (l);
	}
	_g_object_unref0 (child);
}


static void vtg_symbol_completion_provider_parse (VtgSymbolCompletionProvider* self, GeditDocument* doc) {
	char* buffer;
	g_return_if_fail (self != NULL);
	g_return_if_fail (doc != NULL);
	buffer = vtg_symbol_completion_provider_get_document_text (self, doc, self->priv->all_doc);
	vsc_source_buffer_set_source (self->priv->_sb, buffer);
	vsc_parser_manager_reparse_source_buffers (vsc_symbol_completion_get_parser (self->priv->_completion));
	_g_free0 (buffer);
}


static void vtg_symbol_completion_provider_real_finish (GscProvider* base) {
	VtgSymbolCompletionProvider * self;
	GList* _tmp0_;
	self = (VtgSymbolCompletionProvider*) base;
	self->priv->list = (_tmp0_ = NULL, __g_list_free_g_object_unref0 (self->priv->list), _tmp0_);
}


static const char* vtg_symbol_completion_provider_real_get_name (GscProvider* base) {
	VtgSymbolCompletionProvider * self;
	const char* result;
	self = (VtgSymbolCompletionProvider*) base;
	result = "SymbolCompletionProvider";
	return result;
}


static GList* vtg_symbol_completion_provider_real_get_proposals (GscProvider* base, GscTrigger* trigger) {
	VtgSymbolCompletionProvider * self;
	GList* result;
	GTimer* timer;
	VscSymbolCompletionResult* _tmp0_;
	gboolean _tmp1_;
	GList* _tmp4_;
	self = (VtgSymbolCompletionProvider*) base;
	g_return_val_if_fail (trigger != NULL, NULL);
	timer = g_timer_new ();
	vtg_symbol_completion_provider_transform_result (self, _tmp0_ = vtg_symbol_completion_provider_get_completions (self, VTG_SYMBOL_COMPLETION_TRIGGER (trigger)));
	_g_object_unref0 (_tmp0_);
	g_timer_stop (timer);
	_tmp1_ = FALSE;
	if (g_list_length (self->priv->list) == 0) {
		_tmp1_ = self->priv->cache_building;
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		VtgSymbolCompletionTrigger* _tmp2_;
		self->priv->_last_trigger = (_tmp2_ = _g_object_ref0 (VTG_SYMBOL_COMPLETION_TRIGGER (trigger)), _g_object_unref0 (self->priv->_last_trigger), _tmp2_);
	} else {
		VtgSymbolCompletionTrigger* _tmp3_;
		self->priv->_last_trigger = (_tmp3_ = NULL, _g_object_unref0 (self->priv->_last_trigger), _tmp3_);
	}
	result = (_tmp4_ = self->priv->list, self->priv->list = NULL, _tmp4_);
	_g_timer_destroy0 (timer);
	return result;
	_g_timer_destroy0 (timer);
}


static gint _vtg_symbol_completion_provider_proposal_sort_gcompare_func (void* a, void* b) {
	return vtg_symbol_completion_provider_proposal_sort (a, b);
}


static void vtg_symbol_completion_provider_append_symbols (VtgSymbolCompletionProvider* self, GeeList* symbols, GdkPixbuf* icon) {
	GscProposal** _tmp1_;
	gint proposals_size;
	gint proposals_length1;
	gint _tmp0_;
	GscProposal** proposals;
	g_return_if_fail (self != NULL);
	g_return_if_fail (symbols != NULL);
	g_return_if_fail (icon != NULL);
	proposals = (_tmp1_ = vtg_utils_get_proposal_cache (&_tmp0_), proposals_length1 = _tmp0_, proposals_size = proposals_length1, _tmp1_);
	{
		GeeIterator* _symbol_it;
		_symbol_it = gee_iterable_iterator ((GeeIterable*) symbols);
		while (TRUE) {
			VscSymbolCompletionItem* symbol;
			GscProposal* proposal;
			const char* _tmp2_;
			char* name;
			const char* _tmp3_;
			char* info;
			if (!gee_iterator_next (_symbol_it)) {
				break;
			}
			symbol = (VscSymbolCompletionItem*) gee_iterator_get (_symbol_it);
			proposal = NULL;
			_tmp2_ = NULL;
			if (symbol->name != NULL) {
				_tmp2_ = symbol->name;
			} else {
				_tmp2_ = "<null>";
			}
			name = g_strdup (_tmp2_);
			_tmp3_ = NULL;
			if (symbol->info != NULL) {
				_tmp3_ = symbol->info;
			} else {
				_tmp3_ = "";
			}
			info = g_strdup (_tmp3_);
			if (self->priv->prealloc_index < VTG_UTILS_prealloc_count) {
				GscProposal* _tmp4_;
				proposal = (_tmp4_ = _g_object_ref0 (proposals[self->priv->prealloc_index]), _g_object_unref0 (proposal), _tmp4_);
				self->priv->prealloc_index++;
				g_object_set (proposal, "label", name, NULL);
				g_object_set (proposal, "info", info, NULL);
				g_object_set (proposal, "icon", icon, NULL);
			} else {
				GscProposal* _tmp5_;
				proposal = (_tmp5_ = gsc_proposal_new (name, info, icon), _g_object_unref0 (proposal), _tmp5_);
			}
			self->priv->list = g_list_append (self->priv->list, _g_object_ref0 (proposal));
			_g_object_unref0 (symbol);
			_g_object_unref0 (proposal);
			_g_free0 (name);
			_g_free0 (info);
		}
		_gee_collection_object_unref0 (_symbol_it);
	}
	self->priv->list = g_list_sort (self->priv->list, _vtg_symbol_completion_provider_proposal_sort_gcompare_func);
}


static gint vtg_symbol_completion_provider_proposal_sort (void* a, void* b) {
	gint result;
	GscProposal* pa;
	GscProposal* pb;
	pa = _g_object_ref0 (GSC_PROPOSAL (a));
	pb = _g_object_ref0 (GSC_PROPOSAL (b));
	result = strcmp (gsc_proposal_get_label (pa), gsc_proposal_get_label (pb));
	_g_object_unref0 (pa);
	_g_object_unref0 (pb);
	return result;
}


static void vtg_symbol_completion_provider_transform_result (VtgSymbolCompletionProvider* self, VscSymbolCompletionResult* _result_) {
	GTimer* timer;
	GList* _tmp0_;
	gboolean _tmp1_;
	g_return_if_fail (self != NULL);
	timer = g_timer_new ();
	self->priv->prealloc_index = 0;
	self->priv->list = (_tmp0_ = NULL, __g_list_free_g_object_unref0 (self->priv->list), _tmp0_);
	_tmp1_ = FALSE;
	if (_result_ != NULL) {
		_tmp1_ = !vsc_symbol_completion_result_get_is_empty (_result_);
	} else {
		_tmp1_ = FALSE;
	}
	if (_tmp1_) {
		if (gee_collection_get_size ((GeeCollection*) _result_->fields) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->fields, self->priv->_icon_field);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->properties) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->properties, self->priv->_icon_property);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->methods) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->methods, self->priv->_icon_method);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->signals) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->signals, self->priv->_icon_signal);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->classes) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->classes, self->priv->_icon_class);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->interfaces) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->interfaces, self->priv->_icon_iface);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->structs) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->structs, self->priv->_icon_struct);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->enums) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->enums, self->priv->_icon_enum);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->error_domains) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->error_domains, self->priv->_icon_struct);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->constants) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->constants, self->priv->_icon_const);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->namespaces) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->namespaces, self->priv->_icon_namespace);
		}
		if (gee_collection_get_size ((GeeCollection*) _result_->others) > 0) {
			vtg_symbol_completion_provider_append_symbols (self, _result_->others, self->priv->_icon_generic);
		}
	}
	g_timer_stop (timer);
	_g_timer_destroy0 (timer);
}


static void vtg_symbol_completion_provider_parse_current_line (VtgSymbolCompletionProvider* self, gboolean skip_leading_spaces, char** symbolname, char** last_symbolname, char** line, gint* lineno, gint* colno) {
	GeditDocument* doc;
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	gunichar ch;
	char* _tmp0_;
	char* _tmp1_;
	char* _tmp2_;
	char* _tmp3_;
	g_return_if_fail (self != NULL);
	if (symbolname != NULL) {
		*symbolname = NULL;
	}
	if (last_symbolname != NULL) {
		*last_symbolname = NULL;
	}
	if (line != NULL) {
		*line = NULL;
	}
	doc = GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view));
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	ch = 0U;
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	(*lineno) = gtk_text_iter_get_line (&start);
	ch = gtk_text_iter_get_char (&start);
	while (TRUE) {
		gint cline;
		if (!g_unichar_isalnum (ch)) {
			break;
		}
		gtk_text_iter_forward_char (&start);
		cline = gtk_text_iter_get_line (&start);
		if ((*lineno) != cline) {
			gtk_text_iter_backward_char (&start);
			break;
		}
		ch = gtk_text_iter_get_char (&start);
	}
	(*colno) = gtk_text_iter_get_line_offset (&start);
	(*symbolname) = (_tmp0_ = g_strdup (""), _g_free0 ((*symbolname)), _tmp0_);
	(*last_symbolname) = (_tmp1_ = g_strdup (""), _g_free0 ((*last_symbolname)), _tmp1_);
	(*line) = (_tmp2_ = g_strdup (""), _g_free0 ((*line)), _tmp2_);
	end = start;
	gtk_text_iter_set_line_offset (&start, 0);
	(*line) = (_tmp3_ = g_strdup (gtk_text_iter_get_text (&start, &end)), _g_free0 ((*line)), _tmp3_);
	if ((*colno) > 0) {
		GString* tmp;
		gint bracket_lev_1;
		gint bracket_lev_2;
		gint bracket_lev_3;
		gint string_lev;
		gint _tmp4_;
		gint status;
		gunichar prev_char;
		char* _tmp27_;
		tmp = g_string_new ("");
		bracket_lev_1 = 0;
		bracket_lev_2 = 0;
		bracket_lev_3 = 0;
		string_lev = 0;
		_tmp4_ = 0;
		if (skip_leading_spaces) {
			_tmp4_ = 1;
		} else {
			_tmp4_ = 0;
		}
		status = _tmp4_;
		prev_char = gtk_text_iter_get_char (&end);
		while (TRUE) {
			gboolean _tmp5_;
			gboolean _tmp6_;
			gboolean _tmp8_;
			if (!gtk_text_iter_backward_char (&end)) {
				break;
			}
			_tmp5_ = FALSE;
			if (gtk_text_iter_get_line_offset (&end) == 0) {
				_tmp5_ = TRUE;
			} else {
				_tmp5_ = gtk_text_iter_get_line (&end) < (*lineno);
			}
			if (_tmp5_) {
				break;
			}
			ch = gtk_text_iter_get_char (&end);
			_tmp6_ = FALSE;
			if (status == 1) {
				gboolean _tmp7_;
				_tmp7_ = FALSE;
				if (ch != ' ') {
					_tmp7_ = ch != '\t';
				} else {
					_tmp7_ = FALSE;
				}
				_tmp6_ = _tmp7_;
			} else {
				_tmp6_ = FALSE;
			}
			if (_tmp6_) {
				status = 0;
			}
			_tmp8_ = FALSE;
			if (status == 0) {
				gboolean _tmp9_;
				gboolean _tmp10_;
				gboolean _tmp11_;
				gboolean _tmp12_;
				_tmp9_ = FALSE;
				_tmp10_ = FALSE;
				_tmp11_ = FALSE;
				_tmp12_ = FALSE;
				if (ch == ' ') {
					_tmp12_ = TRUE;
				} else {
					_tmp12_ = ch == '\t';
				}
				if (_tmp12_) {
					_tmp11_ = TRUE;
				} else {
					gboolean _tmp13_;
					_tmp13_ = FALSE;
					if (ch == '(') {
						_tmp13_ = bracket_lev_1 == 0;
					} else {
						_tmp13_ = FALSE;
					}
					_tmp11_ = _tmp13_;
				}
				if (_tmp11_) {
					_tmp10_ = TRUE;
				} else {
					gboolean _tmp14_;
					_tmp14_ = FALSE;
					if (ch == '[') {
						_tmp14_ = bracket_lev_2 == 0;
					} else {
						_tmp14_ = FALSE;
					}
					_tmp10_ = _tmp14_;
				}
				if (_tmp10_) {
					_tmp9_ = TRUE;
				} else {
					gboolean _tmp15_;
					_tmp15_ = FALSE;
					if (ch == '{') {
						_tmp15_ = bracket_lev_3 == 0;
					} else {
						_tmp15_ = FALSE;
					}
					_tmp9_ = _tmp15_;
				}
				_tmp8_ = _tmp9_;
			} else {
				_tmp8_ = FALSE;
			}
			if (_tmp8_) {
				break;
			}
			switch (status) {
				case 0:
				{
					if (ch == ')') {
						status = 2;
						bracket_lev_1++;
					} else {
						if (ch == ']') {
							status = 2;
							bracket_lev_2++;
						} else {
							if (ch == '}') {
								status = 2;
								bracket_lev_3++;
							} else {
								gboolean _tmp16_;
								_tmp16_ = FALSE;
								if (ch == '.') {
									_tmp16_ = TRUE;
								} else {
									gboolean _tmp17_;
									_tmp17_ = FALSE;
									if (ch == '-') {
										_tmp17_ = prev_char == '>';
									} else {
										_tmp17_ = FALSE;
									}
									_tmp16_ = _tmp17_;
								}
								if (_tmp16_) {
									gboolean _tmp19_;
									if (_vala_strcmp0 ((*last_symbolname), "") == 0) {
										char* _tmp18_;
										(*last_symbolname) = (_tmp18_ = g_utf8_strreverse (tmp->str, -1), _g_free0 ((*last_symbolname)), _tmp18_);
									}
									_tmp19_ = FALSE;
									if (ch == '-') {
										_tmp19_ = prev_char == '>';
									} else {
										_tmp19_ = FALSE;
									}
									if (_tmp19_) {
										g_string_append_unichar (tmp, (gunichar) '.');
									} else {
										g_string_append_unichar (tmp, ch);
									}
									status = 1;
								} else {
									if (ch == '=') {
										if (_vala_strcmp0 ((*last_symbolname), "") == 0) {
											char* _tmp20_;
											(*last_symbolname) = (_tmp20_ = g_utf8_strreverse (tmp->str, -1), _g_free0 ((*last_symbolname)), _tmp20_);
										}
										g_string_truncate (tmp, (gsize) 0);
										status = 1;
									} else {
										if (ch == '\"') {
											string_lev++;
											status = 3;
										} else {
											if (ch != '\n') {
												g_string_append_unichar (tmp, ch);
											}
										}
									}
								}
							}
						}
					}
					break;
				}
				case 1:
				{
					break;
				}
				case 2:
				{
					gboolean _tmp21_;
					gboolean _tmp24_;
					gboolean _tmp25_;
					_tmp21_ = FALSE;
					if (bracket_lev_1 > 0) {
						_tmp21_ = ch == '(';
					} else {
						_tmp21_ = FALSE;
					}
					if (_tmp21_) {
						bracket_lev_1--;
					} else {
						gboolean _tmp22_;
						_tmp22_ = FALSE;
						if (bracket_lev_2 > 0) {
							_tmp22_ = ch == '[';
						} else {
							_tmp22_ = FALSE;
						}
						if (_tmp22_) {
							bracket_lev_2--;
						} else {
							gboolean _tmp23_;
							_tmp23_ = FALSE;
							if (bracket_lev_3 > 0) {
								_tmp23_ = ch == '{';
							} else {
								_tmp23_ = FALSE;
							}
							if (_tmp23_) {
								bracket_lev_3--;
							} else {
								if (ch == ')') {
									bracket_lev_1++;
								} else {
									if (ch == ']') {
										bracket_lev_2++;
									} else {
										if (ch == '{') {
											bracket_lev_3++;
										}
									}
								}
							}
						}
					}
					_tmp24_ = FALSE;
					_tmp25_ = FALSE;
					if (bracket_lev_1 <= 0) {
						_tmp25_ = bracket_lev_2 <= 0;
					} else {
						_tmp25_ = FALSE;
					}
					if (_tmp25_) {
						_tmp24_ = bracket_lev_3 <= 0;
					} else {
						_tmp24_ = FALSE;
					}
					if (_tmp24_) {
						status = 1;
					}
					break;
				}
				case 3:
				{
					if (ch == '\"') {
						char* _tmp26_;
						g_string_append (tmp, _tmp26_ = g_utf8_strreverse ("string", -1));
						_g_free0 (_tmp26_);
						string_lev--;
						if (string_lev <= 0) {
							status = 0;
						}
					}
					break;
				}
			}
			prev_char = ch;
		}
		(*symbolname) = (_tmp27_ = g_utf8_strreverse (tmp->str, -1), _g_free0 ((*symbolname)), _tmp27_);
		if (_vala_strcmp0 ((*last_symbolname), "") == 0) {
			char* _tmp28_;
			(*last_symbolname) = (_tmp28_ = g_strdup ((*symbolname)), _g_free0 ((*last_symbolname)), _tmp28_);
		}
		_g_string_free0 (tmp);
	}
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


VscSymbolCompletionItem* vtg_symbol_completion_provider_get_current_symbol_completion_item (VtgSymbolCompletionProvider* self) {
	VscSymbolCompletionItem* result;
	char* line;
	char* word;
	char* last_part;
	gint lineno;
	gint colno;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	gboolean _tmp6_;
	char* first_part;
	char* symbol_name;
	VscSymbolCompletionResult* completion_result;
	VscSymbolCompletionResult* _tmp9_;
	gboolean _tmp10_;
	g_return_val_if_fail (self != NULL, NULL);
	line = NULL;
	word = NULL;
	last_part = NULL;
	lineno = 0;
	colno = 0;
	_tmp4_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	((vtg_symbol_completion_provider_parse_current_line (self, TRUE, &_tmp0_, &_tmp2_, &_tmp4_, &lineno, &colno), word = (_tmp1_ = _tmp0_, _g_free0 (word), _tmp1_)), last_part = (_tmp3_ = _tmp2_, _g_free0 (last_part), _tmp3_));
	line = (_tmp5_ = _tmp4_, _g_free0 (line), _tmp5_);
	_tmp6_ = FALSE;
	if (word == NULL) {
		_tmp6_ = TRUE;
	} else {
		_tmp6_ = _vala_strcmp0 (word, "") == 0;
	}
	if (_tmp6_) {
		result = NULL;
		_g_free0 (line);
		_g_free0 (word);
		_g_free0 (last_part);
		return result;
	}
	first_part = NULL;
	if (_vala_strcmp0 (word, last_part) != 0) {
		char* _tmp7_;
		first_part = (_tmp7_ = string_substring (word, (glong) 0, (string_get_length (word) - string_get_length (last_part)) - 1), _g_free0 (first_part), _tmp7_);
	} else {
		char* _tmp8_;
		first_part = (_tmp8_ = g_strdup ("this"), _g_free0 (first_part), _tmp8_);
	}
	symbol_name = g_strdup (last_part);
	if (vtg_symbol_completion_provider_is_vala_keyword (self, symbol_name)) {
		result = NULL;
		_g_free0 (line);
		_g_free0 (word);
		_g_free0 (last_part);
		_g_free0 (first_part);
		_g_free0 (symbol_name);
		return result;
	}
	completion_result = NULL;
	completion_result = (_tmp9_ = vtg_symbol_completion_provider_complete (self, NULL, line, first_part, lineno, colno), _g_object_unref0 (completion_result), _tmp9_);
	_tmp10_ = FALSE;
	if (completion_result != NULL) {
		_tmp10_ = vsc_symbol_completion_result_get_count (completion_result) > 0;
	} else {
		_tmp10_ = FALSE;
	}
	if (_tmp10_) {
		VscSymbolCompletionItem* item;
		VscSymbolCompletionItem* _tmp11_;
		VscSymbolCompletionItem* _tmp12_;
		VscSymbolCompletionItem* _tmp13_;
		VscSymbolCompletionItem* _tmp14_;
		VscSymbolCompletionItem* _tmp15_;
		VscSymbolCompletionItem* _tmp16_;
		VscSymbolCompletionItem* _tmp17_;
		VscSymbolCompletionItem* _tmp18_;
		VscSymbolCompletionItem* _tmp19_;
		VscSymbolCompletionItem* _tmp20_;
		VscSymbolCompletionItem* _tmp21_;
		VscSymbolCompletionItem* _tmp22_;
		item = NULL;
		item = (_tmp11_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->methods), _g_object_unref0 (item), _tmp11_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp12_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->properties), _g_object_unref0 (item), _tmp12_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp13_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->classes), _g_object_unref0 (item), _tmp13_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp14_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->interfaces), _g_object_unref0 (item), _tmp14_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp15_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->structs), _g_object_unref0 (item), _tmp15_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp16_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->fields), _g_object_unref0 (item), _tmp16_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp17_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->signals), _g_object_unref0 (item), _tmp17_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp18_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->others), _g_object_unref0 (item), _tmp18_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp19_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->namespaces), _g_object_unref0 (item), _tmp19_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp20_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->enums), _g_object_unref0 (item), _tmp20_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp21_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->constants), _g_object_unref0 (item), _tmp21_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		item = (_tmp22_ = vtg_symbol_completion_provider_search_completion_item_by_name (self, symbol_name, completion_result->error_domains), _g_object_unref0 (item), _tmp22_);
		if (item != NULL) {
			result = item;
			_g_free0 (line);
			_g_free0 (word);
			_g_free0 (last_part);
			_g_free0 (first_part);
			_g_free0 (symbol_name);
			_g_object_unref0 (completion_result);
			return result;
		}
		_g_object_unref0 (item);
	}
	result = NULL;
	_g_free0 (line);
	_g_free0 (word);
	_g_free0 (last_part);
	_g_free0 (first_part);
	_g_free0 (symbol_name);
	_g_object_unref0 (completion_result);
	return result;
}


static VscSymbolCompletionItem* vtg_symbol_completion_provider_search_completion_item_by_name (VtgSymbolCompletionProvider* self, const char* name, GeeList* items) {
	VscSymbolCompletionItem* result;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (name != NULL, NULL);
	g_return_val_if_fail (items != NULL, NULL);
	{
		GeeIterator* _item_it;
		_item_it = gee_iterable_iterator ((GeeIterable*) items);
		while (TRUE) {
			VscSymbolCompletionItem* item;
			if (!gee_iterator_next (_item_it)) {
				break;
			}
			item = (VscSymbolCompletionItem*) gee_iterator_get (_item_it);
			if (_vala_strcmp0 (item->name, name) == 0) {
				result = item;
				_gee_collection_object_unref0 (_item_it);
				return result;
			}
			_g_object_unref0 (item);
		}
		_gee_collection_object_unref0 (_item_it);
	}
	result = NULL;
	return result;
}


static VscSymbolCompletionResult* vtg_symbol_completion_provider_get_completions (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger) {
	VscSymbolCompletionResult* result;
	char* line;
	char* word;
	char* last_part;
	gint lineno;
	gint colno;
	char* _tmp5_;
	char* _tmp4_;
	char* _tmp3_;
	char* _tmp2_;
	char* _tmp1_;
	char* _tmp0_;
	gboolean _tmp6_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (trigger != NULL, NULL);
	line = NULL;
	word = NULL;
	last_part = NULL;
	lineno = 0;
	colno = 0;
	_tmp4_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	((vtg_symbol_completion_provider_parse_current_line (self, FALSE, &_tmp0_, &_tmp2_, &_tmp4_, &lineno, &colno), word = (_tmp1_ = _tmp0_, _g_free0 (word), _tmp1_)), last_part = (_tmp3_ = _tmp2_, _g_free0 (last_part), _tmp3_));
	line = (_tmp5_ = _tmp4_, _g_free0 (line), _tmp5_);
	_tmp6_ = FALSE;
	if (word == NULL) {
		_tmp6_ = _vala_strcmp0 (word, "") == 0;
	} else {
		_tmp6_ = FALSE;
	}
	if (_tmp6_) {
		result = NULL;
		_g_free0 (line);
		_g_free0 (word);
		_g_free0 (last_part);
		return result;
	}
	result = vtg_symbol_completion_provider_complete (self, trigger, line, word, lineno, colno);
	_g_free0 (line);
	_g_free0 (word);
	_g_free0 (last_part);
	return result;
}


static VscSymbolCompletionResult* vtg_symbol_completion_provider_complete (VtgSymbolCompletionProvider* self, VtgSymbolCompletionTrigger* trigger, const char* line, const char* word, gint lineno, gint colno) {
	VscSymbolCompletionResult* result;
	GError * _inner_error_;
	VscSymbolCompletionResult* _result_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (line != NULL, NULL);
	g_return_val_if_fail (word != NULL, NULL);
	_inner_error_ = NULL;
	_result_ = NULL;
	{
		char* typename;
		GTimer* timer;
		char* _tmp0_;
		char* _tmp1_;
		VscSymbolCompletionFilterOptions* options;
		gboolean _tmp2_;
		gboolean search_error_domains;
		gboolean search_error_base;
		typename = NULL;
		timer = g_timer_new ();
		_tmp0_ = vsc_symbol_completion_get_datatype_name_for_name (self->priv->_completion, word, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &_inner_error_);
		if (_inner_error_ != NULL) {
			_g_free0 (typename);
			_g_timer_destroy0 (timer);
			goto __catch0_g_error;
			goto __finally0;
		}
		typename = (_tmp1_ = _tmp0_, _g_free0 (typename), _tmp1_);
		options = vsc_symbol_completion_filter_options_new ();
		vsc_symbol_completion_filter_options_public_only (options);
		_tmp2_ = FALSE;
		if (strstr (line, "= new ") != NULL) {
			_tmp2_ = TRUE;
		} else {
			_tmp2_ = strstr (line, "=new ") != NULL;
		}
		if (_tmp2_) {
			options->constructors = TRUE;
			options->instance_symbols = FALSE;
			options->static_symbols = FALSE;
		}
		search_error_domains = FALSE;
		search_error_base = FALSE;
		if (strstr (line, "throws ") != NULL) {
			search_error_domains = TRUE;
			search_error_base = TRUE;
		} else {
			if (strstr (line, "throw ") != NULL) {
				search_error_domains = TRUE;
			}
		}
		if (typename != NULL) {
			char* _tmp3_;
			VscSymbolCompletionResult* _tmp4_;
			VscSymbolCompletionResult* _tmp5_;
			options->static_symbols = FALSE;
			options->error_domains = search_error_domains;
			options->error_base = search_error_base;
			if (_vala_strcmp0 (word, "this") == 0) {
				options->private_symbols = TRUE;
				options->protected_symbols = TRUE;
			} else {
				if (_vala_strcmp0 (word, "base") == 0) {
					options->protected_symbols = TRUE;
				}
			}
			options->exclude_type = (_tmp3_ = g_strdup (typename), _g_free0 (options->exclude_type), _tmp3_);
			g_timer_start (timer);
			_tmp4_ = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, typename, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_free0 (typename);
				_g_timer_destroy0 (timer);
				_vsc_symbol_completion_filter_options_unref0 (options);
				goto __catch0_g_error;
				goto __finally0;
			}
			_result_ = (_tmp5_ = _tmp4_, _g_object_unref0 (_result_), _tmp5_);
			g_timer_stop (timer);
		} else {
			gboolean _tmp6_;
			_tmp6_ = FALSE;
			if (g_str_has_prefix (word, "this.") == FALSE) {
				_tmp6_ = g_str_has_prefix (word, "base.") == FALSE;
			} else {
				_tmp6_ = FALSE;
			}
			if (_tmp6_) {
				VscSymbolCompletionResult* _tmp7_;
				VscSymbolCompletionResult* _tmp8_;
				gboolean _tmp9_;
				options->static_symbols = TRUE;
				options->interface_symbols = FALSE;
				options->error_domains = search_error_domains;
				options->error_base = search_error_base;
				g_timer_start (timer);
				_tmp7_ = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, word, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &_inner_error_);
				if (_inner_error_ != NULL) {
					_g_free0 (typename);
					_g_timer_destroy0 (timer);
					_vsc_symbol_completion_filter_options_unref0 (options);
					goto __catch0_g_error;
					goto __finally0;
				}
				_result_ = (_tmp8_ = _tmp7_, _g_object_unref0 (_result_), _tmp8_);
				g_timer_stop (timer);
				_tmp9_ = FALSE;
				if (vsc_symbol_completion_result_get_is_empty (_result_)) {
					gboolean _tmp10_;
					_tmp10_ = FALSE;
					if (trigger == NULL) {
						_tmp10_ = TRUE;
					} else {
						_tmp10_ = trigger->shortcut_triggered;
					}
					_tmp9_ = _tmp10_;
				} else {
					_tmp9_ = FALSE;
				}
				if (_tmp9_) {
					char* _tmp11_;
					char* _tmp12_;
					g_timer_start (timer);
					_tmp11_ = vsc_symbol_completion_get_datatype_name_for_name (self->priv->_completion, "this", vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &_inner_error_);
					if (_inner_error_ != NULL) {
						_g_free0 (typename);
						_g_timer_destroy0 (timer);
						_vsc_symbol_completion_filter_options_unref0 (options);
						goto __catch0_g_error;
						goto __finally0;
					}
					typename = (_tmp12_ = _tmp11_, _g_free0 (typename), _tmp12_);
					if (typename != NULL) {
						VscSymbolCompletionResult* _tmp13_;
						VscSymbolCompletionResult* _tmp14_;
						vsc_symbol_completion_filter_options_defaults (options);
						vsc_symbol_completion_filter_options_public_only (options);
						options->error_domains = search_error_domains;
						options->error_base = search_error_base;
						options->constructors = TRUE;
						options->static_symbols = TRUE;
						options->private_symbols = TRUE;
						options->protected_symbols = TRUE;
						options->local_variables = TRUE;
						options->imported_namespaces = TRUE;
						_tmp13_ = vsc_symbol_completion_get_completions_for_name (self->priv->_completion, options, typename, vsc_source_buffer_get_name (self->priv->_sb), lineno + 1, colno, &_inner_error_);
						if (_inner_error_ != NULL) {
							_g_free0 (typename);
							_g_timer_destroy0 (timer);
							_vsc_symbol_completion_filter_options_unref0 (options);
							goto __catch0_g_error;
							goto __finally0;
						}
						_result_ = (_tmp14_ = _tmp13_, _g_object_unref0 (_result_), _tmp14_);
					}
					g_timer_stop (timer);
				}
			}
		}
		_g_free0 (typename);
		_g_timer_destroy0 (timer);
		_vsc_symbol_completion_filter_options_unref0 (options);
	}
	goto __finally0;
	__catch0_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning ("vtgsymbolcompletionprovider.vala:713: %s", err->message);
			_g_error_free0 (err);
		}
	}
	__finally0:
	if (_inner_error_ != NULL) {
		_g_object_unref0 (_result_);
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
	result = _result_;
	return result;
}


static gboolean vtg_symbol_completion_provider_is_vala_keyword (VtgSymbolCompletionProvider* self, const char* keyword) {
	gboolean result;
	gboolean _tmp0_;
	gboolean _tmp1_;
	gboolean _tmp2_;
	gboolean _tmp3_;
	g_return_val_if_fail (self != NULL, FALSE);
	g_return_val_if_fail (keyword != NULL, FALSE);
	_tmp0_ = FALSE;
	_tmp1_ = FALSE;
	_tmp2_ = FALSE;
	_tmp3_ = FALSE;
	if (_vala_strcmp0 (keyword, "if") == 0) {
		_tmp3_ = TRUE;
	} else {
		_tmp3_ = _vala_strcmp0 (keyword, "for") == 0;
	}
	if (_tmp3_) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = _vala_strcmp0 (keyword, "foreach") == 0;
	}
	if (_tmp2_) {
		_tmp1_ = TRUE;
	} else {
		_tmp1_ = _vala_strcmp0 (keyword, "while") == 0;
	}
	if (_tmp1_) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = _vala_strcmp0 (keyword, "switch") == 0;
	}
	result = _tmp0_;
	return result;
}


static char* vtg_symbol_completion_provider_get_document_text (VtgSymbolCompletionProvider* self, GeditDocument* doc, gboolean all_doc) {
	char* result;
	GtkTextMark* mark;
	GtkTextIter end = {0};
	GtkTextIter start = {0};
	char* doc_text;
	gboolean _tmp0_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (doc != NULL, NULL);
	mark = GTK_TEXT_MARK (gtk_text_buffer_get_insert ((GtkTextBuffer*) doc));
	gtk_text_buffer_get_iter_at_mark ((GtkTextBuffer*) doc, &start, mark);
	doc_text = NULL;
	_tmp0_ = FALSE;
	if (all_doc) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = gedit_document_is_untouched (doc);
	}
	if (_tmp0_) {
		char* _tmp1_;
		end = start;
		gtk_text_iter_set_line_offset (&start, 0);
		while (TRUE) {
			if (!gtk_text_iter_backward_line (&start)) {
				break;
			}
			;
		}
		while (TRUE) {
			if (!gtk_text_iter_forward_line (&end)) {
				break;
			}
			;
		}
		doc_text = (_tmp1_ = g_strdup (gtk_text_iter_get_text (&start, &end)), _g_free0 (doc_text), _tmp1_);
	} else {
		char* text1;
		char* text2;
		char* _tmp3_;
		end = start;
		gtk_text_iter_set_line_offset (&end, 0);
		while (TRUE) {
			if (!gtk_text_iter_backward_line (&start)) {
				break;
			}
			;
		}
		text1 = g_strdup (gtk_text_iter_get_text (&start, &end));
		text2 = g_strdup ("");
		if (gtk_text_iter_forward_line (&end)) {
			char* _tmp2_;
			start = end;
			while (TRUE) {
				if (!gtk_text_iter_forward_line (&end)) {
					break;
				}
				;
			}
			text2 = (_tmp2_ = g_strdup (gtk_text_iter_get_text (&start, &end)), _g_free0 (text2), _tmp2_);
		}
		doc_text = (_tmp3_ = g_strdup_printf ("%s\n%s", text1, text2), _g_free0 (doc_text), _tmp3_);
		_g_free0 (text1);
		_g_free0 (text2);
	}
	result = doc_text;
	return result;
}


static void vtg_symbol_completion_provider_set_completion (VtgSymbolCompletionProvider* self, VscSymbolCompletion* value) {
	VscSymbolCompletion* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_completion = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_completion), _tmp0_);
	g_object_notify ((GObject *) self, "completion");
}


static void vtg_symbol_completion_provider_set_view (VtgSymbolCompletionProvider* self, GeditView* value) {
	GeditView* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_view = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_view), _tmp0_);
	g_object_notify ((GObject *) self, "view");
}


static void vtg_symbol_completion_provider_set_plugin_instance (VtgSymbolCompletionProvider* self, VtgPluginInstance* value) {
	VtgPluginInstance* _tmp0_;
	g_return_if_fail (self != NULL);
	self->priv->_plugin_instance = (_tmp0_ = _g_object_ref0 (value), _g_object_unref0 (self->priv->_plugin_instance), _tmp0_);
	g_object_notify ((GObject *) self, "plugin-instance");
}


static gboolean _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event (GeditView* _sender, const GdkEventKey* event, gpointer self) {
	return vtg_symbol_completion_provider_on_view_key_press (self, _sender, event);
}


static void _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building (VscParserManager* _sender, gpointer self) {
	vtg_symbol_completion_provider_on_cache_building (self, _sender);
}


static void _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded (VscParserManager* _sender, gpointer self) {
	vtg_symbol_completion_provider_on_cache_builded (self, _sender);
}


static GObject * vtg_symbol_completion_provider_constructor (GType type, guint n_construct_properties, GObjectConstructParam * construct_properties) {
	GObject * obj;
	VtgSymbolCompletionProviderClass * klass;
	GObjectClass * parent_class;
	VtgSymbolCompletionProvider * self;
	GError * _inner_error_;
	klass = VTG_SYMBOL_COMPLETION_PROVIDER_CLASS (g_type_class_peek (VTG_TYPE_SYMBOL_COMPLETION_PROVIDER));
	parent_class = G_OBJECT_CLASS (g_type_class_peek_parent (klass));
	obj = parent_class->constructor (type, n_construct_properties, construct_properties);
	self = VTG_SYMBOL_COMPLETION_PROVIDER (obj);
	_inner_error_ = NULL;
	{
		{
			GeditDocument* doc;
			char* name;
			VscSourceBuffer* _tmp3_;
			GdkPixbuf* _tmp4_;
			GdkPixbuf* _tmp5_;
			char* _tmp6_;
			GdkPixbuf* _tmp7_;
			GdkPixbuf* _tmp8_;
			GdkPixbuf* _tmp9_;
			char* _tmp10_;
			GdkPixbuf* _tmp11_;
			GdkPixbuf* _tmp12_;
			GdkPixbuf* _tmp13_;
			char* _tmp14_;
			GdkPixbuf* _tmp15_;
			GdkPixbuf* _tmp16_;
			GdkPixbuf* _tmp17_;
			char* _tmp18_;
			GdkPixbuf* _tmp19_;
			GdkPixbuf* _tmp20_;
			GdkPixbuf* _tmp21_;
			char* _tmp22_;
			GdkPixbuf* _tmp23_;
			GdkPixbuf* _tmp24_;
			GdkPixbuf* _tmp25_;
			char* _tmp26_;
			GdkPixbuf* _tmp27_;
			GdkPixbuf* _tmp28_;
			GdkPixbuf* _tmp29_;
			char* _tmp30_;
			GdkPixbuf* _tmp31_;
			GdkPixbuf* _tmp32_;
			GdkPixbuf* _tmp33_;
			char* _tmp34_;
			GdkPixbuf* _tmp35_;
			GdkPixbuf* _tmp36_;
			GdkPixbuf* _tmp37_;
			char* _tmp38_;
			GdkPixbuf* _tmp39_;
			GdkPixbuf* _tmp40_;
			GdkPixbuf* _tmp41_;
			char* _tmp42_;
			GdkPixbuf* _tmp43_;
			GdkPixbuf* _tmp44_;
			GdkPixbuf* _tmp45_;
			GeditStatusbar* status_bar;
			doc = _g_object_ref0 (GEDIT_DOCUMENT (gtk_text_view_get_buffer ((GtkTextView*) self->priv->_view)));
			name = g_strdup (gedit_document_get_uri (doc));
			if (name == NULL) {
				char* _tmp0_;
				name = (_tmp0_ = g_strdup (gedit_document_get_short_name_for_display (doc)), _g_free0 (name), _tmp0_);
			} else {
				char* _tmp1_;
				char* _tmp2_;
				_tmp1_ = g_filename_from_uri (name, NULL, &_inner_error_);
				if (_inner_error_ != NULL) {
					_g_object_unref0 (doc);
					_g_free0 (name);
					goto __catch1_g_error;
					goto __finally1;
				}
				name = (_tmp2_ = _tmp1_, _g_free0 (name), _tmp2_);
			}
			self->priv->_sb = (_tmp3_ = vsc_source_buffer_new (name, NULL), _g_object_unref0 (self->priv->_sb), _tmp3_);
			g_signal_connect_object ((GtkWidget*) self->priv->_view, "key-press-event", (GCallback) _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event, self, 0);
			vsc_parser_manager_add_source_buffer (vsc_symbol_completion_get_parser (self->priv->_completion), self->priv->_sb, &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			_tmp4_ = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), GTK_STOCK_FILE, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK, &_inner_error_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_generic = (_tmp5_ = _g_object_ref0 (_tmp4_), _g_object_unref0 (self->priv->_icon_generic), _tmp5_);
			_tmp8_ = (_tmp7_ = gdk_pixbuf_new_from_file (_tmp6_ = vtg_utils_get_image_path ("element-field-16.png"), &_inner_error_), _g_free0 (_tmp6_), _tmp7_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_field = (_tmp9_ = _tmp8_, _g_object_unref0 (self->priv->_icon_field), _tmp9_);
			_tmp12_ = (_tmp11_ = gdk_pixbuf_new_from_file (_tmp10_ = vtg_utils_get_image_path ("element-method-16.png"), &_inner_error_), _g_free0 (_tmp10_), _tmp11_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_method = (_tmp13_ = _tmp12_, _g_object_unref0 (self->priv->_icon_method), _tmp13_);
			_tmp16_ = (_tmp15_ = gdk_pixbuf_new_from_file (_tmp14_ = vtg_utils_get_image_path ("element-class-16.png"), &_inner_error_), _g_free0 (_tmp14_), _tmp15_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_class = (_tmp17_ = _tmp16_, _g_object_unref0 (self->priv->_icon_class), _tmp17_);
			_tmp20_ = (_tmp19_ = gdk_pixbuf_new_from_file (_tmp18_ = vtg_utils_get_image_path ("element-structure-16.png"), &_inner_error_), _g_free0 (_tmp18_), _tmp19_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_struct = (_tmp21_ = _tmp20_, _g_object_unref0 (self->priv->_icon_struct), _tmp21_);
			_tmp24_ = (_tmp23_ = gdk_pixbuf_new_from_file (_tmp22_ = vtg_utils_get_image_path ("element-property-16.png"), &_inner_error_), _g_free0 (_tmp22_), _tmp23_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_property = (_tmp25_ = _tmp24_, _g_object_unref0 (self->priv->_icon_property), _tmp25_);
			_tmp28_ = (_tmp27_ = gdk_pixbuf_new_from_file (_tmp26_ = vtg_utils_get_image_path ("element-event-16.png"), &_inner_error_), _g_free0 (_tmp26_), _tmp27_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_signal = (_tmp29_ = _tmp28_, _g_object_unref0 (self->priv->_icon_signal), _tmp29_);
			_tmp32_ = (_tmp31_ = gdk_pixbuf_new_from_file (_tmp30_ = vtg_utils_get_image_path ("element-interface-16.png"), &_inner_error_), _g_free0 (_tmp30_), _tmp31_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_iface = (_tmp33_ = _tmp32_, _g_object_unref0 (self->priv->_icon_iface), _tmp33_);
			_tmp36_ = (_tmp35_ = gdk_pixbuf_new_from_file (_tmp34_ = vtg_utils_get_image_path ("element-enumeration-16.png"), &_inner_error_), _g_free0 (_tmp34_), _tmp35_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_enum = (_tmp37_ = _tmp36_, _g_object_unref0 (self->priv->_icon_enum), _tmp37_);
			_tmp40_ = (_tmp39_ = gdk_pixbuf_new_from_file (_tmp38_ = vtg_utils_get_image_path ("element-literal-16.png"), &_inner_error_), _g_free0 (_tmp38_), _tmp39_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_const = (_tmp41_ = _tmp40_, _g_object_unref0 (self->priv->_icon_const), _tmp41_);
			_tmp44_ = (_tmp43_ = gdk_pixbuf_new_from_file (_tmp42_ = vtg_utils_get_image_path ("element-namespace-16.png"), &_inner_error_), _g_free0 (_tmp42_), _tmp43_);
			if (_inner_error_ != NULL) {
				_g_object_unref0 (doc);
				_g_free0 (name);
				goto __catch1_g_error;
				goto __finally1;
			}
			self->priv->_icon_namespace = (_tmp45_ = _tmp44_, _g_object_unref0 (self->priv->_icon_namespace), _tmp45_);
			g_signal_connect_object (vsc_symbol_completion_get_parser (self->priv->_completion), "cache-building", (GCallback) _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building, self, 0);
			g_signal_connect_object (vsc_symbol_completion_get_parser (self->priv->_completion), "cache-builded", (GCallback) _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded, self, 0);
			status_bar = _g_object_ref0 (GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_instance_get_window (self->priv->_plugin_instance))));
			self->priv->sb_context_id = gtk_statusbar_get_context_id ((GtkStatusbar*) status_bar, "symbol status");
			self->priv->cache_building = TRUE;
			self->priv->all_doc = TRUE;
			vtg_symbol_completion_provider_parse (self, doc);
			_g_object_unref0 (doc);
			_g_free0 (name);
			_g_object_unref0 (status_bar);
		}
		goto __finally1;
		__catch1_g_error:
		{
			GError * err;
			err = _inner_error_;
			_inner_error_ = NULL;
			{
				g_warning ("vtgsymbolcompletionprovider.vala:137: an error occourred: %s", err->message);
				_g_error_free0 (err);
			}
		}
		__finally1:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
		}
	}
	return obj;
}


static void vtg_symbol_completion_provider_class_init (VtgSymbolCompletionProviderClass * klass) {
	vtg_symbol_completion_provider_parent_class = g_type_class_peek_parent (klass);
	g_type_class_add_private (klass, sizeof (VtgSymbolCompletionProviderPrivate));
	G_OBJECT_CLASS (klass)->set_property = vtg_symbol_completion_provider_set_property;
	G_OBJECT_CLASS (klass)->constructor = vtg_symbol_completion_provider_constructor;
	G_OBJECT_CLASS (klass)->finalize = vtg_symbol_completion_provider_finalize;
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION, g_param_spec_object ("completion", "completion", "completion", VSC_TYPE_SYMBOL_COMPLETION, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_VIEW, g_param_spec_object ("view", "view", "view", GEDIT_TYPE_VIEW, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
	g_object_class_install_property (G_OBJECT_CLASS (klass), VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN_INSTANCE, g_param_spec_object ("plugin-instance", "plugin-instance", "plugin-instance", VTG_TYPE_PLUGIN_INSTANCE, G_PARAM_STATIC_NAME | G_PARAM_STATIC_NICK | G_PARAM_STATIC_BLURB | G_PARAM_WRITABLE | G_PARAM_CONSTRUCT_ONLY));
}


static void vtg_symbol_completion_provider_gsc_provider_interface_init (GscProviderIface * iface) {
	vtg_symbol_completion_provider_gsc_provider_parent_iface = g_type_interface_peek_parent (iface);
	iface->finish = vtg_symbol_completion_provider_real_finish;
	iface->get_name = vtg_symbol_completion_provider_real_get_name;
	iface->get_proposals = vtg_symbol_completion_provider_real_get_proposals;
}


static void vtg_symbol_completion_provider_instance_init (VtgSymbolCompletionProvider * self) {
	self->priv = VTG_SYMBOL_COMPLETION_PROVIDER_GET_PRIVATE (self);
	self->priv->_sb = NULL;
	self->priv->_completion = NULL;
	self->priv->_view = NULL;
	self->priv->timeout_id = (guint) 0;
	self->priv->idle_id = (guint) 0;
	self->priv->all_doc = FALSE;
	self->priv->prealloc_index = 0;
	self->priv->counter = 0;
	self->priv->cache_building = FALSE;
	self->priv->prev_cache_building = FALSE;
	self->priv->tooltip_is_visible = FALSE;
	self->priv->_last_trigger = NULL;
	self->priv->sb_msg_id = (guint) 0;
	self->priv->sb_context_id = (guint) 0;
	self->priv->_calltip_window = NULL;
	self->priv->need_parse = TRUE;
	self->priv->current_edited_line = -1;
}


static void vtg_symbol_completion_provider_finalize (GObject* obj) {
	VtgSymbolCompletionProvider * self;
	self = VTG_SYMBOL_COMPLETION_PROVIDER (obj);
	{
		guint _tmp46_;
		guint _tmp47_;
		guint _tmp48_;
		g_signal_handlers_disconnect_matched ((GtkWidget*) self->priv->_view, G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("key-press-event", GTK_TYPE_WIDGET, &_tmp46_, NULL, FALSE), _tmp46_), 0, NULL, (GCallback) _vtg_symbol_completion_provider_on_view_key_press_gtk_widget_key_press_event, self);
		g_signal_handlers_disconnect_matched (vsc_symbol_completion_get_parser (self->priv->_completion), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("cache-building", VSC_TYPE_PARSER_MANAGER, &_tmp47_, NULL, FALSE), _tmp47_), 0, NULL, (GCallback) _vtg_symbol_completion_provider_on_cache_building_vsc_parser_manager_cache_building, self);
		g_signal_handlers_disconnect_matched (vsc_symbol_completion_get_parser (self->priv->_completion), G_SIGNAL_MATCH_ID | G_SIGNAL_MATCH_FUNC | G_SIGNAL_MATCH_DATA, (g_signal_parse_name ("cache-builded", VSC_TYPE_PARSER_MANAGER, &_tmp48_, NULL, FALSE), _tmp48_), 0, NULL, (GCallback) _vtg_symbol_completion_provider_on_cache_builded_vsc_parser_manager_cache_builded, self);
		if (self->priv->sb_msg_id != 0) {
			GeditStatusbar* status_bar;
			status_bar = _g_object_ref0 (GEDIT_STATUSBAR (gedit_window_get_statusbar (vtg_plugin_instance_get_window (self->priv->_plugin_instance))));
			gtk_statusbar_remove ((GtkStatusbar*) status_bar, self->priv->sb_context_id, self->priv->sb_msg_id);
			_g_object_unref0 (status_bar);
		}
		if (self->priv->timeout_id != 0) {
			g_source_remove (self->priv->timeout_id);
		}
		if (self->priv->idle_id != 0) {
			g_source_remove (self->priv->idle_id);
		}
		vsc_parser_manager_remove_source_buffer (vsc_symbol_completion_get_parser (self->priv->_completion), self->priv->_sb);
	}
	_g_object_unref0 (self->priv->_sb);
	_g_object_unref0 (self->priv->_completion);
	_g_object_unref0 (self->priv->_view);
	__g_list_free_g_object_unref0 (self->priv->list);
	_g_object_unref0 (self->priv->_icon_generic);
	_g_object_unref0 (self->priv->_icon_field);
	_g_object_unref0 (self->priv->_icon_method);
	_g_object_unref0 (self->priv->_icon_class);
	_g_object_unref0 (self->priv->_icon_struct);
	_g_object_unref0 (self->priv->_icon_property);
	_g_object_unref0 (self->priv->_icon_signal);
	_g_object_unref0 (self->priv->_icon_iface);
	_g_object_unref0 (self->priv->_icon_const);
	_g_object_unref0 (self->priv->_icon_enum);
	_g_object_unref0 (self->priv->_icon_namespace);
	_g_object_unref0 (self->priv->_last_trigger);
	_g_object_unref0 (self->priv->_plugin_instance);
	_g_object_unref0 (self->priv->_calltip_window);
	G_OBJECT_CLASS (vtg_symbol_completion_provider_parent_class)->finalize (obj);
}


GType vtg_symbol_completion_provider_get_type (void) {
	return vtg_symbol_completion_provider_type_id;
}


GType vtg_symbol_completion_provider_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgSymbolCompletionProviderClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_symbol_completion_provider_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgSymbolCompletionProvider), 0, (GInstanceInitFunc) vtg_symbol_completion_provider_instance_init, NULL };
	static const GInterfaceInfo gsc_provider_info = { (GInterfaceInitFunc) vtg_symbol_completion_provider_gsc_provider_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_symbol_completion_provider_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgSymbolCompletionProvider", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_symbol_completion_provider_type_id, GSC_TYPE_PROVIDER, &gsc_provider_info);
	return vtg_symbol_completion_provider_type_id;
}


static void vtg_symbol_completion_provider_set_property (GObject * object, guint property_id, const GValue * value, GParamSpec * pspec) {
	VtgSymbolCompletionProvider * self;
	self = VTG_SYMBOL_COMPLETION_PROVIDER (object);
	switch (property_id) {
		case VTG_SYMBOL_COMPLETION_PROVIDER_COMPLETION:
		vtg_symbol_completion_provider_set_completion (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_PROVIDER_VIEW:
		vtg_symbol_completion_provider_set_view (self, g_value_get_object (value));
		break;
		case VTG_SYMBOL_COMPLETION_PROVIDER_PLUGIN_INSTANCE:
		vtg_symbol_completion_provider_set_plugin_instance (self, g_value_get_object (value));
		break;
		default:
		G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
		break;
	}
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




