/*
 *  vtgutils.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <gtk/gtk.h>
#include <stdlib.h>
#include <string.h>
#include <gtksourcecompletion/gsc-proposal.h>
#include <gee.h>
#include <vbf.h>
#include <config.h>
#include <gdk-pixbuf/gdk-pixdata.h>
#include <gobject/gvaluecollector.h>


#define VTG_TYPE_VCS_TYPES (vtg_vcs_types_get_type ())
#define _g_object_unref0(var) ((var == NULL) ? NULL : (var = (g_object_unref (var), NULL)))

#define VTG_TYPE_CACHES (vtg_caches_get_type ())
#define VTG_CACHES(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_CACHES, VtgCaches))
#define VTG_CACHES_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_CACHES, VtgCachesClass))
#define VTG_IS_CACHES(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_CACHES))
#define VTG_IS_CACHES_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_CACHES))
#define VTG_CACHES_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_CACHES, VtgCachesClass))

typedef struct _VtgCaches VtgCaches;
typedef struct _VtgCachesClass VtgCachesClass;
typedef struct _VtgCachesPrivate VtgCachesPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
typedef struct _VtgParamSpecCaches VtgParamSpecCaches;
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))

#define VTG_TYPE_UTILS (vtg_utils_get_type ())
#define VTG_UTILS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_TYPE_UTILS, VtgUtils))
#define VTG_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_TYPE_UTILS, VtgUtilsClass))
#define VTG_IS_UTILS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_TYPE_UTILS))
#define VTG_IS_UTILS_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_TYPE_UTILS))
#define VTG_UTILS_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_TYPE_UTILS, VtgUtilsClass))

typedef struct _VtgUtils VtgUtils;
typedef struct _VtgUtilsClass VtgUtilsClass;
typedef struct _VtgUtilsPrivate VtgUtilsPrivate;
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))
#define _g_dir_close0(var) ((var == NULL) ? NULL : (var = (g_dir_close (var), NULL)))
#define __g_list_free_g_free0(var) ((var == NULL) ? NULL : (var = (_g_list_free_g_free (var), NULL)))

typedef enum  {
	VTG_PROJECT_MANAGER_ERROR_NO_BACKEND
} VtgProjectManagerError;
#define VTG_PROJECT_MANAGER_ERROR vtg_project_manager_error_quark ()
typedef enum  {
	VTG_VCS_TYPES_NONE,
	VTG_VCS_TYPES_GIT,
	VTG_VCS_TYPES_BZR,
	VTG_VCS_TYPES_SVN
} VtgVcsTypes;

struct _VtgCaches {
	GTypeInstance parent_instance;
	volatile int ref_count;
	VtgCachesPrivate * priv;
};

struct _VtgCachesClass {
	GTypeClass parent_class;
	void (*finalize) (VtgCaches *self);
};

struct _VtgParamSpecCaches {
	GParamSpec parent_instance;
};

struct _VtgUtils {
	GObject parent_instance;
	VtgUtilsPrivate * priv;
};

struct _VtgUtilsClass {
	GObjectClass parent_class;
};


static GtkListStore* vtg_caches__build_cache;
static GtkListStore* vtg_caches__build_cache = NULL;
static GtkListStore* vtg_caches__compile_cache;
static GtkListStore* vtg_caches__compile_cache = NULL;
static GtkListStore* vtg_caches__configure_cache;
static GtkListStore* vtg_caches__configure_cache = NULL;
static GtkListStore* vtg_caches__executer_cache;
static GtkListStore* vtg_caches__executer_cache = NULL;
static gpointer vtg_caches_parent_class = NULL;
static GType vtg_caches_type_id = 0;
static gboolean vtg_utils__initialized;
static gboolean vtg_utils__initialized = FALSE;
static GscProposal** vtg_utils__proposals;
static gint vtg_utils__proposals_length1;
static GscProposal** vtg_utils__proposals = NULL;
static gint vtg_utils__proposals_length1 = 0;
static gint vtg_utils__proposals_size = 0;
static GeeList* vtg_utils__available_packages;
static GeeList* vtg_utils__available_packages = NULL;
static GtkBuilder* vtg_utils__builder;
static GtkBuilder* vtg_utils__builder = NULL;
static gpointer vtg_utils_parent_class = NULL;
static GType vtg_utils_type_id = 0;

GQuark vtg_project_manager_error_quark (void);
GType vtg_vcs_types_get_type (void);
void vtg_interaction_error_message (const char* message, GError* err);
gpointer vtg_caches_ref (gpointer instance);
void vtg_caches_unref (gpointer instance);
GParamSpec* vtg_param_spec_caches (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_value_set_caches (GValue* value, gpointer v_object);
gpointer vtg_value_get_caches (const GValue* value);
GType vtg_caches_get_type (void);
GType vtg_caches_register_type (GTypeModule * module);
enum  {
	VTG_CACHES_DUMMY_PROPERTY
};
#define VTG_CACHES_CACHE_LIMIT 20
GtkListStore* vtg_caches_get_build_cache (void);
GtkListStore* vtg_caches_get_compile_cache (void);
GtkListStore* vtg_caches_get_configure_cache (void);
GtkListStore* vtg_caches_get_executer_cache (void);
gboolean vtg_caches_cache_contains (GtkListStore* cache, const char* data);
gint vtg_caches_cache_count (GtkListStore* cache);
void vtg_caches_cache_add (GtkListStore* cache, const char* data);
VtgCaches* vtg_caches_new (void);
VtgCaches* vtg_caches_construct (GType object_type);
static void vtg_caches_finalize (VtgCaches* obj);
gboolean vtg_string_utils_is_null_or_empty (const char* data);
char* vtg_string_utils_replace (const char* data, const char* search, const char* replace);
char* vtg_path_utils_normalize_path (const char* name);
gint vtg_path_utils_compare_vala_filenames (const char* filea, const char* fileb);
GType vtg_utils_get_type (void);
GType vtg_utils_register_type (GTypeModule * module);
enum  {
	VTG_UTILS_DUMMY_PROPERTY
};
#define VTG_UTILS_prealloc_count 500
char* vtg_utils_get_ui_path (const char* id);
GtkBuilder* vtg_utils_get_builder (void);
static void vtg_utils_initialize (void);
GscProposal** vtg_utils_get_proposal_cache (int* result_length1);
char* vtg_utils_get_image_path (const char* id);
static void vtg_utils_initialize_packages_cache (void);
GeeList* vtg_utils_get_available_packages (void);
static void _g_list_free_g_free (GList* self);
VtgUtils* vtg_utils_new (void);
VtgUtils* vtg_utils_construct (GType object_type);
static void vtg_utils_finalize (GObject* obj);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);
static int _vala_strcmp0 (const char * str1, const char * str2);



GQuark vtg_project_manager_error_quark (void) {
	return g_quark_from_static_string ("vtg_project_manager_error-quark");
}



GType vtg_vcs_types_get_type (void) {
	static GType vtg_vcs_types_type_id = 0;
	if (G_UNLIKELY (vtg_vcs_types_type_id == 0)) {
		static const GEnumValue values[] = {{VTG_VCS_TYPES_NONE, "VTG_VCS_TYPES_NONE", "none"}, {VTG_VCS_TYPES_GIT, "VTG_VCS_TYPES_GIT", "git"}, {VTG_VCS_TYPES_BZR, "VTG_VCS_TYPES_BZR", "bzr"}, {VTG_VCS_TYPES_SVN, "VTG_VCS_TYPES_SVN", "svn"}, {0, NULL, NULL}};
		vtg_vcs_types_type_id = g_enum_register_static ("VtgVcsTypes", values);
	}
	return vtg_vcs_types_type_id;
}


void vtg_interaction_error_message (const char* message, GError* err) {
	GtkMessageDialog* dialog;
	g_return_if_fail (message != NULL);
	dialog = g_object_ref_sink ((GtkMessageDialog*) gtk_message_dialog_new (NULL, GTK_DIALOG_DESTROY_WITH_PARENT, GTK_MESSAGE_ERROR, GTK_BUTTONS_CLOSE, message));
	g_object_set (dialog, "secondary-text", err->message, NULL);
	gtk_dialog_run ((GtkDialog*) dialog);
	gtk_object_destroy ((GtkObject*) dialog);
	_g_object_unref0 (dialog);
}


static gpointer _g_object_ref0 (gpointer self) {
	return self ? g_object_ref (self) : NULL;
}


GtkListStore* vtg_caches_get_build_cache (void) {
	GtkListStore* result;
	if (vtg_caches__build_cache == NULL) {
		GtkListStore* _tmp0_;
		vtg_caches__build_cache = (_tmp0_ = gtk_list_store_new (1, G_TYPE_STRING, NULL), _g_object_unref0 (vtg_caches__build_cache), _tmp0_);
	}
	result = _g_object_ref0 (vtg_caches__build_cache);
	return result;
}


GtkListStore* vtg_caches_get_compile_cache (void) {
	GtkListStore* result;
	if (vtg_caches__compile_cache == NULL) {
		GtkListStore* _tmp0_;
		vtg_caches__compile_cache = (_tmp0_ = gtk_list_store_new (1, G_TYPE_STRING, NULL), _g_object_unref0 (vtg_caches__compile_cache), _tmp0_);
	}
	result = _g_object_ref0 (vtg_caches__compile_cache);
	return result;
}


GtkListStore* vtg_caches_get_configure_cache (void) {
	GtkListStore* result;
	if (vtg_caches__configure_cache == NULL) {
		GtkListStore* _tmp0_;
		vtg_caches__configure_cache = (_tmp0_ = gtk_list_store_new (1, G_TYPE_STRING, NULL), _g_object_unref0 (vtg_caches__configure_cache), _tmp0_);
	}
	result = _g_object_ref0 (vtg_caches__configure_cache);
	return result;
}


GtkListStore* vtg_caches_get_executer_cache (void) {
	GtkListStore* result;
	if (vtg_caches__executer_cache == NULL) {
		GtkListStore* _tmp0_;
		vtg_caches__executer_cache = (_tmp0_ = gtk_list_store_new (1, G_TYPE_STRING, NULL), _g_object_unref0 (vtg_caches__executer_cache), _tmp0_);
	}
	result = _g_object_ref0 (vtg_caches__executer_cache);
	return result;
}


gboolean vtg_caches_cache_contains (GtkListStore* cache, const char* data) {
	gboolean result;
	GtkTreeIter iter = {0};
	gboolean found;
	g_return_val_if_fail (cache != NULL, FALSE);
	g_return_val_if_fail (data != NULL, FALSE);
	found = FALSE;
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				char* tmp;
				if (!_tmp0_) {
					if (!gtk_tree_model_iter_next ((GtkTreeModel*) cache, &iter)) {
						break;
					}
				}
				_tmp0_ = FALSE;
				tmp = NULL;
				gtk_tree_model_get ((GtkTreeModel*) cache, &iter, 0, &tmp, -1);
				if (_vala_strcmp0 (tmp, data) == 0) {
					found = TRUE;
					_g_free0 (tmp);
					break;
				}
				_g_free0 (tmp);
			}
		}
	}
	result = found;
	return result;
}


void vtg_caches_cache_add (GtkListStore* cache, const char* data) {
	GtkTreeIter iter = {0};
	g_return_if_fail (cache != NULL);
	g_return_if_fail (data != NULL);
	if (vtg_caches_cache_count (cache) > VTG_CACHES_CACHE_LIMIT) {
		if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
			GtkTreeIter target;
			target = iter;
			while (TRUE) {
				if (!gtk_tree_model_iter_next ((GtkTreeModel*) cache, &iter)) {
					break;
				}
				target = iter;
			}
			gtk_list_store_remove (cache, &target);
		}
	}
	gtk_list_store_insert (cache, &iter, 0);
	gtk_list_store_set (cache, &iter, 0, data, -1);
}


gint vtg_caches_cache_count (GtkListStore* cache) {
	gint result;
	gint count;
	GtkTreeIter iter = {0};
	g_return_val_if_fail (cache != NULL, 0);
	count = 0;
	if (gtk_tree_model_get_iter_first ((GtkTreeModel*) cache, &iter)) {
		{
			gboolean _tmp0_;
			_tmp0_ = TRUE;
			while (TRUE) {
				if (!_tmp0_) {
					if (!gtk_tree_model_iter_next ((GtkTreeModel*) cache, &iter)) {
						break;
					}
				}
				_tmp0_ = FALSE;
				count++;
			}
		}
	}
	result = count;
	return result;
}


VtgCaches* vtg_caches_construct (GType object_type) {
	VtgCaches* self;
	self = (VtgCaches*) g_type_create_instance (object_type);
	return self;
}


VtgCaches* vtg_caches_new (void) {
	return vtg_caches_construct (VTG_TYPE_CACHES);
}


static void vtg_value_caches_init (GValue* value) {
	value->data[0].v_pointer = NULL;
}


static void vtg_value_caches_free_value (GValue* value) {
	if (value->data[0].v_pointer) {
		vtg_caches_unref (value->data[0].v_pointer);
	}
}


static void vtg_value_caches_copy_value (const GValue* src_value, GValue* dest_value) {
	if (src_value->data[0].v_pointer) {
		dest_value->data[0].v_pointer = vtg_caches_ref (src_value->data[0].v_pointer);
	} else {
		dest_value->data[0].v_pointer = NULL;
	}
}


static gpointer vtg_value_caches_peek_pointer (const GValue* value) {
	return value->data[0].v_pointer;
}


static gchar* vtg_value_caches_collect_value (GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	if (collect_values[0].v_pointer) {
		VtgCaches* object;
		object = collect_values[0].v_pointer;
		if (object->parent_instance.g_class == NULL) {
			return g_strconcat ("invalid unclassed object pointer for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		} else if (!g_value_type_compatible (G_TYPE_FROM_INSTANCE (object), G_VALUE_TYPE (value))) {
			return g_strconcat ("invalid object type `", g_type_name (G_TYPE_FROM_INSTANCE (object)), "' for value type `", G_VALUE_TYPE_NAME (value), "'", NULL);
		}
		value->data[0].v_pointer = vtg_caches_ref (object);
	} else {
		value->data[0].v_pointer = NULL;
	}
	return NULL;
}


static gchar* vtg_value_caches_lcopy_value (const GValue* value, guint n_collect_values, GTypeCValue* collect_values, guint collect_flags) {
	VtgCaches** object_p;
	object_p = collect_values[0].v_pointer;
	if (!object_p) {
		return g_strdup_printf ("value location for `%s' passed as NULL", G_VALUE_TYPE_NAME (value));
	}
	if (!value->data[0].v_pointer) {
		*object_p = NULL;
	} else if (collect_flags && G_VALUE_NOCOPY_CONTENTS) {
		*object_p = value->data[0].v_pointer;
	} else {
		*object_p = vtg_caches_ref (value->data[0].v_pointer);
	}
	return NULL;
}


GParamSpec* vtg_param_spec_caches (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags) {
	VtgParamSpecCaches* spec;
	g_return_val_if_fail (g_type_is_a (object_type, VTG_TYPE_CACHES), NULL);
	spec = g_param_spec_internal (G_TYPE_PARAM_OBJECT, name, nick, blurb, flags);
	G_PARAM_SPEC (spec)->value_type = object_type;
	return G_PARAM_SPEC (spec);
}


gpointer vtg_value_get_caches (const GValue* value) {
	g_return_val_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_TYPE_CACHES), NULL);
	return value->data[0].v_pointer;
}


void vtg_value_set_caches (GValue* value, gpointer v_object) {
	VtgCaches* old;
	g_return_if_fail (G_TYPE_CHECK_VALUE_TYPE (value, VTG_TYPE_CACHES));
	old = value->data[0].v_pointer;
	if (v_object) {
		g_return_if_fail (G_TYPE_CHECK_INSTANCE_TYPE (v_object, VTG_TYPE_CACHES));
		g_return_if_fail (g_value_type_compatible (G_TYPE_FROM_INSTANCE (v_object), G_VALUE_TYPE (value)));
		value->data[0].v_pointer = v_object;
		vtg_caches_ref (value->data[0].v_pointer);
	} else {
		value->data[0].v_pointer = NULL;
	}
	if (old) {
		vtg_caches_unref (old);
	}
}


static void vtg_caches_class_init (VtgCachesClass * klass) {
	vtg_caches_parent_class = g_type_class_peek_parent (klass);
	VTG_CACHES_CLASS (klass)->finalize = vtg_caches_finalize;
}


static void vtg_caches_instance_init (VtgCaches * self) {
	self->ref_count = 1;
}


static void vtg_caches_finalize (VtgCaches* obj) {
	VtgCaches * self;
	self = VTG_CACHES (obj);
}


GType vtg_caches_get_type (void) {
	return vtg_caches_type_id;
}


GType vtg_caches_register_type (GTypeModule * module) {
	static const GTypeValueTable g_define_type_value_table = { vtg_value_caches_init, vtg_value_caches_free_value, vtg_value_caches_copy_value, vtg_value_caches_peek_pointer, "p", vtg_value_caches_collect_value, "p", vtg_value_caches_lcopy_value };
	static const GTypeInfo g_define_type_info = { sizeof (VtgCachesClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_caches_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgCaches), 0, (GInstanceInitFunc) vtg_caches_instance_init, &g_define_type_value_table };
	static const GTypeFundamentalInfo g_define_type_fundamental_info = { (G_TYPE_FLAG_CLASSED | G_TYPE_FLAG_INSTANTIATABLE | G_TYPE_FLAG_DERIVABLE | G_TYPE_FLAG_DEEP_DERIVABLE) };
	vtg_caches_type_id = g_type_register_fundamental (g_type_fundamental_next (), "VtgCaches", &g_define_type_info, &g_define_type_fundamental_info, 0);
	return vtg_caches_type_id;
}


gpointer vtg_caches_ref (gpointer instance) {
	VtgCaches* self;
	self = instance;
	g_atomic_int_inc (&self->ref_count);
	return instance;
}


void vtg_caches_unref (gpointer instance) {
	VtgCaches* self;
	self = instance;
	if (g_atomic_int_dec_and_test (&self->ref_count)) {
		VTG_CACHES_GET_CLASS (self)->finalize (self);
		g_type_free_instance ((GTypeInstance *) self);
	}
}


gboolean vtg_string_utils_is_null_or_empty (const char* data) {
	gboolean result;
	gboolean _tmp0_;
	_tmp0_ = FALSE;
	if (data == NULL) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = _vala_strcmp0 (data, "") == 0;
	}
	result = _tmp0_;
	return result;
}


char* vtg_string_utils_replace (const char* data, const char* search, const char* replace) {
	char* result;
	GError * _inner_error_;
	g_return_val_if_fail (data != NULL, NULL);
	g_return_val_if_fail (search != NULL, NULL);
	g_return_val_if_fail (replace != NULL, NULL);
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (search, -1), 0, 0, &_inner_error_), _g_free0 (_tmp0_), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch37_g_regex_error;
			}
			goto __finally37;
		}
		_tmp2_ = g_regex_replace_literal (regex, data, (glong) (-1), 0, replace, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			_g_regex_unref0 (regex);
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch37_g_regex_error;
			}
			goto __finally37;
		}
		result = _tmp2_;
		_g_regex_unref0 (regex);
		return result;
	}
	goto __finally37;
	__catch37_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_assert_not_reached ();
			_g_error_free0 (e);
		}
	}
	__finally37:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


char* vtg_path_utils_normalize_path (const char* name) {
	char* result;
	gboolean _tmp0_;
	char** _tmp4_;
	gint name_parts_size;
	gint name_parts_length1;
	char** _tmp2_;
	char* _tmp1_;
	char** _tmp3_;
	char** name_parts;
	char* last_item;
	char* target_name;
	gboolean _tmp9_;
	g_return_val_if_fail (name != NULL, NULL);
	_tmp0_ = FALSE;
	if (name == NULL) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = string_get_length (name) < 2;
	}
	if (_tmp0_) {
		result = g_strdup (name);
		return result;
	}
	name_parts = (_tmp4_ = (_tmp3_ = _tmp2_ = g_strsplit (_tmp1_ = string_substring (name, (glong) 1, string_get_length (name) - 1), "/", 0), _g_free0 (_tmp1_), _tmp3_), name_parts_length1 = _vala_array_length (_tmp2_), name_parts_size = name_parts_length1, _tmp4_);
	last_item = NULL;
	target_name = g_strdup ("");
	{
		char** item_collection;
		int item_collection_length1;
		int item_it;
		item_collection = name_parts;
		item_collection_length1 = name_parts_length1;
		for (item_it = 0; item_it < name_parts_length1; item_it = item_it + 1) {
			char* item;
			item = g_strdup (item_collection[item_it]);
			{
				if (_vala_strcmp0 (item, "..") != 0) {
					char* _tmp7_;
					if (last_item != NULL) {
						char* _tmp6_;
						char* _tmp5_;
						target_name = (_tmp6_ = g_strconcat (target_name, _tmp5_ = g_strconcat ("/", last_item, NULL), NULL), _g_free0 (target_name), _tmp6_);
						_g_free0 (_tmp5_);
					}
					last_item = (_tmp7_ = g_strdup (item), _g_free0 (last_item), _tmp7_);
				} else {
					char* _tmp8_;
					last_item = (_tmp8_ = NULL, _g_free0 (last_item), _tmp8_);
				}
				_g_free0 (item);
			}
		}
	}
	_tmp9_ = FALSE;
	if (last_item != NULL) {
		_tmp9_ = _vala_strcmp0 (last_item, "..") != 0;
	} else {
		_tmp9_ = FALSE;
	}
	if (_tmp9_) {
		char* _tmp11_;
		char* _tmp10_;
		target_name = (_tmp11_ = g_strconcat (target_name, _tmp10_ = g_strconcat ("/", last_item, NULL), NULL), _g_free0 (target_name), _tmp11_);
		_g_free0 (_tmp10_);
	}
	result = target_name;
	name_parts = (_vala_array_free (name_parts, name_parts_length1, (GDestroyNotify) g_free), NULL);
	_g_free0 (last_item);
	return result;
}


gint vtg_path_utils_compare_vala_filenames (const char* filea, const char* fileb) {
	gint result;
	char* dataa;
	char* datab;
	gboolean _tmp0_;
	gboolean _tmp2_;
	g_return_val_if_fail (filea != NULL, 0);
	g_return_val_if_fail (fileb != NULL, 0);
	dataa = g_strdup (filea);
	datab = g_strdup (fileb);
	_tmp0_ = FALSE;
	if (g_str_has_suffix (dataa, ".vala")) {
		_tmp0_ = TRUE;
	} else {
		_tmp0_ = g_str_has_suffix (dataa, ".vapi");
	}
	if (_tmp0_) {
		char* _tmp1_;
		dataa = (_tmp1_ = string_substring (dataa, (glong) 0, string_get_length (dataa) - 5), _g_free0 (dataa), _tmp1_);
	}
	_tmp2_ = FALSE;
	if (g_str_has_suffix (datab, ".vala")) {
		_tmp2_ = TRUE;
	} else {
		_tmp2_ = g_str_has_suffix (datab, ".vapi");
	}
	if (_tmp2_) {
		char* _tmp3_;
		datab = (_tmp3_ = string_substring (datab, (glong) 0, string_get_length (datab) - 5), _g_free0 (datab), _tmp3_);
	}
	result = strcmp (dataa, datab);
	_g_free0 (dataa);
	_g_free0 (datab);
	return result;
}


GtkBuilder* vtg_utils_get_builder (void) {
	GtkBuilder* result;
	GError * _inner_error_;
	_inner_error_ = NULL;
	if (vtg_utils__builder == NULL) {
		GtkBuilder* _tmp0_;
		vtg_utils__builder = (_tmp0_ = gtk_builder_new (), _g_object_unref0 (vtg_utils__builder), _tmp0_);
		{
			char* _tmp1_;
			gtk_builder_add_from_file (vtg_utils__builder, _tmp1_ = vtg_utils_get_ui_path ("vtg.ui"), &_inner_error_);
			if (_inner_error_ != NULL) {
				goto __catch38_g_error;
				goto __finally38;
			}
			_g_free0 (_tmp1_);
		}
		goto __finally38;
		__catch38_g_error:
		{
			GError * err;
			err = _inner_error_;
			_inner_error_ = NULL;
			{
				g_warning ("vtgutils.vala:233: initialize_ui: %s", err->message);
				_g_error_free0 (err);
			}
		}
		__finally38:
		if (_inner_error_ != NULL) {
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
	result = _g_object_ref0 (vtg_utils__builder);
	return result;
}


GscProposal** vtg_utils_get_proposal_cache (int* result_length1) {
	GscProposal** result;
	GscProposal** _tmp0_;
	if (!vtg_utils__initialized) {
		vtg_utils_initialize ();
	}
	result = (_tmp0_ = vtg_utils__proposals, *result_length1 = vtg_utils__proposals_length1, _tmp0_);
	return result;
}


char* vtg_utils_get_image_path (const char* id) {
	char* result;
	char* _result_;
	g_return_val_if_fail (id != NULL, NULL);
	_result_ = g_build_filename (PACKAGE_DATADIR, "images", id, NULL);
	result = _result_;
	return result;
}


char* vtg_utils_get_ui_path (const char* id) {
	char* result;
	char* _result_;
	g_return_val_if_fail (id != NULL, NULL);
	_result_ = g_build_filename (PACKAGE_DATADIR, "ui", id, NULL);
	result = _result_;
	return result;
}


static void vtg_utils_initialize (void) {
	GError * _inner_error_;
	_inner_error_ = NULL;
	{
		GscProposal** _tmp0_;
		GdkPixbuf* _icon_generic;
		vtg_utils__proposals = (_tmp0_ = g_new0 (GscProposal*, VTG_UTILS_prealloc_count + 1), vtg_utils__proposals = (_vala_array_free (vtg_utils__proposals, vtg_utils__proposals_length1, (GDestroyNotify) g_object_unref), NULL), vtg_utils__proposals_length1 = VTG_UTILS_prealloc_count, vtg_utils__proposals_size = vtg_utils__proposals_length1, _tmp0_);
		_icon_generic = _g_object_ref0 (gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), GTK_STOCK_FILE, 16, GTK_ICON_LOOKUP_GENERIC_FALLBACK, &_inner_error_));
		if (_inner_error_ != NULL) {
			goto __catch39_g_error;
			goto __finally39;
		}
		{
			gint idx;
			idx = 0;
			{
				gboolean _tmp1_;
				_tmp1_ = TRUE;
				while (TRUE) {
					GscProposal* obj;
					GscProposal* _tmp2_;
					if (!_tmp1_) {
						idx++;
					}
					_tmp1_ = FALSE;
					if (!(idx < VTG_UTILS_prealloc_count)) {
						break;
					}
					obj = gsc_proposal_new ("", "", _icon_generic);
					vtg_utils__proposals[idx] = (_tmp2_ = _g_object_ref0 (obj), _g_object_unref0 (vtg_utils__proposals[idx]), _tmp2_);
					_g_object_unref0 (obj);
				}
			}
		}
		vtg_utils__initialized = TRUE;
		_g_object_unref0 (_icon_generic);
	}
	goto __finally39;
	__catch39_g_error:
	{
		GError * err;
		err = _inner_error_;
		_inner_error_ = NULL;
		{
			g_warning (err->message);
			_g_error_free0 (err);
		}
	}
	__finally39:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return;
	}
}


static gpointer _gee_collection_object_ref0 (gpointer self) {
	return self ? gee_collection_object_ref (self) : NULL;
}


GeeList* vtg_utils_get_available_packages (void) {
	GeeList* result;
	if (vtg_utils__available_packages == NULL) {
		vtg_utils_initialize_packages_cache ();
	}
	result = _gee_collection_object_ref0 (vtg_utils__available_packages);
	return result;
}


static void _g_list_free_g_free (GList* self) {
	g_list_foreach (self, (GFunc) g_free, NULL);
	g_list_free (self);
}


static void vtg_utils_initialize_packages_cache (void) {
	GError * _inner_error_;
	GList* vapidirs;
	GeeList* _tmp0_;
	_inner_error_ = NULL;
	vapidirs = NULL;
	vapidirs = g_list_append (vapidirs, g_strdup (VALA_VAPIDIR));
	vapidirs = g_list_append (vapidirs, g_strdup ("/usr/local/share/vala/vapi"));
	vtg_utils__available_packages = (_tmp0_ = (GeeList*) gee_array_list_new (VBF_TYPE_PACKAGE, (GBoxedCopyFunc) g_object_ref, g_object_unref, g_direct_equal), _gee_collection_object_unref0 (vtg_utils__available_packages), _tmp0_);
	{
		GList* vapidir_collection;
		GList* vapidir_it;
		vapidir_collection = vapidirs;
		for (vapidir_it = vapidir_collection; vapidir_it != NULL; vapidir_it = vapidir_it->next) {
			char* vapidir;
			vapidir = g_strdup ((const char*) vapidir_it->data);
			{
				GDir* dir;
				char* filename;
				dir = NULL;
				{
					GDir* _tmp1_;
					GDir* _tmp2_;
					_tmp1_ = g_dir_open (vapidir, 0, &_inner_error_);
					if (_inner_error_ != NULL) {
						if (_inner_error_->domain == G_FILE_ERROR) {
							goto __catch40_g_file_error;
						}
						goto __finally40;
					}
					dir = (_tmp2_ = _tmp1_, _g_dir_close0 (dir), _tmp2_);
				}
				goto __finally40;
				__catch40_g_file_error:
				{
					GError * err;
					err = _inner_error_;
					_inner_error_ = NULL;
					{
						_g_error_free0 (err);
						_g_free0 (vapidir);
						_g_dir_close0 (dir);
						continue;
					}
				}
				__finally40:
				if (_inner_error_ != NULL) {
					_g_free0 (vapidir);
					_g_dir_close0 (dir);
					__g_list_free_g_free0 (vapidirs);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return;
				}
				filename = g_strdup (g_dir_read_name (dir));
				while (TRUE) {
					char* _tmp6_;
					if (!(filename != NULL)) {
						break;
					}
					if (g_str_has_suffix (filename, ".vapi")) {
						char* _tmp3_;
						VbfPackage* _tmp5_;
						char* _tmp4_;
						filename = (_tmp3_ = g_utf8_strdown (filename, -1), _g_free0 (filename), _tmp3_);
						gee_collection_add ((GeeCollection*) vtg_utils__available_packages, _tmp5_ = vbf_package_new (_tmp4_ = string_substring (filename, (glong) 0, string_get_length (filename) - 5)));
						_g_object_unref0 (_tmp5_);
						_g_free0 (_tmp4_);
					}
					filename = (_tmp6_ = g_strdup (g_dir_read_name (dir)), _g_free0 (filename), _tmp6_);
				}
				_g_free0 (vapidir);
				_g_dir_close0 (dir);
				_g_free0 (filename);
			}
		}
	}
	__g_list_free_g_free0 (vapidirs);
}


VtgUtils* vtg_utils_construct (GType object_type) {
	VtgUtils * self;
	self = (VtgUtils*) g_object_new (object_type, NULL);
	return self;
}


VtgUtils* vtg_utils_new (void) {
	return vtg_utils_construct (VTG_TYPE_UTILS);
}


static void vtg_utils_class_init (VtgUtilsClass * klass) {
	vtg_utils_parent_class = g_type_class_peek_parent (klass);
	G_OBJECT_CLASS (klass)->finalize = vtg_utils_finalize;
}


static void vtg_utils_instance_init (VtgUtils * self) {
}


static void vtg_utils_finalize (GObject* obj) {
	VtgUtils * self;
	self = VTG_UTILS (obj);
	G_OBJECT_CLASS (vtg_utils_parent_class)->finalize (obj);
}


GType vtg_utils_get_type (void) {
	return vtg_utils_type_id;
}


GType vtg_utils_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgUtilsClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_utils_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgUtils), 0, (GInstanceInitFunc) vtg_utils_instance_init, NULL };
	vtg_utils_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgUtils", &g_define_type_info, 0);
	return vtg_utils_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}


static int _vala_strcmp0 (const char * str1, const char * str2) {
	if (str1 == NULL) {
		return -(str1 != str2);
	}
	if (str2 == NULL) {
		return str1 != str2;
	}
	return strcmp (str1, str2);
}




