/*
 *  vtgvcsbackendsbzr.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <sys/wait.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>


#define VTG_VCS_BACKENDS_TYPE_IVCS (vtg_vcs_backends_ivcs_get_type ())
#define VTG_VCS_BACKENDS_IVCS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_IVCS, VtgVcsBackendsIVcs))
#define VTG_VCS_BACKENDS_IS_IVCS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_IVCS))
#define VTG_VCS_BACKENDS_IVCS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VTG_VCS_BACKENDS_TYPE_IVCS, VtgVcsBackendsIVcsIface))

typedef struct _VtgVcsBackendsIVcs VtgVcsBackendsIVcs;
typedef struct _VtgVcsBackendsIVcsIface VtgVcsBackendsIVcsIface;

#define VTG_VCS_BACKENDS_TYPE_ITEM (vtg_vcs_backends_item_get_type ())
#define VTG_VCS_BACKENDS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItem))
#define VTG_VCS_BACKENDS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItemClass))
#define VTG_VCS_BACKENDS_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_ITEM))
#define VTG_VCS_BACKENDS_IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_ITEM))
#define VTG_VCS_BACKENDS_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItemClass))

typedef struct _VtgVcsBackendsItem VtgVcsBackendsItem;
typedef struct _VtgVcsBackendsItemClass VtgVcsBackendsItemClass;

#define VTG_VCS_BACKENDS_TYPE_BZR (vtg_vcs_backends_bzr_get_type ())
#define VTG_VCS_BACKENDS_BZR(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_BZR, VtgVcsBackendsBzr))
#define VTG_VCS_BACKENDS_BZR_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_BZR, VtgVcsBackendsBzrClass))
#define VTG_VCS_BACKENDS_IS_BZR(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_BZR))
#define VTG_VCS_BACKENDS_IS_BZR_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_BZR))
#define VTG_VCS_BACKENDS_BZR_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_BZR, VtgVcsBackendsBzrClass))

typedef struct _VtgVcsBackendsBzr VtgVcsBackendsBzr;
typedef struct _VtgVcsBackendsBzrClass VtgVcsBackendsBzrClass;
typedef struct _VtgVcsBackendsBzrPrivate VtgVcsBackendsBzrPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

#define VTG_VCS_BACKENDS_TYPE_STATES (vtg_vcs_backends_states_get_type ())
typedef struct _VtgVcsBackendsItemPrivate VtgVcsBackendsItemPrivate;
#define _vtg_vcs_backends_item_unref0(var) ((var == NULL) ? NULL : (var = (vtg_vcs_backends_item_unref (var), NULL)))

struct _VtgVcsBackendsIVcsIface {
	GTypeInterface parent_iface;
	GeeList* (*get_items) (VtgVcsBackendsIVcs* self, const char* path, GError** error);
	gboolean (*test) (VtgVcsBackendsIVcs* self, const char* path);
};

struct _VtgVcsBackendsBzr {
	GObject parent_instance;
	VtgVcsBackendsBzrPrivate * priv;
};

struct _VtgVcsBackendsBzrClass {
	GObjectClass parent_class;
};

typedef enum  {
	VTG_VCS_BACKENDS_STATES_UNTRACKED,
	VTG_VCS_BACKENDS_STATES_ADDED,
	VTG_VCS_BACKENDS_STATES_MODIFIED
} VtgVcsBackendsStates;

struct _VtgVcsBackendsItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	VtgVcsBackendsItemPrivate * priv;
	char* name;
	VtgVcsBackendsStates state;
};

struct _VtgVcsBackendsItemClass {
	GTypeClass parent_class;
	void (*finalize) (VtgVcsBackendsItem *self);
};

typedef enum  {
	VTG_VCS_BACKENDS_VCS_ERROR_CommandFailed
} VtgVcsBackendsVcsError;
#define VTG_VCS_BACKENDS_VCS_ERROR vtg_vcs_backends_vcs_error_quark ()

static gpointer vtg_vcs_backends_bzr_parent_class = NULL;
static VtgVcsBackendsIVcsIface* vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_parent_iface = NULL;
static GType vtg_vcs_backends_bzr_type_id = 0;

gpointer vtg_vcs_backends_item_ref (gpointer instance);
void vtg_vcs_backends_item_unref (gpointer instance);
GParamSpec* vtg_vcs_backends_param_spec_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_vcs_backends_value_set_item (GValue* value, gpointer v_object);
gpointer vtg_vcs_backends_value_get_item (const GValue* value);
GType vtg_vcs_backends_item_get_type (void);
GType vtg_vcs_backends_item_register_type (GTypeModule * module);
GType vtg_vcs_backends_ivcs_get_type (void);
GType vtg_vcs_backends_bzr_get_type (void);
GType vtg_vcs_backends_bzr_register_type (GTypeModule * module);
enum  {
	VTG_VCS_BACKENDS_BZR_DUMMY_PROPERTY
};
VtgVcsBackendsBzr* vtg_vcs_backends_bzr_new (void);
VtgVcsBackendsBzr* vtg_vcs_backends_bzr_construct (GType object_type);
VtgVcsBackendsItem* vtg_vcs_backends_item_new (void);
VtgVcsBackendsItem* vtg_vcs_backends_item_construct (GType object_type);
GType vtg_vcs_backends_states_get_type (void);
GQuark vtg_vcs_backends_vcs_error_quark (void);
static GeeList* vtg_vcs_backends_bzr_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error);
static gboolean vtg_vcs_backends_bzr_real_test (VtgVcsBackendsIVcs* base, const char* path);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



VtgVcsBackendsBzr* vtg_vcs_backends_bzr_construct (GType object_type) {
	VtgVcsBackendsBzr * self;
	self = (VtgVcsBackendsBzr*) g_object_new (object_type, NULL);
	return self;
}


VtgVcsBackendsBzr* vtg_vcs_backends_bzr_new (void) {
	return vtg_vcs_backends_bzr_construct (VTG_VCS_BACKENDS_TYPE_BZR);
}


static char* string_substring (const char* self, glong offset, glong len) {
	char* result;
	glong string_length;
	const char* start;
	g_return_val_if_fail (self != NULL, NULL);
	string_length = g_utf8_strlen (self, -1);
	if (offset < 0) {
		offset = string_length + offset;
		g_return_val_if_fail (offset >= 0, NULL);
	} else {
		g_return_val_if_fail (offset <= string_length, NULL);
	}
	if (len < 0) {
		len = string_length - offset;
	}
	g_return_val_if_fail ((offset + len) <= string_length, NULL);
	start = g_utf8_offset_to_pointer (self, offset);
	result = g_strndup (start, ((gchar*) g_utf8_offset_to_pointer (start, len)) - ((gchar*) start));
	return result;
}


static glong string_get_length (const char* self) {
	glong result;
	g_return_val_if_fail (self != NULL, 0L);
	result = g_utf8_strlen (self, -1);
	return result;
}


static GeeList* vtg_vcs_backends_bzr_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error) {
	VtgVcsBackendsBzr * self;
	GeeList* result;
	GError * _inner_error_;
	GeeList* results;
	char* stdo;
	char* stde;
	gint exit_status;
	char* _tmp7_;
	gboolean _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	gboolean _tmp3_;
	char* _tmp2_;
	gint _tmp1__length1;
	char** _tmp1_;
	char** _tmp0_;
	gboolean _tmp8_;
	gboolean _tmp9_;
	self = (VtgVcsBackendsBzr*) base;
	g_return_val_if_fail (path != NULL, NULL);
	_inner_error_ = NULL;
	results = (GeeList*) gee_array_list_new (VTG_VCS_BACKENDS_TYPE_ITEM, (GBoxedCopyFunc) vtg_vcs_backends_item_ref, vtg_vcs_backends_item_unref, g_direct_equal);
	stdo = NULL;
	stde = NULL;
	exit_status = 0;
	_tmp5_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	_tmp9_ = (_tmp8_ = (_tmp6_ = (_tmp3_ = g_spawn_sync (path, (_tmp1_ = (_tmp0_ = g_new0 (char*, 2 + 1), _tmp0_[0] = g_strdup ("bzr"), _tmp0_[1] = g_strdup ("status"), _tmp0_), _tmp1__length1 = 2, _tmp1_), NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &_tmp2_, &_tmp5_, &exit_status, &_inner_error_), stdo = (_tmp4_ = _tmp2_, _g_free0 (stdo), _tmp4_), _tmp3_), stde = (_tmp7_ = _tmp5_, _g_free0 (stde), _tmp7_), _tmp6_), _tmp1_ = (_vala_array_free (_tmp1_, _tmp1__length1, (GDestroyNotify) g_free), NULL), _tmp8_);
	if (_inner_error_ != NULL) {
		g_propagate_error (error, _inner_error_);
		_gee_collection_object_unref0 (results);
		_g_free0 (stdo);
		_g_free0 (stde);
		return NULL;
	}
	if (_tmp9_) {
		gint exit;
		exit = WEXITSTATUS (exit_status);
		if (exit == 0) {
			char** _tmp11_;
			gint lines_size;
			gint lines_length1;
			char** _tmp10_;
			char** lines;
			gint idx;
			gint status;
			lines = (_tmp11_ = _tmp10_ = g_strsplit (stdo, "\n", 0), lines_length1 = _vala_array_length (_tmp10_), lines_size = lines_length1, _tmp11_);
			idx = 0;
			status = 0;
			while (TRUE) {
				char* line;
				if (!(lines[idx] != NULL)) {
					break;
				}
				line = g_strdup (lines[idx]);
				if (g_str_has_prefix (line, "added:")) {
					status = 1;
				} else {
					if (g_str_has_prefix (line, "modified:")) {
						status = 2;
					} else {
						if (!g_str_has_prefix (line, "  ")) {
							status = 0;
						} else {
							if (g_str_has_prefix (line, "  ")) {
								switch (status) {
									case 1:
									case 2:
									{
										VtgVcsBackendsItem* item;
										VtgVcsBackendsStates _tmp12_;
										char* _tmp13_;
										item = vtg_vcs_backends_item_new ();
										_tmp12_ = 0;
										if (status == 1) {
											_tmp12_ = VTG_VCS_BACKENDS_STATES_ADDED;
										} else {
											_tmp12_ = VTG_VCS_BACKENDS_STATES_MODIFIED;
										}
										item->state = _tmp12_;
										item->name = (_tmp13_ = string_substring (line, (glong) 2, string_get_length (line) - 2), _g_free0 (item->name), _tmp13_);
										gee_collection_add ((GeeCollection*) results, item);
										_vtg_vcs_backends_item_unref0 (item);
										break;
									}
								}
							}
						}
					}
				}
				idx++;
				_g_free0 (line);
			}
			lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
		} else {
			char* _tmp14_;
			GError* _tmp15_;
			_inner_error_ = (_tmp15_ = g_error_new_literal (VTG_VCS_BACKENDS_VCS_ERROR, VTG_VCS_BACKENDS_VCS_ERROR_CommandFailed, _tmp14_ = g_strdup_printf (_ ("error executing the bzr status command.\n%s"), stde)), _g_free0 (_tmp14_), _tmp15_);
			if (_inner_error_ != NULL) {
				g_propagate_error (error, _inner_error_);
				_gee_collection_object_unref0 (results);
				_g_free0 (stdo);
				_g_free0 (stde);
				return NULL;
			}
		}
	}
	result = results;
	_g_free0 (stdo);
	_g_free0 (stde);
	return result;
}


static gboolean vtg_vcs_backends_bzr_real_test (VtgVcsBackendsIVcs* base, const char* path) {
	VtgVcsBackendsBzr * self;
	gboolean result;
	char* bzr_dir;
	self = (VtgVcsBackendsBzr*) base;
	g_return_val_if_fail (path != NULL, FALSE);
	bzr_dir = g_build_filename (path, ".bzr", NULL);
	if (g_file_test (bzr_dir, G_FILE_TEST_IS_DIR)) {
		result = TRUE;
		_g_free0 (bzr_dir);
		return result;
	}
	result = FALSE;
	_g_free0 (bzr_dir);
	return result;
}


static void vtg_vcs_backends_bzr_class_init (VtgVcsBackendsBzrClass * klass) {
	vtg_vcs_backends_bzr_parent_class = g_type_class_peek_parent (klass);
}


static void vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_interface_init (VtgVcsBackendsIVcsIface * iface) {
	vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_items = vtg_vcs_backends_bzr_real_get_items;
	iface->test = vtg_vcs_backends_bzr_real_test;
}


static void vtg_vcs_backends_bzr_instance_init (VtgVcsBackendsBzr * self) {
}


GType vtg_vcs_backends_bzr_get_type (void) {
	return vtg_vcs_backends_bzr_type_id;
}


GType vtg_vcs_backends_bzr_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgVcsBackendsBzrClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_vcs_backends_bzr_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgVcsBackendsBzr), 0, (GInstanceInitFunc) vtg_vcs_backends_bzr_instance_init, NULL };
	static const GInterfaceInfo vtg_vcs_backends_ivcs_info = { (GInterfaceInitFunc) vtg_vcs_backends_bzr_vtg_vcs_backends_ivcs_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_vcs_backends_bzr_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgVcsBackendsBzr", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_vcs_backends_bzr_type_id, VTG_VCS_BACKENDS_TYPE_IVCS, &vtg_vcs_backends_ivcs_info);
	return vtg_vcs_backends_bzr_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




