/*
 *  vtgvcsbackendsgit.vala - Vala developer toys for GEdit
 *  
 *  Copyright (C) 2008 - Andrea Del Signore <sejerpz@tin.it>
 *  
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *   
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *   
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place, Suite 330,
 *  Boston, MA 02111-1307, USA.
 */

#include <glib.h>
#include <glib-object.h>
#include <stdlib.h>
#include <string.h>
#include <gee.h>
#include <sys/wait.h>
#include <glib/gi18n-lib.h>
#include <glib/gstdio.h>


#define VTG_VCS_BACKENDS_TYPE_IVCS (vtg_vcs_backends_ivcs_get_type ())
#define VTG_VCS_BACKENDS_IVCS(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_IVCS, VtgVcsBackendsIVcs))
#define VTG_VCS_BACKENDS_IS_IVCS(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_IVCS))
#define VTG_VCS_BACKENDS_IVCS_GET_INTERFACE(obj) (G_TYPE_INSTANCE_GET_INTERFACE ((obj), VTG_VCS_BACKENDS_TYPE_IVCS, VtgVcsBackendsIVcsIface))

typedef struct _VtgVcsBackendsIVcs VtgVcsBackendsIVcs;
typedef struct _VtgVcsBackendsIVcsIface VtgVcsBackendsIVcsIface;

#define VTG_VCS_BACKENDS_TYPE_ITEM (vtg_vcs_backends_item_get_type ())
#define VTG_VCS_BACKENDS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItem))
#define VTG_VCS_BACKENDS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItemClass))
#define VTG_VCS_BACKENDS_IS_ITEM(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_ITEM))
#define VTG_VCS_BACKENDS_IS_ITEM_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_ITEM))
#define VTG_VCS_BACKENDS_ITEM_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_ITEM, VtgVcsBackendsItemClass))

typedef struct _VtgVcsBackendsItem VtgVcsBackendsItem;
typedef struct _VtgVcsBackendsItemClass VtgVcsBackendsItemClass;

#define VTG_VCS_BACKENDS_TYPE_GIT (vtg_vcs_backends_git_get_type ())
#define VTG_VCS_BACKENDS_GIT(obj) (G_TYPE_CHECK_INSTANCE_CAST ((obj), VTG_VCS_BACKENDS_TYPE_GIT, VtgVcsBackendsGit))
#define VTG_VCS_BACKENDS_GIT_CLASS(klass) (G_TYPE_CHECK_CLASS_CAST ((klass), VTG_VCS_BACKENDS_TYPE_GIT, VtgVcsBackendsGitClass))
#define VTG_VCS_BACKENDS_IS_GIT(obj) (G_TYPE_CHECK_INSTANCE_TYPE ((obj), VTG_VCS_BACKENDS_TYPE_GIT))
#define VTG_VCS_BACKENDS_IS_GIT_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), VTG_VCS_BACKENDS_TYPE_GIT))
#define VTG_VCS_BACKENDS_GIT_GET_CLASS(obj) (G_TYPE_INSTANCE_GET_CLASS ((obj), VTG_VCS_BACKENDS_TYPE_GIT, VtgVcsBackendsGitClass))

typedef struct _VtgVcsBackendsGit VtgVcsBackendsGit;
typedef struct _VtgVcsBackendsGitClass VtgVcsBackendsGitClass;
typedef struct _VtgVcsBackendsGitPrivate VtgVcsBackendsGitPrivate;
#define _g_free0(var) (var = (g_free (var), NULL))
#define _gee_collection_object_unref0(var) ((var == NULL) ? NULL : (var = (gee_collection_object_unref (var), NULL)))

#define VTG_VCS_BACKENDS_TYPE_STATES (vtg_vcs_backends_states_get_type ())
typedef struct _VtgVcsBackendsItemPrivate VtgVcsBackendsItemPrivate;
#define _g_regex_unref0(var) ((var == NULL) ? NULL : (var = (g_regex_unref (var), NULL)))
#define _g_error_free0(var) ((var == NULL) ? NULL : (var = (g_error_free (var), NULL)))
#define _vtg_vcs_backends_item_unref0(var) ((var == NULL) ? NULL : (var = (vtg_vcs_backends_item_unref (var), NULL)))
#define _g_pattern_spec_free0(var) ((var == NULL) ? NULL : (var = (g_pattern_spec_free (var), NULL)))

struct _VtgVcsBackendsIVcsIface {
	GTypeInterface parent_iface;
	GeeList* (*get_items) (VtgVcsBackendsIVcs* self, const char* path, GError** error);
	gboolean (*test) (VtgVcsBackendsIVcs* self, const char* path);
};

struct _VtgVcsBackendsGit {
	GObject parent_instance;
	VtgVcsBackendsGitPrivate * priv;
};

struct _VtgVcsBackendsGitClass {
	GObjectClass parent_class;
};

typedef enum  {
	VTG_VCS_BACKENDS_STATES_UNTRACKED,
	VTG_VCS_BACKENDS_STATES_ADDED,
	VTG_VCS_BACKENDS_STATES_MODIFIED
} VtgVcsBackendsStates;

struct _VtgVcsBackendsItem {
	GTypeInstance parent_instance;
	volatile int ref_count;
	VtgVcsBackendsItemPrivate * priv;
	char* name;
	VtgVcsBackendsStates state;
};

struct _VtgVcsBackendsItemClass {
	GTypeClass parent_class;
	void (*finalize) (VtgVcsBackendsItem *self);
};

typedef enum  {
	VTG_VCS_BACKENDS_VCS_ERROR_CommandFailed
} VtgVcsBackendsVcsError;
#define VTG_VCS_BACKENDS_VCS_ERROR vtg_vcs_backends_vcs_error_quark ()

static gpointer vtg_vcs_backends_git_parent_class = NULL;
static VtgVcsBackendsIVcsIface* vtg_vcs_backends_git_vtg_vcs_backends_ivcs_parent_iface = NULL;
static GType vtg_vcs_backends_git_type_id = 0;

gpointer vtg_vcs_backends_item_ref (gpointer instance);
void vtg_vcs_backends_item_unref (gpointer instance);
GParamSpec* vtg_vcs_backends_param_spec_item (const gchar* name, const gchar* nick, const gchar* blurb, GType object_type, GParamFlags flags);
void vtg_vcs_backends_value_set_item (GValue* value, gpointer v_object);
gpointer vtg_vcs_backends_value_get_item (const GValue* value);
GType vtg_vcs_backends_item_get_type (void);
GType vtg_vcs_backends_item_register_type (GTypeModule * module);
GType vtg_vcs_backends_ivcs_get_type (void);
GType vtg_vcs_backends_git_get_type (void);
GType vtg_vcs_backends_git_register_type (GTypeModule * module);
enum  {
	VTG_VCS_BACKENDS_GIT_DUMMY_PROPERTY
};
VtgVcsBackendsGit* vtg_vcs_backends_git_new (void);
VtgVcsBackendsGit* vtg_vcs_backends_git_construct (GType object_type);
VtgVcsBackendsItem* vtg_vcs_backends_item_new (void);
VtgVcsBackendsItem* vtg_vcs_backends_item_construct (GType object_type);
GType vtg_vcs_backends_states_get_type (void);
GQuark vtg_vcs_backends_vcs_error_quark (void);
static GeeList* vtg_vcs_backends_git_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error);
static gboolean vtg_vcs_backends_git_real_test (VtgVcsBackendsIVcs* base, const char* path);
static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func);
static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func);
static gint _vala_array_length (gpointer array);



VtgVcsBackendsGit* vtg_vcs_backends_git_construct (GType object_type) {
	VtgVcsBackendsGit * self;
	self = (VtgVcsBackendsGit*) g_object_new (object_type, NULL);
	return self;
}


VtgVcsBackendsGit* vtg_vcs_backends_git_new (void) {
	return vtg_vcs_backends_git_construct (VTG_VCS_BACKENDS_TYPE_GIT);
}


static char* string_replace (const char* self, const char* old, const char* replacement) {
	char* result;
	GError * _inner_error_;
	g_return_val_if_fail (self != NULL, NULL);
	g_return_val_if_fail (old != NULL, NULL);
	g_return_val_if_fail (replacement != NULL, NULL);
	_inner_error_ = NULL;
	{
		char* _tmp0_;
		GRegex* _tmp1_;
		GRegex* regex;
		char* _tmp2_;
		regex = (_tmp1_ = g_regex_new (_tmp0_ = g_regex_escape_string (old, -1), 0, 0, &_inner_error_), _g_free0 (_tmp0_), _tmp1_);
		if (_inner_error_ != NULL) {
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch27_g_regex_error;
			}
			goto __finally27;
		}
		_tmp2_ = g_regex_replace_literal (regex, self, (glong) (-1), 0, replacement, 0, &_inner_error_);
		if (_inner_error_ != NULL) {
			_g_regex_unref0 (regex);
			if (_inner_error_->domain == G_REGEX_ERROR) {
				goto __catch27_g_regex_error;
			}
			goto __finally27;
		}
		result = _tmp2_;
		_g_regex_unref0 (regex);
		return result;
	}
	goto __finally27;
	__catch27_g_regex_error:
	{
		GError * e;
		e = _inner_error_;
		_inner_error_ = NULL;
		{
			g_assert_not_reached ();
			_g_error_free0 (e);
		}
	}
	__finally27:
	if (_inner_error_ != NULL) {
		g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
		g_clear_error (&_inner_error_);
		return NULL;
	}
}


static GeeList* vtg_vcs_backends_git_real_get_items (VtgVcsBackendsIVcs* base, const char* path, GError** error) {
	VtgVcsBackendsGit * self;
	GeeList* result;
	GError * _inner_error_;
	GeeList* results;
	char* stdo;
	char* stde;
	gint exit_status;
	char* _tmp7_;
	gboolean _tmp6_;
	char* _tmp5_;
	char* _tmp4_;
	gboolean _tmp3_;
	char* _tmp2_;
	gint _tmp1__length1;
	char** _tmp1_;
	char** _tmp0_;
	gboolean _tmp8_;
	gboolean _tmp9_;
	self = (VtgVcsBackendsGit*) base;
	g_return_val_if_fail (path != NULL, NULL);
	_inner_error_ = NULL;
	results = (GeeList*) gee_array_list_new (VTG_VCS_BACKENDS_TYPE_ITEM, (GBoxedCopyFunc) vtg_vcs_backends_item_ref, vtg_vcs_backends_item_unref, g_direct_equal);
	stdo = NULL;
	stde = NULL;
	exit_status = 0;
	_tmp5_ = NULL;
	_tmp2_ = NULL;
	_tmp0_ = NULL;
	_tmp9_ = (_tmp8_ = (_tmp6_ = (_tmp3_ = g_spawn_sync (path, (_tmp1_ = (_tmp0_ = g_new0 (char*, 2 + 1), _tmp0_[0] = g_strdup ("git"), _tmp0_[1] = g_strdup ("status"), _tmp0_), _tmp1__length1 = 2, _tmp1_), NULL, G_SPAWN_SEARCH_PATH, NULL, NULL, &_tmp2_, &_tmp5_, &exit_status, &_inner_error_), stdo = (_tmp4_ = _tmp2_, _g_free0 (stdo), _tmp4_), _tmp3_), stde = (_tmp7_ = _tmp5_, _g_free0 (stde), _tmp7_), _tmp6_), _tmp1_ = (_vala_array_free (_tmp1_, _tmp1__length1, (GDestroyNotify) g_free), NULL), _tmp8_);
	if (_inner_error_ != NULL) {
		if (_inner_error_->domain == G_SPAWN_ERROR) {
			g_propagate_error (error, _inner_error_);
			_gee_collection_object_unref0 (results);
			_g_free0 (stdo);
			_g_free0 (stde);
			return NULL;
		} else {
			_gee_collection_object_unref0 (results);
			_g_free0 (stdo);
			_g_free0 (stde);
			g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
			g_clear_error (&_inner_error_);
			return NULL;
		}
	}
	if (_tmp9_) {
		gint exit;
		gboolean _tmp10_;
		exit = WEXITSTATUS (exit_status);
		_tmp10_ = FALSE;
		if (exit == 0) {
			_tmp10_ = TRUE;
		} else {
			_tmp10_ = exit == 1;
		}
		if (_tmp10_) {
			char** _tmp12_;
			gint lines_size;
			gint lines_length1;
			char** _tmp11_;
			char** lines;
			gint idx;
			GPatternSpec* modp;
			GPatternSpec* newp;
			lines = (_tmp12_ = _tmp11_ = g_strsplit (stdo, "\n", 0), lines_length1 = _vala_array_length (_tmp11_), lines_size = lines_length1, _tmp12_);
			idx = 0;
			modp = g_pattern_spec_new ("#\tmodified:   *");
			newp = g_pattern_spec_new ("#\tnew file:   *");
			while (TRUE) {
				char* line;
				if (!(lines[idx] != NULL)) {
					break;
				}
				line = g_strdup (lines[idx]);
				if (g_pattern_match_string (modp, line)) {
					VtgVcsBackendsItem* item;
					char* _tmp13_;
					item = vtg_vcs_backends_item_new ();
					item->state = VTG_VCS_BACKENDS_STATES_MODIFIED;
					item->name = (_tmp13_ = string_replace (line, "#\tmodified:   ", ""), _g_free0 (item->name), _tmp13_);
					gee_collection_add ((GeeCollection*) results, item);
					_vtg_vcs_backends_item_unref0 (item);
				} else {
					if (g_pattern_match_string (newp, line)) {
						VtgVcsBackendsItem* item;
						char* _tmp14_;
						item = vtg_vcs_backends_item_new ();
						item->state = VTG_VCS_BACKENDS_STATES_ADDED;
						item->name = (_tmp14_ = string_replace (line, "#\tnew file:   ", ""), _g_free0 (item->name), _tmp14_);
						gee_collection_add ((GeeCollection*) results, item);
						_vtg_vcs_backends_item_unref0 (item);
					}
				}
				idx++;
				_g_free0 (line);
			}
			lines = (_vala_array_free (lines, lines_length1, (GDestroyNotify) g_free), NULL);
			_g_pattern_spec_free0 (modp);
			_g_pattern_spec_free0 (newp);
		} else {
			char* _tmp15_;
			GError* _tmp16_;
			_inner_error_ = (_tmp16_ = g_error_new_literal (VTG_VCS_BACKENDS_VCS_ERROR, VTG_VCS_BACKENDS_VCS_ERROR_CommandFailed, _tmp15_ = g_strdup_printf (_ ("error executing the git status command.\n%s"), stde)), _g_free0 (_tmp15_), _tmp16_);
			if (_inner_error_ != NULL) {
				if (_inner_error_->domain == G_SPAWN_ERROR) {
					g_propagate_error (error, _inner_error_);
					_gee_collection_object_unref0 (results);
					_g_free0 (stdo);
					_g_free0 (stde);
					return NULL;
				} else {
					_gee_collection_object_unref0 (results);
					_g_free0 (stdo);
					_g_free0 (stde);
					g_critical ("file %s: line %d: uncaught error: %s", __FILE__, __LINE__, _inner_error_->message);
					g_clear_error (&_inner_error_);
					return NULL;
				}
			}
		}
	}
	result = results;
	_g_free0 (stdo);
	_g_free0 (stde);
	return result;
}


static gboolean vtg_vcs_backends_git_real_test (VtgVcsBackendsIVcs* base, const char* path) {
	VtgVcsBackendsGit * self;
	gboolean result;
	char* git_dir;
	self = (VtgVcsBackendsGit*) base;
	g_return_val_if_fail (path != NULL, FALSE);
	git_dir = g_build_filename (path, ".git", NULL);
	if (g_file_test (git_dir, G_FILE_TEST_IS_DIR)) {
		result = TRUE;
		_g_free0 (git_dir);
		return result;
	}
	result = FALSE;
	_g_free0 (git_dir);
	return result;
}


static void vtg_vcs_backends_git_class_init (VtgVcsBackendsGitClass * klass) {
	vtg_vcs_backends_git_parent_class = g_type_class_peek_parent (klass);
}


static void vtg_vcs_backends_git_vtg_vcs_backends_ivcs_interface_init (VtgVcsBackendsIVcsIface * iface) {
	vtg_vcs_backends_git_vtg_vcs_backends_ivcs_parent_iface = g_type_interface_peek_parent (iface);
	iface->get_items = vtg_vcs_backends_git_real_get_items;
	iface->test = vtg_vcs_backends_git_real_test;
}


static void vtg_vcs_backends_git_instance_init (VtgVcsBackendsGit * self) {
}


GType vtg_vcs_backends_git_get_type (void) {
	return vtg_vcs_backends_git_type_id;
}


GType vtg_vcs_backends_git_register_type (GTypeModule * module) {
	static const GTypeInfo g_define_type_info = { sizeof (VtgVcsBackendsGitClass), (GBaseInitFunc) NULL, (GBaseFinalizeFunc) NULL, (GClassInitFunc) vtg_vcs_backends_git_class_init, (GClassFinalizeFunc) NULL, NULL, sizeof (VtgVcsBackendsGit), 0, (GInstanceInitFunc) vtg_vcs_backends_git_instance_init, NULL };
	static const GInterfaceInfo vtg_vcs_backends_ivcs_info = { (GInterfaceInitFunc) vtg_vcs_backends_git_vtg_vcs_backends_ivcs_interface_init, (GInterfaceFinalizeFunc) NULL, NULL};
	vtg_vcs_backends_git_type_id = g_type_module_register_type (module, G_TYPE_OBJECT, "VtgVcsBackendsGit", &g_define_type_info, 0);
	g_type_add_interface_static (vtg_vcs_backends_git_type_id, VTG_VCS_BACKENDS_TYPE_IVCS, &vtg_vcs_backends_ivcs_info);
	return vtg_vcs_backends_git_type_id;
}


static void _vala_array_destroy (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	if ((array != NULL) && (destroy_func != NULL)) {
		int i;
		for (i = 0; i < array_length; i = i + 1) {
			if (((gpointer*) array)[i] != NULL) {
				destroy_func (((gpointer*) array)[i]);
			}
		}
	}
}


static void _vala_array_free (gpointer array, gint array_length, GDestroyNotify destroy_func) {
	_vala_array_destroy (array, array_length, destroy_func);
	g_free (array);
}


static gint _vala_array_length (gpointer array) {
	int length;
	length = 0;
	if (array) {
		while (((gpointer*) array)[length]) {
			length++;
		}
	}
	return length;
}




